@@ -193,6 +193,10 @@ GA_NO_ENEMY   = 512
 GA_NO_NEUTRAL = 1024
 GA_NO_SELF    = 2048
 
+# Game features, for Interface::SetFeatures()
+# Defined in globals.h
+GF_ALL_STRINGS_TAGGED = 1
+
 # Shadow color for ShowModal()
 # !!! Keep these synchronized with Interface.h !!!
 MODAL_SHADOW_NONE = 0
@@ -254,6 +258,7 @@ IE_ITEM_UNSELLABLE   = (IE_ITEM_CRITICAL | IE_ITEM_STOLEN)
 IE_INV_ITEM_IDENTIFIED    = 0x01
 IE_INV_ITEM_UNSTEALABLE   = 0x02
 IE_INV_ITEM_STOLEN        = 0x04
+IE_INV_ITEM_STEEL         = 0x04 # pst only
 IE_INV_ITEM_UNDROPPABLE   = 0x08
 # GemRB extensions
 IE_INV_ITEM_ACQUIRED      = 0x10
@@ -285,6 +290,7 @@ SHOP_SELL   = 2
 SHOP_ID     = 4
 SHOP_STEAL  = 8
 SHOP_SELECT = 0x40
+SHOP_FULL   = 0x8000
 
 #game constants
 PARTY_SIZE = 6
