@@ -23,12 +23,14 @@
 ###################################################
 
 import GemRB
+import GameCheck
 import GUICommon
 import GUICommonWindows
 from GUIDefines import *
 from ie_stats import *
 from ie_slots import *
 from ie_sounds import *
+from ie_feats import FEAT_MERCANTILE_BACKGROUND
 
 StoreWindow = None
 MessageWindow = None
@@ -50,13 +52,12 @@ ITEM_PC    = 0
 ITEM_STORE = 1
 
 Inventory = None
-RentIndex = -1
 Store = None
 Buttons = [-1,-1,-1,-1]
 inventory_slots = ()
 total_price = 0
 total_income = 0
-if GUICommon.GameIsIWD2():
+if GameCheck.IsIWD2():
 	ItemButtonCount = 6
 else:
 	ItemButtonCount = 4
@@ -80,7 +81,7 @@ BarteringPC = 0
 # 5 - drink
 # 6 - rent
 
-if GUICommon.GameIsIWD1():
+if GameCheck.IsIWD1():
 	# no bam for bags
 	storebams = ("STORSTOR","STORTVRN","STORINN","STORTMPL","STORSTOR","STORSTOR")
 else:
@@ -111,17 +112,17 @@ def CloseStoreWindow ():
 		StoreWindow.Unload ()
 	if ActionWindow:
 		ActionWindow.Unload ()
-	if not GUICommon.GameIsBG1():
+	if not GameCheck.IsBG1():
 		if PortraitWindow:
 			PortraitWindow.Unload ()
 	StoreWindow = None
 	GemRB.LeaveStore ()
-	if not GUICommon.GameIsBG1():
+	if not GameCheck.IsBG1():
 		GUICommonWindows.PortraitWindow = OldPortraitWindow
 	if Inventory:
 		GUIINV.OpenInventoryWindow ()
 	else:
-		GemRB.GamePause (0, 1)
+		GemRB.GamePause (0, 3)
 		GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE) #enabling the game control screen
 		GemRB.UnhideGUI () #enabling the other windows
 		GUICommonWindows.SetSelectionChangeHandler( None )
@@ -153,21 +154,14 @@ def OpenStoreWindow ():
 	else:
 		Inventory = None
 		# pause the game, so we don't get interrupted
-		GemRB.GamePause (1, 1)
+		GemRB.GamePause (1, 3)
 
 	GemRB.SetVar ("Action", 0)
-	if GUICommon.GameIsIWD2():
+	if GameCheck.IsIWD2():
 		GemRB.LoadWindowPack ("GUISTORE", 800, 600)
 	else:
 		GemRB.LoadWindowPack ("GUISTORE", 640, 480)
 	StoreWindow = Window = GemRB.LoadWindow (3)
-	#saving the original portrait window
-	OldPortraitWindow = GUICommonWindows.PortraitWindow
-	if GUICommon.GameIsIWD2() or GUICommon.GameIsBG1():
-		#PortraitWindow = GUICommonWindows.OpenPortraitWindow ()
-		pass
-	else:
-		PortraitWindow = GUICommonWindows.OpenPortraitWindow (0)
 	ActionWindow = GemRB.LoadWindow (0)
 	#this window is static and grey, but good to stick the frame onto
 	ActionWindow.SetFrame ()
@@ -181,7 +175,7 @@ def OpenStoreWindow ():
 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseStoreWindow)
 
 	#Store type icon
-	if not GUICommon.GameIsIWD2():
+	if not GameCheck.IsIWD2():
 		Button = Window.GetControl (5)
 		Button.SetSprites (storebams[Store['StoreType']],0,0,0,0,0)
 
@@ -194,7 +188,7 @@ def OpenStoreWindow ():
 		Button.SetVarAssoc ("Action", i)
 		if Action>=0:
 			Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-			if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
+			if GameCheck.IsIWD1() or GameCheck.IsIWD2():
 				Button.SetSprites ("GUISTBBC", Action, 1,2,0,0)
 			else:
 				Button.SetSprites ("GUISTBBC", Action, 0,1,2,0)
@@ -210,8 +204,19 @@ def OpenStoreWindow ():
 	ActionWindow.SetVisible (WINDOW_VISIBLE)
 	Window.SetVisible (WINDOW_VISIBLE)
 	store_funcs[store_buttons[0]] ()
-	if not GUICommon.GameIsIWD2():
-		if GUICommon.GameIsBG1():
+
+	# Portrait window must be loaded after store, because UpdatePortraitWindow
+	# checks GUISTORE.StoreHealWindow
+	#saving the original portrait window
+	OldPortraitWindow = GUICommonWindows.PortraitWindow
+	if GameCheck.IsIWD2() or GameCheck.IsBG1():
+		#PortraitWindow = GUICommonWindows.OpenPortraitWindow ()
+		pass
+	else:
+		PortraitWindow = GUICommonWindows.OpenPortraitWindow (0)
+
+	if not GameCheck.IsIWD2():
+		if GameCheck.IsBG1():
 			GUICommonWindows.PortraitWindow.SetVisible (WINDOW_VISIBLE)
 		else:
 			PortraitWindow.SetVisible (WINDOW_VISIBLE)
@@ -223,6 +228,11 @@ def OpenStoreShoppingWindow ():
 
 	CloseWindows()
 
+	GemRB.SetVar ("LeftIndex", 0) # reset the shopkeeps list
+	GemRB.SetVar ("LeftTopIndex", 0)
+	GemRB.SetVar ("RightTopIndex", 0)
+	GemRB.SetVar ("LeftTopIndex", 0)
+
 	StoreShoppingWindow = Window = GemRB.LoadWindow (2)
 
 	# left scrollbar
@@ -236,9 +246,9 @@ def OpenStoreShoppingWindow ():
 	if Inventory:
 		# Title
 		Label = Window.GetControl (0xfffffff)
-		if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
+		if GameCheck.IsIWD1() or GameCheck.IsIWD2():
 			Label.SetText (26291)
-		elif GUICommon.GameIsBG2():
+		elif GameCheck.IsBG2():
 			Label.SetText (51881)
 		else:
 			Label.SetText ("")
@@ -265,18 +275,18 @@ def OpenStoreShoppingWindow ():
 
 	for i in range (ItemButtonCount):
 		Button = Window.GetControl (i+5)
-		if GUICommon.GameIsBG2():
+		if GameCheck.IsBG2():
 			Button.SetBorder (0,0,0,0,0,0,0,128,160,0,1)
 		else:
 			Button.SetBorder (0,0,0,0,0,32,32,192,128,0,1)
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectBuy)
 		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InfoLeftWindow)
 		Button.SetFont ("NUMBER")
-		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_TOP, OP_OR)
+		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)
 		Button.AttachScrollBar (ScrollBarLeft)
 
 		Button = Window.GetControl (i+13)
-		if GUICommon.GameIsBG2():
+		if GameCheck.IsBG2():
 			Button.SetBorder (0,0,0,0,0,0,0,128,160,0,1)
 			Button.SetSprites ("GUIBTBUT", 0, 0,1,2,5)
 		else:
@@ -285,17 +295,17 @@ def OpenStoreShoppingWindow ():
 			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectSell)
 		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InfoRightWindow)
 		Button.SetFont ("NUMBER")
-		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_TOP, OP_OR)
+		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)
 		Button.AttachScrollBar (ScrollBarRight)
 
+	UnselectNoRedraw ()
+
 	# Buy
 	LeftButton = Button = Window.GetControl (2)
 	if Inventory:
-		if GUICommon.GameIsIWD2():
+		if GameCheck.IsIWD1() or GameCheck.IsIWD2():
 			Button.SetText (26287)
-		elif GUICommon.GameIsIWD1():
-			Button.SetText (26288)
-		elif GUICommon.GameIsBG2():
+		elif GameCheck.IsBG2():
 			Button.SetText (51882)
 		else:
 			Button.SetText ("")
@@ -307,9 +317,9 @@ def OpenStoreShoppingWindow ():
 	# Sell
 	RightButton = Button = Window.GetControl (3)
 	if Inventory:
-		if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
+		if GameCheck.IsIWD1() or GameCheck.IsIWD2():
 			Button.SetText (26288)
-		elif GUICommon.GameIsBG2():
+		elif GameCheck.IsBG2():
 			Button.SetText (51883)
 		else:
 			Button.SetText ("")
@@ -320,7 +330,7 @@ def OpenStoreShoppingWindow ():
 			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SellPressed)
 
 	# inactive button
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		Button = Window.GetControl (50)
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
@@ -330,8 +340,11 @@ def OpenStoreShoppingWindow ():
 	Button.SetState (IE_GUI_BUTTON_LOCKED)
 
 	# encumbrance
-	Label = Window.CreateLabel (0x10000043, 15,325,60,15,"NUMBER","0:",IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_TOP)
-	Label = Window.CreateLabel (0x10000044, 15,365,80,15,"NUMBER","0:",IE_FONT_ALIGN_RIGHT|IE_FONT_ALIGN_TOP)
+	r = Button.GetRect ()
+	Label = Window.CreateLabel (0x10000043, r["X"],r["Y"],r["Width"],15,
+		"NUMBER","0:",IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_TOP)
+	Label = Window.CreateLabel (0x10000044, r["X"],r["Y"]+r["Height"]-15,r["Width"],15,
+		"NUMBER","0:",IE_FONT_ALIGN_RIGHT|IE_FONT_ALIGN_BOTTOM)
 
 	GUICommonWindows.SetSelectionChangeHandler( UpdateStoreShoppingWindow )
 	UpdateStoreShoppingWindow ()
@@ -368,19 +381,21 @@ def OpenStoreIdentifyWindow ():
 	for i in range (ItemButtonCount):
 		Button = Window.GetControl (i+8)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-		if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
+		if GameCheck.IsIWD1() or GameCheck.IsIWD2():
 			Button.SetSprites ("GUISTMSC", 0, 1,2,0,3)
 			Button.SetBorder (0,0,0,0,0,32,32,192,128,0,1)
-		elif GUICommon.GameIsBG1():
+		elif GameCheck.IsBG1():
 			Button.SetBorder (0,0,0,0,0,32,32,192,128,0,1)
 		else:
 			Button.SetBorder (0,0,0,0,0,0,0,128,160,0,1)
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectID)
 		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InfoIdentifyWindow)
 		Button.SetFont ("NUMBER")
-		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_TOP, OP_OR)
+		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)
 		Button.AttachScrollBar (ScrollBar)
 
+	UnselectNoRedraw ()
+
 	GUICommonWindows.SetSelectionChangeHandler( UpdateStoreIdentifyWindow )
 	UpdateStoreIdentifyWindow ()
 	Window.SetVisible (WINDOW_VISIBLE)
@@ -408,23 +423,27 @@ def OpenStoreStealWindow ():
 
 	for i in range (ItemButtonCount):
 		Button = Window.GetControl (i+4)
-		if GUICommon.GameIsBG2():
+		if GameCheck.IsBG2():
 			Button.SetBorder (0,0,0,0,0,0,0,128,160,0,1)
 		else:
 			Button.SetBorder (0,0,0,0,0,32,32,192,128,0,1)
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RedrawStoreStealWindow)
+		Button.SetFont ("NUMBER")
+		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)
 		Button.AttachScrollBar (ScrollBarLeft)
 
 		Button = Window.GetControl (i+11)
-		if GUICommon.GameIsBG2():
+		if GameCheck.IsBG2():
 			Button.SetBorder (0,0,0,0,0,0,0,128,160,0,1)
 		else:
 			Button.SetBorder (0,0,0,0,0,32,32,192,128,0,1)
 		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InfoRightWindow)
 		Button.SetFont ("NUMBER")
-		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_TOP, OP_OR)
+		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)
 		Button.AttachScrollBar (ScrollBarRight)
 
+	UnselectNoRedraw ()
+
 	# Steal
 	LeftButton = Button = Window.GetControl (1)
 	Button.SetText (14179)
@@ -434,8 +453,11 @@ def OpenStoreStealWindow ():
 	Button.SetState (IE_GUI_BUTTON_LOCKED)
 
 	# encumbrance
-	Label = Window.CreateLabel (0x10000043, 15,325,60,15,"NUMBER","0:",IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_TOP)
-	Label = Window.CreateLabel (0x10000044, 15,365,80,15,"NUMBER","0:",IE_FONT_ALIGN_RIGHT|IE_FONT_ALIGN_TOP)
+	r = Button.GetRect ()
+	Label = Window.CreateLabel (0x10000043, r["X"],r["Y"],r["Width"],15,
+		"NUMBER","0:",IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_TOP)
+	Label = Window.CreateLabel (0x10000044, r["X"],r["Y"]+r["Height"]-15,r["Width"],15,
+		"NUMBER","0:",IE_FONT_ALIGN_RIGHT|IE_FONT_ALIGN_BOTTOM)
 
 	GUICommonWindows.SetSelectionChangeHandler( UpdateStoreStealWindow )
 	UpdateStoreStealWindow ()
@@ -451,7 +473,7 @@ def OpenStoreDonateWindow ():
 
 	# graphics
 	Button = Window.GetControl (10)
-	Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ANIMATED|IE_GUI_BUTTON_PLAYONCE, OP_OR)
+	Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ANIMATED|IE_GUI_BUTTON_PLAYONCE|IE_GUI_BUTTON_PLAYALWAYS, OP_OR)
 	Button.SetState (IE_GUI_BUTTON_LOCKED)
 
 	# Donate
@@ -498,6 +520,8 @@ def OpenStoreHealWindow ():
 		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, InfoHealWindow)
 		#Button.AttachScrollBar (ScrollBar)
 
+	UnselectNoRedraw ()
+
 	# price tag
 	Label = Window.GetControl (0x10000003)
 	Label.SetText ("0")
@@ -513,7 +537,7 @@ def OpenStoreHealWindow ():
 		Count = Count-4
 	else:
 		Count = 0
-	ScrollBar.SetVarAssoc ("TopIndex", Count+1)
+	ScrollBar.SetVarAssoc ("TopIndex", Count)
 
 	GUICommonWindows.SetSelectionChangeHandler( UpdateStoreHealWindow )
 	UpdateStoreHealWindow ()
@@ -529,14 +553,19 @@ def OpenStoreRumourWindow ():
 	StoreRumourWindow = Window = GemRB.LoadWindow (8)
 
 	#removing those pesky labels
-	for i in range (5):
-		Window.DeleteControl (0x10000005+i)
+	if not GameCheck.IsIWD2():
+		for i in range (5):
+			Window.DeleteControl (0x10000005+i)
 
-	TextArea = Window.GetControl (11)
+	TextArea = None
+	if GameCheck.IsIWD2():
+		TextArea = Window.GetControl (13)
+	else:
+		TextArea = Window.GetControl (11)
 	TextArea.SetText (14144)
 
 	#tavern quality image
-	if GUICommon.GameIsBG1() or GUICommon.GameIsBG2():
+	if GameCheck.IsBG1() or GameCheck.IsBG2():
 		BAM = "TVRNQUL%d"% ((Store['StoreFlags']>>9)&3)
 		Button = Window.GetControl (12)
 		Button.SetSprites (BAM, 0, 0, 0, 0, 0)
@@ -545,11 +574,11 @@ def OpenStoreRumourWindow ():
 	ScrollBar = Window.GetControl (5)
 	ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, UpdateStoreRumourWindow)
 	Count = Store['StoreDrinkCount']
-	if Count>4:
-		Count = Count-4
+	if Count>5:
+		Count = Count-5
 	else:
 		Count = 0
-	ScrollBar.SetVarAssoc ("TopIndex", Count+1)
+	ScrollBar.SetVarAssoc ("TopIndex", Count)
 
 	GUICommonWindows.SetSelectionChangeHandler( UpdateStoreRumourWindow )
 	UpdateStoreRumourWindow ()
@@ -557,7 +586,7 @@ def OpenStoreRumourWindow ():
 	return
 
 def OpenStoreRentWindow ():
-	global StoreRentWindow, RentIndex
+	global StoreRentWindow
 
 	CloseWindows()
 
@@ -565,6 +594,15 @@ def OpenStoreRentWindow ():
 
 	# room types
 	RentIndex = -1
+	for i in range (4):
+		ok = Store['StoreRoomPrices'][i]
+		if ok >= 0:
+			RentIndex = i
+			break
+
+	# RentIndex needs to be set before SetVarAssoc
+	GemRB.SetVar ("RentIndex", RentIndex)
+
 	for i in range (4):
 		ok = Store['StoreRoomPrices'][i]
 		Button = Window.GetControl (i)
@@ -573,15 +611,13 @@ def OpenStoreRentWindow ():
 			Button.SetState (IE_GUI_BUTTON_DISABLED) #disabled room icons are selected, not disabled
 		else:
 			Button.SetVarAssoc ("RentIndex", i)
-			if RentIndex==-1:
-				RentIndex = i
 
 		Button = Window.GetControl (i+4)
 		Button.SetText (14294+i)
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateStoreRentWindow)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 		Button.SetVarAssoc ("RentIndex", i)
-		if GUICommon.GameIsBG1():
+		if GameCheck.IsBG1():
 			#these bioware guys screw up everything possible
 			#remove this line if you fixed guistore
 			Button.SetSprites ("GUISTROC",0, 1,2,0,3)
@@ -593,8 +629,6 @@ def OpenStoreRentWindow ():
 	Button.SetText (14293)
 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RentRoom)
 
-	GemRB.SetVar ("RentIndex", RentIndex)
-
 	GUICommonWindows.SetSelectionChangeHandler( UpdateStoreRentWindow )
 	UpdateStoreRentWindow ()
 	Window.SetVisible (WINDOW_VISIBLE)
@@ -602,8 +636,9 @@ def OpenStoreRentWindow ():
 
 def UpdateStoreCommon (Window, title, name, gold):
 
-	Label = Window.GetControl (title)
-	Label.SetText (Store['StoreName'])
+	if Store['StoreName'] != -1:
+		Label = Window.GetControl (title)
+		Label.SetText (Store['StoreName'])
 
 	if name:
 		pc = GemRB.GameGetSelectedPCSingle ()
@@ -623,11 +658,10 @@ def GetPC():
 			PreviousPC = pc
 			# reset the store indices, to prevent overscrolling
 			GemRB.SetVar ("RightIndex", 0)
-			GemRB.SetVar ("LeftIndex", 0)
 			GemRB.SetVar ("RightTopIndex", 0)
-			GemRB.SetVar ("LeftTopIndex", 0)
 			GemRB.SetVar ("Index", 0)
 			GemRB.SetVar ("TopIndex", 0)
+			UnselectNoRedraw ()
 	else:
 		PreviousPC = GemRB.GameGetSelectedPCSingle ()
 		pc = PreviousPC
@@ -642,7 +676,7 @@ def UpdateStoreShoppingWindow ():
 	Store = GemRB.GetStore ()
 	pc = GetPC()
 
-	LeftCount = Store['StoreItemCount'] - ItemButtonCount + 1
+	LeftCount = Store['StoreItemCount'] - ItemButtonCount
 	if LeftCount<0:
 		LeftCount=0
 	ScrollBar = Window.GetControl (11)
@@ -653,7 +687,7 @@ def UpdateStoreShoppingWindow ():
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 	inventory_slots = GemRB.GetSlots (pc, SLOT_INVENTORY)
-	RightCount = len(inventory_slots) - ItemButtonCount + 1
+	RightCount = len(inventory_slots) - ItemButtonCount
 	if RightCount<0:
 		RightCount=0
 	ScrollBar = Window.GetControl (12)
@@ -665,6 +699,22 @@ def UpdateStoreShoppingWindow ():
 	RedrawStoreShoppingWindow ()
 	return
 
+# Unselects all the selected buttons, so they are not preselected in other windows
+def UnselectNoRedraw ():
+	pc = GemRB.GameGetSelectedPCSingle ()
+	LeftCount = Store['StoreItemCount']
+	for i in range (LeftCount, 0, -1):
+		Flags = GemRB.IsValidStoreItem (pc, i-1, ITEM_STORE)&SHOP_SELECT
+		if Flags:
+			GemRB.ChangeStoreItem (pc, i-1, SHOP_BUY|SHOP_SELECT)
+
+	RightCount = len (inventory_slots)
+	for Slot in range (RightCount):
+		Flags = GemRB.IsValidStoreItem (pc, inventory_slots[Slot], ITEM_PC) & SHOP_SELECT
+		if Flags:
+			GemRB.ChangeStoreItem (pc, inventory_slots[Slot], SHOP_SELL|SHOP_SELECT)
+			# same code for ID, so no repeat needed
+
 def SelectID ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	Index = GemRB.GetVar ("Index")
@@ -806,8 +856,17 @@ def RedrawStoreShoppingWindow ():
 		Label.SetText ("")
 	else:
 		Label.SetText (str(BuySum) )
+
+	# shade the inventory icon if it is full
+	free_slots = len(GemRB.GetSlots (pc, SLOT_INVENTORY, -1))
+	Button = Window.GetControl (44)
+	if free_slots == 0:
+		Button.SetState (IE_GUI_BUTTON_PRESSED)
+	else:
+		Button.SetState (IE_GUI_BUTTON_LOCKED)
+
 	# also disable the button if the inventory is full
-	if BuySum and selected_count <= len(GemRB.GetSlots (pc, SLOT_INVENTORY, -1)):
+	if BuySum and selected_count <= free_slots:
 		LeftButton.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
 		LeftButton.SetState (IE_GUI_BUTTON_DISABLED)
@@ -841,6 +900,7 @@ def RedrawStoreShoppingWindow ():
 		Button.SetVarAssoc ("RightIndex", RightTopIndex+i)
 		SetupItems (pc, Slot, Button, Label, i, ITEM_PC, idx)
 
+	GUICommon.SetEncumbranceLabels ( Window, 0x10000043, 0x10000044, pc)
 	return
 
 def UpdateStoreIdentifyWindow ():
@@ -852,7 +912,7 @@ def UpdateStoreIdentifyWindow ():
 	inventory_slots = GemRB.GetSlots (pc, SLOT_INVENTORY)
 	Count = len(inventory_slots)
 	ScrollBar = Window.GetControl (7)
-	ScrollBar.SetVarAssoc ("TopIndex", Count-ItemButtonCount+1)
+	ScrollBar.SetVarAssoc ("TopIndex", Count-ItemButtonCount)
 	GemRB.SetVar ("Index", -1)
 	RedrawStoreIdentifyWindow ()
 	return
@@ -880,7 +940,7 @@ def RedrawStoreIdentifyWindow ():
 			Slot = None
 		Button = Window.GetControl (i+8)
 		# TODO: recheck they really differ
-		if GUICommon.GameIsIWD2():
+		if GameCheck.IsIWD2():
 			Label = Window.GetControl (0x1000000d+i)
 		else:
 			Label = Window.GetControl (0x1000000c+i)
@@ -915,6 +975,7 @@ def RedrawStoreIdentifyWindow ():
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
 			Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
+			Button.SetText ("")
 			Label.SetText ("")
 
 	Button = Window.GetControl (5)
@@ -997,7 +1058,7 @@ def InfoWindow (Slot, Item):
 	MessageWindow = Window = GemRB.LoadWindow (12)
 
 	# TODO: check non-bg2 games to see which label is which
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		NameLabel = Window.GetControl (0x10000000)
 		FakeLabel = Window.GetControl (0x10000007)
 	else:
@@ -1008,7 +1069,7 @@ def InfoWindow (Slot, Item):
 	FakeLabel.SetText ("")
 
 	#description bam
-	if GUICommon.GameIsBG1() or GUICommon.GameIsBG2():
+	if GameCheck.IsBG1() or GameCheck.IsBG2():
 		Button = Window.GetControl (7)
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_CENTER_PICTURES | IE_GUI_BUTTON_NO_IMAGE, OP_OR)
 		Button.SetItemIcon (Slot['ItemResRef'], 2)
@@ -1031,7 +1092,7 @@ def InfoWindow (Slot, Item):
 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ErrorDone)
 
 	# hide the empty button
-	if GUICommon.GameIsBG2() or GUICommon.GameIsIWD2():
+	if GameCheck.IsBG2() or GameCheck.IsIWD2():
 		Window.DeleteControl (9)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
@@ -1046,13 +1107,13 @@ def UpdateStoreStealWindow ():
 	Store = GemRB.GetStore ()
 	LeftCount = Store['StoreItemCount']
 	ScrollBar = Window.GetControl (9)
-	ScrollBar.SetVarAssoc ("LeftTopIndex", LeftCount-ItemButtonCount+1)
+	ScrollBar.SetVarAssoc ("LeftTopIndex", LeftCount-ItemButtonCount)
 
 	pc = GetPC()
 	inventory_slots = GemRB.GetSlots (pc, SLOT_INVENTORY)
 	RightCount = len(inventory_slots)
 	ScrollBar = Window.GetControl (10)
-	ScrollBar.SetVarAssoc ("RightTopIndex", RightCount-ItemButtonCount+1)
+	ScrollBar.SetVarAssoc ("RightTopIndex", RightCount-ItemButtonCount)
 	GemRB.SetVar ("LeftIndex", -1)
 	LeftButton.SetState (IE_GUI_BUTTON_DISABLED)
 	RedrawStoreStealWindow ()
@@ -1111,11 +1172,21 @@ def RedrawStoreStealWindow ():
 		if Flags & SHOP_SELECT:
 			selected_count += 1
 
+	# shade the inventory icon if it is full
+	Button = Window.GetControl (37)
+	free_slots = len(GemRB.GetSlots (pc, SLOT_INVENTORY, -1))
+	if free_slots == 0:
+		Button.SetState (IE_GUI_BUTTON_PRESSED)
+	else:
+		Button.SetState (IE_GUI_BUTTON_LOCKED)
+
 	# also disable the button if the inventory is full
-	if LeftIndex>=0 and selected_count <= len(GemRB.GetSlots (pc, SLOT_INVENTORY, -1)):
+	if LeftIndex>=0 and selected_count <= free_slots:
 		LeftButton.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
 		LeftButton.SetState (IE_GUI_BUTTON_DISABLED)
+
+	GUICommon.SetEncumbranceLabels ( Window, 0x10000043, 0x10000044, pc)
 	return
 
 def SetupItems (pc, Slot, Button, Label, i, type, idx, steal=0):
@@ -1191,9 +1262,9 @@ def SetupItems (pc, Slot, Button, Label, i, type, idx, steal=0):
 
 		GemRB.SetToken ("ITEMNAME", Name)
 		if Inventory:
-			if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
+			if GameCheck.IsIWD1() or GameCheck.IsIWD2():
 				LabelText = GemRB.GetString (24890)
-			elif GUICommon.GameIsBG2():
+			elif GameCheck.IsBG2():
 				LabelText = GemRB.GetString (28337)
 			else:
 				LabelText = ""
@@ -1212,16 +1283,20 @@ def GetRealPrice (pc, mode, Item, Slot):
 	# get the base from the item
 	price = Item['Price']
 
-	# FIXME: actually, the price for wands depends on their charge
-	# how ...
 	if Item['MaxStackAmount']>1:
 		price = price * Slot['Usages0']
+	elif Item['MaxCharge']>0:
+		price = price * Slot['Usages0'] / Item['MaxCharge']
 
 	# modifier from store properties (in percent)
 	if mode == "buy":
 		mod = Store['BuyMarkup']
+		if GemRB.HasFeat(pc, FEAT_MERCANTILE_BACKGROUND):
+			mod -= 5
 	else:
 		mod = Store['SellMarkup']
+		if GemRB.HasFeat(pc, FEAT_MERCANTILE_BACKGROUND):
+			mod += 5
 
 	# depreciation works like this:
 	# - if you sell the item the first time, SellMarkup is used;
@@ -1319,11 +1394,14 @@ def UpdateStoreHealWindow ():
 	TopIndex = GemRB.GetVar ("TopIndex")
 	Index = GemRB.GetVar ("Index")
 	pc = GemRB.GameGetSelectedPCSingle ()
+	labelOffset = 0x1000000c
+	if GameCheck.IsIWD2():
+		labelOffset += 1 # grrr
 	for i in range (ItemButtonCount):
 		Cure = GemRB.GetStoreCure (TopIndex+i)
 
 		Button = Window.GetControl (i+8)
-		Label = Window.GetControl (0x1000000c+i)
+		Label = Window.GetControl (labelOffset+i)
 		Button.SetVarAssoc ("Index", TopIndex+i)
 		if Cure:
 			Spell = GemRB.GetSpell (Cure['CureResRef'])
@@ -1333,8 +1411,9 @@ def UpdateStoreHealWindow ():
 			dead = GemRB.GetPlayerStat (pc, IE_STATE_ID) & STATE_DEAD
 			# toggle raise dead/resurrect based on state
 			# unfortunately the flags are not set properly in iwd
-			if (dead and Spell["SpellTargetType"] != 3) or \
-			   (not dead and Spell["SpellTargetType"] == 3): # 3 - non-living
+			if not GameCheck.IsIWD1() and (  # 3 - non-living
+					(dead and Spell["SpellTargetType"] != 3) or \
+					(not dead and Spell["SpellTargetType"] == 3)):
 				# locked and shaded
 				Button.SetState (IE_GUI_BUTTON_DISABLED)
 				Button.SetBorder (0, 0,0, 0,0, 200,0,0,100, 1,1)
@@ -1398,7 +1477,15 @@ def BuyHeal ():
 
 	GemRB.GameSetPartyGold (gold-Cure['Price'])
 	pc = GemRB.GameGetSelectedPCSingle ()
-	GemRB.ApplySpell (pc, Cure['CureResRef'], pc)
+	Spell = GemRB.GetSpell (Cure['CureResRef'])
+	# for anything but raise/resurrect, the talker should be the caster, so
+	# self-targetting spells work properly. Raise dead is an exception as
+	# the teleporting to the temple would not work otherwise
+	if Spell["SpellTargetType"] == 3: # non-living
+		GemRB.ApplySpell (pc, Cure['CureResRef'], Store['StoreOwner'])
+	else:
+		GemRB.ApplySpell (pc, Cure['CureResRef'], pc)
+
 	if SpellTable:
 		sound = SpellTable.GetValue(Cure['CureResRef'], "SOUND")
 	else:
@@ -1414,19 +1501,32 @@ def UpdateStoreRumourWindow ():
 
 	UpdateStoreCommon (Window, 0x10000011, 0, 0x10000012)
 	TopIndex = GemRB.GetVar ("TopIndex")
-	for i in range (5):
+	DrinkButtonCount = ItemButtonCount + 1
+	offset = 0
+	if GameCheck.IsIWD2():
+		offset = 40
+		DrinkButtonCount += 1 # shows even more than with inventories
+	for i in range (DrinkButtonCount):
 		Drink = GemRB.GetStoreDrink (TopIndex+i)
-		Button = Window.GetControl (i)
+		Button = Window.GetControl (i+offset)
 		Button.SetVarAssoc ("Index", i)
 		if Drink:
-			GemRB.SetToken ("ITEMNAME", GemRB.GetString (Drink['DrinkName']))
-			GemRB.SetToken ("ITEMCOST", str(Drink['Price']) )
-			Button.SetText (10162)
+			if GameCheck.IsIWD2():
+				Button.SetText (GemRB.GetString (Drink['DrinkName']))
+				CostLabel = Window.GetControl (0x10000000+29+i)
+				CostLabel.SetText (str(Drink['Price']))
+			else:
+				GemRB.SetToken ("ITEMNAME", GemRB.GetString (Drink['DrinkName']))
+				GemRB.SetToken ("ITEMCOST", str(Drink['Price']) )
+				Button.SetText (10162)
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GulpDrink)
 		else:
 			Button.SetText ("")
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
+			if GameCheck.IsIWD2():
+				CostLabel = Window.GetControl (0x10000000+29+i)
+				CostLabel.SetText ("")
 	return
 
 def GulpDrink ():
@@ -1457,8 +1557,6 @@ def GulpDrink ():
 	return
 
 def UpdateStoreRentWindow ():
-	global RentIndex
-
 	Window = StoreRentWindow
 	UpdateStoreCommon (Window, 0x10000008, 0, 0x10000009)
 	RentIndex = GemRB.GetVar ("RentIndex")
@@ -1480,19 +1578,22 @@ def RentConfirm ():
 	price = Store['StoreRoomPrices'][RentIndex]
 	Gold = GemRB.GameGetPartyGold ()
 	GemRB.GameSetPartyGold (Gold-price)
-	GemRB.RestParty (13, 1, RentIndex+1)
+	# TODO: run GemRB.RunRestScripts ()
+	cutscene = GemRB.RestParty (13, 1, RentIndex+1)
 	if RentConfirmWindow:
 		RentConfirmWindow.Unload ()
 	Window = StoreRentWindow
-	TextArea = Window.GetControl (12)
-	#is there any way to change this???
-	GemRB.SetToken ("HOUR", "8")
-	GemRB.SetToken ("HP", "%d"%(RentIndex+1))
-	TextArea.SetText (16476)
-	GemRB.SetVar ("RentIndex", -1)
-	Button = Window.GetControl (RentIndex+4)
-	Button.SetState (IE_GUI_BUTTON_ENABLED)
-	UpdateStoreRentWindow ()
+	if cutscene:
+		CloseStoreWindow ()
+	else:
+		TextArea = Window.GetControl (12)
+		#is there any way to change this???
+		GemRB.SetToken ("HP", "%d"%(RentIndex+1))
+		TextArea.SetText (16476)
+		GemRB.SetVar ("RentIndex", -1)
+		Button = Window.GetControl (RentIndex+4)
+		Button.SetState (IE_GUI_BUTTON_ENABLED)
+		UpdateStoreRentWindow ()
 	return
 
 def RentDeny () :
@@ -1502,7 +1603,7 @@ def RentDeny () :
 	return
 
 def RentRoom ():
-	global RentIndex, RentConfirmWindow
+	global RentConfirmWindow
 
 	RentIndex = GemRB.GetVar ("RentIndex")
 	price = Store['StoreRoomPrices'][RentIndex]
