@@ -18,11 +18,14 @@
 #
 
 import GemRB
+import GameCheck
 import GUICommon
+import Spellbook
 from GUIDefines import *
 from ie_stats import *
 from ie_slots import *
 from ie_spells import *
+from ie_sounds import DEF_IDENTIFY
 
 OverSlot = None
 UsedSlot = None
@@ -32,6 +35,7 @@ ItemIdentifyWindow = None
 ItemAbilitiesWindow = None
 ErrorWindow = None
 StackAmount = 0
+pause = None
 
 def OnDragItemGround ():
 	"""Drops and item to the ground."""
@@ -43,7 +47,7 @@ def OnDragItemGround ():
 		slot_item = GemRB.GetContainerItem (pc, slot)
 		item = GemRB.GetItem (slot_item["ItemResRef"])
 		GemRB.DragItem (pc, slot, item["ItemIcon"], 0, 1) #container
-		if GUICommon.GameIsPST():
+		if GameCheck.IsPST():
 			GemRB.PlaySound (item["DescIcon"])
 	else:
 		GemRB.DropDraggedItem (pc, -2) #dropping on ground
@@ -104,14 +108,24 @@ def OnDragItem ():
 					#enter the store
 					GemRB.EnterStore (slot_item["ItemResRef"])
 					#if it is possible to add, then do it
-					if GemRB.IsValidStoreItem (pc, slot, 0)&SHOP_SELL:
+					ret = GemRB.IsValidStoreItem (pc, slot, 0)
+					if ret&SHOP_SELL:
 						GemRB.ChangeStoreItem (pc, slot, SHOP_SELL)
 					else:
-						GemRB.DisplayString( 9375, 0xffffff)
+						msg = 9375
+						if ret&SHOP_FULL:
+							if GameCheck.IsIWD1() or GameCheck.IsIWD2():
+								msg = 24893
+							elif GameCheck.HasTOB():
+								msg = 54692
+						GemRB.DisplayString(msg, 0xffffff)
 					#leave (save) store
 					GemRB.LeaveStore()
 
 			GemRB.DropDraggedItem (pc, slot)
+			# drop item if it caused us to disable the inventory view (example: cursed berserking sword)
+			if GemRB.GetPlayerStat (pc, IE_STATE_ID) & (STATE_BERSERK) and GemRB.IsDraggingItem ():
+				GemRB.DropDraggedItem (pc, -3)
 
 	GUIINV.UpdateInventoryWindow ()
 	return
@@ -123,14 +137,6 @@ def OnDropItemToPC ():
 
 	#-3 : drop stuff in inventory (but not equippable slots)
 	GemRB.DropDraggedItem (pc, -3)
-	if GemRB.IsDraggingItem ()==1:
-		if GUICommon.HasTOB ():
-			GemRB.DisplayString (61794, 0xffffff)
-		elif GUICommon.GameIsPST ():
-			GemRB.DisplayString (19257, 0xffffff)
-		else:
-			GemRB.DisplayString (17999, 0xffffff)
-
 	GUIINV.UpdateInventoryWindow ()
 	return
 
@@ -199,7 +205,7 @@ def MouseLeaveSlot ():
 def MouseEnterGround ():
 	Window = GUIINV.InventoryWindow
 
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		offset = 47
 	else:
 		offset = 68
@@ -213,7 +219,7 @@ def MouseEnterGround ():
 def MouseLeaveGround ():
 	Window = GUIINV.InventoryWindow
 
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		offset = 47
 	else:
 		offset = 68
@@ -221,7 +227,7 @@ def MouseLeaveGround ():
 	Button = Window.GetControl (i+offset)
 
 	if GemRB.IsDraggingItem ()==1:
-		Button.SetState (IE_GUI_BUTTON_SECOND)
+		Button.SetState (IE_GUI_BUTTON_FAKEPRESSED)
 	return
 
 def CloseItemInfoWindow ():
@@ -236,7 +242,7 @@ def DisplayItem (itemresref, type):
 	item = GemRB.GetItem (itemresref)
 	ItemInfoWindow = Window = GemRB.LoadWindow (5)
 
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		strrefs = [ 1403, 4256, 4255, 4251, 4252, 4254, 4279 ]
 	else:
 		strrefs = [ 11973, 14133, 11960, 19392, 17104, item["DialogName"], 17108 ]
@@ -251,7 +257,7 @@ def DisplayItem (itemresref, type):
 
 	#item icon
 	Button = Window.GetControl (2)
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		Button.SetItemIcon (itemresref)
 	else:
@@ -271,7 +277,9 @@ def DisplayItem (itemresref, type):
 		text = item["ItemDesc"]
 	else:
 		text = item["ItemDescIdentified"]
-	Text.SetText (text)
+
+	Text.Clear ()
+	Text.Append (text)
 
 	#left button
 	Button = Window.GetControl(8)
@@ -280,7 +288,7 @@ def DisplayItem (itemresref, type):
 	if type&2:
 		Button.SetText (strrefs[1])
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, IdentifyItemWindow)
-	elif select and not GUICommon.GameIsPST():
+	elif select and not GameCheck.IsPST():
 		Button.SetText (strrefs[2])
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, AbilitiesItemWindow)
 	else:
@@ -288,10 +296,10 @@ def DisplayItem (itemresref, type):
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 
 	# description icon (not present in iwds)
-	if not GUICommon.GameIsIWD1() and not GUICommon.GameIsIWD2():
+	if not GameCheck.IsIWD1() and not GameCheck.IsIWD2():
 		Button = Window.GetControl (7)
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_CENTER_PICTURES | IE_GUI_BUTTON_NO_IMAGE, OP_OR)
-		if GUICommon.GameIsPST():
+		if GameCheck.IsPST():
 			Button.SetItemIcon (itemresref, 1) # no DescIcon
 		else:
 			Button.SetItemIcon (itemresref, 2)
@@ -301,7 +309,7 @@ def DisplayItem (itemresref, type):
 	Button = Window.GetControl(9)
 	drink = (type&1) and (item["Function"]&1)
 	read = (type&1) and (item["Function"]&2)
-	#sorcerors cannot learn spells
+	# sorcerers cannot learn spells
 	# FIXME: unhardcode
 	pc = GemRB.GameGetSelectedPCSingle ()
 	if GemRB.GetPlayerStat (pc, IE_CLASS) == 19:
@@ -316,9 +324,13 @@ def DisplayItem (itemresref, type):
 		Button.SetText (strrefs[4])
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ReadItemWindow)
 	elif container:
-		if GUICommon.GameIsIWD2() or GUICommon.GameIsHOW():
+		# Just skip the redundant info page and go directly to the container
+		if GemRB.GetVar("GUIEnhancements")&GE_ALWAYS_OPEN_CONTAINER_ITEMS:
+			OpenItemWindow()
+			return
+		if GameCheck.IsIWD2() or GameCheck.IsHOW():
 			Button.SetText (24891) # Open Container
-		elif GUICommon.GameIsBG2():
+		elif GameCheck.IsBG2():
 			Button.SetText (44002) # open container
 		else:
 			# a fallback, since the originals have nothing appropriate
@@ -357,19 +369,21 @@ def DisplayItem (itemresref, type):
 	ItemInfoWindow.ShowModal(MODAL_SHADOW_GRAY)
 	return
 
-def OpenItemInfoWindow ():
+def OpenItemInfoWindow (slot = None):
 	pc = GemRB.GameGetSelectedPCSingle ()
-	if GUICommon.GameIsPST():
-		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
+
+	if slot == None:
+		if GameCheck.IsPST():
+			slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
+		else:
+			slot = GemRB.GetVar ("ItemButton")
+			slot_item = GemRB.GetSlotItem (pc, slot)
 	else:
-		slot = GemRB.GetVar ("ItemButton")
 		slot_item = GemRB.GetSlotItem (pc, slot)
+
 	item = GemRB.GetItem (slot_item["ItemResRef"])
 
-	#auto identify when lore is high enough
-	if item["LoreToID"]<=GemRB.GetPlayerStat (pc, IE_LORE):
-		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
-		slot_item["Flags"] |= IE_INV_ITEM_IDENTIFIED
+	if TryAutoIdentification(pc, item, slot, slot_item, True):
 		GUIINV.UpdateInventoryWindow ()
 
 	if slot_item["Flags"] & IE_INV_ITEM_IDENTIFIED:
@@ -379,6 +393,14 @@ def OpenItemInfoWindow ():
 	DisplayItem (slot_item["ItemResRef"], value)
 	return
 
+#auto identify when lore is high enough
+def TryAutoIdentification(pc, item, slot, slot_item, enabled=0):
+	if enabled and item["LoreToID"]<=GemRB.GetPlayerStat (pc, IE_LORE):
+		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
+		slot_item["Flags"] |= IE_INV_ITEM_IDENTIFIED
+		return True
+	return False
+
 def OpenGroundItemInfoWindow ():
 	global ItemInfoWindow
 
@@ -445,7 +467,7 @@ def OpenItemAmountWindow ():
 	Text = Window.GetControl (6)
 	# FIXME: use a proper size
 	# FIXME: fix it for all the games
-	if GUICommon.GameIsIWD2():
+	if GameCheck.IsIWD2():
 		Text.SetSize (40, 40)
 	Text.SetText (str (StackAmount//2))
 	Text.SetStatus (IE_GUI_EDIT_NUMBER|IE_GUI_CONTROL_FOCUSED)
@@ -498,6 +520,12 @@ def UpdateSlot (pc, slot):
 
 	Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, OnDragItem)
 	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
+
+	# characters should auto-identify any item they recieve
+	if slot_item:
+		item = GemRB.GetItem (slot_item["ItemResRef"])
+		TryAutoIdentification(pc, item, slot+1, slot_item, GemRB.GetVar("GUIEnhancements")&GE_TRY_IDENTIFY_ON_TRANSFER)
+
 	GUICommon.UpdateInventorySlot (pc, Button, slot_item, "inventory", SlotType["Type"]&SLOT_INVENTORY == 0)
 
 	if slot_item:
@@ -532,10 +560,10 @@ def UpdateSlot (pc, slot):
 		if (SlotType["Type"]&SLOT_INVENTORY) or not GemRB.CanUseItemType (SlotType["Type"], itemname):
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		else:
-			Button.SetState (IE_GUI_BUTTON_SECOND)
+			Button.SetState (IE_GUI_BUTTON_FAKEPRESSED)
 
 		if slot_item and (GemRB.GetEquippedQuickSlot (pc)==slot+1 or GemRB.GetEquippedAmmunition (pc)==slot+1):
-			Button.SetState (IE_GUI_BUTTON_THIRD)
+			Button.SetState (IE_GUI_BUTTON_FAKEDISABLED)
 
 	return
 
@@ -613,7 +641,7 @@ def GetColor():
 	GUIINV.InventoryWindow.SetVisible (WINDOW_GRAYED) #darken it
 	ColorPicker=GemRB.LoadWindow (3)
 	GemRB.SetVar ("Selected",-1)
-	if GUICommon.GameIsIWD2():
+	if GameCheck.IsIWD2():
 		Button = ColorPicker.GetControl (35)
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CancelColor)
 		Button.SetText(13727)
@@ -681,63 +709,53 @@ def CloseErrorWindow ():
 	return
 
 def ReadItemWindow ():
-	global level, spell_ref
+	global pause
 	"""Tries to learn the mage scroll."""
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 	slot = GemRB.GetVar ("ItemButton")
-	ret = GUICommon.CannotLearnSlotSpell()
+	ret = Spellbook.CannotLearnSlotSpell()
 
 	if ret:
 		# couldn't find any strrefs for the other undhandled values (stat, level)
-		strref = 72873
-		CloseItemInfoWindow ()
-		if GUICommon.HasTOB() and ret == LSR_KNOWN:
-			OpenErrorWindow (strref)
-		return
-
-	if GUICommon.GameIsPST():
-		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
-	else:
-		slot_item = GemRB.GetSlotItem (pc, slot)
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
-	spell = GemRB.GetSpell (spell_ref)
-	if spell:
-		# can we learn more spells of this level?
-		level = spell['SpellLevel']-1
-		spell_count = GemRB.GetKnownSpellsCount (pc, IE_SPELL_TYPE_WIZARD, level)
-		if spell_count > GemRB.GetAbilityBonus (IE_INT, 2, GemRB.GetPlayerStat (pc, IE_INT)):
-			ret = LSR_FULL
-			if GUICommon.GameIsBG2():
-				strref = 32097
-			else:
-				strref = -1
+		if ret == LSR_KNOWN and GameCheck.HasTOB():
+			strref = 72873
+		elif ret == LSR_KNOWN and GameCheck.IsPST():
+			strref = 50394
+		elif ret == LSR_FULL and GameCheck.IsBG2():
+			strref = 32097
+		elif ret == LSR_FULL and GameCheck.IsPST():
+			strref = 50395
+		elif GameCheck.IsPST():
+			strref = 4250
 		else:
-			GemRB.UseItem (pc, slot, 1, 5)
-			GemRB.SetTimedEvent(DelayedReadItemWindow, 1)
-			return
-	else:
-		print "WARNING: invalid spell header in item", slot_item['ItemResRef']
-		CloseItemInfoWindow ()
-		return -1
+			strref = 10831
 
-	CloseItemInfoWindow ()
-	OpenErrorWindow (strref)
+		CloseItemInfoWindow ()
+		OpenErrorWindow (strref)
+
+	else:
+		if GameCheck.IsPST():
+			slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
+
+		pause = GemRB.GamePause(3,1) #query the pause state
+		GemRB.GamePause(0,1)
+		GemRB.UseItem (pc, slot, 1, 5)
+		GemRB.SetTimedEvent(DelayedReadItemWindow, 1)
 
 	return ret
 
 def DelayedReadItemWindow ():
-	global level, spell_ref
+	#restore the pause state
+	if pause:
+		GemRB.GamePause(1,1)
 
-	pc = GemRB.GameGetSelectedPCSingle ()
-	if GUICommon.HasSpell (pc, IE_SPELL_TYPE_WIZARD, level, spell_ref):
-		strref = 10830
-	else:
-		ret = LSR_FAILED
-		strref = 10831
 	CloseItemInfoWindow ()
-	if not GUICommon.GameIsPST():
-		OpenErrorWindow (strref)
+	if GameCheck.IsPST():
+		strref = 4249
+	else:
+		strref = 10830
+	OpenErrorWindow (strref)
 	return
 
 def OpenItemWindow ():
@@ -745,11 +763,11 @@ def OpenItemWindow ():
 
 	#close inventory
 	GemRB.SetVar ("Inventory", 1)
+	slot = GemRB.GetVar ("ItemButton") #get this before closing win
 	if ItemInfoWindow:
 		ItemInfoWindow.Unload ()
 	GUIINV.OpenInventoryWindow ()
 	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = GemRB.GetVar ("ItemButton")
 	slot_item = GemRB.GetSlotItem (pc, slot)
 	ResRef = slot_item['ItemResRef']
 	#the store will have to reopen the inventory
@@ -763,7 +781,7 @@ def DialogItemWindow ():
 		ItemInfoWindow.Unload ()
 	GUIINV.OpenInventoryWindow ()
 	pc = GemRB.GameGetSelectedPCSingle ()
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
 	else:
 		slot = GemRB.GetVar ("ItemButton")
@@ -783,12 +801,12 @@ def IdentifyUseSpell ():
 	slot = GemRB.GetVar ("ItemButton")
 	if ItemIdentifyWindow:
 		ItemIdentifyWindow.Unload ()
-	GemRB.HasSpecialSpell (pc, 1, 1)
+	GemRB.HasSpecialSpell (pc, SP_IDENTIFY, 1)
 	if ItemInfoWindow:
 		ItemInfoWindow.Unload ()
 	GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
 	GemRB.PlaySound(DEF_IDENTIFY)
-	OpenItemInfoWindow()
+	OpenItemInfoWindow(slot)
 	return
 
 def IdentifyUseScroll ():
@@ -805,7 +823,7 @@ def IdentifyUseScroll ():
 	if GemRB.HasSpecialItem (pc, 1, 1):
 		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
 	GemRB.PlaySound(DEF_IDENTIFY)
-	OpenItemInfoWindow()
+	OpenItemInfoWindow(slot)
 	return
 
 def CloseIdentifyItemWindow ():
@@ -823,16 +841,16 @@ def IdentifyItemWindow ():
 
 	ItemIdentifyWindow = Window = GemRB.LoadWindow (9)
 	Button = Window.GetControl (0)
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		Button.SetText (4259)
 	else:
 		Button.SetText (17105)
 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, IdentifyUseSpell)
-	if not GemRB.HasSpecialSpell (pc, 1, 0):
+	if not GemRB.HasSpecialSpell (pc, SP_IDENTIFY, 0):
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
 
 	Button = Window.GetControl (1)
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		Button.SetText (4260)
 	else:
 		Button.SetText (17106)
@@ -841,7 +859,7 @@ def IdentifyItemWindow ():
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
 
 	Button = Window.GetControl (2)
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		Button.SetText (4196)
 	else:
 		Button.SetText (13727)
@@ -849,7 +867,7 @@ def IdentifyItemWindow ():
 	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
 	TextArea = Window.GetControl (3)
-	if GUICommon.GameIsPST():
+	if GameCheck.IsPST():
 		TextArea.SetText (4258)
 	else:
 		TextArea.SetText (19394)
