@@ -22,11 +22,13 @@ import GemRB
 from GUIDefines import *
 from ie_stats import *
 from ie_restype import RES_2DA
+import GameCheck
 import GUICommon
+import Spellbook
 import CommonTables
 import LUSpellSelection
 import LUCommon
-if GUICommon.HasTOB():
+if GameCheck.HasTOB():
 	import LUHLASelection
 import LUProfsSelection
 import LUSkillsSelection
@@ -75,7 +77,7 @@ def OpenLevelUpWindow():
 
 	LevelUpWindow = GemRB.LoadWindow (3)
 
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		InfoButton = LevelUpWindow.GetControl (125)
 		InfoButton.SetText (13707)
 		InfoButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUpInfoPress)
@@ -95,7 +97,7 @@ def OpenLevelUpWindow():
 	Label = LevelUpWindow.GetControl (0x10000000+90)
 	Label.SetText (GemRB.GetPlayerName (pc))
 
-	if GUICommon.GameIsBG1() or GUICommon.GameIsIWD1():
+	if GameCheck.IsBG1() or GameCheck.IsIWD1():
 		# armorclass
 		Label = LevelUpWindow.GetControl (0x10000057)
 		ac = GemRB.GetPlayerStat (pc, IE_ARMORCLASS)
@@ -128,11 +130,10 @@ def OpenLevelUpWindow():
 
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
 	print "Class:",Class,"\tActor Class:",actor.classid
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
 	SkillTable = GemRB.LoadTable("skills")
 
 	# kit
-	ClassName = CommonTables.Classes.GetRowName(ClassIndex)
+	ClassName = GUICommon.GetClassRowName (Class, "class")
 	Kit = GUICommon.GetKitIndex (pc)
 	print "Kit:", Kit, "\tActor Kit:",actor.KitIndex()
 	print "ClassName:",ClassName,"\tActor ClassNames:",actor.ClassNames()
@@ -173,7 +174,8 @@ def OpenLevelUpWindow():
 			if IsDual[0] == 1: # kit
 				Classes[1] = CommonTables.KitList.GetValue (IsDual[1], 7)
 			else: # class
-				Classes[1] = CommonTables.Classes.GetValue (Classes[1], 5)
+				TmpClassName = GUICommon.GetClassRowName (Classes[1], "index")
+				Classes[1] = CommonTables.Classes.GetValue (TmpClassName, "ID")
 
 		# store a boolean for IsDual
 		IsDual = IsDual[0] > 0
@@ -188,9 +190,9 @@ def OpenLevelUpWindow():
 	if IsDual:
 		# convert the classes from indicies to class id's
 		DualSwap = GUICommon.IsDualSwap (pc)
-		ClassName = CommonTables.Classes.GetRowName (Classes[0])
+		ClassName = GUICommon.GetClassRowName (Classes[0], "index")
 		KitName = ClassName # for simplicity throughout the code
-		Classes[0] = CommonTables.Classes.GetValue (Classes[0], 5)
+		Classes[0] = CommonTables.Classes.GetValue (ClassName, "ID")
 		# Class[1] is taken care of above
 
 		# we need the old level as well
@@ -218,10 +220,7 @@ def OpenLevelUpWindow():
 		# we don't care about the current level, but about the to-be-achieved one
 		# get the next level
 		Level[i] = LUCommon.GetNextLevelFromExp (GemRB.GetPlayerStat (pc, IE_XP)/NumClasses, Classes[i])
-		TmpIndex = CommonTables.Classes.FindValue (5, Classes[i])
-		TmpName = CommonTables.Classes.GetRowName (TmpIndex) 
-
-#		print "Name:",TmpName
+		TmpClassName = GUICommon.GetClassRowName (Classes[i], "class")
 
 		# find the level diff for each classes (3 max, obviously)
 		if i == 0:
@@ -239,9 +238,9 @@ def OpenLevelUpWindow():
 
 		# save our current and next spell amounts
 		StartLevel = Level[i] - LevelDiff[i]
-		DruidTable = CommonTables.ClassSkills.GetValue (Classes[i], 0, 0)
-		ClericTable = CommonTables.ClassSkills.GetValue (Classes[i], 1, 0)
-		MageTable = CommonTables.ClassSkills.GetValue (Classes[i], 2, 0)
+		DruidTable = CommonTables.ClassSkills.GetValue (TmpClassName, "DRUIDSPELL", 0)
+		ClericTable = CommonTables.ClassSkills.GetValue (TmpClassName, "CLERICSPELL", 0)
+		MageTable = CommonTables.ClassSkills.GetValue (TmpClassName, "MAGESPELL", 0)
 
 		# see if we have mage spells
 		if MageTable != "*":
@@ -287,7 +286,7 @@ def OpenLevelUpWindow():
 
 		# setup class bonuses for this class
 		if IsMulti or IsDual or Kit == 0:
-			ABTable = CommonTables.ClassSkills.GetValue (TmpName, "ABILITIES")
+			ABTable = CommonTables.ClassSkills.GetValue (TmpClassName, "ABILITIES")
 		else: # single-classed with a kit
 			ABTable = CommonTables.KitList.GetValue (str(Kit), "ABILITIES")
 
@@ -307,7 +306,7 @@ def OpenLevelUpWindow():
 
 	# use total levels for HLAs
 	HLACount = 0
-	if GUICommon.HasTOB(): # make sure SoA doesn't try to get it
+	if GameCheck.HasTOB(): # make sure SoA doesn't try to get it
 		HLATable = GemRB.LoadTable("lunumab")
 		# we need to check each level against a multi value (this is kinda screwy)
 		if actor.multiclass:
@@ -319,8 +318,7 @@ def OpenLevelUpWindow():
 		for i in range (NumClasses):
 			if IsMulti:
 				# get the row name for lookup ex. MULTI2FIGHTER, MULTI3THIEF
-				MultiName = CommonTables.Classes.FindValue (5, Classes[i])
-				MultiName = CommonTables.Classes.GetRowName (MultiName)
+				MultiName = GUICommon.GetClassRowName (Classes[i], "class")
 				MultiName = "MULTI" + str(NumClasses) + MultiName
 			else:
 				MultiName = ClassName
@@ -340,7 +338,7 @@ def OpenLevelUpWindow():
 		# set values required by the hla level up code
 		HLACount = HLACount / HLATable.GetValue (ClassName, "RATE", 1)
 		GemRB.SetVar ("HLACount", HLACount)
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		HLAButton = LevelUpWindow.GetControl (126)
 		if HLACount:
 			HLAButton.SetText (4954)
@@ -352,7 +350,7 @@ def OpenLevelUpWindow():
 	Level1 = []
 	for i in range (len (Level)):
 		Level1.append (Level[i]-LevelDiff[i])
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		LUProfsSelection.SetupProfsWindow (pc, LUProfsSelection.LUPROFS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level)
 	else:
 		LUProfsSelection.SetupProfsWindow (pc, LUProfsSelection.LUPROFS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level, 0, False, 0)
@@ -361,13 +359,13 @@ def OpenLevelUpWindow():
 	#we autohide the skills and let SetupSkillsWindow show them if needbe
 	for i in range (4):
 		HideSkills (i)
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		LUSkillsSelection.SetupSkillsWindow (pc, LUSkillsSelection.LUSKILLS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level)
 	else:
 		LUSkillsSelection.SetupSkillsWindow (pc, LUSkillsSelection.LUSKILLS_TYPE_LEVELUP, LevelUpWindow, RedrawSkills, Level1, Level, 0, False)
 	NewSkillPoints = GemRB.GetVar ("SkillPointsLeft")
 
-	if GUICommon.GameIsBG2():
+	if GameCheck.IsBG2():
 		TextAreaControl = LevelUpWindow.GetControl(110)
 		TextAreaControl.SetText(GetLevelUpNews())
 	else:
@@ -407,7 +405,7 @@ def RedrawSkills():
 
 	# we need to disable the HLA button if we don't have any HLAs left
 	HLACount = GemRB.GetVar ("HLACount")
-	if GUICommon.GameIsBG2() and HLACount == 0:
+	if GameCheck.IsBG2() and HLACount == 0:
 		# turn the HLA button off
 		HLAButton = LevelUpWindow.GetControl (126)
 		HLAButton.SetState(IE_GUI_BUTTON_DISABLED)
@@ -444,8 +442,7 @@ def GetLevelUpNews():
 
 	for i in range(NumClasses):
 		# get the class name
-		TmpClassName = CommonTables.Classes.FindValue (5, Classes[i])
-		TmpClassName = CommonTables.Classes.GetRowName (TmpClassName)
+		TmpClassName = GUICommon.GetClassRowName (Classes[i], "class")
 
 		# backstab
 		# NOTE: Stalkers and assassins should get the correct mods at the correct levels based
@@ -458,12 +455,12 @@ def GetLevelUpNews():
 			BackstabMult = BackstabTable.GetValue (0, Level[i])
 
 		# lay on hands
-		if (CommonTables.ClassSkills.GetValue (Classes[i], 6) != "*"):
+		if (CommonTables.ClassSkills.GetValue (TmpClassName, "LAYHANDS") != "*"):
 			# inquisitors and undead hunters don't get lay on hands out the chute, whereas cavaliers
 			# and unkitted paladins do; therefore, we check for the existence of lay on hands to ensure
 			# the character should get the new value; LoH is defined in GA_SPCL211 if anyone wants to
 			# make a pally kit with LoH
-			if (GUICommon.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
+			if (Spellbook.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
 				LOHTable = GemRB.LoadTable ("layhands")
 				LOHGain = LOHTable.GetValue (0, Level[i]) - LOHTable.GetValue (0, Level[i]-LevelDiff[i])
 
@@ -605,8 +602,9 @@ def LevelUpDonePress():
 			# bonus spells don't count in determining if we can use this level
 			if GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_PRIEST, i, 0) > 0: # we can memorize spells of this level
 				for j in range(NumClasses): # loop through each class
-					IsDruid = CommonTables.ClassSkills.GetValue (Classes[j], 0, 0)
-					IsCleric = CommonTables.ClassSkills.GetValue (Classes[j], 1, 0)
+					TmpClassName = GUICommon.GetClassRowName (Classes[j], "class")
+					IsDruid = CommonTables.ClassSkills.GetValue (TmpClassName, "DRUIDSPELL", 0)
+					IsCleric = CommonTables.ClassSkills.GetValue (TmpClassName, "CLERICSPELL", 0)
 					if IsCleric == "*" and IsDruid == "*": # no divine spells (so mage/cleric multis don't screw up)
 						continue
 					elif IsCleric == "*": # druid spells
@@ -614,9 +612,9 @@ def LevelUpDonePress():
 					else: # cleric spells
 						ClassFlag = 0x4000
 
-					Learnable = GUICommon.GetLearnablePriestSpells(ClassFlag, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), i+1)
+					Learnable = Spellbook.GetLearnablePriestSpells(ClassFlag, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), i+1)
 					for k in range(len(Learnable)): # loop through all the learnable spells
-						if GUICommon.HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, Learnable[k]) < 0: # only write it if we don't yet know it
+						if Spellbook.HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, Learnable[k]) < 0: # only write it if we don't yet know it
 							GemRB.LearnSpell(pc, Learnable[k])
 
 	# hlas
@@ -657,19 +655,19 @@ def ReactivateBaseClass ():
 	Proficiencies, THACO, saves, spells, and innate abilites,
 	most noteably."""
 
-	ClassIndex = CommonTables.Classes.FindValue (5, Classes[1])
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
+	# we construct the Classes array, so that the active class is always first and the base is second
+	ClassName = GUICommon.GetClassRowName (Classes[1], "class")
 	KitIndex = GUICommon.GetKitIndex (pc)
 
 	# reactivate all our proficiencies
 	TmpTable = GemRB.LoadTable ("weapprof")
 	ProfsTableOffset = 0
-	if GUICommon.GameIsBG2 ():
+	if GameCheck.IsBG2 ():
 		ProfsTableOffset = 8 # skip bg1 weapprof.2da proficiencies
 	ProfCount = TmpTable.GetRowCount () - ProfsTableOffset
 	for i in range(ProfCount):
 		ProfID = TmpTable.GetValue (i+ProfsTableOffset, 0)
-		if GUICommon.GameIsBG1():
+		if GameCheck.IsBG1():
 			ProfID = ProfID + IE_PROFICIENCYBASTARDSWORD
 		Value = GemRB.GetPlayerStat (pc, ProfID)
 		OldProf = (Value & 0x38) >> 3
@@ -677,7 +675,7 @@ def ReactivateBaseClass ():
 		if OldProf > NewProf:
 			Value = (OldProf << 3) | OldProf
 			print "Value:",Value
-			if GUICommon.GameIsBG2():
+			if GameCheck.IsBG2():
 				GemRB.ApplyEffect (pc, "Proficiency", Value, ProfID )
 			else:
 				GemRB.SetPlayerStat (pc, ProfID, Value)
@@ -689,7 +687,7 @@ def ReactivateBaseClass ():
 		GemRB.SetPlayerStat (pc, IE_TOHIT, TmpThaco)
 
 	# see if all our saves are lower than our current saves
-	SavesTable = CommonTables.Classes.GetValue (ClassIndex, 3, 0)
+	SavesTable = CommonTables.Classes.GetValue (ClassName, "SAVE", 0)
 	SavesTable = GemRB.LoadTable (SavesTable)
 	for i in range (5):
 		# see if this save is lower than our old save
@@ -698,7 +696,7 @@ def ReactivateBaseClass ():
 			GemRB.SetPlayerStat (pc, IE_SAVEVSDEATH+i, TmpSave)
 
 	# see if we're a caster
-	SpellTables = [CommonTables.ClassSkills.GetValue (Classes[1], 0, 0), CommonTables.ClassSkills.GetValue (Classes[1], 1, 0), CommonTables.ClassSkills.GetValue (Classes[1], 2, 0)]
+	SpellTables = [CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL", 0), CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL", 0), CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL", 0)]
 	if SpellTables[2] != "*": # casts mage spells
 		# set up our memorizations
 		SpellTable = GemRB.LoadTable (SpellTables[2])
@@ -713,7 +711,7 @@ def ReactivateBaseClass ():
 			SpellTable = GemRB.LoadTable (SpellTables[1])
 			ClassMask = 0x4000
 		else: # druidic spells
-			if not GameRB.HasResource(SpellTables[0], RES_2DA):
+			if not GemRB.HasResource(SpellTables[0], RES_2DA):
 				SpellTables[0] = "MXSPLPRS"
 			SpellTable = GemRB.LoadTable (SpellTables[0])
 			ClassMask = 0x8000
@@ -732,9 +730,9 @@ def ReactivateBaseClass ():
 			#	they will now only get druid spells up to the level they could cast
 			#	this should probably be noted somewhere (ranger/cleric multis still function the same,
 			#	but that could be remedied if desired)
-			Learnable = GUICommon.GetLearnablePriestSpells(ClassMask, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), i+1)
+			Learnable = Spellbook.GetLearnablePriestSpells(ClassMask, GemRB.GetPlayerStat (pc, IE_ALIGNMENT), i+1)
 			for k in range (len (Learnable)): # loop through all the learnable spells
-				if GUICommon.HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, Learnable[k]) < 0: # only write it if we don't yet know it
+				if Spellbook.HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, Learnable[k]) < 0: # only write it if we don't yet know it
 					GemRB.LearnSpell(pc, Learnable[k])
 
 	# setup class bonuses for this class
