@@ -369,24 +369,7 @@ def setAccept():
 	GemRB.SetEquippedQuickSlot (MyChar, 0)
 
 	# apply class/kit abilities
-	IsMulti = GUICommon.IsMultiClassed (MyChar, 1)
-	Levels = [GemRB.GetPlayerStat (MyChar, IE_LEVEL), GemRB.GetPlayerStat (MyChar, IE_LEVEL2), \
-			GemRB.GetPlayerStat (MyChar, IE_LEVEL3)]
-	KitIndex = GUICommon.GetKitIndex (MyChar)
-	if IsMulti[0]>1:
-		#get the class abilites for each class
-		for i in range (IsMulti[0]):
-			TmpClassName = GUICommon.GetClassRowName (IsMulti[i+1], "class")
-			ABTable = CommonTables.ClassSkills.GetValue (TmpClassName, "ABILITIES")
-			if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
-				GUICommon.AddClassAbilities (MyChar, ABTable, Levels[i], Levels[i])
-	else:
-		if KitIndex:
-			ABTable = CommonTables.KitList.GetValue (str(KitIndex), "ABILITIES")
-		else:
-			ABTable = CommonTables.ClassSkills.GetValue (ClassName, "ABILITIES")
-		if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
-			GUICommon.AddClassAbilities (MyChar, ABTable, Levels[0], Levels[0])
+	GUICommon.ResolveClassAbilities (MyChar, ClassName)
 
 	#LETS PLAY!!
 	playmode = GemRB.GetVar ("PlayMode")
