@@ -262,7 +262,7 @@ def UpdateRecordsWindow ():
 
 
 	# class
-	text = CommonTables.Classes.GetValue (GemRB.GetPlayerStat (pc, IE_CLASS) - 1, 0)
+	text = CommonTables.Classes.GetValue (GUICommon.GetClassRowName (pc), "NAME_REF")
 
 	Label = Window.GetControl (0x10000016)
 	Label.SetText (text)
@@ -303,38 +303,25 @@ def UpdateRecordsWindow ():
 
 # puts default info to textarea (overview of PC's bonuses, saves, etc.
 def OnRecordsButtonLeave ():
-	Window = RecordsWindow
-	# help, info textarea
-	Text = Window.GetControl (0)
-	Text.SetText (stats_overview)
+	OnRecordsHelpStat (-1, 0, stats_overview)
 	return
 
 def OnRecordsHelpFaction ():
-	Window = RecordsWindow
 	Help = GemRB.GetString (20106) + "\n\n" + faction_help
-	TextArea = Window.GetControl (0)
-	TextArea.SetText (Help)
+	OnRecordsHelpStat (-1, 0, Help)
 	return
 
 def OnRecordsHelpArmorClass ():
-	Window = RecordsWindow
-	Help = GemRB.GetString (18493)
-	TextArea = Window.GetControl (0)
-	TextArea.SetText (Help)
+	OnRecordsHelpStat (-1, 0, 18493)
 	return
 
 def OnRecordsHelpHitPoints ():
-	Window = RecordsWindow
-	Help = GemRB.GetString (18494)
-	TextArea = Window.GetControl (0)
-	TextArea.SetText (Help)
+	OnRecordsHelpStat (-1, 0, 18494)
 	return
 
 def OnRecordsHelpAlignment ():
-	Window = RecordsWindow
 	Help = GemRB.GetString (20105) + "\n\n" + alignment_help
-	TextArea = Window.GetControl (0)
-	TextArea.SetText (Help)
+	OnRecordsHelpStat (-1, 0, Help)
 	return
 
 #Bio:
@@ -347,10 +334,14 @@ def OnRecordsHelpAlignment ():
 # 39428 nordom
 # 39429 vhailor
 
-def OnRecordsHelpStrength ():
-	Window = RecordsWindow
-	TextArea = Window.GetControl (0)
+def OnRecordsHelpStat (row, col, strref, bon1=0, bon2=0):
+	TextArea = RecordsWindow.GetControl (0)
+	TextArea.SetText (strref)
+	if row != -1:
+		TextArea.Append ("\n\n" + GemRB.StatComment (StatTable.GetValue(str(row),col), bon1, bon2) )
+	return
 
+def OnRecordsHelpStrength ():
 	# These are used to get the stats
 	pc = GemRB.GameGetSelectedPCSingle ()
 
@@ -365,15 +356,11 @@ def OnRecordsHelpStrength ():
 		y=0
 	if e>60:
 		s=19
-	TextArea.SetText(18489)
-	TextArea.Append("\n\n"+GemRB.StatComment(StatTable.GetValue(s,0),x,y) )
 
+	OnRecordsHelpStat (s, "STR", 18489, x, y)
 	return
 
 def OnRecordsHelpDexterity ():
-	Window = RecordsWindow
-	TextArea = Window.GetControl (0)
-
 	# Loading table of modifications
 	Table = GemRB.LoadTable("dexmod")
 
@@ -386,42 +373,28 @@ def OnRecordsHelpDexterity ():
 	# Getting the dexterity description
 	x = -Table.GetValue(Dex,2)
 
-	TextArea.SetText(18487)
-	TextArea.Append("\n\n"+GemRB.StatComment(StatTable.GetValue(Dex,3),x,0) )
+	OnRecordsHelpStat (Dex, "DEX", 18487, x)
 	return
 
 def OnRecordsHelpIntelligence ():
-	Window = RecordsWindow
-	TextArea = Window.GetControl (0)
-
 	# These are used to get the stats
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	# Getting the character's intelligence
 	Int = GemRB.GetPlayerStat (pc, IE_INT)
-
-	TextArea.SetText(18488)
-	TextArea.Append("\n\n"+GemRB.StatComment(StatTable.GetValue(Int,1),0,0) )
+	OnRecordsHelpStat (Int, "INT", 18488)
 	return
 
 def OnRecordsHelpWisdom ():
-	Window = RecordsWindow
-	TextArea = Window.GetControl (0)
-
 	# These are used to get the stats
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	# Getting the character's wisdom
 	Wis = GemRB.GetPlayerStat (pc, IE_WIS)
-
-	TextArea.SetText(18490)
-	TextArea.Append("\n\n"+GemRB.StatComment(StatTable.GetValue(Wis,2),0,0) )
+	OnRecordsHelpStat (Wis, "WIS", 18490)
 	return
 
 def OnRecordsHelpConstitution ():
-	Window = RecordsWindow
-	TextArea = Window.GetControl (0)
-
 	# Loading table of modifications
 	Table = GemRB.LoadTable("hpconbon")
 
@@ -434,22 +407,16 @@ def OnRecordsHelpConstitution ():
 	# Getting the constitution description
 	x = Table.GetValue(Con-1,1)
 
-	TextArea.SetText(18491)
-	TextArea.Append("\n\n"+GemRB.StatComment(StatTable.GetValue(Con,4),x,0) )
+	OnRecordsHelpStat (Con, "CON", 18491, x)
 	return
 
 def OnRecordsHelpCharisma ():
-	Window = RecordsWindow
-	TextArea = Window.GetControl (0)
-
 	# These are used to get the stats
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	# Getting the character's charisma
 	Cha = GemRB.GetPlayerStat (pc, IE_CHR)
-
-	TextArea.SetText(1903)
-	TextArea.Append("\n\n"+GemRB.StatComment(StatTable.GetValue(Cha,5),0,0) )
+	OnRecordsHelpStat (Cha, "CHR", 1903)
 	return
 
 def GetCharacterHeader (pc):
@@ -458,7 +425,7 @@ def GetCharacterHeader (pc):
 	BioTable = GemRB.LoadTable ("bios")
 
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS) - 1
-	Multi = CommonTables.Classes.GetValue (Class, 4)
+	Multi = GUICommon.HasMultiClassBits (pc)
 	Specific = "%d"%GemRB.GetPlayerStat (pc, IE_SPECIFIC)
 
 	#Nameless is Specific == 1
@@ -611,9 +578,10 @@ def GetStatOverview (pc):
 	#   4208 THAC0
 	stats.append ((4208, GS (IE_TOHIT), ''))
 	#   4209 Number of Attacks
-	tmp = GS (IE_NUMBEROFATTACKS)
+	tmp = GemRB.GetCombatDetails(pc, 0)["APR"]
+
 	if (tmp&1):
-		tmp2 = str(tmp/2) + chr(188)
+		tmp2 = str(tmp/2) + chr(189)
 	else:
 		tmp2 = str(tmp/2)
 
@@ -746,30 +714,35 @@ def OpenInformationWindow ():
 	InformationWindow = Window = GemRB.LoadWindow (5)
 	GemRB.SetVar ("FloatWindow", InformationWindow.ID)
 
+	# Biography
+	Button = Window.GetControl (1)
+	Button.SetText (4247)
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyWindow)
+
+	# Done
+	Button = Window.GetControl (0)
+	Button.SetText (1403)
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenInformationWindow)
+	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
 	TotalPartyExp = 0
 	TotalPartyKills = 0
 	for i in range (1, GemRB.GetPartySize() + 1):
 		stat = GemRB.GetPCStats(i)
-		TotalPartyExp = TotalPartyExp + stat['KillsChapterXP']
-		TotalPartyKills = TotalPartyKills + stat['KillsChapterCount']
+		TotalPartyExp = TotalPartyExp + stat['KillsTotalXP']
+		TotalPartyKills = TotalPartyKills + stat['KillsTotalCount']
 
 	# These are used to get the stats
 	pc = GemRB.GameGetSelectedPCSingle ()
-
 	stat = GemRB.GetPCStats (pc)
 
 	Label = Window.GetControl (0x10000001)
 	Label.SetText (GemRB.GetPlayerName (pc, 1))
 
-
 	# class
-	text = CommonTables.Classes.GetValue (GemRB.GetPlayerStat (pc, IE_CLASS) - 1, 0)
-
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	Label = Window.GetControl (0x1000000A)
-	Label.SetText (text)
-
-
+	Label.SetText (ClassTitle)
 
 	Label = Window.GetControl (0x10000002)
 	if stat['BestKilledName'] == -1:
@@ -777,22 +750,8 @@ def OpenInformationWindow ():
 	else:
 		Label.SetText (GemRB.GetString (stat['BestKilledName']))
 
-	# NOTE: currentTime is in seconds, joinTime is in seconds * 15
-	#   (script updates???). In each case, there are 60 seconds
-	#   in a minute, 24 hours in a day, but ONLY 5 minutes in an hour!!
-	# Hence currentTime (and joinTime after div by 15) has
-	#   7200 secs a day (60 * 5 * 24)
-	currentTime = GemRB.GetGameTime()
-	joinTime = stat['JoinDate'] - stat['AwayTime']
-
-	party_time = currentTime - (joinTime / 15) 
-	days = party_time / 7200
-	hours = (party_time % 7200) / 300
-
-	GemRB.SetToken ('GAMEDAYS', str (days))
-	GemRB.SetToken ('HOUR', str (hours))
-	
 	Label = Window.GetControl (0x10000003)
+	GUICommon.SetCurrentDateTokens (stat)
 	Label.SetText (41277)
 
 	Label = Window.GetControl (0x10000004)
@@ -803,17 +762,17 @@ def OpenInformationWindow ():
 
 	Label = Window.GetControl (0x10000006)
 	if TotalPartyExp != 0:
-		PartyExp = int ((stat['KillsChapterXP'] * 100) / TotalPartyExp)
+		PartyExp = int ((stat['KillsTotalXP'] * 100) / TotalPartyExp)
 		Label.SetText (str (PartyExp) + '%')
 	else:
 		Label.SetText ("0%")
 
 	Label = Window.GetControl (0x10000007)
 	if TotalPartyKills != 0:
-		PartyKills = int ((stat['KillsChapterCount'] * 100) / TotalPartyKills)
+		PartyKills = int ((stat['KillsTotalCount'] * 100) / TotalPartyKills)
 		Label.SetText (str (PartyKills) + '%')
 	else:
-		Label.SetText ('0%')
+		Label.SetText ("0%")
 
 	Label = Window.GetControl (0x10000008)
 	Label.SetText (str (stat['KillsTotalXP']))
@@ -846,24 +805,10 @@ def OpenInformationWindow ():
 	Label = Window.GetControl (0x10000012)
 	Label.SetTextColor (255, 255, 255)
 
-
-
-	# Biography
-	Button = Window.GetControl (1)
-	Button.SetText (4247)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyWindow)
-
-	# Done
-	Button = Window.GetControl (0)
-	Button.SetText (1403)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenInformationWindow)
-	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
-
 	GemRB.UnhideGUI ()
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 
 
-
 def OpenBiographyWindow ():
 	global BiographyWindow
 
@@ -1000,8 +945,7 @@ def OpenLevelUpWindow ():
 	# Recording this avatar's current proficiency level
 	# Since Nameless one is not covered, hammer and club can't occur
 	# What is the avatar's class (Which we can use to lookup XP)
-	ClassRow = GemRB.GetPlayerStat (pc, IE_CLASS)-1
-	Class = CommonTables.Classes.GetRowName (ClassRow)
+	Class = GUICommon.GetClassRowName (pc)
 
 	# name
 	Label = Window.GetControl (0x10000000)
@@ -1009,7 +953,7 @@ def OpenLevelUpWindow ():
 
 	# class
 	Label = Window.GetControl (0x10000001)
-	Label.SetText (CommonTables.Classes.GetValue (ClassRow, 0))
+	Label.SetText (CommonTables.Classes.GetValue (Class, "NAME_REF"))
 
 	# Armor Class
 	Label = Window.GetControl (0x10000023)
@@ -1130,7 +1074,7 @@ def OpenLevelUpWindow ():
 			# Thac0
 			Thac0 = GetThac0 (Class, NextLevel)
 			# Is the new thac0 better than old? (The smaller, the better)
-			if Thac0 < GemRB.GetPlayerStat (pc, IE_TOHIT):
+			if Thac0 < GemRB.GetPlayerStat (pc, IE_TOHIT, 1):
 				Thac0Updated = True
 
 	else:
@@ -1195,7 +1139,7 @@ def OpenLevelUpWindow ():
 		# Multi class use the primary class level to determine Thac0
 		Thac0 = GetThac0 (Class, PrimNextLevel)
 		# Is the new thac0 better than old? (The smaller the better)
-		if Thac0 < GemRB.GetPlayerStat (pc, IE_TOHIT):
+		if Thac0 < GemRB.GetPlayerStat (pc, IE_TOHIT, 1):
 			Thac0Updated = True
 
 
