@@ -27,6 +27,8 @@
 #include "AnimationFactory.h"
 #include "Plugin.h"
 
+namespace GemRB {
+
 class Font;
 
 class GEM_EXPORT AnimationMgr : public Plugin {
@@ -37,12 +39,12 @@ public:
 	virtual int GetCycleSize(unsigned char Cycle) = 0;
 	virtual AnimationFactory* GetAnimationFactory(const char* ResRef,
 		unsigned char mode = IE_NORMAL) = 0;
-	/** This function will load the Animation as a Font */
-	virtual Font* GetFont() = 0;
 	/** Debug Function: Returns the Global Animation Palette as a Sprite2D Object.
 	If the Global Animation Palette is NULL, returns NULL. */
 	virtual Sprite2D* GetPalette() = 0;
 	virtual int GetCycleCount() = 0;
 };
 
+}
+
 #endif
