@@ -30,7 +30,6 @@
 #include "Interface.h"
 #include "Map.h"
 #include "Palette.h"
-#include "Video.h"
 
 namespace GemRB {
 
@@ -76,7 +75,6 @@ struct EquipResRefData {
 	unsigned char Cycle;
 };
 
-
 void CharAnimations::ReleaseMemory()
 {
 	if (AvatarTable) {
@@ -163,6 +161,7 @@ int CharAnimations::GetActorPartCount() const
 	case IE_ANI_NINE_FRAMES: //dragon animations
 		return 9;
 	case IE_ANI_FOUR_FRAMES: //wyvern animations
+	case IE_ANI_FOUR_FRAMES_2: //demogorgon animations
 		return 4;
 	case IE_ANI_TWO_PIECE:   //ankheg animations
 		return 2;
@@ -287,15 +286,21 @@ void CharAnimations::LockPalette(const ieDword *gradients)
 	}
 }
 
-//                            0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18
-static const char *StancePrefix[]={"3","2","5","5","4","4","2","2","5","4","1","3","3","3","4","1","4","4","4"};
-static const char *CyclePrefix[]= {"0","0","1","1","1","1","0","0","1","1","0","1","1","1","1","1","1","1","1"};
-static const unsigned int CycleOffset[] = {0,  0,  0,  0,  0,  9,  0,  0,  0, 18,  0,  0,  9,  18,  0,  0,  0,  0,  0};
+//                                          0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18
+static const char StancePrefix[] =        {'3','2','5','5','4','4','2','2','5','4','1','3','3','3','4','1','4','4','4'};
+static const char CyclePrefix[] =         {'0','0','1','1','1','1','0','0','1','1','0','0','0','0','1','1','1','1','1'};
+static const unsigned int CycleOffset[] = { 0,  0,  0,  0,  0,  9,  0,  0,  0, 18,  0,  0,  9, 18,  0,  0,  0,  0,  0 };
+
+#define NINE_FRAMES_PALETTE(stance)	((PaletteType) (StancePrefix[stance] - '1'))
 
 void CharAnimations::SetColors(const ieDword *arg)
 {
 	Colors = arg;
 	SetupColors(PAL_MAIN);
+	SetupColors(PAL_MAIN_2);
+	SetupColors(PAL_MAIN_3);
+	SetupColors(PAL_MAIN_4);
+	SetupColors(PAL_MAIN_5);
 	SetupColors(PAL_WEAPON);
 	SetupColors(PAL_OFFHAND);
 	SetupColors(PAL_HELMET);
@@ -307,12 +312,14 @@ void CharAnimations::CheckColorMod()
 		if (GlobalColorMod.type != RGBModifier::NONE) {
 			GlobalColorMod.type = RGBModifier::NONE;
 			GlobalColorMod.speed = 0;
-		  change[0]=change[1]=change[2]=change[3]=true;
+			for (int i = 0; i < PAL_MAX; ++i) {
+				change[i] = true;
+			}
 		}
 	}
 	unsigned int location;
 
-	for (location = 0; location < 32; ++location) {
+	for (location = 0; location < PAL_MAX * 8; ++location) {
 		if (!ColorMods[location].phase) {
 		  if (ColorMods[location].type != RGBModifier::NONE) {
 				ColorMods[location].type = RGBModifier::NONE;
@@ -327,7 +334,7 @@ void CharAnimations::CheckColorMod()
 
 void CharAnimations::SetupColors(PaletteType type)
 {
-	Palette* pal = palette[(int)type];
+	Palette* pal = palette[type];
 
 	if (!pal) {
 		return;
@@ -386,41 +393,37 @@ void CharAnimations::SetupColors(PaletteType type)
 	}
 
 	int PType = NoPalette();
-	if ( PType && (type == PAL_MAIN) ) {
-		bool needmod = false;
-		if (GlobalColorMod.type != RGBModifier::NONE) {
-			needmod = true;
-		}
-		if (!needmod && PaletteResRef[0]) {
-			gamedata->FreePalette(palette[PAL_MAIN], PaletteResRef);
-		}
-		PaletteResRef[0]=0;
+	if ( PType && (type <= PAL_MAIN_5) ) {
 		//handling special palettes like MBER_BL (black bear)
 		if (PType!=1) {
+			ieResRef oldResRef;
+			CopyResRef(oldResRef, PaletteResRef[type]);
 			if (GetAnimType()==IE_ANI_NINE_FRAMES) {
-				snprintf(PaletteResRef,9,"%.4s_%-.2s%s",ResRef, (char *) &PType, StancePrefix[StanceID]);
+				snprintf(PaletteResRef[type],9,"%.4s_%-.2s%c",ResRef, (char *) &PType, '1'+type);
 			} else {
-				snprintf(PaletteResRef,9,"%.4s_%-.2s",ResRef, (char *) &PType);
+				snprintf(PaletteResRef[type],9,"%.4s_%-.2s",ResRef, (char *) &PType);
 			}
-			strlwr(PaletteResRef);
-			Palette *tmppal = gamedata->GetPalette(PaletteResRef);
+			strlwr(PaletteResRef[type]);
+			Palette *tmppal = gamedata->GetPalette(PaletteResRef[type]);
 			if (tmppal) {
-				palette[PAL_MAIN] = tmppal;
+				gamedata->FreePalette(palette[type], oldResRef);
+				palette[type] = tmppal;
 			} else {
-				PaletteResRef[0]=0;
+				PaletteResRef[type][0]=0;
 			}
 		}
+		bool needmod = GlobalColorMod.type != RGBModifier::NONE;
 		if (needmod) {
-			if (!modifiedPalette[PAL_MAIN])
-				modifiedPalette[PAL_MAIN] = new Palette();
-			modifiedPalette[PAL_MAIN]->SetupGlobalRGBModification(palette[PAL_MAIN], GlobalColorMod);
+			if (!modifiedPalette[type])
+				modifiedPalette[type] = new Palette();
+			modifiedPalette[type]->SetupGlobalRGBModification(palette[type], GlobalColorMod);
 		} else {
-			gamedata->FreePalette(modifiedPalette[PAL_MAIN], 0);
+			gamedata->FreePalette(modifiedPalette[type], 0);
 		}
 		return;
 	}
 
-	pal->SetupPaperdollColours(Colors, (int)type);
+	pal->SetupPaperdollColours(Colors, type);
 	if (lockPalette) {
 		return;
 	}
@@ -431,23 +434,22 @@ void CharAnimations::SetupColors(PaletteType type)
 		needmod = true;
 	} else {
 		for (i = 0; i < 7; ++i) {
-			if (ColorMods[i+8*((int)type)].type != RGBModifier::NONE)
+			if (ColorMods[i+8*type].type != RGBModifier::NONE)
 				needmod = true;
 		}
 	}
 
-
 	if (needmod) {
-		if (!modifiedPalette[(int)type])
-			modifiedPalette[(int)type] = new Palette();
+		if (!modifiedPalette[type])
+			modifiedPalette[type] = new Palette();
 
 		if (GlobalColorMod.type != RGBModifier::NONE) {
-			modifiedPalette[(int)type]->SetupGlobalRGBModification(palette[(int)type], GlobalColorMod);
+			modifiedPalette[type]->SetupGlobalRGBModification(palette[type], GlobalColorMod);
 		} else {
-			modifiedPalette[(int)type]->SetupRGBModification(palette[(int)type],ColorMods, (int)type);
+			modifiedPalette[type]->SetupRGBModification(palette[type],ColorMods, type);
 		}
 	} else {
-		gamedata->FreePalette(modifiedPalette[(int)type], 0);
+		gamedata->FreePalette(modifiedPalette[type], 0);
 	}
 
 }
@@ -456,17 +458,21 @@ Palette* CharAnimations::GetPartPalette(int part)
 {
 	int actorPartCount = GetActorPartCount();
 	PaletteType type = PAL_MAIN;
-
+	if (GetAnimType() == IE_ANI_NINE_FRAMES) {
+		//these animations use several palettes
+		type = NINE_FRAMES_PALETTE(StanceID);
+	}
+	else if (GetAnimType() == IE_ANI_FOUR_FRAMES_2) return NULL;
 	// always use unmodified BAM palette for the supporting part
-	if (GetAnimType() == IE_ANI_TWO_PIECE && part == 1) return NULL;
+	else if (GetAnimType() == IE_ANI_TWO_PIECE && part == 1) return NULL;
 	else if (part == actorPartCount) type = PAL_WEAPON;
 	else if (part == actorPartCount+1) type = PAL_OFFHAND;
 	else if (part == actorPartCount+2) type = PAL_HELMET;
 
-	if (modifiedPalette[(int)type])
-		return modifiedPalette[(int)type];
+	if (modifiedPalette[type])
+		return modifiedPalette[type];
 
-	return palette[(int)type];
+	return palette[type];
 }
 
 static int compare_avatars(const void *a, const void *b)
@@ -621,7 +627,7 @@ CharAnimations::CharAnimations(unsigned int AnimID, ieDword ArmourLevel)
 {
 	Colors = NULL;
 	int i,j;
-	for (i = 0; i < 4; ++i) {
+	for (i = 0; i < PAL_MAX; ++i) {
 		change[i] = true;
 		modifiedPalette[i] = NULL;
 		palette[i] = NULL;
@@ -642,11 +648,13 @@ CharAnimations::CharAnimations(unsigned int AnimID, ieDword ArmourLevel)
 	ArmorType = 0;
 	RangedType = 0;
 	WeaponType = 0;
-	PaletteResRef[0] = 0;
+	for (i = 0; i < 5; ++i) {
+		PaletteResRef[i][0] = 0;
+	}
 	WeaponRef[0] = 0;
 	HelmetRef[0] = 0;
 	OffhandRef[0] = 0;
-	for (i = 0; i < 32; ++i) {
+	for (i = 0; i < PAL_MAX * 8; ++i) {
 		ColorMods[i].type = RGBModifier::NONE;
 		ColorMods[i].speed = 0;
 		// make initial phase depend on location to make the pulse appear
@@ -707,11 +715,12 @@ void CharAnimations::DropAnims()
 CharAnimations::~CharAnimations(void)
 {
 	DropAnims();
-	gamedata->FreePalette(palette[PAL_MAIN], PaletteResRef);
 	int i;
-	for (i = 1; i < 4; ++i)
+	for (i = 0; i <= PAL_MAIN_5; ++i)
+		gamedata->FreePalette(palette[i], PaletteResRef[i]);
+	for (; i < PAL_MAX; ++i)
 		gamedata->FreePalette(palette[i], 0);
-	for (i = 0; i < 4; ++i)
+	for (i = 0; i < PAL_MAX; ++i)
 		gamedata->FreePalette(modifiedPalette[i], 0);
 }
 /*
@@ -799,11 +808,15 @@ IE_ANI_TWO_FILES_3B:	Animations using this type are stored using the following t
 			This is the standard IWD animation, but BG2 also has it.
 			See MOR2
 
+IE_ANI_TWO_FILES_4: This type stores animations in two files (G1 and G2) which each having only
+			one cycle. And both of those seem to be identical.
+
 IE_ANI_TWO_PIECE: This is a modified IE_ANI_SIX_FILES with supporting still frames (using a
 			different palette) stored in a second set of files. Currently only used by MAKH
 
 IE_ANI_FOUR_FRAMES:	These animations are large, four bams make a frame.
 
+IE_ANI_FOUR_FRAMES_2:	Another variation of the four frames format with more files.
 
 IE_ANI_NINE_FRAMES:     These animations are huge, nine bams make a frame.
 
@@ -854,7 +867,7 @@ WSW 003      |      013 ESE
 
 Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Orient)
 {
-	if (StanceID>=MAX_ANIMS) {
+	if (Stance >= MAX_ANIMS) {
 		error("CharAnimation", "Illegal stance ID\n");
 	}
 
@@ -1037,16 +1050,22 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 		}
 
 		if (part < actorPartCount) {
+			PaletteType ptype = PAL_MAIN;
+			if (AnimType == IE_ANI_NINE_FRAMES) {
+				//these animations use several palettes
+				ptype = NINE_FRAMES_PALETTE(StanceID);
+			}
+
 			//if you need to revert this change, consider true paletted
 			//animations which need a GlobalColorMod (mgir for example)
 
 			//if (!palette[PAL_MAIN] && ((GlobalColorMod.type!=RGBModifier::NONE) || (NoPalette()!=1)) ) {
-			if(!palette[PAL_MAIN]) {
+			if(!palette[ptype]) {
 				// This is the first time we're loading an Animation.
 				// We copy the palette of its first frame into our own palette
-				palette[PAL_MAIN] = a->GetFrame(0)->GetPalette()->Copy();
+				palette[ptype] = a->GetFrame(0)->GetPalette()->Copy();
 				// ...and setup the colours properly
-				SetupColors(PAL_MAIN);
+				SetupColors(ptype);
 			}
 		} else if (part == actorPartCount) {
 			if (!palette[PAL_WEAPON]) {
@@ -1094,6 +1113,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 		switch (GetAnimType()) {
 			case IE_ANI_NINE_FRAMES: //dragon animations
 			case IE_ANI_FOUR_FRAMES: //wyvern animations
+			case IE_ANI_FOUR_FRAMES_2:
 			case IE_ANI_BIRD:
 			case IE_ANI_CODE_MIRROR:
 			case IE_ANI_CODE_MIRROR_2: //9 orientations
@@ -1119,6 +1139,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 	switch (GetAnimType()) {
 		case IE_ANI_NINE_FRAMES: //dragon animations
 		case IE_ANI_FOUR_FRAMES: //wyvern animations
+		case IE_ANI_FOUR_FRAMES_2:
 		case IE_ANI_BIRD:
 		case IE_ANI_CODE_MIRROR:
 		case IE_ANI_SIX_FILES: //16 anims some are stored elsewhere
@@ -1151,6 +1172,13 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 				Anims[StanceID][Orient] = anims;
 			}
 			break;
+		case IE_ANI_TWO_FILES_4: {
+			for (int i = 0; i < MAX_ANIMS; ++i) {
+				for (int j = 0; j < MAX_ORIENT; ++j) {
+					Anims[i][j] = anims;
+				}
+			}}
+			break; 
 
 		case IE_ANI_PST_ANIMATION_3: //no stc just std
 		case IE_ANI_PST_ANIMATION_2: //no std just stc
@@ -1201,6 +1229,10 @@ void CharAnimations::GetAnimResRef(unsigned char StanceID,
 			AddFFSuffix( NewResRef, StanceID, Cycle, Orient, Part );
 			break;
 
+		case IE_ANI_FOUR_FRAMES_2:
+			AddFF2Suffix( NewResRef, StanceID, Cycle, Orient, Part );
+			break;
+
 		case IE_ANI_NINE_FRAMES:
 			AddNFSuffix( NewResRef, StanceID, Cycle, Orient, Part );
 			break;
@@ -1242,6 +1274,11 @@ void CharAnimations::GetAnimResRef(unsigned char StanceID,
 			AddMMR2Suffix( NewResRef, StanceID, Cycle, Orient );
 			break;
 
+		case IE_ANI_TWO_FILES_4:
+			strcat( NewResRef, "g1");
+			Cycle = 0;
+			break;
+
 		case IE_ANI_TWO_FILES:
 			AddTwoFileSuffix(NewResRef, StanceID, Cycle, Orient );
 			break;
@@ -1436,12 +1473,8 @@ void CharAnimations::AddVHR2Suffix(char* ResRef, unsigned char StanceID,
 			break;
 
 		case IE_ANI_SLEEP:
-			strcat( ResRef, "g15" );
-			Cycle+=45;
-			break;
-
 		case IE_ANI_TWITCH:
-			strcat( ResRef, "g14" );
+			strcat( ResRef, "g15" );
 			Cycle+=45;
 			break;
 
@@ -1485,6 +1518,7 @@ void CharAnimations::AddVHR3Suffix(char* ResRef, unsigned char StanceID,
 		case IE_ANI_ATTACK: //temporarily
 		case IE_ANI_ATTACK_BACKSLASH:
 			strcat( ResRef, "g21" );
+			Cycle+=9;
 			break;
 
 		case IE_ANI_ATTACK_SLASH:
@@ -1492,22 +1526,18 @@ void CharAnimations::AddVHR3Suffix(char* ResRef, unsigned char StanceID,
 			break;
 
 		case IE_ANI_ATTACK_JAB:
-			strcat( ResRef, "g26" );
+		case IE_ANI_CONJURE://ending
+			strcat( ResRef, "g22" );
 			Cycle+=18;
 			break;
 
 		case IE_ANI_CAST: //looping
-			strcat( ResRef, "g25" );
-			Cycle+=45;
-			break;
-
-		case IE_ANI_CONJURE://ending
-			strcat( ResRef, "g26" );
-			Cycle+=36;
+			strcat( ResRef, "g22" );
+			Cycle+=27;
 			break;
 
 		case IE_ANI_SHOOT:
-			strcat( ResRef, "g24" );
+			strcat( ResRef, "g23" );
 			Cycle+=27;
 			break;
 
@@ -1518,12 +1548,8 @@ void CharAnimations::AddVHR3Suffix(char* ResRef, unsigned char StanceID,
 			break;
 
 		case IE_ANI_SLEEP:
-			strcat( ResRef, "g15" );
-			Cycle+=45;
-			break;
-
 		case IE_ANI_TWITCH:
-			strcat( ResRef, "g14" );
+			strcat( ResRef, "g15" );
 			Cycle+=45;
 			break;
 
@@ -1619,13 +1645,75 @@ void CharAnimations::AddFFSuffix(char* ResRef, unsigned char StanceID,
 	ResRef[7]=0;
 }
 
+void CharAnimations::AddFF2Suffix(char* ResRef, unsigned char StanceID,
+	unsigned char& Cycle, unsigned char Orient, int Part) const
+{
+	Cycle = SixteenToNine[Orient];
+	switch (StanceID) {
+		case IE_ANI_HEAD_TURN:
+			strcat( ResRef, "g101" );
+			break;
+
+		case IE_ANI_READY:
+		case IE_ANI_AWAKE:
+			strcat( ResRef, "g102" );
+			Cycle += 9;
+			break;
+
+		case IE_ANI_WALK:
+			strcat( ResRef, "g101" );
+			break;
+
+		case IE_ANI_CAST:
+		case IE_ANI_CONJURE:
+			strcat( ResRef, "g205" );
+			Cycle += 45;
+			break;
+
+		case IE_ANI_ATTACK:
+		case IE_ANI_ATTACK_SLASH:
+			strcat( ResRef, "g206" );
+			Cycle += 54;
+			break;
+
+		case IE_ANI_ATTACK_BACKSLASH:
+			strcat( ResRef, "g202" );
+			break;
+
+		case IE_ANI_ATTACK_JAB:
+			strcat( ResRef, "g203" );
+			Cycle += 18;
+			break;
+
+		case IE_ANI_DIE:
+		case IE_ANI_GET_UP:
+		case IE_ANI_EMERGE:
+			strcat( ResRef, "g104" );
+			Cycle += 36;
+			break;
+
+		case IE_ANI_SLEEP:
+		case IE_ANI_TWITCH:
+		case IE_ANI_DAMAGE:
+			strcat( ResRef, "g103" );
+			Cycle += 27;
+			break;
+
+		default:
+			error("CharAnimation", "Four frames 2 Animation: unhandled stance: %s %d\n", ResRef, StanceID);
+			break;
+
+	}
+	ResRef[6]=(char) (Part+'1');
+}
+
 void CharAnimations::AddNFSuffix(char* ResRef, unsigned char StanceID,
 	unsigned char& Cycle, unsigned char Orient, int Part)
 {
 	char prefix[10];
 
 	Cycle = SixteenToNine[Orient];
-	snprintf(prefix, 9, "%s%s%d%s%d", ResRef, StancePrefix[StanceID], Part+1,
+	snprintf(prefix, 9, "%s%c%d%c%d", ResRef, StancePrefix[StanceID], Part+1,
 			 CyclePrefix[StanceID], Cycle);
 	strnlwrcpy(ResRef,prefix,8);
 	Cycle=(ieByte) (Cycle+CycleOffset[StanceID]);
@@ -2544,6 +2632,7 @@ void CharAnimations::AddHLSuffix(char* ResRef, unsigned char StanceID,
 void CharAnimations::PulseRGBModifiers()
 {
 	unsigned long time = core->GetGame()->Ticks;
+	int i;
 
 	if (time - lastModUpdate <= 40)
 		return;
@@ -2556,7 +2645,9 @@ void CharAnimations::PulseRGBModifiers()
 		GlobalColorMod.speed > 0)
 	{
 		GlobalColorMod.phase += inc;
-		change[0] = change[1] = change[2] = change[3] = true;
+		for (i = 0; i < PAL_MAX; ++i) {
+			change[i] = true;
+		}
 
 		// reset if done
 		if (GlobalColorMod.phase > 2*GlobalColorMod.speed) {
@@ -2567,12 +2658,12 @@ void CharAnimations::PulseRGBModifiers()
 		}
 	}
 
-	for (int i = 0; i < 32; ++i) {
+	for (i = 0; i < PAL_MAX * 8; ++i) {
 		if (ColorMods[i].type != RGBModifier::NONE &&
 			ColorMods[i].speed > 0)
 		{
-			ColorMods[i].phase += inc;
 			change[i>>3] = true;
+			ColorMods[i].phase += inc;
 			if (ColorMods[i].phase > 2*ColorMods[i].speed) {
 				ColorMods[i].type = RGBModifier::NONE;
 				ColorMods[i].phase = 0;
@@ -2582,21 +2673,11 @@ void CharAnimations::PulseRGBModifiers()
 		}
 	}
 
-	if (change[0]) {
-		change[0]=0;
-		SetupColors(PAL_MAIN);
-	}
-	if (change[1]) {
-		change[1]=0;
-		SetupColors(PAL_WEAPON);
-	}
-	if (change[2]) {
-		change[2]=0;
-		SetupColors(PAL_OFFHAND);
-	}
-	if (change[3]) {
-		change[3]=0;
-		SetupColors(PAL_HELMET);
+	for (i = 0; i < PAL_MAX; ++i) {
+		if (change[i]) {
+			change[i] = false;
+			SetupColors((PaletteType) i);
+		}
 	}
 
 	lastModUpdate += inc*40;
