@@ -84,6 +84,8 @@ namespace GemRB {
 #define IE_ANI_TWO_FILES_3B		16    //iwd animations (eg. MBBM)
 #define IE_ANI_TWO_PIECE		17    //MAKH
 #define IE_ANI_FOUR_FILES_3		18    //mostly civilians
+#define IE_ANI_TWO_FILES_4		19
+#define IE_ANI_FOUR_FRAMES_2	20    //MDEM
 
 //PST animation types
 #define IE_ANI_PST_ANIMATION_1		56   //full animation
@@ -109,6 +111,18 @@ namespace GemRB {
 //special flags
 #define AV_NO_BODY_HEAT                 1
 
+enum PaletteType {
+	PAL_MAIN,
+	PAL_MAIN_2,
+	PAL_MAIN_3,
+	PAL_MAIN_4,
+	PAL_MAIN_5,
+	PAL_WEAPON,
+	PAL_OFFHAND,
+	PAL_HELMET,
+	PAL_MAX
+};
+
 struct AvatarStruct {
 	/* entries from avatars.2da */
 	unsigned int AnimID;
@@ -145,17 +159,17 @@ private:
 	char OffhandRef[2];
 public:
 	const ieDword *Colors; //these are the custom color indices
-	RGBModifier ColorMods[32]; // color modification effects
+	RGBModifier ColorMods[PAL_MAX*8]; // color modification effects
 	unsigned long lastModUpdate;
 	RGBModifier GlobalColorMod; // global color modification effect
 
-	bool change[4];
-	Palette* palette[4];
-	Palette* modifiedPalette[4];
+	bool change[PAL_MAX];
+	Palette* palette[PAL_MAX];
+	Palette* modifiedPalette[PAL_MAX];
 	unsigned int AvatarsRowNum;
 	unsigned char ArmorType, WeaponType, RangedType;
 	ieResRef ResRef;
-	ieResRef PaletteResRef;
+	ieResRef PaletteResRef[5];
 	unsigned char nextStanceID, StanceID;
 	bool autoSwitchOnEnd;
 	bool lockPalette;
@@ -205,6 +219,8 @@ private:
 		unsigned char& Cycle, unsigned char Orient);
 	void AddFFSuffix(char* ResRef, unsigned char AnimID,
 		unsigned char& Cycle, unsigned char Orient, int Part);
+	void AddFF2Suffix(char* ResRef, unsigned char AnimID,
+		unsigned char& Cycle, unsigned char Orient, int Part) const;
 	void AddHLSuffix(char* ResRef, unsigned char AnimID,
 		unsigned char& Cycle, unsigned char Orient);
 	void AddNFSuffix(char* ResRef, unsigned char AnimID,
