@@ -22,12 +22,14 @@
 
 #include "win32def.h"
 
+#include "AnimationFactory.h"
 #include "GameData.h"
+#include "GlobalTimer.h"
 #include "Interface.h"
-#include "Palette.h"    /* needed only for paperdoll palettes */
-#include "Video.h"      /* needed only for paperdoll palettes */
 #include "GUI/Button.h"
 
+namespace GemRB {
+
 ControlAnimation::ControlAnimation(Control* ctl, const ieResRef ResRef, int Cycle)
 {
 	control = NULL;
@@ -45,6 +47,7 @@ ControlAnimation::ControlAnimation(Control* ctl, const ieResRef ResRef, int Cycl
 	control = ctl;
 	control->animation = this;
 	has_palette = false;
+	is_blended = false;
 }
 
 //freeing the bitmaps only once, but using an intelligent algorithm
@@ -69,11 +72,17 @@ bool ControlAnimation::SameResource(const ieResRef ResRef, int Cycle)
 	return true;
 }
 
-void ControlAnimation::UpdateAnimation(void)
+void ControlAnimation::UpdateAnimation(bool paused)
 {
 	unsigned long time;
 	int Cycle = cycle;
 
+	if (paused && !(control->Flags & IE_GUI_BUTTON_PLAYALWAYS)) {
+		// try again later
+		core->timer->AddAnimation( this, 1 );
+		return;
+	}
+
 	if (control->Flags & IE_GUI_BUTTON_PLAYRANDOM) {
 		// simple Finite-State Machine
 		if (anim_phase == 0) {
@@ -108,6 +117,7 @@ void ControlAnimation::UpdateAnimation(void)
 		//stopping at end frame
 		if (control->Flags & IE_GUI_BUTTON_PLAYONCE) {
 			core->timer->RemoveAnimation( this );
+			control->SetAnimPicture( NULL );
 			return;
 		}
 		anim_phase = 0;
@@ -122,8 +132,18 @@ void ControlAnimation::UpdateAnimation(void)
 	if (has_palette) {
 		Palette* palette = pic->GetPalette();
 		palette->SetupPaperdollColours(colors, 0);
+		if (is_blended) {
+			palette->CreateShadedAlphaChannel();
+		}
 		pic->SetPalette(palette);
-		palette->Release();
+		palette->release();
+	} else {
+		if (is_blended) {
+			Palette* palette = pic->GetPalette();
+			palette->CreateShadedAlphaChannel();
+			pic->SetPalette(palette);
+			palette->release();
+		}
 	}
 
 	control->SetAnimPicture( pic );
@@ -136,3 +156,9 @@ void ControlAnimation::SetPaletteGradients(ieDword *col)
 	has_palette = true;
 }
 
+void ControlAnimation::SetBlend(bool b)
+{
+	is_blended = b;
+}
+
+}
