@@ -23,13 +23,15 @@
 
 #include "RGBAColor.h"
 #include "exports.h"
-
-#include "AnimationFactory.h"
-#include "Sprite2D.h"
-#include "GUI/Control.h"
+#include "globals.h"
 
 #include <vector>
 
+namespace GemRB {
+
+class AnimationFactory;
+class Control;
+
 class GEM_EXPORT ControlAnimation {
 private:
 	AnimationFactory* bam;
@@ -38,14 +40,18 @@ private:
 	unsigned int frame;
 	unsigned int anim_phase;
 	bool has_palette;
+	bool is_blended;
 	ieDword colors[8];
 public:
 	ControlAnimation(Control* ctl, const ieResRef ResRef, int Cycle = 0);
 	~ControlAnimation(void);
-	void UpdateAnimation();
+	void UpdateAnimation(bool paused);
 	//report if the current resource is the same as descripted by the params
 	bool SameResource(const ieResRef ResRef, int Cycle);
 	void SetPaletteGradients(ieDword *col);
+	void SetBlend(bool b);
 };
 
+}
+
 #endif
