@@ -46,67 +46,10 @@ BOOL WINAPI DllEntryPoint(HINSTANCE /*hinstDLL*/, DWORD /*fdwReason*/,
 }
 #endif
 
+namespace GemRB {
+
 //// Globally used functions
 
-ieByte pl_uppercase[256];
-ieByte pl_lowercase[256];
-
-// these 3 functions will copy a string to a zero terminated string with a maximum length
-void strnlwrcpy(char *dest, const char *source, int count)
-{
-	while(count--) {
-		*dest++ = pl_lowercase[(ieByte) *source];
-		if(!*source++) {
-			while(count--) *dest++=0;
-			break;
-		}
-	}
-	*dest=0;
-}
-
-void strnuprcpy(char* dest, const char *source, int count)
-{
-	while(count--) {
-		*dest++ = pl_uppercase[(ieByte) *source];
-		if(!*source++) {
-			while(count--) *dest++=0;
-			break;
-		}
-	}
-	*dest=0;
-}
-
-// this one also filters spaces, used to copy variables 
-void strnspccpy(char* dest, const char *source, int count)
-{
-	memset(dest,0,count);
-	while(count--) {
-		char c = pl_uppercase[(ieByte) *source];
-		if (c!=' ') {
-			*dest++=c;
-		}
-		if(!*source++) {
-			return;
-		}
-	}
-}
-
-#ifndef HAVE_STRNLEN
-int strnlen(const char* string, int maxlen)
-{
-	if (!string) {
-		return -1;
-	}
-	int i = 0;
-	while (maxlen-- > 0) {
-		if (!string[i])
-			break;
-		i++;
-	}
-	return i;
-}
-#endif // ! HAVE_STRNLEN
-
 static const unsigned char orientations[25]={
 6,7,8,9,10,
 5,6,8,10,11,
@@ -270,54 +213,11 @@ int EARelation(Scriptable* Owner, Actor* target)
 	return EAR_NEUTRAL;
 }
 
-/** Returns the length of string (up to a delimiter) */
-GEM_EXPORT int strlench(const char* string, char ch)
+// safely copies a ResRef (ie. nulls out the unused buffer size)
+void CopyResRef(ieResRef d, const ieResRef s)
 {
-	int i;
-	for (i = 0; string[i] && string[i] != ch; i++)
-		;
-	return i;
+	strncpy(d, s, sizeof(ieResRef) - 1);
+	d[sizeof(ieResRef) - 1] = '\0';
 }
 
-//// Compatibility functions
-#ifndef HAVE_STRNDUP
-GEM_EXPORT char* strndup(const char* s, size_t l)
-{
-	size_t len = strlen( s );
-	if (len < l) {
-		l = len;
-	}
-	char* string = ( char* ) malloc( l + 1 );
-	strncpy( string, s, l );
-	string[l] = 0;
-	return string;
 }
-#endif
-
-#ifdef WIN32
-
-#else
-
-char* strupr(char* string)
-{
-	char* s;
-	if (string) {
-		for (s = string; *s; ++s)
-			*s = toupper( *s );
-	}
-	return string;
-}
-
-char* strlwr(char* string)
-{
-	char* s;
-	if (string) {
-		for (s = string; *s; ++s)
-			*s = tolower( *s );
-	}
-	return string;
-}
-
-
-#endif // ! WIN32
-
