@@ -34,6 +34,8 @@
 #include "GUI/GameControl.h"
 #include "GUI/TextArea.h"
 
+namespace GemRB {
+
 //translate section values (journal, solved, unsolved, user)
 static int sectionMap[4]={4,1,2,0};
 static const int bg2Sections[4]={4,1,2,0};
@@ -45,6 +47,7 @@ DialogHandler::DialogHandler(void)
 	targetID = 0;
 	originalTargetID = 0;
 	speakerID = 0;
+	initialState = -1;
 	if (core->HasFeature(GF_JOURNAL_HAS_SECTIONS) ) {
 		memcpy(sectionMap, bg2Sections, sizeof(sectionMap) );
 	} else {
@@ -72,7 +75,7 @@ bool DialogHandler::InitDialog(Scriptable* spk, Scriptable* tgt, const char* dlg
 	dlg = dm->GetDialog();
 
 	if (!dlg) {
-		printMessage("GameControl", "Cannot start dialog: %s\n", LIGHT_RED, dlgref);
+		Log(ERROR, "GameControl", "Cannot start dialog: %s", dlgref);
 		return false;
 	}
 
@@ -100,13 +103,20 @@ bool DialogHandler::InitDialog(Scriptable* spk, Scriptable* tgt, const char* dlg
 	if (!gc)
 		return false;
 
+	Video *video = core->GetVideoDriver();
+	Region vp = video->GetViewport();
+	//allow mouse selection from dialog (even though screen is locked)
+	video->SetMouseEnabled(true);
+	core->timer->SetMoveViewPort( tgt->Pos.x, tgt->Pos.y, 0, true );
+	video->MoveViewportTo( tgt->Pos.x-vp.w/2, tgt->Pos.y-vp.h/2 );
+
 	//check if we are already in dialog
 	if (gc->GetDialogueFlags()&DF_IN_DIALOG) {
 		return true;
 	}
 
-	int si = dlg->FindFirstState( tgt );
-	if (si < 0) {
+	initialState = dlg->FindFirstState( tgt );
+	if (initialState < 0) {
 		return false;
 	}
 
@@ -116,6 +126,7 @@ bool DialogHandler::InitDialog(Scriptable* spk, Scriptable* tgt, const char* dlg
 
 	//no exploring while in dialogue
 	gc->SetScreenFlags(/*SF_GUIENABLED|*/SF_DISABLEMOUSE|SF_LOCKSCROLL, BM_OR);
+	Log(WARNING, "DialogHandler", "Errors occuring while in dialog mode cannot be logged in the MessageWindow.");
 	gc->SetDialogueFlags(DF_IN_DIALOG, BM_OR);
 
 	if (tgt->Type==ST_ACTOR) {
@@ -123,12 +134,6 @@ bool DialogHandler::InitDialog(Scriptable* spk, Scriptable* tgt, const char* dlg
 		tar->DialogInterrupt();
 	}
 
-	//allow mouse selection from dialog (even though screen is locked)
-	Video *video = core->GetVideoDriver();
-	Region vp = video->GetViewport();
-	video->SetMouseEnabled(true);
-	core->timer->SetMoveViewPort( tgt->Pos.x, tgt->Pos.y, 0, true );
-	video->MoveViewportTo( tgt->Pos.x-vp.w/2, tgt->Pos.y-vp.h/2 );
 	//there are 3 bits, if they are all unset, the dialog freezes scripts
 	if (!(dlg->Flags&7) ) {
 		gc->SetDialogueFlags(DF_FREEZE_SCRIPTS, BM_OR);
@@ -174,8 +179,11 @@ void DialogHandler::EndDialog(bool try_to_break)
 	core->GetGUIScriptEngine()->RunFunction( "GUIWORLD", "DialogEnded" );
 	//restoring original size
 	core->GetGame()->SetControlStatus(CS_DIALOG, BM_NAND);
-	core->GetGameControl()->SetScreenFlags(SF_DISABLEMOUSE|SF_LOCKSCROLL, BM_NAND);
-	core->GetGameControl()->SetDialogueFlags(0, BM_SET);
+	GameControl* gc = core->GetGameControl();
+	if ( !(gc->GetScreenFlags()&SF_CUTSCENE)) {
+		gc->SetScreenFlags(SF_DISABLEMOUSE|SF_LOCKSCROLL, BM_NAND);
+	}
+	gc->SetDialogueFlags(0, BM_SET);
 	core->SetEventFlag(EF_PORTRAIT);
 }
 
@@ -184,21 +192,21 @@ void DialogHandler::DialogChoose(unsigned int choose)
 {
 	TextArea* ta = core->GetMessageTextArea();
 	if (!ta) {
-		printMessage("GameControl","Dialog aborted???",LIGHT_RED);
+		Log(ERROR, "GameControl", "Dialog aborted???");
 		EndDialog();
 		return;
 	}
 
 	Actor *speaker = GetSpeaker();
 	if (!speaker) {
-		printMessage("GameControl","Speaker gone???",LIGHT_RED);
+		Log(ERROR, "GameControl", "Speaker gone???");
 		EndDialog();
 		return;
 	}
 
 	Scriptable *target = GetTarget();
 	if (!target) {
-		printMessage("GameControl","Target gone???",LIGHT_RED);
+		Log(ERROR, "GameControl", "Target gone???");
 		EndDialog();
 		return;
 	}
@@ -207,16 +215,11 @@ void DialogHandler::DialogChoose(unsigned int choose)
 		tgt = (Actor *)target;
 	}
 
-	Video *video = core->GetVideoDriver();
-	Region vp = video->GetViewport();
-	video->SetMouseEnabled(true);
-	core->timer->SetMoveViewPort( target->Pos.x, target->Pos.y, 0, true );
-	video->MoveViewportTo( target->Pos.x-vp.w/2, target->Pos.y-vp.h/2 );
-
+	int si;
 	if (choose == (unsigned int) -1) {
 		//increasing talkcount after top level condition was determined
 
-		int si = dlg->FindFirstState( tgt );
+		si = initialState;
 		if (si<0) {
 			EndDialog();
 			return;
@@ -231,9 +234,9 @@ void DialogHandler::DialogChoose(unsigned int choose)
 				tgt->InteractCount++;
 			}
 		}
-		ds = dlg->GetState( si );
+		//ds = dlg->GetState( si );
 	} else {
-		if (ds->transitionsCount <= choose) {
+		if (!ds || ds->transitionsCount <= choose) {
 			return;
 		}
 
@@ -250,7 +253,7 @@ void DialogHandler::DialogChoose(unsigned int choose)
 				Section |= 2;
 			}
 			if (core->GetGame()->AddJournalEntry(tr->journalStrRef, sectionMap[Section], tr->Flags>>16) ) {
-				displaymsg->DisplayConstantString(STR_JOURNALCHANGE,0xffff00);
+				displaymsg->DisplayConstantString(STR_JOURNALCHANGE, DMC_BG2XPGREEN);
 				char *string = core->GetString( tr->journalStrRef );
 				//cutting off the strings at the first crlf
 				char *poi = strchr(string,'\n');
@@ -264,11 +267,13 @@ void DialogHandler::DialogChoose(unsigned int choose)
 
 		if (tr->textStrRef != 0xffffffff) {
 			//allow_zero is for PST (deionarra's text)
-			displaymsg->DisplayStringName( (int) (tr->textStrRef), 0x8080FF, speaker, IE_STR_SOUND|IE_STR_SPEECH|IE_STR_ALLOW_ZERO);
+			displaymsg->DisplayStringName( (int) (tr->textStrRef), DMC_DIALOGPARTY, speaker, IE_STR_SOUND|IE_STR_SPEECH|IE_STR_ALLOW_ZERO);
 			if (core->HasFeature( GF_DIALOGUE_SCROLLS )) {
 				ta->AppendText( "", -1 );
 			}
 		}
+		target->ImmediateEvent();
+		target->ProcessActions(); //run the action queue now
 
 		if (tr->actions.size()) {
 			// does this belong here? we must clear actions somewhere before
@@ -276,18 +281,15 @@ void DialogHandler::DialogChoose(unsigned int choose)
 			// executing actions directly does not work, because dialog
 			// needs to end before final actions are executed due to
 			// actions making new dialogs!
-			if (target->Type == ST_ACTOR) ((Movable *)target)->ClearPath(); // fuzzie added this
-			target->ClearActions();
+			target->Stop();
 
 			// do not interrupt during dialog actions (needed for aerie.d polymorph block)
-			char buf[20];
-			strcpy(buf, "SetInterrupt(FALSE)");
-			target->AddAction( GenerateAction( buf ) );
+			target->AddAction( GenerateAction( "BreakInstants()" ) );
+			target->AddAction( GenerateAction( "SetInterrupt(FALSE)" ) );
 			for (unsigned int i = 0; i < tr->actions.size(); i++) {
 				target->AddAction(tr->actions[i]);
 			}
-			strcpy(buf, "SetInterrupt(TRUE)");
-			target->AddAction( GenerateAction( buf ) );
+			target->AddAction( GenerateAction( "SetInterrupt(TRUE)" ) );
 		}
 
 		int final_dialog = tr->Flags & IE_DLG_TR_FINAL;
@@ -306,7 +308,7 @@ void DialogHandler::DialogChoose(unsigned int choose)
 		core->SetCutSceneMode( false );
 
 		//displaying dialog for selected option
-		int si = tr->stateIndex;
+		si = tr->stateIndex;
 		//follow external linkage, if required
 		if (tr->Dialog[0] && strnicmp( tr->Dialog, dlg->ResRef, 8 )) {
 			//target should be recalculated!
@@ -346,7 +348,7 @@ void DialogHandler::DialogChoose(unsigned int choose)
 			}
 			target = tgt;
 			if (!target) {
-				printMessage("Dialog","Can't redirect dialog\n",YELLOW);
+				Log(WARNING, "Dialog", "Can't redirect dialog");
 				ta->SetMinRow( false );
 				EndDialog();
 				return;
@@ -361,7 +363,7 @@ void DialogHandler::DialogChoose(unsigned int choose)
 			/*if (target->GetInternalFlag()&IF_NOINT) {
 				// this whole check moved out of InitDialog by fuzzie, see comments
 				// for the IF_NOINT check in BeginDialog
-				displaymsg->DisplayConstantString(STR_TARGETBUSY,0xff0000);
+				displaymsg->DisplayConstantString(STR_TARGETBUSY, DMC_RED);
 				ta->SetMinRow( false );
 				EndDialog();
 				return;
@@ -373,16 +375,18 @@ void DialogHandler::DialogChoose(unsigned int choose)
 				return;
 			}
 		}
-		ds = dlg->GetState( si );
-		if (!ds) {
-			printMessage("Dialog","Can't find next dialog\n",YELLOW);
-			ta->SetMinRow( false );
-			EndDialog();
-			return;
-		}
 	}
+
+	ds = dlg->GetState( si );
+	if (!ds) {
+		Log(WARNING, "Dialog", "Can't find next dialog");
+		ta->SetMinRow( false );
+		EndDialog();
+		return;
+	}
+
 	//displaying npc text
-	displaymsg->DisplayStringName( ds->StrRef, 0x70FF70, target, IE_STR_SOUND|IE_STR_SPEECH);
+	displaymsg->DisplayStringName( ds->StrRef, DMC_DIALOG, target, IE_STR_SOUND|IE_STR_SPEECH);
 	//adding a gap between options and npc text
 	ta->AppendText("",-1);
 	int i;
@@ -433,7 +437,7 @@ void DialogHandler::DialogChoose(unsigned int choose)
 	}
 	// this happens if a trigger isn't implemented or the dialog is wrong
 	if (!idx) {
-		printMessage("Dialog", "There were no valid dialog options!\n", YELLOW);
+		Log(WARNING, "Dialog", "There were no valid dialog options!");
 		core->GetGameControl()->SetDialogueFlags(DF_OPENENDWINDOW, BM_OR);
 	}
 end_of_choose:
@@ -491,3 +495,5 @@ Actor *DialogHandler::GetSpeaker()
 	return GetActorByGlobalID(speakerID);
 }
 
+
+}
