@@ -25,6 +25,8 @@
 
 #include "Dialog.h"
 
+namespace GemRB {
+
 class GEM_EXPORT DialogHandler {
 public:
 	DialogHandler();
@@ -35,6 +37,7 @@ private:
 private:
 	DialogState* ds;
 	Dialog* dlg;
+	int initialState;
 public:
 	ieDword speakerID;
 	ieDword targetID;
@@ -43,9 +46,11 @@ public:
 	Scriptable *GetTarget();
 	Actor *GetSpeaker();
 
-	int InitDialog(Scriptable* speaker, Scriptable* target, const char* dlgref);
+	bool InitDialog(Scriptable* speaker, Scriptable* target, const char* dlgref);
 	void EndDialog(bool try_to_break=false);
 	void DialogChoose(unsigned int choose);
 };
 
+}
+
 #endif
