@@ -25,12 +25,16 @@
 #include "Plugin.h"
 #include "System/DataStream.h"
 
+namespace GemRB {
+
 class GEM_EXPORT DialogMgr : public Plugin {
 public:
 	DialogMgr(void);
 	virtual ~DialogMgr(void);
-	virtual bool Open(DataStream* stream, bool autoFree = true) = 0;
+	virtual bool Open(DataStream* stream) = 0;
 	virtual Dialog* GetDialog() const = 0;
 };
 
+}
+
 #endif
