@@ -32,6 +32,16 @@
 
 #include <cstdlib>
 
+namespace GemRB {
+
+#define DMC_WHITE 0xf0f0f0
+#define DMC_RED 0xff0000
+#define DMC_LIGHTGREY 0xd7d7be
+#define DMC_BG2XPGREEN 0xbcefbc
+#define DMC_GOLD 0xc0c000
+#define DMC_DIALOG DMC_WHITE
+#define DMC_DIALOGPARTY 0x8080FF
+
 class Scriptable;
 
 class GEM_EXPORT DisplayMessage
@@ -58,6 +68,8 @@ public:
 	void DisplayConstantStringValue(int stridx, unsigned int color, ieDword value) const;
 	/** displays a string constant in the textarea, starting with speaker's name */
 	void DisplayConstantStringName(int stridx, unsigned int color, const Scriptable *speaker) const;
+	/** displays a string constant in the textarea, starting with speaker's name, also replaces one numeric value (it is a format string) */
+	void DisplayConstantStringNameValue(int stridx, unsigned int color, const Scriptable *speaker, int value) const;
 	/** displays a string constant in the textarea, starting with actor, and ending with target */
 	void DisplayConstantStringAction(int stridx, unsigned int color, const Scriptable *actor, const Scriptable *target) const;
 	/** displays a string in the textarea */
@@ -66,8 +78,12 @@ public:
 	/** displays a string in the textarea, starting with speaker's name */
 	void DisplayStringName(int stridx, unsigned int color, const Scriptable *speaker, ieDword flags) const;
 	void DisplayStringName(const char *text, unsigned int color, const Scriptable *speaker) const;
+	/** iwd2 hidden roll debugger */
+	void DisplayRollStringName(int stridx, unsigned int color, const Scriptable *speaker, ...) const;
 };
 
 extern GEM_EXPORT DisplayMessage * displaymsg;
 
+}
+
 #endif
