@@ -20,6 +20,8 @@
 
 #include "EffectQueue.h"
 
+#include "ie_feats.h"
+#include "overlays.h"
 #include "strrefs.h"
 
 #include "DisplayMessage.h"
@@ -29,11 +31,14 @@
 #include "Map.h"
 #include "SymbolMgr.h"
 #include "Scriptable/Actor.h"
-#include "Spell.h"  //needs for the source flags bitfield
+#include "Spell.h" //needs for the source flags bitfield
 #include "TableMgr.h"
+#include "System/StringBuffer.h"
 
 #include <cstdio>
 
+namespace GemRB {
+
 static struct {
 	const char* Name;
 	EffectFunction Function;
@@ -45,6 +50,49 @@ static int initialized = 0;
 static EffectDesc *effectnames = NULL;
 static int effectnames_count = 0;
 static int pstflags = false;
+static bool iwd2fx = false;
+
+static EffectRef fx_unsummon_creature_ref = { "UnsummonCreature", -1 };
+static EffectRef fx_ac_vs_creature_type_ref = { "ACVsCreatureType", -1 };
+static EffectRef fx_spell_focus_ref = { "SpellFocus", -1 };
+static EffectRef fx_spell_resistance_ref = { "SpellResistance", -1 };
+static EffectRef fx_protection_from_display_string_ref = { "Protection:String", -1 };
+static EffectRef fx_variable_ref = { "Variable:StoreLocalVariable", -1 };
+
+//immunity effects
+static EffectRef fx_level_immunity_ref = { "Protection:Spelllevel", -1 };
+static EffectRef fx_opcode_immunity_ref = { "Protection:Opcode", -1 }; //bg2
+static EffectRef fx_opcode_immunity2_ref = { "Protection:Opcode2", -1 };//iwd
+static EffectRef fx_spell_immunity_ref = { "Protection:Spell", -1 }; //bg2
+static EffectRef fx_spell_immunity2_ref = { "Protection:Spell2", -1 };//iwd
+static EffectRef fx_store_spell_sequencer_ref = { "Sequencer:Store", -1 }; //bg2, works against sequencers
+static EffectRef fx_school_immunity_ref = { "Protection:School", -1 };
+static EffectRef fx_secondary_type_immunity_ref = { "Protection:SecondaryType", -1 };
+
+//decrementing immunity effects
+static EffectRef fx_level_immunity_dec_ref = { "Protection:SpellLevelDec", -1 };
+static EffectRef fx_spell_immunity_dec_ref = { "Protection:SpellDec", -1 };
+static EffectRef fx_school_immunity_dec_ref = { "Protection:SchoolDec", -1 };
+static EffectRef fx_secondary_type_immunity_dec_ref = { "Protection:SecondaryTypeDec", -1 };
+
+//bounce effects
+static EffectRef fx_level_bounce_ref = { "Bounce:SpellLevel", -1 };
+//static EffectRef fx_opcode_bounce_ref = { "Bounce:Opcode", -1 };
+static EffectRef fx_spell_bounce_ref = { "Bounce:Spell", -1 };
+static EffectRef fx_school_bounce_ref = { "Bounce:School", -1 };
+static EffectRef fx_secondary_type_bounce_ref = { "Bounce:SecondaryType", -1 };
+
+//decrementing bounce effects
+static EffectRef fx_level_bounce_dec_ref = { "Bounce:SpellLevelDec", -1 };
+static EffectRef fx_spell_bounce_dec_ref = { "Bounce:SpellDec", -1 };
+static EffectRef fx_school_bounce_dec_ref = { "Bounce:SchoolDec", -1 };
+static EffectRef fx_secondary_type_bounce_dec_ref = { "Bounce:SecondaryTypeDec", -1 };
+
+//spelltrap (multiple decrementing immunity)
+static EffectRef fx_spelltrap = { "SpellTrap", -1 };
+
+//weapon immunity
+static EffectRef fx_weapon_immunity_ref = { "Protection:Weapons", -1 };
 
 bool EffectQueue::match_ids(Actor *target, int table, ieDword value)
 {
@@ -55,6 +103,10 @@ bool EffectQueue::match_ids(Actor *target, int table, ieDword value)
 	int a, stat;
 
 	switch (table) {
+	case 0:
+		stat = IE_FACTION; break;
+	case 1:
+		stat = IE_TEAM; break;
 	case 2: //EA
 		stat = IE_EA; break;
 	case 3: //GENERAL
@@ -89,6 +141,10 @@ bool EffectQueue::match_ids(Actor *target, int table, ieDword value)
 			}
 		}
 		return true;
+	case 9:
+		stat = target->GetClassMask();
+		if (value&stat) return true;
+		return false;
 	default:
 		return false;
 	}
@@ -176,13 +232,11 @@ static EffectDesc* FindEffect(const char* effectname)
 	}
 	void *tmp = bsearch(effectname, effectnames, effectnames_count, sizeof(EffectDesc), find_effect);
 	if( !tmp) {
-		printMessage("EffectQueue", "Couldn't assign effect: %s\n", YELLOW, effectname);
+		Log(WARNING, "EffectQueue", "Couldn't assign effect: %s", effectname);
 	}
 	return (EffectDesc *) tmp;
 }
 
-static EffectRef fx_protection_from_display_string_ref = { "Protection:String", -1 };
-
 static inline void ResolveEffectRef(EffectRef &effect_reference)
 {
 	if( effect_reference.opcode==-1) {
@@ -203,6 +257,7 @@ bool Init_EffectQueue()
 		return true;
 	}
 	pstflags = !!core->HasFeature(GF_PST_STATE_FLAGS);
+	iwd2fx = !!core->HasFeature(GF_ENHANCED_EFFECTS);
 
 	memset( Opcodes, 0, sizeof( Opcodes ) );
 	for(i=0;i<MAX_EFFECTS;i++) {
@@ -215,12 +270,12 @@ bool Init_EffectQueue()
 
 	int eT = core->LoadSymbol( "effects" );
 	if( eT < 0) {
-		printMessage( "EffectQueue","A critical scripting file is missing!\n",LIGHT_RED );
+		Log(ERROR, "EffectQueue", "A critical scripting file is missing!");
 		return false;
 	}
 	Holder<SymbolMgr> effectsTable = core->GetSymbol( eT );
 	if( !effectsTable) {
-		printMessage( "EffectQueue","A critical scripting file is damaged!\n",LIGHT_RED );
+		Log(ERROR, "EffectQueue", "A critical scripting file is damaged!");
 		return false;
 	}
 
@@ -249,7 +304,7 @@ bool Init_EffectQueue()
 			}
 			poi->opcode = i;
 		}
-		//print("-------- FN: %d, %s\n", i, effectname);
+		//print("-------- FN: %d, %s", i, effectname);
 	}
 	core->DelSymbol( eT );
 
@@ -308,8 +363,7 @@ Effect *EffectQueue::CreateEffect(ieDword opcode, ieDword param1, ieDword param2
 	memset(fx,0,sizeof(Effect));
 	fx->Target = FX_TARGET_SELF;
 	fx->Opcode = opcode;
-	//probability2 is the low number (by effectqueue 331)
-	fx->Probability1 = 100;
+	fx->ProbabilityRangeMax = 100;
 	fx->Parameter1 = param1;
 	fx->Parameter2 = param2;
 	fx->TimingMode = timing;
@@ -395,17 +449,17 @@ Effect *EffectQueue::CreateEffectCopy(Effect *oldfx, EffectRef &effect_reference
 	return CreateEffectCopy(oldfx, effect_reference.opcode, param1, param2);
 }
 
-static EffectRef fx_unsummon_creature_ref = { "UnsummonCreature", -1 };
-
 Effect *EffectQueue::CreateUnsummonEffect(Effect *fx)
 {
 	Effect *newfx = NULL;
 	if( (fx->TimingMode&0xff) == FX_DURATION_INSTANT_LIMITED) {
 		newfx = CreateEffectCopy(fx, fx_unsummon_creature_ref, 0, 0);
 		newfx->TimingMode = FX_DURATION_DELAY_PERMANENT;
+		newfx->Target = FX_TARGET_PRESET;
 		if( newfx->Resource3[0]) {
 			strnuprcpy(newfx->Resource,newfx->Resource3, sizeof(ieResRef)-1 );
 		} else {
+			//FIXME: unhardcode; should probably use a spell hit (shtable) in some way
 			strnuprcpy(newfx->Resource,"SPGFLSH1", sizeof(ieResRef)-1 );
 		}
 		if( fx->TimingMode == FX_DURATION_ABSOLUTE) {
@@ -585,7 +639,9 @@ int EffectQueue::AddEffect(Effect* fx, Scriptable* self, Actor* pretarget, const
 		flg = FX_APPLIED;
 		break;
 	case FX_TARGET_PRESET:
-		fx->SetPosition(pretarget->Pos);
+		//fx->SetPosition(pretarget->Pos);
+		//knock needs this
+		fx->SetPosition(dest);
 
 		flg = ApplyEffect( pretarget, fx, 1 );
 		if( fx->TimingMode != FX_DURATION_JUST_EXPIRED) {
@@ -653,7 +709,7 @@ all_party:
 
 	case FX_TARGET_UNKNOWN:
 	default:
-		print( "Unknown FX target type: %d\n", fx->Target);
+		Log(MESSAGE, "EffectQueue", "Unknown FX target type: %d", fx->Target);
 		flg = FX_ABORT;
 		break;
 	}
@@ -738,8 +794,14 @@ static inline bool check_level(Actor *target, Effect *fx)
 
 	ieDword level = (ieDword) target->GetXPLevel( true );
 	//return true if resisted
-	if (fx->MinAffectedLevel > 0 || fx->MaxAffectedLevel > 0) {
-		if (level < fx->MinAffectedLevel || level > fx->MaxAffectedLevel) {
+	if ((signed)fx->MinAffectedLevel > 0) {
+		if ((signed)level < (signed)fx->MinAffectedLevel) {
+			return true;
+		}
+	}
+
+	if ((signed)fx->MaxAffectedLevel > 0) {
+		if ((signed)level > (signed)fx->MaxAffectedLevel) {
 			return true;
 		}
 	}
@@ -750,55 +812,21 @@ static inline bool check_level(Actor *target, Effect *fx)
 //roll should hit in the middle
 static inline bool check_probability(Effect* fx)
 {
-	//watch for this, probability1 is the high number
-	//probability2 is the low number
 	//random value is 0-99
-	if( fx->random_value<=fx->Probability2 || fx->random_value>fx->Probability1) {
+	if (fx->random_value<fx->ProbabilityRangeMin || fx->random_value>fx->ProbabilityRangeMax) {
 		return false;
 	}
 	return true;
 }
 
-//immunity effects
-static EffectRef fx_level_immunity_ref = { "Protection:Spelllevel", -1 };
-static EffectRef fx_opcode_immunity_ref = { "Protection:Opcode", -1 }; //bg2
-static EffectRef fx_opcode_immunity2_ref = { "Protection:Opcode2", -1 };//iwd
-static EffectRef fx_spell_immunity_ref = { "Protection:Spell", -1 }; //bg2
-static EffectRef fx_spell_immunity2_ref = { "Protection:Spell2", -1 };//iwd
-static EffectRef fx_store_spell_sequencer_ref = { "Sequencer:Store", -1 }; //bg2, works against sequencers
-static EffectRef fx_school_immunity_ref = { "Protection:School", -1 };
-static EffectRef fx_secondary_type_immunity_ref = { "Protection:SecondaryType", -1 };
-
-//decrementing immunity effects
-static EffectRef fx_level_immunity_dec_ref = { "Protection:SpellLevelDec", -1 };
-static EffectRef fx_spell_immunity_dec_ref = { "Protection:SpellDec", -1 };
-static EffectRef fx_school_immunity_dec_ref = { "Protection:SchoolDec", -1 };
-static EffectRef fx_secondary_type_immunity_dec_ref = { "Protection:SecondaryTypeDec", -1 };
-
-//bounce effects
-static EffectRef fx_level_bounce_ref = { "Bounce:SpellLevel", -1 };
-//static EffectRef fx_opcode_bounce_ref = { "Bounce:Opcode", -1 };
-static EffectRef fx_spell_bounce_ref = { "Bounce:Spell", -1 };
-static EffectRef fx_school_bounce_ref = { "Bounce:School", -1 };
-static EffectRef fx_secondary_type_bounce_ref = { "Bounce:SecondaryType", -1 };
-
-//decrementing bounce effects
-static EffectRef fx_level_bounce_dec_ref = { "Bounce:SpellLevelDec", -1 };
-static EffectRef fx_spell_bounce_dec_ref = { "Bounce:SpellDec", -1 };
-static EffectRef fx_school_bounce_dec_ref = { "Bounce:SchoolDec", -1 };
-static EffectRef fx_secondary_type_bounce_dec_ref = { "Bounce:SecondaryTypeDec", -1 };
-
-//spelltrap (multiple decrementing immunity)
-static EffectRef fx_spelltrap = { "SpellTrap", -1 };
-
 //this is for whole spell immunity/bounce
-static inline void DecreaseEffect(Effect *efx)
+static inline int DecreaseEffect(Effect *efx)
 {
-	efx->Parameter1--;
-	if( (int) efx->Parameter1<1) {
-		//don't remove effects directly!!!
-		efx->TimingMode = FX_DURATION_JUST_EXPIRED;
+	if (efx->Parameter1) {
+		efx->Parameter1--;
+	return true;
 	}
+	return false;
 }
 
 //lower decreasing immunities/bounces
@@ -819,7 +847,7 @@ static int check_type(Actor* actor, Effect* fx)
 	}
 */
 	//spell level immunity
-	if(fx->Power && actor->fxqueue.HasEffectWithParamPair(fx_level_immunity_ref, fx->Power, 0) ) {
+	if(fx->Power && actor->fxqueue.HasEffectWithParamPair(fx_level_immunity_ref, 0, fx->Power) ) {
 		return 0;
 	}
 
@@ -853,26 +881,28 @@ static int check_type(Actor* actor, Effect* fx)
 
 	//decrementing immunity checks
 	//decrementing level immunity
-	efx = actor->fxqueue.HasEffectWithParamPair(fx_level_immunity_dec_ref, fx->Power, 0);
-	if( efx ) {
-		DecreaseEffect(efx);
-		return 0;
+	if (fx->Power) {
+		efx = actor->fxqueue.HasEffectWithParamPair(fx_level_immunity_dec_ref, 0, fx->Power);
+		if( efx ) {
+			if (DecreaseEffect(efx))
+				return 0;
+		}
 	}
 
 	//decrementing spell immunity
 	if( fx->Source[0]) {
 		efx = actor->fxqueue.HasEffectWithResource(fx_spell_immunity_dec_ref, fx->Source);
 		if( efx) {
-			DecreaseEffect(efx);
-			return 0;
+			if (DecreaseEffect(efx))
+				return 0;
 		}
 	}
 	//decrementing primary type immunity (school)
 	if( fx->PrimaryType) {
 		efx = actor->fxqueue.HasEffectWithParam(fx_school_immunity_dec_ref, fx->PrimaryType);
 		if( efx) {
-			DecreaseEffect(efx);
-			return 0;
+			if (DecreaseEffect(efx))
+				return 0;
 		}
 	}
 
@@ -880,8 +910,8 @@ static int check_type(Actor* actor, Effect* fx)
 	if( fx->SecondaryType) {
 		efx = actor->fxqueue.HasEffectWithParam(fx_secondary_type_immunity_dec_ref, fx->SecondaryType);
 		if( efx) {
-			DecreaseEffect(efx);
-			return 0;
+			if (DecreaseEffect(efx))
+				return 0;
 		}
 	}
 
@@ -890,21 +920,25 @@ static int check_type(Actor* actor, Effect* fx)
 	//if the spelltrap effect already absorbed enough levels
 	//but still didn't get removed, it will absorb levels it shouldn't
 	//it will also absorb multiple spells in a single round
-	efx=actor->fxqueue.HasEffectWithParamPair(fx_spelltrap, 0, fx->Power);
-	if( efx) {
-		//storing the absorbed spell level
-		efx->Parameter3+=fx->Power;
-		//instead of a single effect, they had to create an effect for each level
-		//HOW DAMN LAME
-		//if decrease needs the spell level, use fx->Power here
-		actor->fxqueue.DecreaseParam1OfEffect(fx_spelltrap, 1);
-		//efx->Parameter1--;
-		return 0;
+	if (fx->Power) {
+		efx=actor->fxqueue.HasEffectWithParamPair(fx_spelltrap, 0, fx->Power);
+		if( efx) {
+			//storing the absorbed spell level
+			efx->Parameter3+=fx->Power;
+			//instead of a single effect, they had to create an effect for each level
+			//HOW DAMN LAME
+			//if decrease needs the spell level, use fx->Power here
+			actor->fxqueue.DecreaseParam1OfEffect(fx_spelltrap, 1);
+			//efx->Parameter1--;
+			return 0;
+		}
 	}
 
 	//bounce checks
-	if( (bounce&BNC_LEVEL) && actor->fxqueue.HasEffectWithParamPair(fx_level_bounce_ref, fx->Power, 0) ) {
-		return 0;
+	if (fx->Power) {
+		if( (bounce&BNC_LEVEL) && actor->fxqueue.HasEffectWithParamPair(fx_level_bounce_ref, 0, fx->Power) ) {
+			return 0;
+		}
 	}
 
 	if( fx->Source[0] && (bounce&BNC_RESOURCE) && actor->fxqueue.HasEffectWithResource(fx_spell_bounce_ref, fx->Source) ) {
@@ -925,35 +959,37 @@ static int check_type(Actor* actor, Effect* fx)
 	//decrementing bounce checks
 
 	//level decrementing bounce check
-	if( (bounce&BNC_LEVEL_DEC)) {
-		efx=actor->fxqueue.HasEffectWithParamPair(fx_level_bounce_dec_ref, fx->Power, 0);
-		if( efx) {
-			DecreaseEffect(efx);
-			return -1;
+	if (fx->Power) {
+		if( (bounce&BNC_LEVEL_DEC)) {
+			efx=actor->fxqueue.HasEffectWithParamPair(fx_level_bounce_dec_ref, 0, fx->Power);
+			if( efx) {
+				if (DecreaseEffect(efx))
+					return -1;
+			}
 		}
 	}
 
 	if( fx->Source[0] && (bounce&BNC_RESOURCE_DEC)) {
 		efx=actor->fxqueue.HasEffectWithResource(fx_spell_bounce_dec_ref, fx->Resource);
 		if( efx) {
-			DecreaseEffect(efx);
-			return -1;
+			if (DecreaseEffect(efx))
+				return -1;
 		}
 	}
 
 	if( fx->PrimaryType && (bounce&BNC_SCHOOL_DEC) ) {
 		efx=actor->fxqueue.HasEffectWithParam(fx_school_bounce_dec_ref, fx->PrimaryType);
 		if( efx) {
-			DecreaseEffect(efx);
-			return -1;
+			if (DecreaseEffect(efx))
+				return -1;
 		}
 	}
 
 	if( fx->SecondaryType && (bounce&BNC_SECTYPE_DEC) ) {
 		efx=actor->fxqueue.HasEffectWithParam(fx_secondary_type_bounce_dec_ref, fx->SecondaryType);
 		if( efx) {
-			DecreaseEffect(efx);
-			return -1;
+			if (DecreaseEffect(efx))
+				return -1;
 		}
 	}
 
@@ -963,17 +999,27 @@ static int check_type(Actor* actor, Effect* fx)
 //check resistances, saving throws
 static bool check_resistance(Actor* actor, Effect* fx)
 {
+	Actor *caster;
+	Scriptable *cob;
+
 	if( !actor) {
 		return false;
 	}
 
+	cob = GetCasterObject();
+	if (cob && cob->Type==ST_ACTOR) {
+		caster = (Actor *) cob;
+	} else {
+		caster = 0;
+	}
+
 	//opcode immunity
 	if( actor->fxqueue.HasEffectWithParam(fx_opcode_immunity_ref, fx->Opcode) ) {
-		print ("immune to effect: %s\n", (char*) Opcodes[fx->Opcode].Name);
+		Log(MESSAGE, "EffectQueue", "immune to effect: %s", (char*) Opcodes[fx->Opcode].Name);
 		return true;
 	}
 	if( actor->fxqueue.HasEffectWithParam(fx_opcode_immunity2_ref, fx->Opcode) ) {
-		print ("immune2 to effect: %s\n", (char*) Opcodes[fx->Opcode].Name);
+		Log(MESSAGE, "EffectQueue", "immune2 to effect: %s", (char*) Opcodes[fx->Opcode].Name);
 		return true;
 	}
 
@@ -995,7 +1041,7 @@ static bool check_resistance(Actor* actor, Effect* fx)
 
 	//don't resist self
 	bool selective_mr = core->HasFeature(GF_SELECTIVE_MAGIC_RES);
-	if (fx->Target==FX_TARGET_SELF) {
+	if (fx->CasterID == actor->GetGlobalID()) {
 		if (selective_mr) {
 			return false;
 		}
@@ -1003,18 +1049,50 @@ static bool check_resistance(Actor* actor, Effect* fx)
 
 	//magic immunity
 	ieDword val = actor->GetStat(IE_RESISTMAGIC);
-	if( (signed) fx->random_value < (signed) val) {
+	bool resisted = false;
+
+	if (iwd2fx) {
+		// 3ed style check
+		// TODO: check if luck really affects it (i doubt it does)
+		int roll = actor->LuckyRoll(1, 20, 0);
+		ieDword check = fx->CasterLevel + roll;
+		int penetration = 0;
+		// +2/+4 level bonus from the (greater) spell penetration feat
+		if (caster && caster->HasFeat(FEAT_SPELL_PENETRATION)) {
+			penetration += 2 * caster->GetStat(IE_FEAT_SPELL_PENETRATION);
+		}
+		check += penetration;
+		resisted = (signed) check < (signed) val;
+		// ~Spell Resistance check (Spell resistance:) %d vs. (d20 + caster level + spell resistance mod)  = %d + %d + %d.~
+		displaymsg->DisplayRollStringName(39673, DMC_LIGHTGREY, actor, val, roll, fx->CasterLevel, penetration);
+	} else {
+		// 2.5 style check
+		resisted = (signed) fx->random_value < (signed) val;
+	}
+	if (resisted) {
 		// we take care of irresistible spells a few checks above, so selective mr has no impact here anymore
 		displaymsg->DisplayConstantStringName(STR_MAGIC_RESISTED, DMC_WHITE, actor);
-		print ("effect resisted: %s\n", (char*) Opcodes[fx->Opcode].Name);
+		Log(MESSAGE, "EffectQueue", "effect resisted: %s", (char*) Opcodes[fx->Opcode].Name);
 		return true;
 	}
 
-	//saving throws
+	//saving throws, bonus can be improved by school specific bonus
+	int bonus = fx->SavingThrowBonus + actor->fxqueue.BonusForParam2(fx_spell_resistance_ref, fx->PrimaryType);
+	if (caster) {
+		bonus += actor->fxqueue.BonusAgainstCreature(fx_ac_vs_creature_type_ref,caster);
+		//saving throw could be made difficult by caster's school specific bonus
+		bonus -= caster->fxqueue.BonusForParam2(fx_spell_focus_ref, fx->PrimaryType);
+	}
+
 	bool saved = false;
 	for (int i=0;i<5;i++) {
 		if( fx->SavingThrowType&(1<<i)) {
-			saved = actor->GetSavingThrow(i, fx->SavingThrowBonus);
+			// FIXME: first bonus handling for iwd2 is just a guess
+			if (iwd2fx) {
+				saved = actor->GetSavingThrow(i, bonus-fx->SavingThrowBonus, fx->SpellLevel, fx->SavingThrowBonus);
+			} else {
+				saved = actor->GetSavingThrow(i, bonus);
+			}
 			if( saved) {
 				break;
 			}
@@ -1024,7 +1102,7 @@ static bool check_resistance(Actor* actor, Effect* fx)
 		if( fx->IsSaveForHalfDamage) {
 			fx->Parameter1/=2;
 		} else {
-			print ("%s saved against effect: %s\n", actor->GetName(1), (char*) Opcodes[fx->Opcode].Name);
+			Log(MESSAGE, "EffectQueue", "%s saved against effect: %s", actor->GetName(1), (char*) Opcodes[fx->Opcode].Name);
 			return true;
 		}
 	}
@@ -1043,7 +1121,10 @@ static bool check_resistance(Actor* actor, Effect* fx)
 
 int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieDword resistance) const
 {
-	//print( "FX 0x%02x: %s(%d, %d)\n", fx->Opcode, effectnames[fx->Opcode].Name, fx->Parameter1, fx->Parameter2 );
+	//print("FX 0x%02x: %s(%d, %d)", fx->Opcode, effectnames[fx->Opcode].Name, fx->Parameter1, fx->Parameter2);
+	if (fx->TimingMode == FX_DURATION_JUST_EXPIRED) {
+		return FX_NOT_APPLIED;
+	}
 	if( fx->Opcode >= MAX_EFFECTS) {
 		fx->TimingMode = FX_DURATION_JUST_EXPIRED;
 		return FX_NOT_APPLIED;
@@ -1059,9 +1140,9 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieD
 		}
 
 		//gemrb specific, stat based chance
-		if ((fx->Probability2 == 100) && Owner && (Owner->Type==ST_ACTOR) ) {
-			fx->Probability2 = 0;
-			fx->Probability1 = ((Actor *) Owner)->GetSafeStat(fx->Probability1);
+		if ((fx->ProbabilityRangeMin == 100) && Owner && (Owner->Type==ST_ACTOR) ) {
+			fx->ProbabilityRangeMin = 0;
+			fx->ProbabilityRangeMax = ((Actor *) Owner)->GetSafeStat(fx->ProbabilityRangeMax);
 		}
 
 		if (resistance) {
@@ -1133,7 +1214,7 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieD
 	if( fx->Opcode<MAX_EFFECTS) {
 		fn = Opcodes[fx->Opcode].Function;
 		if (!(target || (Opcodes[fx->Opcode].Flags & EFFECT_NO_ACTOR))) {
-			print("targetless opcode without EFFECT_NO_ACTOR: %d, skipping\n", fx->Opcode);
+			Log(MESSAGE, "EffectQueue", "targetless opcode without EFFECT_NO_ACTOR: %d, skipping", fx->Opcode);
 			return FX_NOT_APPLIED;
 		}
 	}
@@ -1175,7 +1256,7 @@ int EffectQueue::ApplyEffect(Actor* target, Effect* fx, ieDword first_apply, ieD
 			case FX_ABORT:
 				break;
 			default:
-				abort();
+				error("EffectQueue", "Unknown effect result '%x', aborting ...\n", res);
 		}
 	} else {
 		//effect not found, it is going to be discarded
@@ -1291,6 +1372,7 @@ void EffectQueue::RemoveAllEffectsWithResource(ieDword opcode, const ieResRef re
 	}
 }
 
+//this will modify effect reference
 void EffectQueue::RemoveAllEffectsWithResource(EffectRef &effect_reference, const ieResRef resource) const
 {
 	ResolveEffectRef(effect_reference);
@@ -1322,6 +1404,13 @@ void EffectQueue::RemoveAllDetrimentalEffects(ieDword opcode, ieDword current) c
 	}
 }
 
+//this will modify effect reference
+void EffectQueue::RemoveAllDetrimentalEffects(EffectRef &effect_reference, ieDword current) const
+{
+	ResolveEffectRef(effect_reference);
+	RemoveAllDetrimentalEffects(effect_reference.opcode, current);
+}
+
 //Removes all effects with a matching param2
 //param2 is usually an effect's subclass (quality) while param1 is more like quantity.
 //So opcode+param2 usually pinpoints an effect better when not all effects of a given
@@ -1338,6 +1427,36 @@ void EffectQueue::RemoveAllEffectsWithParam(ieDword opcode, ieDword param2) cons
 	}
 }
 
+//this will modify effect reference
+void EffectQueue::RemoveAllEffectsWithParam(EffectRef &effect_reference, ieDword param2) const
+{
+	ResolveEffectRef(effect_reference);
+	RemoveAllEffectsWithParam(effect_reference.opcode, param2);
+}
+
+//Removes all effects with a matching resource field
+void EffectQueue::RemoveAllEffectsWithParamAndResource(ieDword opcode, ieDword param2, const ieResRef resource) const
+{
+	std::list< Effect* >::const_iterator f;
+	for ( f = effects.begin(); f != effects.end(); f++ ) {
+		MATCH_OPCODE();
+		MATCH_LIVE_FX();
+		MATCH_PARAM2();
+		if(resource[0]) {
+			MATCH_RESOURCE();
+		}
+
+		(*f)->TimingMode = FX_DURATION_JUST_EXPIRED;
+	}
+}
+
+//this will modify effect reference
+void EffectQueue::RemoveAllEffectsWithParamAndResource(EffectRef &effect_reference, ieDword param2, const ieResRef resource) const
+{
+	ResolveEffectRef(effect_reference);
+	RemoveAllEffectsWithParamAndResource(effect_reference.opcode, param2, resource);
+}
+
 //this function is called by FakeEffectExpiryCheck
 //probably also called by rest
 void EffectQueue::RemoveExpiredEffects(ieDword futuretime) const
@@ -1373,20 +1492,6 @@ void EffectQueue::RemoveAllNonPermanentEffects() const
 	}
 }
 
-//this will modify effect reference
-
-void EffectQueue::RemoveAllDetrimentalEffects(EffectRef &effect_reference, ieDword current) const
-{
-	ResolveEffectRef(effect_reference);
-	RemoveAllDetrimentalEffects(effect_reference.opcode, current);
-}
-
-void EffectQueue::RemoveAllEffectsWithParam(EffectRef &effect_reference, ieDword param2) const
-{
-	ResolveEffectRef(effect_reference);
-	RemoveAllEffectsWithParam(effect_reference.opcode, param2);
-}
-
 //remove certain levels of effects, possibly matching school/secondary type
 //this method removes whole spells (tied together by their source)
 //FIXME: probably this isn't perfect
@@ -1460,6 +1565,7 @@ Effect *EffectQueue::HasOpcodeWithParam(ieDword opcode, ieDword param2) const
 	return NULL;
 }
 
+//this will modify effect reference
 Effect *EffectQueue::HasEffectWithParam(EffectRef &effect_reference, ieDword param2) const
 {
 	ResolveEffectRef(effect_reference);
@@ -1489,6 +1595,7 @@ Effect *EffectQueue::HasOpcodeWithParamPair(ieDword opcode, ieDword param1, ieDw
 	return NULL;
 }
 
+//this will modify effect reference
 Effect *EffectQueue::HasEffectWithParamPair(EffectRef &effect_reference, ieDword param1, ieDword param2) const
 {
 	ResolveEffectRef(effect_reference);
@@ -1498,30 +1605,6 @@ Effect *EffectQueue::HasEffectWithParamPair(EffectRef &effect_reference, ieDword
 	return HasOpcodeWithParamPair(effect_reference.opcode, param1, param2);
 }
 
-// sums all the values of the specific damage bonus effects of the passed "damage type"
-int EffectQueue::SpecificDamageBonus(ieDword opcode, ieDword param2) const
-{
-	int bonus = 0;
-	std::list< Effect* >::const_iterator f;
-	for ( f = effects.begin(); f != effects.end(); f++ ) {
-		MATCH_OPCODE();
-		MATCH_LIVE_FX();
-		MATCH_PARAM2();
-		bonus += (signed) (*f)->Parameter1;
-	}
-	return bonus;
-}
-
-static EffectRef fx_damage_bonus_modifier_ref = { "DamageBonusModifier", -1 };
-int EffectQueue::SpecificDamageBonus(ieDword damage_type) const
-{
-	ResolveEffectRef(fx_damage_bonus_modifier_ref);
-	if(fx_damage_bonus_modifier_ref.opcode < 0) {
-		return 0;
-	}
-	return SpecificDamageBonus(fx_damage_bonus_modifier_ref.opcode, damage_type);
-}
-
 //this could be used for stoneskins and mirror images as well
 void EffectQueue::DecreaseParam1OfEffect(ieDword opcode, ieDword amount) const
 {
@@ -1590,8 +1673,10 @@ int EffectQueue::DecreaseParam3OfEffect(EffectRef &effect_reference, ieDword amo
 }
 
 //this function does IDS targeting for effects (extra damage/thac0 against creature)
-static const int ids_stats[7]={IE_EA, IE_GENERAL, IE_RACE, IE_CLASS, IE_SPECIFIC, IE_SEX, IE_ALIGNMENT};
+//faction/team may be useful for grouping creatures differently, without messing with existing general/specific values
+static const int ids_stats[9]={IE_FACTION, IE_TEAM, IE_EA, IE_GENERAL, IE_RACE, IE_CLASS, IE_SPECIFIC, IE_SEX, IE_ALIGNMENT};
 
+//0,1 and 9 are only in GemRB
 int EffectQueue::BonusAgainstCreature(ieDword opcode, Actor *actor) const
 {
 	int sum = 0;
@@ -1600,14 +1685,33 @@ int EffectQueue::BonusAgainstCreature(ieDword opcode, Actor *actor) const
 		MATCH_OPCODE();
 		MATCH_LIVE_FX();
 		if( (*f)->Parameter1) {
+			ieDword param1;
 			ieDword ids = (*f)->Parameter2;
-			if( ids<2 || ids>8) {
-				ids=2;
+			switch(ids) {
+			case 0:
+			case 1:
+			case 2:
+			case 3:
+			case 4:
+			case 5:
+			case 6:
+			case 7:
+			case 8:
+				param1 = actor->GetStat(ids_stats[ids]);
+				MATCH_PARAM1();
+				break;
+			case 9:
+				//pseudo stat/classmask
+				param1 = actor->GetClassMask() & (*f)->Parameter1;
+				if (!param1) continue;
+				break;
+			default:
+				break;
 			}
-			ieDword param1 = actor->GetStat(ids_stats[ids-2]);
-			MATCH_PARAM1();
 		}
 		int val = (int) (*f)->Parameter3;
+		//we are really lucky with this, most of these boni are using +2 (including fiendslayer feat)
+		//it would be much more inconvenient if we had to use v2 effect files
 		if( !val) val = 2;
 		sum += val;
 	}
@@ -1623,6 +1727,28 @@ int EffectQueue::BonusAgainstCreature(EffectRef &effect_reference, Actor *actor)
 	return BonusAgainstCreature(effect_reference.opcode, actor);
 }
 
+int EffectQueue::BonusForParam2(ieDword opcode, ieDword param2) const
+{
+	int sum = 0;
+	std::list< Effect* >::const_iterator f;
+	for ( f = effects.begin(); f != effects.end(); f++ ) {
+		MATCH_OPCODE();
+		MATCH_LIVE_FX();
+		MATCH_PARAM2();
+		sum += (*f)->Parameter1;
+	}
+	return sum;
+}
+
+int EffectQueue::BonusForParam2(EffectRef &effect_reference, ieDword param2) const
+{
+	ResolveEffectRef(effect_reference);
+	if( effect_reference.opcode<0) {
+		return 0;
+	}
+	return BonusForParam2(effect_reference.opcode, param2);
+}
+
 bool EffectQueue::WeaponImmunity(ieDword opcode, int enchantment, ieDword weapontype) const
 {
 	std::list< Effect* >::const_iterator f;
@@ -1647,8 +1773,6 @@ bool EffectQueue::WeaponImmunity(ieDword opcode, int enchantment, ieDword weapon
 	return false;
 }
 
-static EffectRef fx_weapon_immunity_ref = { "Protection:Weapons", -1 };
-
 bool EffectQueue::WeaponImmunity(int enchantment, ieDword weapontype) const
 {
 	ResolveEffectRef(fx_weapon_immunity_ref);
@@ -1680,58 +1804,34 @@ void EffectQueue::AddWeaponEffects(EffectQueue *fxqueue, EffectRef &fx_ref) cons
 	}
 }
 
-/* no longer needed, use IE_CASTING stat
-static EffectRef fx_disable_spellcasting_ref = { "DisableCasting", -1 };
-int EffectQueue::DisabledSpellcasting(int types) const
+// figure out how much damage reduction applies for a given weapon enchantment and damage type
+int EffectQueue::SumDamageReduction(EffectRef &effect_reference, ieDword weaponEnchantment, int &total) const
 {
-	ResolveEffectRef(fx_disable_spellcasting_ref);
-	if( fx_disable_spellcasting_ref.opcode < 0) {
-		return 0;
-	}
+	ResolveEffectRef(effect_reference);
+	ieDword opcode = effect_reference.opcode;
+	int remaining = 0;
+	int count = 0;
 
-	unsigned int spelltype_mask = 0;
-	bool iwd2 = !!core->HasFeature(GF_ENHANCED_EFFECTS);
-	ieDword opcode = fx_disable_spellcasting_ref.opcode;
 	std::list< Effect* >::const_iterator f;
-	for ( f = effects.begin(); f != effects.end(); f++ ) {
+	for (f = effects.begin(); f != effects.end(); f++) {
 		MATCH_OPCODE();
 		MATCH_LIVE_FX();
 
-		if (iwd2) {
-			switch((*f)->Parameter2) {
-				case 0: // all
-					spelltype_mask |= 7;
-					break;
-				case 1: // mage and cleric
-					spelltype_mask |= 3;
-					break;
-				case 2: // mage
-					spelltype_mask |= 2;
-					break;
-				case 3: // cleric
-					spelltype_mask |= 1;
-					break;
-				case 4: // innate
-					spelltype_mask |= 4;
-					break;
-			}
-		} else {
-			switch((*f)->Parameter2) {
-				case 0: // mage
-					spelltype_mask |= 2;
-					break;
-				case 1: // cleric
-					spelltype_mask |= 1;
-					break;
-				case 2: // innate
-					spelltype_mask |= 4;
-					break;
-			}
+		Effect* fx = *f;
+		if (!fx) continue;
+		count++;
+		// add up if the effect has enough enchantment (or ignores it)
+		if (!fx->Parameter2 || fx->Parameter2 > weaponEnchantment) {
+			remaining += fx->Parameter1;
 		}
+		total += fx->Parameter1;
+	}
+	if (count) {
+		return remaining;
+	} else {
+		return -1;
 	}
-	return spelltype_mask & types;
 }
-*/
 
 //useful for immunity vs spell, can't use item, etc.
 Effect *EffectQueue::HasOpcodeWithResource(ieDword opcode, const ieResRef resource) const
@@ -1753,6 +1853,19 @@ Effect *EffectQueue::HasEffectWithResource(EffectRef &effect_reference, const ie
 	return HasOpcodeWithResource(effect_reference.opcode, resource);
 }
 
+//returns the first effect with source 'Removed'
+Effect *EffectQueue::HasSource(const ieResRef Removed) const
+{
+	std::list< Effect* >::const_iterator f;
+	for ( f = effects.begin(); f != effects.end(); f++ ) {
+		MATCH_LIVE_FX();
+		MATCH_SOURCE();
+
+		return (*f);
+	}
+	return NULL;
+}
+
 //used in contingency/sequencer code (cannot have the same contingency twice)
 Effect *EffectQueue::HasOpcodeWithSource(ieDword opcode, const ieResRef Removed) const
 {
@@ -1786,7 +1899,14 @@ bool EffectQueue::HasAnyDispellableEffect() const
 
 void EffectQueue::dump() const
 {
-	print( "EFFECT QUEUE:\n" );
+	StringBuffer buffer;
+	dump(buffer);
+	Log(DEBUG, "EffectQueue", buffer);
+}
+
+void EffectQueue::dump(StringBuffer& buffer) const
+{
+	buffer.append("EFFECT QUEUE:\n");
 	int i = 0;
 	std::list< Effect* >::const_iterator f;
 	for ( f = effects.begin(); f != effects.end(); f++ ) {
@@ -1796,7 +1916,7 @@ void EffectQueue::dump() const
 			if( fx->Opcode < MAX_EFFECTS)
 				Name = (char*) Opcodes[fx->Opcode].Name;
 
-			print( " %2d: 0x%02x: %s (%d, %d) S:%s\n", i++, fx->Opcode, Name, fx->Parameter1, fx->Parameter2, fx->Source );
+			buffer.appendFormatted(" %2d: 0x%02x: %s (%d, %d) S:%s\n", i++, fx->Opcode, Name, fx->Parameter1, fx->Parameter2, fx->Source);
 		}
 	}
 }
@@ -1822,8 +1942,6 @@ bool EffectQueue::HasDuration(Effect *fx)
 	return false;
 }
 
-static EffectRef fx_variable_ref = { "Variable:StoreLocalVariable", -1 };
-
 //returns true if the effect must be saved
 bool EffectQueue::Persistent(Effect* fx)
 {
@@ -1954,14 +2072,23 @@ int EffectQueue::CheckImmunity(Actor *target) const
 		if( target->ImmuneToProjectile(fx->Projectile)) return 0;
 
 		//don't resist item projectile payloads based on spell school, bounce, etc.
-		if( fx->InventorySlot) {
-			return 1;
-		}
+		//FIXME: -Uh, why not ?
+		//Allegedly, the book of infinite spells needed this, but irresistable by level
+		//spells got fx->Power = 0, so i added those exceptions and removed this
+		//if( fx->InventorySlot) {
+		//	return 1;
+		//}
 
 		//check level resistances
 		//check specific spell immunity
 		//check school/sectype immunity
-		return check_type(target, fx);
+		int ret = check_type(target, fx);
+		if (ret<0) {
+			if (target->Modified[IE_SANCTUARY]&(1<<OV_BOUNCE) ) {
+				target->Modified[IE_SANCTUARY]|=(1<<OV_BOUNCE2);
+			}
+		}
+		return ret;
 	}
 	return 0;
 }
@@ -1984,10 +2111,12 @@ void EffectQueue::AffectAllInRange(Map *map, const Point &pos, int idstype, int
 			continue;
 		}
 		//line of sight
-		if( !map->IsVisible(actor->Pos, pos)) {
+		if( !map->IsVisibleLOS(actor->Pos, pos)) {
 			continue;
 		}
 		AddAllEffects(actor, actor->Pos);
 	}
 }
 
+
+}
