@@ -37,6 +37,8 @@
 
 #include <list>
 
+namespace GemRB {
+
 class Palette;
 
 // NOTE: keep these synchronized with GUIDefines.py!!!
@@ -47,14 +49,14 @@ class Palette;
 // Like DISABLED, but processes MouseOver events and draws UNPRESSED bitmap
 #define IE_GUI_BUTTON_LOCKED    4
 // Draws the disabled bitmap, but otherwise works like unpressed
-#define IE_GUI_BUTTON_THIRD     5
-#define IE_GUI_BUTTON_SECOND    6
+#define IE_GUI_BUTTON_FAKEDISABLED     5
+#define IE_GUI_BUTTON_FAKEPRESSED    6
 #define IE_GUI_BUTTON_LOCKED_PRESSED    7  //all the same as LOCKED
 
 #define IE_GUI_BUTTON_NO_IMAGE     0x00000001   // don't draw image (BAM)
 #define IE_GUI_BUTTON_PICTURE      0x00000002   // draw picture (BMP, MOS, ...)
 #define IE_GUI_BUTTON_SOUND        0x00000004
-#define IE_GUI_BUTTON_ALT_SOUND    0x00000008
+#define IE_GUI_BUTTON_CAPS         0x00000008   // convert text to uppercase
 #define IE_GUI_BUTTON_CHECKBOX     0x00000010   // or radio button
 #define IE_GUI_BUTTON_RADIOBUTTON  0x00000020   // sticks in a state
 #define IE_GUI_BUTTON_DEFAULT      0x00000040   // enter key triggers it
@@ -73,12 +75,14 @@ class Palette;
 #define IE_GUI_BUTTON_NO_TEXT      0x00010000   // don't draw button label
 #define IE_GUI_BUTTON_PLAYRANDOM   0x00020000
 #define IE_GUI_BUTTON_PLAYONCE     0x00040000
+#define IE_GUI_BUTTON_PLAYALWAYS   0x00080000   // play even when game is paused
 
-#define IE_GUI_BUTTON_CENTER_PICTURES 0x00080000 // center button's PictureList
-#define IE_GUI_BUTTON_BG1_PAPERDOLL   0x00100000 // BG1-style paperdoll PictureList
-#define IE_GUI_BUTTON_HORIZONTAL      0x00200000 // horizontal clipping of overlay
-#define IE_GUI_BUTTON_CANCEL          0x00400000 // cancel key triggers it
-#define IE_GUI_BUTTON_CAPS            0x00800000 // convert text to uppercase
+#define IE_GUI_BUTTON_CENTER_PICTURES 0x00100000 // center button's PictureList
+#define IE_GUI_BUTTON_BG1_PAPERDOLL   0x00200000 // BG1-style paperdoll PictureList
+#define IE_GUI_BUTTON_HORIZONTAL      0x00400000 // horizontal clipping of overlay
+#define IE_GUI_BUTTON_CANCEL          0x00800000 // cancel key triggers it
+//WARNING: we cannot add any more flags here
+//the top 8 bits are reserved for the control ID (see setFlags)
 
 //composite button flags
 #define IE_GUI_BUTTON_NORMAL       0x00000004   // default button, doesn't stick
@@ -110,6 +114,14 @@ struct ButtonBorder {
 
 #define MAX_NUM_BORDERS 3
 
+enum BUTTON_IMAGE_TYPE {
+	BUTTON_IMAGE_NONE = -1,
+	BUTTON_IMAGE_UNPRESSED,
+	BUTTON_IMAGE_PRESSED,
+	BUTTON_IMAGE_SELECTED,
+	BUTTON_IMAGE_DISABLED,
+	BUTTON_IMAGE_TYPE_COUNT
+};
 
 /**
  * @class Button
@@ -118,18 +130,16 @@ struct ButtonBorder {
  */
 
 class GEM_EXPORT Button : public Control {
+protected:
+	/** Draws the Control on the Output Display */
+	void DrawInternal(Region& drawRegion);
+	virtual bool HasBackground();
 public: 
-	Button();
+	Button(Region& frame);
 	~Button();
 	/** Sets the 'type' Image of the Button to 'img'.
-	'type' may assume the following values:
-	- IE_GUI_BUTTON_UNPRESSED
-	- IE_GUI_BUTTON_PRESSED
-	- IE_GUI_BUTTON_SELECTED
-	- IE_GUI_BUTTON_DISABLED */
-	void SetImage(unsigned char type, Sprite2D* img);
-	/** Draws the Control on the Output Display */
-	void Draw(unsigned short x, unsigned short y);
+	see 'BUTTON_IMAGE_TYPE' */
+	void SetImage(BUTTON_IMAGE_TYPE, Sprite2D* img);
 	/** Sets the Button State */
 	void SetState(unsigned char state);
 	/** Sets the Text of the current control */
@@ -149,6 +159,7 @@ public:
 	/** Enables or disables specified border/frame */
 	void EnableBorder(int index, bool enabled);
 public: // Public Events
+	bool WantsDragOperation();
 	/** Mouse Enter */
 	void OnMouseEnter(unsigned short x, unsigned short y);
 	/** Mouse Leave */
@@ -164,7 +175,7 @@ public: // Public Events
 	/** Mouse wheel scroll */
 	void OnMouseWheelScroll(short x, short y);
 	/** A special key has been pressed */
-	void OnSpecialKeyPress(unsigned char Key);
+	bool OnSpecialKeyPress(unsigned char Key);
 	/** Set handler for specified event */
 	bool SetEvent(int eventType, EventHandler handler);
 	/** Button Pressed Event Script Function Name */
@@ -179,12 +190,15 @@ public: // Public Events
 	EventHandler MouseLeaveButton;
 	EventHandler MouseOverButton;
 	/** Refreshes the button from a radio group */
-	void RedrawButton(const char* VariableName, unsigned int Sum);
+	void UpdateState(const char* VariableName, unsigned int Sum);
 	/** Set palette used for drawing button label in normal state.  */
 	void SetTextColor(const Color &fore, const Color &back);
 	/** Sets percent (0-1.0) of width for clipping picture */
 	void SetPictureClipping(double clip)  { Clipping = clip; }
+	/** Set explicit anchor point for text */
 	void SetAnchor(ieWord x, ieWord y);
+	/** Set offset pictures and label move when button is pressed */
+	void SetPushOffset(ieWord x, ieWord y);
 private: // Private attributes
 	char* Text;
 	bool hasText;
@@ -192,14 +206,7 @@ private: // Private attributes
 	bool ToggleState;
 	Palette* normal_palette;
 	Palette* disabled_palette;
-	/** Button Unpressed Image */
-	Sprite2D* Unpressed;
-	/** Button Pressed Image */
-	Sprite2D* Pressed;
-	/** Button Selected Image */
-	Sprite2D* Selected;
-	/** Button Disabled Image */
-	Sprite2D* Disabled;
+	Sprite2D* buttonImages[BUTTON_IMAGE_TYPE_COUNT];
 	/** Pictures to Apply when the hasPicture flag is set */
 	Sprite2D* Picture;
 	/** If non-empty, list of Pictures to draw when hasPicture is set */
@@ -211,11 +218,16 @@ private: // Private attributes
 	/** HP Bar over portraits */
 	unsigned long starttime;
 	Color SourceRGB, DestRGB;
+	/** Explicit text anchor point if IE_GUI_BUTTON_ANCHOR is set */
 	Point Anchor;
+	/** Offset pictures and label move when the button is pressed. */
+	Point PushOffset;
 	/** frame settings */
 	ButtonBorder borders[MAX_NUM_BORDERS];
 	bool IsPixelTransparent (unsigned short x, unsigned short y);
 	void CloseUpColor();
 };
 
+}
+
 #endif
