@@ -37,18 +37,23 @@
 #include <cstdio>
 #include <cstring>
 
-Control::Control()
+namespace GemRB {
+
+Control::Control(const Region& frame)
 {
 	hasFocus = false;
 	Changed = true;
 	InHandler = false;
 	VarName[0] = 0;
+	ControlID = 0;
 	Value = 0;
 	Flags = 0;
 	Tooltip = NULL;
 	Owner = NULL;
-	XPos = 0;
-	YPos = 0;
+	XPos = frame.x;
+	YPos = frame.y;
+	Width = frame.w;
+	Height = frame.h;
 
 	sb = NULL;
 	animation = NULL;
@@ -60,7 +65,7 @@ Control::Control()
 Control::~Control()
 {
 	if (InHandler) {
-		printMessage("Control","Destroying control inside event handler, crash may occur!\n", LIGHT_RED);
+		Log(ERROR, "Control", "Destroying control inside event handler, crash may occur!");
 	}
 	core->DisplayTooltip( 0, 0, NULL );
 	free (Tooltip);
@@ -70,6 +75,51 @@ Control::~Control()
 	core->GetVideoDriver()->FreeSprite(AnimPicture);
 }
 
+Region Control::ControlFrame()
+{
+	return Region(XPos, YPos, Width, Height);
+}
+
+void Control::MarkDirty() {
+	if (!Changed) {
+		Changed = true;
+		if (Owner && !HasBackground()) {
+			// the window is our background so we must redraw the entire thing :(
+			Owner->InvalidateForControl(this);
+		}
+	}
+}
+
+bool Control::NeedsDraw()
+{
+	return (Changed || (Owner->Flags&WF_FLOAT));
+}
+
+void Control::Draw(unsigned short x, unsigned short y)
+{
+	// FIXME: Draw shouldnt be getting called on controls that are offscreen...
+	if (XPos == 65535) {
+		return;
+	}
+	// no point in drawing something with a 0 w/h
+	if (!Width || !Height) {
+		return;
+	}
+	if (!NeedsDraw()) {
+		return;
+	}
+
+	Region drawFrame = Region(x + XPos, y + YPos, Width, Height);
+	Region clip;
+	Video* video = core->GetVideoDriver();
+	// clip drawing to the control bounds, then restore after drawing
+	video->GetClipRect(clip);
+	video->SetClipRect(&drawFrame);
+	DrawInternal(drawFrame);
+	video->SetClipRect(&clip);
+	Changed = false; // set *after* calling DrawInternal
+}
+
 /** Sets the Tooltip text of the current control */
 int Control::SetTooltip(const char* string)
 {
@@ -93,7 +143,7 @@ void Control::DisplayTooltip()
 		core->DisplayTooltip( 0, 0, NULL );
 }
 
-void Control::ResetEventHandler(EventHandler handler)
+void Control::ResetEventHandler(EventHandler &handler)
 {
 	handler = NULL;
 }
@@ -102,10 +152,13 @@ void Control::SetText(const char* /*string*/)
 {
 }
 
+//return -1 if there is an error
+//return 1 if there is no handler (not an error)
+//return 0 if the handler ran as intended
 int Control::RunEventHandler(EventHandler handler)
 {
 	if (InHandler) {
-		printMessage("Control","Nested event handlers are not supported!\n", YELLOW);
+		Log(WARNING, "Control", "Nested event handlers are not supported!");
 		return -1;
 	}
 	if (handler) {
@@ -116,48 +169,20 @@ int Control::RunEventHandler(EventHandler handler)
 		unsigned short WID = wnd->WindowID;
 		unsigned short ID = (unsigned short) ControlID;
 		InHandler = true;
+		//TODO: detect caller errors, trap them???
 		handler->call();
+		InHandler = false;
 		if (!core->IsValidWindow(WID,wnd) ) {
-			printMessage ("Control","Owner window destructed!\n", LIGHT_RED);
+			Log(ERROR, "Control", "Owner window destructed!");
 			return -1;
 		}
 		if (!wnd->IsValidControl(ID,this) ) {
-			printMessage ("Control","Control destructed!\n", LIGHT_RED);
+			Log(ERROR, "Control", "Control destructed!");
 			return -1;
 		}
-		InHandler = false;
+		return 0;
 	}
-	return 0;
-}
-
-/** Key Press Event */
-void Control::OnKeyPress(unsigned char /*Key*/, unsigned short /*Mod*/)
-{
-	//print("OnKeyPress: CtrlID = 0x%08X, Key = %c (0x%02hX)\n", (unsigned int) ControlID, Key, Key);
-}
-
-/** Key Release Event */
-void Control::OnKeyRelease(unsigned char /*Key*/, unsigned short /*Mod*/)
-{
-	//print( "OnKeyRelease: CtrlID = 0x%08X, Key = %c (0x%02hX)\n", (unsigned int) ControlID, Key, Key );
-}
-
-/** Mouse Enter Event */
-void Control::OnMouseEnter(unsigned short /*x*/, unsigned short /*y*/)
-{
-	//print("OnMouseEnter: CtrlID = 0x%08X, x = %hd, y = %hd\n", (unsigned int) ControlID, x, y);
-}
-
-/** Mouse Leave Event */
-void Control::OnMouseLeave(unsigned short /*x*/, unsigned short /*y*/)
-{
-	//print("OnMouseLeave: CtrlID = 0x%08X, x = %hd, y = %hd\n", (unsigned int) ControlID, x, y);
-}
-
-/** Mouse Over Event */
-void Control::OnMouseOver(unsigned short /*x*/, unsigned short /*y*/)
-{
-	//print("OnMouseOver: CtrlID = 0x%08X, x = %hd, y = %hd\n", (unsigned int) ControlID, x, y);
+	return 1;
 }
 
 /** Mouse Button Down */
@@ -176,7 +201,7 @@ void Control::OnMouseDown(unsigned short x, unsigned short y,
 void Control::OnMouseUp(unsigned short /*x*/, unsigned short /*y*/,
 	unsigned short /*Button*/, unsigned short /*Mod*/)
 {
-	//print("OnMouseUp: CtrlID = 0x%08X, x = %hd, y = %hd, Button = %d, Mos = %hd\n", (unsigned int) ControlID, x, y, Button, Mod);
+	//print("OnMouseUp: CtrlID = 0x%08X, x = %hd, y = %hd, Button = %d, Mos = %hd",(unsigned int) ControlID, x, y, Button, Mod);
 }
 
 /** Mouse scroll wheel */
@@ -189,18 +214,20 @@ void Control::OnMouseWheelScroll( short x, short y)
 }
 
 /** Special Key Press */
-void Control::OnSpecialKeyPress(unsigned char Key)
+bool Control::OnSpecialKeyPress(unsigned char Key)
 {
 	if (Key == GEM_UP || Key == GEM_DOWN) {
 		Control *ctrl = Owner->GetScrollControl();
 		if (ctrl && (ctrl!=this)) {
-			ctrl->OnSpecialKeyPress(Key);
+			return ctrl->OnSpecialKeyPress(Key);
 		}
 	}
+	return false;
 }
 void Control::SetFocus(bool focus)
 {
 	hasFocus = focus;
+	Changed = true;
 }
 
 bool Control::isFocused()
@@ -210,8 +237,11 @@ bool Control::isFocused()
 /** Sets the Display Flags */
 int Control::SetFlags(int arg_flags, int opcode)
 {
-	if ((arg_flags >>24) != ControlType)
+	if ((arg_flags >>24) != ControlType) {
+		Log(WARNING, "Control", "Trying to modify invalid flag %x on control %d (opcode %d)",
+			arg_flags, ControlID, opcode);
 		return -2;
+	}
 	switch (opcode) {
 		case BM_SET:
 			Flags = arg_flags;  //set
@@ -253,7 +283,7 @@ int Control::SetScrollBar(Control* ptr)
 {
 	if (ptr && (ptr->ControlType!=IE_GUI_SCROLLBAR)) {
 		ptr = NULL;
-		printMessage("Control","Attached control is not a ScrollBar!\n",YELLOW);
+		Log(WARNING, "Control", "Attached control is not a ScrollBar!");
 		return -1;
 	}
 	sb = ptr;
@@ -261,3 +291,5 @@ int Control::SetScrollBar(Control* ptr)
 	if (ptr) return 1;
 	return 0;
 }
+
+}
