@@ -63,14 +63,23 @@ class Window;
  */
 
 class GEM_EXPORT Control {
+private:
+	/** If true, control is redrawn during next call to gc->DrawWindows.
+	 * Then it's set back to false. */
+	bool Changed;
 protected:
 	/** Focused Control */
 	bool hasFocus;
+	virtual void DrawInternal(Region& drawFrame)=0;
+	virtual bool NeedsDraw();
+	virtual bool HasBackground() { return true; }
 public:
-	Control();
+	Control(const Region& frame);
 	virtual ~Control();
+	Region ControlFrame();
 	/** Draws the Control on the Output Display */
-	virtual void Draw(unsigned short x, unsigned short y) = 0;
+	void Draw(unsigned short x, unsigned short y);
+	void MarkDirty();
 	/** Sets the Text of the current control */
 	virtual void SetText(const char* string);
 	/** Sets the Tooltip text of the current control */
@@ -104,9 +113,6 @@ public: // Public attributes
 	/** Text to display as a tooltip when the mouse cursor hovers
 	 * for some time over the control */
 	char* Tooltip;
-	/** If true, control is redrawn during next call to gc->DrawWindows.
-	 * Then it's set back to false. */
-	bool Changed;
 	/** True if we are currently in an event handler */
 	bool InHandler;
 	/** Owner Window */
@@ -124,6 +130,7 @@ public: //Events
 	int SetFlags(int arg_flags, int opcode);
 	virtual void SetFocus(bool focus);
 	bool isFocused();
+	virtual bool WantsDragOperation() { return false; };
 	/** Set handler for specified event. Override in child classes */
 	virtual bool SetEvent(int eventType, EventHandler handler) = 0;
 	/** Run specified handler, it may return error code */
@@ -151,6 +158,7 @@ public: //Events
 	virtual bool IsPixelTransparent(unsigned short /*x*/, unsigned short /*y*/) {
 		return false;
 	}
+	virtual const char* QueryText() const { return ""; }
 	/** Sets the animation picture ref */
 	void SetAnimPicture(Sprite2D* Picture);
 	/** Sets the Scroll Bar Pointer */
