@@ -35,6 +35,8 @@
 #include "Interface.h"
 #include "Map.h"
 
+namespace GemRB {
+
 class GameControl;
 class Window;
 class DialogHandler;
@@ -92,18 +94,33 @@ static const unsigned long tp_steps[8]={3,2,1,0,1,2,3,4};
  */
 
 class GEM_EXPORT GameControl : public Control {
+	enum WINDOW_GROUP {
+		WINDOW_GROUP_LEFT,
+		WINDOW_GROUP_BOTTOM,
+		WINDOW_GROUP_RIGHT,
+		WINDOW_GROUP_TOP,
+		WINDOW_GROUP_COUNT
+	};
+	enum WINDOW_RESIZE_OPERATION {
+		WINDOW_EXPAND = -1,
+		WINDOW_CONTRACT = 1
+	};
 public:
-	GameControl(void);
+	GameControl(const Region& frame);
 	~GameControl(void);
-public:
+protected:
 	/** Draws the Control on the Output Display */
-	void Draw(unsigned short x, unsigned short y);
+	void DrawInternal(Region& drawFrame);
+	// GameControl always needs to redraw
+	bool NeedsDraw() { return true; };
+public:
 	/** Draws the target reticle for Actor movement. */
-	void DrawTargetReticle(Point p, int size, bool animate);
+	void DrawTargetReticle(Point p, int size, bool animate, bool flash=false, bool actorSelected=false);
 	/** Sets multiple quicksaves flag*/
 	//static void MultipleQuickSaves(int arg);
 	void SetTracker(Actor *actor, ieDword dist);
 private:
+	int windowGroupCounts[WINDOW_GROUP_COUNT];
 	ieDword lastActorID;
 	ieDword trackerID;
 	ieDword distance;  //tracking distance
@@ -113,24 +130,18 @@ private:
 	bool MouseIsDown;
 	bool DoubleClick;
 	Region SelectionRect;
-	Point FormationPivotPoint;
 	Point FormationApplicationPoint;
-	short StartX, StartY;
-	// following variables used for touch scroll areas
-	bool touchScrollAreasEnabled; // true, if scroll areas enabled
-	bool touched; // true, if player touched screen (left button down and hold)
-	unsigned int scrollAreasWidth; // scroll areas width
-
+	Point ClickPoint;
 public:
 	Door* overDoor;
 	Container* overContainer;
 	InfoPoint* overInfoPoint;
 
-	// allow targetting allies, enemies and/or neutrals (bitmask)
+	// allow targeting allies, enemies and/or neutrals (bitmask)
 	int target_types;
 	unsigned short lastMouseX, lastMouseY;
 private:
-	// currently selected targetting type, such as talk, attack, cast, ...
+	// currently selected targeting type, such as talk, attack, cast, ...
 	// private to enforce proper cursor changes
 	int target_mode;
 	unsigned char lastCursor;
@@ -145,12 +156,12 @@ private:
 	int DialogueFlags;
 	char *DisplayText;
 	unsigned int DisplayTextTime;
-	bool EnableRunning;
+	bool AlwaysRun;
 public: //Events
 	/** Key Press Event */
-	void OnKeyPress(unsigned char Key, unsigned short Mod);
+	bool OnKeyPress(unsigned char Key, unsigned short Mod);
 	/** Key Release Event */
-	void OnKeyRelease(unsigned char Key, unsigned short Mod);
+	bool OnKeyRelease(unsigned char Key, unsigned short Mod);
 	/** Mouse Over Event */
 	void OnMouseOver(unsigned short x, unsigned short y);
 	/** Global Mouse Move Event */
@@ -163,9 +174,10 @@ public: //Events
 		unsigned short Mod);
 	void OnMouseWheelScroll(short x, short y);
 	/** Special Key Press */
-	void OnSpecialKeyPress(unsigned char Key);
+	bool OnSpecialKeyPress(unsigned char Key);
 	void DisplayTooltip();
 	void UpdateScrolling();
+	void SetScrolling(bool scroll);
 	void SetTargetMode(int mode);
 	int GetTargetMode() { return target_mode; }
 	void SetScreenFlags(int value, int mode);
@@ -176,20 +188,17 @@ public: //Events
 	void SetDisplayText(ieStrRef text, unsigned int time);
 	/* centers viewport to the points specified */
 	void Center(unsigned short x, unsigned short y);
+	void ClearMouseState();
 private:
 	/** this function safely retrieves an Actor by ID */
 	Actor *GetActorByGlobalID(ieDword ID);
 	void CalculateSelection(const Point &p);
-	void ResizeDel(Window* win, int type);
-	void ResizeAdd(Window* win, int type);
-	void HandleWindowHide(const char *WindowName, const char *WindowPosition);
-	void HandleWindowReveal(const char *WindowName, const char *WindowPosition);
+	void ResizeParentWindowFor(Window* win, int type, WINDOW_RESIZE_OPERATION);
 	void ReadFormations();
 	/** Draws an arrow on the edge of the screen based on the point (points at offscreen actors) */
-	void DrawArrowMarker(const Region &screen, Point p, const Region &viewport);
+	void DrawArrowMarker(const Region &screen, Point p, const Region &viewport, const Color& color);
 
 private:
-	unsigned char LeftCount, BottomCount, RightCount, TopCount;
 	Actor *user;     //the user of item or spell
 public:
 	DialogHandler *dialoghandler;
@@ -206,16 +215,13 @@ public:
 	void SelectActor(int whom, int type = -1);
 	void SetLastActor(Actor *actor, Actor *prevActor);
 	void SetCutSceneMode(bool active);
-	int HideGUI();
-	int UnhideGUI();
+	bool SetGUIHidden(bool hide);
 	void TryToAttack(Actor *source, Actor *target);
 	void TryToCast(Actor *source, const Point &p);
 	void TryToCast(Actor *source, Actor *target);
 	void TryToDefend(Actor *source, Actor *target);
 	void TryToTalk(Actor *source, Actor *target);
-	void TryToPick(Actor *source, Actor *tgt);
-	void TryToPick(Actor *source, Door *tgt);
-	void TryToPick(Actor *source, Container *tgt);
+	void TryToPick(Actor *source, Scriptable *tgt);
 	void TryToDisarm(Actor *source, InfoPoint *tgt);
 	void PerformActionOn(Actor *actor);
 	void ResetTargetMode();
@@ -245,7 +251,7 @@ public:
 	/** changes map to the current PC */
 	void ChangeMap(Actor *pc, bool forced);
 	/** Returns game screenshot, with or without GUI controls */
-	Sprite2D* GetScreenshot( bool show_gui = 0 );
+	Sprite2D* GetScreenshot(const Region& rgn, bool show_gui = false );
 	/** Returns current area preview for saving a game */
 	Sprite2D* GetPreview();
 	/** Returns PC portrait for a currently running game */
@@ -255,6 +261,9 @@ public:
 	/** Page is the spell type + spell level info */
 	void SetupCasting(ieResRef spellname, int type, int level, int slot, Actor *actor, int targettype, int cnt);
 	bool SetEvent(int eventType, EventHandler handler);
+	void ToggleAlwaysRun();
 };
 
+}
+
 #endif
