@@ -18,33 +18,31 @@
  *
  */
 
-#include "GUI/TextArea.h"
+#include <list>
 
-#include "GUI/GameControl.h"
+#include "TextArea.h"
 
 #include "win32def.h"
 
-#include "Audio.h"
 #include "DialogHandler.h"
 #include "GameData.h"
 #include "ImageMgr.h"
-#include "Interface.h"
-#include "Palette.h"
-#include "Variables.h"
 #include "Video.h"
 #include "GUI/EventMgr.h"
+#include "GUI/GameControl.h"
 #include "GUI/Window.h"
 #include "Scriptable/Actor.h"
 
-#include <cstdio>
-#include <cstdlib>
+namespace GemRB {
 
-TextArea::TextArea(Color hitextcolor, Color initcolor, Color lowtextcolor)
+TextArea::TextArea(const Region& frame, Color hitextcolor, Color initcolor, Color lowtextcolor)
+	: Control(frame)
 {
+	ControlType = IE_GUI_TEXTAREA;
 	keeplines = 100;
 	rows = 0;
-	smooth = 0;
 	TextYPos = 0;
+	ticks = starttime = 0;
 	startrow = 0;
 	minrow = 0;
 	Cursor = NULL;
@@ -53,7 +51,6 @@ TextArea::TextArea(Color hitextcolor, Color initcolor, Color lowtextcolor)
 	seltext = -1;
 	Value = 0xffffffff;
 	ResetEventHandler( TextAreaOnChange );
-	ResetEventHandler( TextAreaOutOfText );
 	PortraitResRef[0]=0;
 	palette = core->CreatePalette( hitextcolor, lowtextcolor );
 	initpalette = core->CreatePalette( initcolor, lowtextcolor );
@@ -65,12 +62,7 @@ TextArea::TextArea(Color hitextcolor, Color initcolor, Color lowtextcolor)
 	tmp.g = 152;
 	tmp.b = 102;
 	lineselpal = core->CreatePalette( tmp, lowtextcolor );
-	InternalFlags = 1;
-	//Drop Capitals means initials on!
-	core->GetDictionary()->Lookup("Drop Capitals", InternalFlags);
-	if (InternalFlags) {
-		InternalFlags = TA_INITIALS;
-	}
+	ftext = finit = NULL;
 }
 
 TextArea::~TextArea(void)
@@ -94,10 +86,7 @@ void TextArea::RefreshSprite(const char *portrait)
 		SetAnimPicture(NULL);
 	}
 	strnlwrcpy(PortraitResRef, portrait, 8);
-	if (!strnicmp(PortraitResRef, "none", 8) ) {
-		return;
-	}
-	ResourceHolder<ImageMgr> im(PortraitResRef);
+	ResourceHolder<ImageMgr> im(PortraitResRef, true);
 	if (im == NULL) {
 		return;
 	}
@@ -105,80 +94,55 @@ void TextArea::RefreshSprite(const char *portrait)
 	SetAnimPicture ( im->GetSprite2D() );
 }
 
-void TextArea::Draw(unsigned short x, unsigned short y)
+bool TextArea::NeedsDraw()
 {
-	/** Don't come back recursively */
-	if (InternalFlags&TA_BITEMYTAIL) {
+	if (Flags&IE_GUI_TEXTAREA_SMOOTHSCROLL) {
+		if (startrow == rows) { // the text is offscreen
+			return false;
+		}
+		MarkDirty();
+		return true;
+	}
+	return Control::NeedsDraw();
+}
+
+void TextArea::DrawInternal(Region& clip)
+{
+	if (lines.size() == 0) {
 		return;
 	}
-	int tx=x+XPos;
-	int ty=y+YPos;
-	Region clip( tx, ty, Width, Height );
-	Video *video = core->GetVideoDriver();
 
+	Video *video = core->GetVideoDriver();
 	if (Flags&IE_GUI_TEXTAREA_SPEAKER) {
 		if (AnimPicture) {
-			video->BlitSprite(AnimPicture, tx,ty, true, &clip);
+			video->BlitSprite(AnimPicture, clip.x, clip.y, true, &clip);
 			clip.x+=AnimPicture->Width;
 			clip.w-=AnimPicture->Width;
 		}
 	}
 
-	//this might look better in GlobalTimer
-	//or you might want to change the animated button to work like this
-	if (Flags &IE_GUI_TEXTAREA_SMOOTHSCROLL)
+	if (Flags&IE_GUI_TEXTAREA_SMOOTHSCROLL)
 	{
-		unsigned long thisTime;
-
-		thisTime = GetTickCount();
+		unsigned long thisTime = GetTickCount();
 		if (thisTime>starttime) {
 			starttime = thisTime+ticks;
-			smooth--;
-			while (smooth<=0) {
-				smooth+=ftext->maxHeight;
-				if (startrow<rows) {
-					startrow++;
-				}
-			}
-
-			/** Forcing redraw of whole screen before drawing text*/
-			Owner->Invalidate();
-			InternalFlags |= TA_BITEMYTAIL;
-			Owner->DrawWindow();
-			InternalFlags &= ~TA_BITEMYTAIL;
+			TextYPos++;// can't use ScrollToY
+			if (TextYPos % ftext->maxHeight == 0) SetRow(startrow + 1);
 		}
 	}
 
-	if (!Changed && !(Owner->Flags&WF_FLOAT) ) {
-		return;
-	}
-	Changed = false;
-
-	if (XPos == 65535) {
-		return;
-	}
-	size_t linesize = lines.size();
-	if (linesize == 0) {
-		return;
-	}
-
-	//smooth vertical scrolling up
-	if (Flags & IE_GUI_TEXTAREA_SMOOTHSCROLL) {
-		clip.y+=smooth;
-		clip.h-=smooth;
-	}
-
 	//if textarea is 'selectable' it actually means, it is a listbox
 	//in this case the selected value equals the line number
 	//if it is 'not selectable' it can still have selectable lines
 	//but then it is like the dialog window in the main game screen:
 	//the selected value is encoded into the line
+	size_t linesize = lines.size() - 1; // -1 because 0 counts
 	if (!(Flags & IE_GUI_TEXTAREA_SELECTABLE) ) {
 		char* Buffer = (char *) malloc( 1 );
 		Buffer[0] = 0;
 		int len = 0;
 		int lastlen = 0;
-		for (size_t i = 0; i < linesize; i++) {
+		for (size_t i = 0; i <= linesize; i++) {
 			if (strnicmp( "[s=", lines[i], 3 ) == 0) {
 				int tlen;
 				unsigned long acolor, bcolor;
@@ -210,12 +174,11 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 				memcpy( &Buffer[lastlen], lines[i], len - lastlen );
 			}
 			lastlen = len;
-			if (i != linesize - 1) {
+			if (i != linesize) {
 				Buffer[lastlen - 1] = '\n';
 				Buffer[lastlen] = 0;
 			}
 		}
-		video->SetClipRect( &clip );
 
 		int pos;
 
@@ -230,29 +193,10 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 		short LineOffset = (short)(TextYPos % ftext->maxHeight);
 		Region textClip(clip.x, clip.y - LineOffset, clip.w, clip.h + LineOffset);
 
-
 		ftext->PrintFromLine( startrow, textClip,
 							 ( unsigned char * ) Buffer, palette,
 							 IE_FONT_ALIGN_LEFT, finit, Cursor, pos );
 		free( Buffer );
-		video->SetClipRect( NULL );
-		//streaming text
-		if (linesize>50) {
-			//the buffer is filled enough
-			return;
-		}
-		if (core->GetAudioDrv()->IsSpeaking() ) {
-			//the narrator is still talking
-			return;
-		}
-		if (RunEventHandler( TextAreaOutOfText )) {
-			return;
-		}
-		if (linesize==lines.size()) {
-			ResetEventHandler( TextAreaOutOfText );
-			return;
-		}
-		AppendText("\n",-1);
 		return;
 	}
 	// normal scrolling textarea
@@ -276,7 +220,7 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 		ftext->PrintFromLine( sr, clip,
 			( unsigned char * ) lines[i], pal,
 			IE_FONT_ALIGN_LEFT, finit, NULL );
-		yl = ftext->size[1].h*(lrows[i]-sr);
+		yl = ftext->maxHeight * (lrows[i]-sr);
 		clip.y+=yl;
 		clip.h-=yl;
 		break;
@@ -291,7 +235,7 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 			pal = palette;
 		ftext->Print( clip, ( unsigned char * ) lines[i], pal,
 			IE_FONT_ALIGN_LEFT, true );
-		yl = ftext->size[1].h*lrows[i];
+		yl = ftext->maxHeight * lrows[i];
 		clip.y+=yl;
 		clip.h-=yl;
 
@@ -336,7 +280,6 @@ void TextArea::SetMinRow(bool enable)
 	} else {
 		minrow = 0;
 	}
-	Changed = true;
 }
 
 //drop lines scrolled out at the top.
@@ -444,7 +387,6 @@ void TextArea::UpdateControls()
 	int pos;
 
 	CalcRowCount();
-	Changed = true;
 	if (sb) {
 		ScrollBar* bar = ( ScrollBar* ) sb;
 		if (Flags & IE_GUI_TEXTAREA_AUTOSCROLL)
@@ -460,6 +402,18 @@ void TextArea::UpdateControls()
 			SetRow(pos);
 		}
 	}
+
+	GameControl* gc = core->GetGameControl();
+	if (gc && gc->GetDialogueFlags()&DF_IN_DIALOG) {
+		// This hack is to refresh the mouse cursor so that reply below cursor gets
+		// highlighted during a dialog
+		// FIXME: we check DF_IN_DIALOG here to avoid recurssion in the MessageWindowLogger, but what happens when an error happens during dialog?
+		// I'm not super sure about how to avoid that. for now the logger will not log anything in dialog mode.
+		int x,y;
+		core->GetVideoDriver()->GetMousePos(x,y);
+		core->GetEventMgr()->MouseMove(x,y);
+	}
+
 	core->RedrawAll();
 }
 
@@ -468,18 +422,17 @@ void TextArea::SetFonts(Font* init, Font* text)
 {
 	finit = init;
 	ftext = text;
-	Changed = true;
+	MarkDirty();
 }
 
 /** Key Press Event */
-void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
+bool TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 {
 	if (Flags & IE_GUI_TEXTAREA_EDITABLE) {
 		if (Key >= 0x20) {
-			Owner->Invalidate();
-			Changed = true;
+			MarkDirty();
 			int len = GetRowLength(CurLine);
-			//print("len: %d Before: %s\n",len, lines[CurLine]);
+			//print("len: %d Before: %s", len, lines[CurLine]);
 			lines[CurLine] = (char *) realloc( lines[CurLine], len + 2 );
 			for (int i = len; i > CurPos; i--) {
 				lines[CurLine][i] = lines[CurLine][i - 1];
@@ -487,28 +440,28 @@ void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 			lines[CurLine][CurPos] = Key;
 			lines[CurLine][len + 1] = 0;
 			CurPos++;
-			//print("pos: %d After: %s\n",CurPos, lines[CurLine]);
+			//print("pos: %d After: %s", CurPos, lines[CurLine]);
 			CalcRowCount();
 			RunEventHandler( TextAreaOnChange );
 		}
-		return;
+		return true;
 	}
 
 	//Selectable=false for dialogs, rather unintuitive, but fact
 	if ((Flags & IE_GUI_TEXTAREA_SELECTABLE) || ( Key < '1' ) || ( Key > '9' ))
-		return;
+		return false;
 	GameControl *gc = core->GetGameControl();
 	if (gc && (gc->GetDialogueFlags()&DF_IN_DIALOG) ) {
-		Changed = true;
+		MarkDirty();
 		seltext=minrow-1;
 		if ((unsigned int) seltext>=lines.size()) {
-			return;
+			return true;
 		}
 		for(int i=0;i<Key-'0';i++) {
 			do {
 				seltext++;
 				if ((unsigned int) seltext>=lines.size()) {
-					return;
+					return true;
 				}
 			}
 			while (strnicmp( lines[seltext], "[s=", 3 ) != 0 );
@@ -518,23 +471,24 @@ void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 		if (idx==-1) {
 			//this kills this object, don't use any more data!
 			gc->dialoghandler->EndDialog();
-			return;
+			return true;
 		}
 		gc->dialoghandler->DialogChoose( idx );
+		return true;
 	}
+	return false;
 }
 
 /** Special Key Press */
-void TextArea::OnSpecialKeyPress(unsigned char Key)
+bool TextArea::OnSpecialKeyPress(unsigned char Key)
 {
 	int len;
 	int i;
 
 	if (!(Flags&IE_GUI_TEXTAREA_EDITABLE)) {
-		return;
+		return false;
 	}
-	Owner->Invalidate();
-	Changed = true;
+	MarkDirty();
 	switch (Key) {
 		case GEM_HOME:
 			CurPos = 0;
@@ -577,7 +531,7 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 			break;
 		case GEM_DELETE:
 			len = GetRowLength(CurLine);
-			//print("len: %d Before: %s\n",len, lines[CurLine]);
+			//print("len: %d Before: %s", len, lines[CurLine]);
 			if (CurPos>=len) {
 				//TODO: merge next line
 				break;
@@ -586,12 +540,12 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 			for (i = CurPos; i < len; i++) {
 				lines[CurLine][i] = lines[CurLine][i + 1];
 			}
-			//print("pos: %d After: %s\n",CurPos, lines[CurLine]);
+			//print("pos: %d After: %s", CurPos, lines[CurLine]);
 			break;
 		case GEM_BACKSP:
 			len = GetRowLength(CurLine);
 			if (CurPos != 0) {
-				//print("len: %d Before: %s\n",len, lines[CurLine]);
+				//print("len: %d Before: %s", len, lines[CurLine]);
 				if (len<1) {
 					break;
 				}
@@ -601,15 +555,15 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 				}
 				lines[CurLine][len - 1] = 0;
 				CurPos--;
-				//print("pos: %d After: %s\n",CurPos, lines[CurLine]);
+				//print("pos: %d After: %s", CurPos, lines[CurLine]);
 			} else {
 				if (CurLine) {
 					//TODO: merge lines
 					int oldline = CurLine;
 					CurLine--;
 					int old = GetRowLength(CurLine);
-					//print("len: %d Before: %s\n",old, lines[CurLine]);
-					//print("len: %d Before: %s\n",len, lines[oldline]);
+					//print("len: %d Before: %s", old, lines[CurLine]);
+					//print("len: %d Before: %s", len, lines[oldline]);
 					lines[CurLine] = (char *) realloc (lines[CurLine], len+old);
 					memcpy(lines[CurLine]+old, lines[oldline],len);
 					free(lines[oldline]);
@@ -617,13 +571,13 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 					lines.erase(lines.begin()+oldline);
 					lrows.erase(lrows.begin()+oldline);
 					CurPos = old;
-					//print("pos: %d len: %d After: %s\n",CurPos, GetRowLength(CurLine), lines[CurLine]);
+					//print("pos: %d len: %d After: %s", CurPos, GetRowLength(CurLine), lines[CurLine]);
 				}
 			}
 			break;
 		 case GEM_RETURN:
 			//add an empty line after CurLine
-			//print("pos: %d Before: %s\n",CurPos, lines[CurLine]);
+			//print("pos: %d Before: %s", CurPos, lines[CurLine]);
 			lrows.insert(lrows.begin()+CurLine, 0);
 			len = GetRowLength(CurLine);
 			//copy the text after the cursor into the new line
@@ -637,12 +591,13 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 			//move cursor to next line beginning
 			CurLine++;
 			CurPos=0;
-			//print("len: %d After: %s\n",GetRowLength(CurLine-1), lines[CurLine-1]);
-			//print("len: %d After: %s\n",GetRowLength(CurLine), lines[CurLine]);
+			//print("len: %d After: %s", GetRowLength(CurLine-1), lines[CurLine-1]);
+			//print("len: %d After: %s", GetRowLength(CurLine), lines[CurLine]);
 			break;
 	}
 	CalcRowCount();
 	RunEventHandler( TextAreaOnChange );
+	return true;
 }
 
 /** Returns Row count */
@@ -696,17 +651,17 @@ void TextArea::ScrollToY(unsigned long y, Control* sender)
 /** Set Starting Row */
 void TextArea::SetRow(int row)
 {
-	if (row < rows) {
+	if (row <= rows) {
 		startrow = row;
 		TextYPos = row * ftext->maxHeight;
 	}
-	Changed = true;
+	MarkDirty();
 }
 
 void TextArea::CalcRowCount()
 {
 	int tr;
-	int w = Width;
+	size_t w = Width;
 
 	if (Flags&IE_GUI_TEXTAREA_SPEAKER) {
 		const char *portrait = NULL;
@@ -734,12 +689,11 @@ void TextArea::CalcRowCount()
 		for (size_t i = 0; i < lines.size(); i++) {
 //			rows++;
 			tr = 0;
-			int len = ( int ) strlen( lines[i] );
-			char* tmp = (char *) malloc( len + 1 );
-			memcpy( tmp, lines[i], len + 1 );
+			ieWord* tmp = NULL;
+			size_t len = ftext->GetDoubleByteString((unsigned char*)lines[i], tmp);
 			ftext->SetupString( tmp, w );
-			for (int p = 0; p <= len; p++) {
-				if (( ( unsigned char ) tmp[p] ) == '[') {
+			for (size_t p = 0; p <= len; p++) {
+				if (( tmp[p] ) == '[') {
 					p++;
 					//char tag[256];
 					int k = 0;
@@ -784,7 +738,7 @@ void TextArea::CalcRowCount()
 		return;
 	}
 	ScrollBar* bar = ( ScrollBar* ) sb;
-	tr = rows - Height/ftext->size[1].h + 1;
+	tr = rows - Height/ftext->maxHeight + 1;
 	if (tr<0) {
 		tr = 0;
 	}
@@ -795,16 +749,18 @@ void TextArea::CalcRowCount()
 /** This method is key to touchscreen scrolling */
 void TextArea::OnMouseWheelScroll(short /*x*/, short y)
 {
-	unsigned long fauxY = TextYPos;
-	if ((long)fauxY + y <= 0) fauxY = 0;
-	else fauxY += y;
-	ScrollToY(fauxY, this);
+	if (!(IE_GUI_TEXTAREA_SMOOTHSCROLL & Flags)){
+		unsigned long fauxY = TextYPos;
+		if ((long)fauxY + y <= 0) fauxY = 0;
+		else fauxY += y;
+		ScrollToY(fauxY, this);
+	}
 }
 
 /** Mouse Over Event */
 void TextArea::OnMouseOver(unsigned short /*x*/, unsigned short y)
 {
-	int height = ftext->maxHeight; //size[1].h;
+	int height = ftext->maxHeight;
 	int r = y / height;
 	int row = 0;
 
@@ -812,26 +768,29 @@ void TextArea::OnMouseOver(unsigned short /*x*/, unsigned short y)
 		row += lrows[i];
 		if (r < ( row - startrow )) {
 			if (seltext != (int) i)
-				core->RedrawAll();
+				MarkDirty();
 			seltext = ( int ) i;
-			//print("CtrlId = 0x%08lx, seltext = %d, rows = %d, row = %d, r = %d\n", ControlID, i, rows, row, r);
+			//print("CtrlId = 0x%08lx, seltext = %d, rows = %d, row = %d, r = %d", ControlID, i, rows, row, r);
 			return;
 		}
 	}
 	if (seltext != -1) {
-		core->RedrawAll();
+		MarkDirty();
 	}
 	seltext = -1;
-	//print("CtrlId = 0x%08lx, seltext = %d, rows %d, row %d, r = %d\n", ControlID, seltext, rows, row, r);
+	//print("CtrlId = 0x%08lx, seltext = %d, rows %d, row %d, r = %d", ControlID, seltext, rows, row, r);
 }
 
 /** Mouse Button Up */
-void TextArea::OnMouseUp(unsigned short x, unsigned short y, unsigned short /*Button*/,
+void TextArea::OnMouseUp(unsigned short x, unsigned short y, unsigned short Button,
 	unsigned short /*Mod*/)
 {
-	if (( x <= Width ) && ( y <= ( Height - 5 ) ) && ( seltext != -1 )) {
+	if (!(Button & (GEM_MB_ACTION|GEM_MB_MENU)))
+		return;
+
+	if ((x < Width) && (y < Height - 5) && (seltext != -1)) {
 		Value = (unsigned int) seltext;
-		Changed = true;
+		MarkDirty();
 		if (strnicmp( lines[seltext], "[s=", 3 ) == 0) {
 			if (minrow > seltext)
 				return;
@@ -877,13 +836,13 @@ void TextArea::CopyTo(TextArea *ta)
 	ta->SetText(lines);
 }
 
-void TextArea::RedrawTextArea(const char* VariableName, unsigned int Sum)
+void TextArea::UpdateState(const char* VariableName, unsigned int Sum)
 {
 	if (strnicmp( VarName, VariableName, MAX_VARIABLE_LENGTH )) {
 		return;
 	}
 	Value = Sum;
-	Changed = true;
+	MarkDirty();
 }
 
 void TextArea::SelectText(const char *select)
@@ -898,34 +857,28 @@ void TextArea::SelectText(const char *select)
 			} else {
 				SetRow( i );
 			}
-			RedrawTextArea( VarName, i);
+			UpdateState(VarName, i);
 			CalcRowCount();
-			Owner->Invalidate();
 			core->RedrawAll();
 			break;
 		}
 	}
 }
 
-const char* TextArea::QueryText()
+const char* TextArea::QueryText() const
 {
 	if ( Value<lines.size() ) {
 		return ( const char * ) lines[Value];
 	}
-	return ( const char *) "";
+	return "";
 }
 
 bool TextArea::SetEvent(int eventType, EventHandler handler)
 {
-	Changed = true;
-
 	switch (eventType) {
 	case IE_GUI_TEXTAREA_ON_CHANGE:
 		TextAreaOnChange = handler;
 		break;
-	case IE_GUI_TEXTAREA_OUT_OF_TEXT:
-		TextAreaOutOfText = handler;
-		break;
 	default:
 		return false;
 	}
@@ -968,34 +921,23 @@ void TextArea::Clear()
 
 //setting up the textarea for smooth scrolling, the first
 //TEXTAREA_OUTOFTEXT callback is called automatically
-void TextArea::SetupScroll(unsigned long tck)
+void TextArea::SetupScroll()
 {
 	SetPreservedRow(0);
-	smooth = ftext->maxHeight;
 	startrow = 0;
-	ticks = tck;
+	// ticks is the number of ticks it takes to scroll this font 1 px
+	ticks = 2400 / ftext->maxHeight;
 	//clearing the textarea
 	Clear();
-	unsigned int i = (unsigned int) (Height/smooth);
-	while (i--) {
+	unsigned int i = (unsigned int) (1 + ((Height - 1) / ftext->maxHeight)); // ceiling
+	while (i--) { //push empty lines so that the text starts out of view.
 		char *str = (char *) malloc(1);
 		str[0]=0;
 		lines.push_back(str);
 		lrows.push_back(0);
 	}
-	i = (unsigned int) lines.size();
 	Flags |= IE_GUI_TEXTAREA_SMOOTHSCROLL;
 	starttime = GetTickCount();
-	if (RunEventHandler( TextAreaOutOfText )) {
-		//event handler destructed this object?
-		return;
-	}
-	if (i==lines.size()) {
-		ResetEventHandler( TextAreaOutOfText );
-		return;
-	}
-	//recalculates rows
-	AppendText("\n",-1);
 }
 
 void TextArea::OnMouseDown(unsigned short /*x*/, unsigned short /*y*/, unsigned short Button,
@@ -1014,11 +956,9 @@ void TextArea::OnMouseDown(unsigned short /*x*/, unsigned short /*y*/, unsigned
 		switch(Button) {
 		case GEM_MB_SCRLUP:
 			scrlbr->ScrollUp();
-			core->RedrawAll();
 			break;
 		case GEM_MB_SCRLDOWN:
 			scrlbr->ScrollDown();
-			core->RedrawAll();
 			break;
 		}
 	}
@@ -1031,3 +971,17 @@ void TextArea::SetFocus(bool focus)
 		core->GetVideoDriver()->ShowSoftKeyboard();
 	}
 }
+
+static bool charSorter(const char *a, const char *b) {
+	return stricmp(a, b) < 0;
+}
+
+void TextArea::SortText()
+{
+	std::list<char*> sorter(lines.begin(), lines.end());
+	sorter.sort(charSorter);
+	lines.assign(sorter.begin(), sorter.end());
+	CalcRowCount();
+}
+
+}
