@@ -23,16 +23,21 @@
 
 #include "GameData.h"
 #include "Interface.h"
-#include "Palette.h"
+#include "Sprite2D.h"
 #include "Video.h"
 #include "GUI/EventMgr.h"
 #include "GUI/Window.h"
 
-TextEdit::TextEdit(unsigned short maxLength, unsigned short px, unsigned short py)
+namespace GemRB {
+
+TextEdit::TextEdit(const Region& frame, unsigned short maxLength, unsigned short px, unsigned short py)
+	: Control(frame)
 {
+	ControlType = IE_GUI_EDIT;
 	max = maxLength;
 	FontPosX = px;
 	FontPosY = py;
+	Alignment = IE_FONT_ALIGN_MIDDLE | IE_FONT_ALIGN_LEFT;
 	Buffer = ( unsigned char * ) malloc( max + 1 );
 	font = NULL;
 	Cursor = NULL;
@@ -42,10 +47,9 @@ TextEdit::TextEdit(unsigned short maxLength, unsigned short px, unsigned short p
 	ResetEventHandler( EditOnChange );
 	ResetEventHandler( EditOnDone );
 	ResetEventHandler( EditOnCancel );
-	Color white = {0xff, 0xff, 0xff, 0x00}, black = {0x00, 0x00, 0x00, 0x00};
 	//Original engine values
 	//Color white = {0xc8, 0xc8, 0xc8, 0x00}, black = {0x3c, 0x3c, 0x3c, 0x00};
-	palette = core->CreatePalette( white, black );
+	palette = core->CreatePalette( ColorWhite, ColorBlack );
 }
 
 TextEdit::~TextEdit(void)
@@ -57,28 +61,32 @@ TextEdit::~TextEdit(void)
 	video->FreeSprite( Cursor );
 }
 
-/** Draws the Control on the Output Display */
-void TextEdit::Draw(unsigned short x, unsigned short y)
+void TextEdit::SetAlignment(unsigned char Alignment)
 {
-	if (!Changed && !(Owner->Flags&WF_FLOAT)) {
-		return;
-	}
-	Changed = false;
-	if (Back) {
-		core->GetVideoDriver()->BlitSprite( Back, x + XPos, y + YPos, true );
+    this->Alignment = Alignment;
+    MarkDirty();
+}
 
+/** Draws the Control on the Output Display */
+void TextEdit::DrawInternal(Region& rgn)
+{
+	ieWord yOff = FontPosY;
+	if (Back) {
+		core->GetVideoDriver()->BlitSprite( Back, rgn.x, rgn.y, true );
+		if (yOff) yOff += Back->Height;
 	}
 	if (!font)
 		return;
 
+	// FIXME: we should clip text to the background right?
 	//The aligning of textedit fields is done by absolute positioning (FontPosX, FontPosY)
 	if (hasFocus) {
-		font->Print( Region( x + XPos + FontPosX, y + YPos + FontPosY, Width, Height ), Buffer,
-				palette, IE_FONT_ALIGN_LEFT | IE_FONT_ALIGN_TOP,
+		font->Print( Region( rgn.x + FontPosX, rgn.y - yOff, rgn.w, rgn.h ), Buffer,
+				palette, Alignment,
 				true, NULL, Cursor, CurPos );
 	} else {
-		font->Print( Region( x + XPos + FontPosX, y + YPos + FontPosY, Width, Height ), Buffer,
-				palette, IE_FONT_ALIGN_LEFT | IE_FONT_ALIGN_TOP, true );
+		font->Print( Region( rgn.x + FontPosX, rgn.y - yOff, rgn.w, rgn.h ), Buffer,
+				palette, Alignment, true );
 	}
 }
 
@@ -87,10 +95,10 @@ void TextEdit::SetFont(Font* f)
 {
 	if (f != NULL) {
 		font = f;
-		Changed = true;
+		MarkDirty();
 		return;
 	}
-	printMessage("TextEdit","Invalid font set!\n", LIGHT_RED);
+	Log(ERROR, "TextEdit", "Invalid font set!");
 }
 
 Font *TextEdit::GetFont() { return font; }
@@ -102,7 +110,7 @@ void TextEdit::SetCursor(Sprite2D* cur)
 	if (cur != NULL) {
 		Cursor = cur;
 	}
-	Changed = true;
+	MarkDirty();
 }
 
 /** Set BackGround */
@@ -112,17 +120,16 @@ void TextEdit::SetBackGround(Sprite2D* back)
 	if (Back)
 		core->GetVideoDriver()->FreeSprite(Back);
 	Back = back;
-	Changed = true;
+	MarkDirty();
 }
 
 /** Key Press Event */
-void TextEdit::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
+bool TextEdit::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 {
 	if (Key >= 0x20) {
 		if (Value && ( (Key<'0') || (Key>'9') ) )
-			return;
-		Owner->Invalidate();
-		Changed = true;
+			return false;
+		MarkDirty();
 		int len = ( int ) strlen( ( char* ) Buffer );
 		if (len + 1 < max) {
 			for (int i = len; i > CurPos; i--) {
@@ -133,15 +140,16 @@ void TextEdit::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 			CurPos++;
 		}
 		RunEventHandler( EditOnChange );
+		return true;
 	}
+	return false;
 }
 /** Special Key Press */
-void TextEdit::OnSpecialKeyPress(unsigned char Key)
+bool TextEdit::OnSpecialKeyPress(unsigned char Key)
 {
 	int len;
 
-	Owner->Invalidate();
-	Changed = true;
+	MarkDirty();
 	switch (Key) {
 		case GEM_HOME:
 			CurPos = 0;
@@ -179,10 +187,9 @@ void TextEdit::OnSpecialKeyPress(unsigned char Key)
 			break;
 		case GEM_RETURN:
 			RunEventHandler( EditOnDone );
-			return;
-
 	}
 	RunEventHandler( EditOnChange );
+	return true;
 }
 
 void TextEdit::SetFocus(bool focus)
@@ -196,12 +203,10 @@ void TextEdit::SetFocus(bool focus)
 /** Sets the Text of the current control */
 void TextEdit::SetText(const char* string)
 {
-	strncpy( ( char * ) Buffer, string, max );
-	Buffer[max]=0;
-	CurPos = (ieWord) strlen((char *) Buffer);
-	if (Owner) {
-		Owner->Invalidate();
-	}
+	int len = strlcpy( ( char * ) Buffer, string, max + 1 );
+	if (len > max) CurPos = max + 1;
+	else CurPos = len;
+	MarkDirty();
 }
 
 void TextEdit::SetBufferLength(ieWord buflen)
@@ -215,15 +220,13 @@ void TextEdit::SetBufferLength(ieWord buflen)
 }
 
 /** Simply returns the pointer to the text, don't modify it! */
-const char* TextEdit::QueryText()
+const char* TextEdit::QueryText() const
 {
 	return ( const char * ) Buffer;
 }
 
 bool TextEdit::SetEvent(int eventType, EventHandler handler)
 {
-	Changed = true;
-
 	switch (eventType) {
 	case IE_GUI_EDIT_ON_CHANGE:
 		EditOnChange = handler;
@@ -240,3 +243,5 @@ bool TextEdit::SetEvent(int eventType, EventHandler handler)
 
 	return true;
 }
+
+}
