@@ -34,6 +34,8 @@
 
 #include "Font.h"
 
+namespace GemRB {
+
 class Palette;
 
 // !!! Keep these synchronized with GUIDefines.py
@@ -50,11 +52,13 @@ class Palette;
  */
 
 class GEM_EXPORT TextEdit : public Control {
-public:
-	TextEdit(unsigned short maxLength, unsigned short x, unsigned short y);
-	~TextEdit(void);
+protected:
 	/** Draws the Control on the Output Display */
-	void Draw(unsigned short x, unsigned short y);
+	void DrawInternal(Region& drawFrame);
+	bool HasBackground() { return Back; }
+public:
+	TextEdit(const Region& frame, unsigned short maxLength, unsigned short x, unsigned short y);
+	~TextEdit(void);
 	/** Set Font */
 	void SetFont(Font* f);
 	Font *GetFont();
@@ -64,15 +68,18 @@ public:
 	void SetBackGround(Sprite2D* back);
 	/** Sets the Text of the current control */
 	void SetText(const char* string);
-	/** Sets the Text of the current control */
-	const char* QueryText();
+	/** Gets the Text of the current control */
+	const char* QueryText() const;
 	/** Sets the buffer length */
 	void SetBufferLength(ieWord buflen);
+	/** Sets the alignment */
+	void SetAlignment(unsigned char Alignment);
 private:
 	/** Text Editing Cursor Sprite */
 	Sprite2D* Cursor;
 	/** Text Font */
 	Font* font;
+	unsigned char Alignment;
 	/** Background */
 	Sprite2D* Back;
 	/** Max Edit Text Length */
@@ -87,9 +94,9 @@ private:
 	Palette* palette;
 public: //Events
 	/** Key Press Event */
-	void OnKeyPress(unsigned char Key, unsigned short Mod);
+	bool OnKeyPress(unsigned char Key, unsigned short Mod);
 	/** Special Key Press */
-	void OnSpecialKeyPress(unsigned char Key);
+	bool OnSpecialKeyPress(unsigned char Key);
 	/** Set handler for specified event */
 	bool SetEvent(int eventType, EventHandler handler);
 	void SetFocus(bool focus);
@@ -99,4 +106,6 @@ public: //Events
 	EventHandler EditOnCancel;
 };
 
+}
+
 #endif
