@@ -814,7 +814,7 @@ void Game::PlacePersistents(Map *newMap, const char *ResRef)
 	last = NPCs.size()-1;
 	for (i = 0; i < NPCs.size(); i++) {
 		if (stricmp( NPCs[i]->Area, ResRef ) == 0) {
-			if (i < last && CheckForReplacementActor(i)) {
+			if (i <= last && CheckForReplacementActor(i)) {
 				i--;
 				last--;
 				continue;
@@ -840,7 +840,6 @@ int Game::LoadMap(const char* ResRef, bool loadscreen)
 
 	int index = FindMap(ResRef);
 	if (index>=0) {
-		PlacePersistents(GetMap(index), ResRef);
 		return index;
 	}
 
@@ -928,8 +927,10 @@ bool Game::CheckForReplacementActor(int i)
 					error("Game::CheckForReplacementActor", "GetNPC failed: cannot find act!\n");
 				} else {
 					newact->Pos = act->Pos; // the map is not loaded yet, so no SetPosition
+					newact->TalkCount = act->TalkCount;
+					newact->InteractCount = act->InteractCount;
 					CopyResRef(newact->Area, act->Area);
-					DelNPC(InStore(act));
+					DelNPC(InStore(act), true);
 					return true;
 				}
 			}
@@ -1644,11 +1645,15 @@ bool Game::RestParty(int checks, int dream, int hp)
 
 	while (i--) {
 		Actor *tar = GetPC(i, true);
-		tar->ClearPath();
-		tar->ClearActions();
+		tar->Stop();
 		tar->SetModal(MS_NONE, 0);
 		//if hp = 0, then healing will be complete
 		tar->Heal(hp);
+		// auto-cast memorized healing spells if requested and available
+		// run it only once, since it loops itself to save time
+		if (i+1 == GetPartySize(true)) {
+			CastOnRest();
+		}
 		//removes fatigue, recharges spells
 		tar->Rest(hours);
 		if (!hoursLeft)
@@ -1667,7 +1672,8 @@ bool Game::RestParty(int checks, int dream, int hp)
 		if (gamedata->Exists("player1d",IE_BCS_CLASS_ID, true)) {
 			cutscene = true;
 			PlayerDream();
-		} else if (gamedata->Exists("drmtxt2", IE_2DA_CLASS_ID, true)) {
+		// all games have these bg1 leftovers, but only bg2 replaced the content
+		} else if (gamedata->GetResource("drmtxt2", IE_2DA_CLASS_ID, true)->Size() > 0) {
 			TextDream();
 		}
 
@@ -1709,6 +1715,50 @@ bool Game::RestParty(int checks, int dream, int hp)
 	return cutscene;
 }
 
+// heal on rest and similar
+void Game::CastOnRest()
+{
+	ieDword tmp = 0;
+	core->GetDictionary()->Lookup("Heal Party on Rest", tmp);
+	int specialCount = core->GetSpecialSpellsCount();
+
+	if (tmp && specialCount != -1) {
+		int ps = GetPartySize(true);
+		int ps2 = ps;
+		std::vector<Actor *> injurees;
+		for (int idx = 1; idx <= ps; idx++) {
+			Actor *tar = FindPC(idx);
+			if (tar && tar->GetStat(IE_HITPOINTS) < tar->GetStat(IE_MAXHITPOINTS)) {
+				injurees.push_back(tar);
+			}
+		}
+		if (injurees.empty()) injurees = PCs; // enable some nonhealing magic too
+
+		SpellDescType *special_spells = core->GetSpecialSpells();
+		while (specialCount--) {
+			if (special_spells[specialCount].value & SP_REST) {
+				if (injurees.empty()) injurees = PCs; // enable some nonhealing magic too
+				// check each party member for the spell
+				while (ps--) {
+					Actor *tar = GetPC(ps, true);
+					while (tar && tar->spellbook.HaveSpell(special_spells[specialCount].resref, 0)) {
+						// we have the spell, so cast it on most injured
+						// NOTE: no distinction is made about the potency of spells
+						Actor *injuree = injurees[0];
+						tar->DirectlyCastSpell(injuree, special_spells[specialCount].resref, 0, 1, true, true, true);
+						if (injuree->GetStat(IE_HITPOINTS) == injuree->GetStat(IE_MAXHITPOINTS)) {
+							if (!injurees.empty()) {
+								injurees.erase(injurees.begin());
+							}
+						}
+					}
+				}
+				ps = ps2;
+			}
+		}
+	}
+}
+
 //timestop effect
 void Game::TimeStop(Actor* owner, ieDword end)
 {
