@@ -41,6 +41,7 @@
 #include "Spell.h"
 #include "SpellMgr.h"
 #include "StoreMgr.h"
+#include "VEFObject.h"
 #include "Scriptable/Actor.h"
 #include "System/FileStream.h"
 
@@ -68,7 +69,7 @@ static void ReleasePalette(void *poi)
 	//we allow nulls, but we shouldn't release them
 	if (!poi) return;
 	//as long as palette has its own refcount, this should be Release
-	((Palette *) poi)->Release();
+	((Palette *) poi)->release();
 }
 
 GEM_EXPORT GameData* gamedata;
@@ -271,7 +272,7 @@ void GameData::FreePalette(Palette *&pal, const ieResRef name)
 		if(pal->named) {
 			error("GameData", "Palette is supposed to be named, but got no name!\n");
 		} else {
-			pal->Release();
+			pal->release();
 			pal=NULL;
 		}
 		return;
@@ -284,7 +285,7 @@ void GameData::FreePalette(Palette *&pal, const ieResRef name)
 		error("Core", "Corrupted Palette cache encountered (reference count went below zero), Palette name is: %.8s\n", name);
 	}
 	if (!res) {
-		pal->Release();
+		pal->release();
 	}
 	pal = NULL;
 }
@@ -431,6 +432,29 @@ ScriptedAnimation* GameData::GetScriptedAnimation( const char *effect, bool doub
 	return ret;
 }
 
+VEFObject* GameData::GetVEFObject(const char *effect, bool doublehint)
+{
+	VEFObject *ret = NULL;
+
+	if (Exists( effect, IE_VEF_CLASS_ID, true ) ) {
+		DataStream *ds = GetResource( effect, IE_VEF_CLASS_ID );
+		ret = new VEFObject();
+		strnlwrcpy(ret->ResName, effect, 8);
+		ret->LoadVEF(ds);
+	} else {
+		if (Exists( effect, IE_2DA_CLASS_ID, true ) ) {
+			ret = new VEFObject();
+			ret->Load2DA(effect);
+		} else {
+			ScriptedAnimation *sca = GetScriptedAnimation(effect, doublehint);
+			if (sca) {
+				ret = new VEFObject(sca);
+			}
+		}
+	}
+	return ret;
+}
+
 // Return single BAM frame as a sprite. Use if you want one frame only,
 // otherwise it's not efficient
 Sprite2D* GameData::GetBAMSprite(const ieResRef ResRef, int cycle, int frame, bool silent)
