@@ -60,6 +60,7 @@ static const TriggerLink triggernames[] = {
 	{"assaltedby", GameScript::AttackedBy, 0},//pst
 	{"attackedby", GameScript::AttackedBy, 0},
 	{"becamevisible", GameScript::BecameVisible, 0},
+	{"beeninparty", GameScript::BeenInParty, 0},
 	{"bitcheck", GameScript::BitCheck,TF_MERGESTRINGS},
 	{"bitcheckexact", GameScript::BitCheckExact,TF_MERGESTRINGS},
 	{"bitglobal", GameScript::BitGlobal_Trigger,TF_MERGESTRINGS},
@@ -157,6 +158,7 @@ static const TriggerLink triggernames[] = {
 	{"harmlessentered", GameScript::HarmlessEntered, 0}, //pst
 	{"harmlessopened", GameScript::HarmlessOpened, 0}, //pst
 	{"hasbounceeffects", GameScript::HasBounceEffects, 0},
+	{"hasdlc", GameScript::HasDLC, 0},
 	{"hasimmunityeffects", GameScript::HasImmunityEffects, 0},
 	{"hasinnateability", GameScript::HaveSpell, 0}, //these must be the same
 	{"hasitem", GameScript::HasItem, 0},
@@ -225,6 +227,7 @@ static const TriggerLink triggernames[] = {
 	{"isscriptname", GameScript::CalledByName, 0}, //seems the same
 	{"isspelltargetvalid", GameScript::IsSpellTargetValid, 0},
 	{"isteambiton", GameScript::IsTeamBitOn, 0},
+	{"istouchgui", GameScript::IsTouchGUI, 0},
 	{"isvalidforpartydialog", GameScript::IsValidForPartyDialog, 0},
 	{"isvalidforpartydialogue", GameScript::IsValidForPartyDialog, 0},
 	{"isweaponranged", GameScript::IsWeaponRanged, 0},
@@ -676,7 +679,7 @@ static const ActionLink actionnames[] = {
 	{"leavearealua", GameScript::LeaveAreaLUA, 0},
 	{"leavearealuaentry", GameScript::LeaveAreaLUAEntry,AF_BLOCKING},
 	{"leavearealuapanic", GameScript::LeaveAreaLUAPanic, 0},
-	{"leavearealuapanicentry", GameScript::LeaveAreaLUAPanicEntry,AF_BLOCKING},
+	{"leavearealuapanicentry", GameScript::LeaveAreaLUAPanicEntry, 0},
 	{"leaveparty", GameScript::LeaveParty, 0},
 	{"lock", GameScript::Lock, 0},//key not checked at this time!
 	{"lockscroll", GameScript::LockScroll, 0},
@@ -714,6 +717,7 @@ static const ActionLink actionnames[] = {
 	{"moveviewobject", GameScript::MoveViewObject, AF_BLOCKING},
 	{"moveviewpoint", GameScript::MoveViewPoint, AF_BLOCKING},
 	{"moveviewpointuntildone", GameScript::MoveViewPoint, 0},
+	{"multiplayersync", GameScript::MultiPlayerSync, 0},
 	{"nidspecial1", GameScript::NIDSpecial1,AF_BLOCKING|AF_DIRECT|AF_ALIVE},//we use this for dialogs, hack
 	{"nidspecial2", GameScript::NIDSpecial2,AF_BLOCKING},//we use this for worldmap, another hack
 	{"nidspecial3", GameScript::Attack,AF_BLOCKING|AF_DIRECT|AF_ALIVE},//this hack is for attacking preset target
@@ -747,6 +751,7 @@ static const ActionLink actionnames[] = {
 	{"polymorph", GameScript::Polymorph, 0},
 	{"polymorphcopy", GameScript::PolymorphCopy, 0},
 	{"polymorphcopybase", GameScript::PolymorphCopyBase, 0},
+	{"polymorphex", GameScript::Polymorph, 0}, //pst FIXME: has RangeModes int1 that we ignore
 	{"protectobject", GameScript::ProtectObject, 0},
 	{"protectpoint", GameScript::ProtectPoint, AF_BLOCKING},
 	{"quitgame", GameScript::QuitGame, 0},
@@ -757,9 +762,12 @@ static const ActionLink actionnames[] = {
 	{"randomwalkcontinuous", GameScript::RandomWalkContinuous, AF_BLOCKING|AF_ALIVE},
 	{"realsetglobaltimer", GameScript::RealSetGlobalTimer,AF_MERGESTRINGS},
 	{"reallyforcespell", GameScript::ReallyForceSpell, AF_BLOCKING|AF_ALIVE},
+	{"reallyforcespellres", GameScript::ReallyForceSpell, AF_BLOCKING|AF_ALIVE},
 	{"reallyforcespelldead", GameScript::ReallyForceSpellDead, AF_BLOCKING},
+	{"reallyforcespelldeadres", GameScript::ReallyForceSpellDead, AF_BLOCKING},
 	{"reallyforcespelllevel", GameScript::ReallyForceSpell, AF_BLOCKING|AF_ALIVE},//this is the same action
 	{"reallyforcespellpoint", GameScript::ReallyForceSpellPoint, AF_BLOCKING|AF_ALIVE},
+	{"reallyforcespellpointres", GameScript::ReallyForceSpellPoint, AF_BLOCKING|AF_ALIVE},
 	{"recoil", GameScript::Recoil, AF_ALIVE},
 	{"regainpaladinhood", GameScript::RegainPaladinHood, 0},
 	{"regainrangerhood", GameScript::RegainRangerHood, 0},
@@ -787,6 +795,7 @@ static const ActionLink actionnames[] = {
 	{"revealareaonmap", GameScript::RevealAreaOnMap, 0},
 	{"runawayfrom", GameScript::RunAwayFrom,AF_BLOCKING|AF_ALIVE},
 	{"runawayfromnointerrupt", GameScript::RunAwayFromNoInterrupt,AF_BLOCKING|AF_ALIVE},
+	{"runawayfromnointerruptnoleavearea", GameScript::RunAwayFromNoInterrupt,AF_BLOCKING|AF_ALIVE},
 	{"runawayfromnoleavearea", GameScript::RunAwayFromNoLeaveArea,AF_BLOCKING|AF_ALIVE},
 	{"runawayfrompoint", GameScript::RunAwayFromPoint,AF_BLOCKING|AF_ALIVE},
 	{"runfollow", GameScript::RunAwayFrom,AF_BLOCKING|AF_ALIVE},
@@ -807,6 +816,7 @@ static const ActionLink actionnames[] = {
 	{"setapparentnamestrref", GameScript::SetApparentName, 0},
 	{"setareaflags", GameScript::SetAreaFlags, 0},
 	{"setarearestflag", GameScript::SetAreaRestFlag, 0},
+	{"setareascript", GameScript::ChangeAIScript, 0},
 	{"setbeeninpartyflags", GameScript::SetBeenInPartyFlags, 0},
 	{"setbestweapon", GameScript::SetBestWeapon, 0},
 	{"setcorpseenabled", GameScript::AmbientActivate, 0},//another weird name
@@ -843,6 +853,7 @@ static const ActionLink actionnames[] = {
 	{"setmazeharder", GameScript::SetMazeHarder, 0}, //pst specific crap
 	{"setmoraleai", GameScript::SetMoraleAI, 0},
 	{"setmusic", GameScript::SetMusic, 0},
+	{"setmytarget", GameScript::SetMyTarget, 0},
 	{"setname", GameScript::SetApparentName, 0},
 	{"setnamelessclass", GameScript::SetNamelessClass, 0},
 	{"setnamelessdeath", GameScript::SetNamelessDeath, 0},
@@ -1972,12 +1983,9 @@ bool GameScript::Update(bool *continuing, bool *done)
 
 					//movetoobjectfollow would break if this isn't called
 					//(what is broken if it is here?)
-					MySelf->ClearActions();
 					//IE even clears the path, shall we?
 					//yes we must :)
-					if (MySelf->Type == ST_ACTOR) {
-						((Movable *)MySelf)->ClearPath();
-					}
+					MySelf->Stop();
 				}
 				lastAction=a;
 			}
@@ -2177,6 +2185,9 @@ bool Condition::Evaluate(Scriptable* Sender)
 			//we started an Or() block
 			if (ORcount) {
 				Log(WARNING, "GameScript", "Unfinished OR block encountered!");
+				if (!subresult) {
+					return 0;
+				}
 			}
 			ORcount = result;
 			subresult = false;
@@ -2195,6 +2206,7 @@ bool Condition::Evaluate(Scriptable* Sender)
 	}
 	if (ORcount) {
 		Log(WARNING, "GameScript", "Unfinished OR block encountered!");
+		return subresult;
 	}
 	return 1;
 }
@@ -2269,6 +2281,14 @@ int Response::Execute(Scriptable* Sender)
 {
 	int ret = 0; // continue or not
 	for (size_t i = 0; i < actions.size(); i++) {
+		if (!CheckCanary()) {
+			// FIXME: hack to prevent crashing when a script deletes itself.
+			// this object has been deleted and this should not be considered a fix (it may cause unforseen problems too).
+			Log(ERROR, "GameScript", "Aborting response execution due to object deletion.\n \
+									  This should not happen and we need to fix it.");
+			ret = 0;
+			break;
+		}
 		Action* aC = actions[i];
 		switch (actionflags[aC->actionID] & AF_MASK) {
 			case AF_IMMEDIATE:
@@ -2406,42 +2426,47 @@ Trigger* GenerateTrigger(char* String)
 	return trigger;
 }
 
-Action* GenerateAction(char* String)
+Action* GenerateAction(const char* String)
 {
-	strlwr( String );
+	Action* action = NULL;
+	char* actionString = strdup(String);
+	// the only thing we seem to need a copy for is the call to strlwr...
+	strlwr( actionString );
 	if (InDebug&ID_ACTIONS) {
 		Log(WARNING, "GameScript", "Compiling:%s", String);
 	}
 	int len = strlench(String,'(')+1; //including (
-	char *src = String+len;
+	char *src = actionString+len;
 	int i = -1;
 	char *str;
 	unsigned short actionID;
 	if (overrideActionsTable) {
-		i = overrideActionsTable->FindString(String, len);
+		i = overrideActionsTable->FindString(actionString, len);
 		if (i >= 0) {
 			str = overrideActionsTable->GetStringIndex( i )+len;
 			actionID = overrideActionsTable->GetValueIndex(i);
 		}
 	}
 	if (i<0) {
-		i = actionsTable->FindString(String, len);
+		i = actionsTable->FindString(actionString, len);
 		if (i < 0) {
 			Log(ERROR, "GameScript", "Invalid scripting action: %s", String);
-			return NULL;
+			goto done;
 		}
 		str = actionsTable->GetStringIndex( i )+len;
 		actionID = actionsTable->GetValueIndex(i);
 	}
-	Action *action = GenerateActionCore( src, str, actionID);
+	action = GenerateActionCore( src, str, actionID);
 	if (!action) {
 		Log(ERROR, "GameScript", "Malformed scripting action: %s", String);
-		return NULL;
+		goto done;
 	}
+	done:
+	free(actionString);
 	return action;
 }
 
-Action* GenerateActionDirect(char *String, Scriptable *object)
+Action* GenerateActionDirect(const char *String, Scriptable *object)
 {
 	Action* action = GenerateAction(String);
 	if (!action) return NULL;
@@ -2464,7 +2489,7 @@ void Object::dump(StringBuffer& buffer) const
 {
 	int i;
 
-	GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
+	AssertCanary(__FUNCTION__);
 	if(objectName[0]) {
 		buffer.appendFormatted("Object: %s\n",objectName);
 		return;
@@ -2507,7 +2532,7 @@ void Trigger::dump() const
 
 void Trigger::dump(StringBuffer& buffer) const
 {
-	GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
+	AssertCanary(__FUNCTION__);
 	buffer.appendFormatted("Trigger: %d\n", triggerID);
 	buffer.appendFormatted("Int parameters: %d %d %d\n", int0Parameter, int1Parameter, int2Parameter);
 	buffer.appendFormatted("Point: [%d.%d]\n", pointParameter.x, pointParameter.y);
@@ -2532,7 +2557,7 @@ void Action::dump(StringBuffer& buffer) const
 {
 	int i;
 
-	GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
+	AssertCanary(__FUNCTION__);
 	buffer.appendFormatted("Int0: %d, Int1: %d, Int2: %d\n",int0Parameter, int1Parameter, int2Parameter);
 	buffer.appendFormatted("String0: %s, String1: %s\n", string0Parameter?string0Parameter:"<NULL>", string1Parameter?string1Parameter:"<NULL>");
 	for (i=0;i<3;i++) {
