@@ -29,6 +29,8 @@
 #include "Game.h"
 #include "GUI/GameControl.h"
 
+namespace GemRB {
+
 //-------------------------------------------------------------
 // Object Functions
 //-------------------------------------------------------------
@@ -136,9 +138,13 @@ Targets *GameScript::Gabber(Scriptable* /*Sender*/, Targets *parameters, int ga_
 
 Targets *GameScript::LastTrigger(Scriptable *Sender, Targets *parameters, int ga_flags)
 {
+	Scriptable *target = parameters->GetTarget(0, -1);
 	parameters->Clear();
-	if (Sender->LastTrigger) {
-		Actor *target = Sender->GetCurrentArea()->GetActorByGlobalID(Sender->LastTrigger);
+	if (target) {
+		target = Sender->GetCurrentArea()->GetActorByGlobalID(target->LastTrigger);
+		parameters->AddTarget(target, 0, ga_flags);
+	} else if (Sender->LastTrigger) {
+		target = Sender->GetCurrentArea()->GetActorByGlobalID(Sender->LastTrigger);
 		parameters->AddTarget(target, 0, ga_flags);
 	}
 	return parameters;
@@ -162,6 +168,24 @@ Targets *GameScript::LastMarkedObject(Scriptable *Sender, Targets *parameters, i
 	return parameters;
 }
 
+Targets *GameScript::SpellTarget(Scriptable *Sender, Targets *parameters, int ga_flags)
+{
+	Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
+	if (!actor) {
+		if (Sender->Type==ST_ACTOR) {
+			actor = (Actor *) Sender;
+		}
+	}
+	parameters->Clear();
+	if (actor) {
+		Actor *target = actor->GetCurrentArea()->GetActorByGlobalID(actor->LastTarget);
+		if (target) {
+			parameters->AddTarget(target, 0, ga_flags);
+		}
+	}
+	return parameters;
+}
+
 //actions should always use LastMarkedObject, because LastSeen could be deleted
 Targets *GameScript::LastSeenBy(Scriptable *Sender, Targets *parameters, int ga_flags)
 {
@@ -523,164 +547,66 @@ Targets *GameScript::Player8Fill(Scriptable* /*Sender*/, Targets *parameters, in
 	return parameters;
 }
 
-Targets *GameScript::BestAC(Scriptable* /*Sender*/, Targets *parameters, int ga_flags)
+//This filter works only on the Party - silly restriction, but the dataset expects this
+Targets *GameScript::StrongestOfMale(Scriptable* Sender, Targets *parameters, int ga_flags)
 {
-	targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
-	}
-	Scriptable *scr=t->actor;
-	Actor *actor=(Actor *) scr;
-	int bestac=actor->GetStat(IE_ARMORCLASS);
-	// assignment in while
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		actor = (Actor *) t->actor;
-		int ac=actor->GetStat(IE_ARMORCLASS);
-		if (bestac<ac) {
-			bestac=ac;
-			scr=t->actor;
-		}
-	}
-
-	parameters->Clear();
-	parameters->AddTarget(scr, 0, ga_flags);
-	return parameters;
-}
-
-/*no idea why this object exists since the gender could be filtered easier*/
-Targets *GameScript::StrongestOfMale(Scriptable* /*Sender*/, Targets *parameters, int ga_flags)
-{
-	targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
-	}
-	int pos=-1;
-	int worsthp=-1;
-	Scriptable *scr = NULL;
-	//assignment intentional
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		Actor *actor = (Actor *) t->actor;
-		if (actor->GetStat(IE_SEX)!=SEX_MALE) continue;
-		int hp=actor->GetStat(IE_HITPOINTS);
-		if ((pos==-1) || (worsthp<hp)) {
-			worsthp=hp;
-			scr=t->actor;
-		}
-	}
-	parameters->Clear();
-	if (scr) {
-		parameters->AddTarget(scr, 0, ga_flags);
-	}
-	return parameters;
-}
-
-Targets *GameScript::StrongestOf(Scriptable* /*Sender*/, Targets *parameters, int ga_flags)
-{
-	targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
-	}
-	Scriptable *scr=t->actor;
-	Actor *actor=(Actor *) scr;
-	int besthp=actor->GetStat(IE_HITPOINTS);
-	// assignment in while
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		actor = (Actor *) t->actor;
-		int hp=actor->GetStat(IE_HITPOINTS);
-		if (besthp<hp) {
-			besthp=hp;
-			scr=t->actor;
-		}
-	}
-	parameters->Clear();
-	parameters->AddTarget(scr, 0, ga_flags);
-	return parameters;
-}
-
-Targets *GameScript::WeakestOf(Scriptable* /*Sender*/, Targets *parameters, int ga_flags)
-{
-	targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
-	}
-	Scriptable *scr=t->actor;
-	Actor *actor=(Actor *) scr;
-	int worsthp=actor->GetStat(IE_HITPOINTS);
-	// assignment in while
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		actor = (Actor *) t->actor;
-		int hp=actor->GetStat(IE_HITPOINTS);
-		if (worsthp>hp) {
-			worsthp=hp;
-			scr=t->actor;
-		}
-	}
-	parameters->Clear();
-	parameters->AddTarget(scr, 0, ga_flags);
-	return parameters;
-}
-
-Targets *GameScript::WorstAC(Scriptable* /*Sender*/, Targets *parameters, int ga_flags)
-{
-	targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
-	}
-	Scriptable *scr=t->actor;
-	Actor *actor=(Actor *) scr;
-	int worstac=actor->GetStat(IE_ARMORCLASS);
-	// assignment in while
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		actor = (Actor *) t->actor;
-		int ac=actor->GetStat(IE_ARMORCLASS);
-		if (worstac>ac) {
-			worstac=ac;
-			scr=t->actor;
-		}
-	}
-	parameters->Clear();
-	parameters->AddTarget(scr, 0, ga_flags);
-	return parameters;
-}
-
-Targets *GameScript::MostDamagedOf(Scriptable* Sender, Targets *parameters, int ga_flags)
-{
-	//Original engines restrict this to the PCs...
-	/*targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
-	}
-	Scriptable *scr = t->actor;
-	Actor *actor=(Actor *) scr;
-	int worsthp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
-	// assignment in while
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		actor = (Actor *) t->actor;
-		int hp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
-		if (worsthp>hp) {
-			worsthp=hp;
-			scr=t->actor;
-		}
-	}
-	parameters->Clear();
-	parameters->AddTarget(scr, 0, ga_flags);
-	return parameters;*/
-	Map* area = Sender->GetCurrentArea() ;
+	Map* area = Sender->GetCurrentArea();
 	Game *game = core->GetGame();
-	Scriptable* scr = NULL ;
-	int worsthp = 0xffff ;
+	Scriptable* scr = NULL;
+	int besthp = 0;
+	int i = game->GetPartySize(false);
+	while (i--) {
+		Actor *actor = game->GetPC(i, false);
+		if (actor->GetStat(IE_SEX)!=SEX_MALE) continue;
+		if(actor->GetCurrentArea() == area) {
+			int hp = actor->GetStat(IE_HITPOINTS);
+			if (!scr || besthp<hp) {
+				besthp=hp;
+				scr=actor;
+			}
+		}
+	}
+	parameters->Clear();
+	parameters->AddTarget(scr, 0, ga_flags);
+	return parameters;
+}
+
+//This filter works only on the Party - silly restriction, but the dataset expects this
+Targets *GameScript::StrongestOf(Scriptable* Sender, Targets *parameters, int ga_flags)
+{
+	Map* area = Sender->GetCurrentArea();
+	Game *game = core->GetGame();
+	Scriptable* scr = NULL;
+	int besthp = 0;
 	int i = game->GetPartySize(false);
 	while (i--) {
 		Actor *actor = game->GetPC(i, false);
 		if(actor->GetCurrentArea() == area) {
-			int hp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
-			if (worsthp>hp) {
+			int hp = actor->GetStat(IE_HITPOINTS);
+			if (!scr || besthp<hp) {
+				besthp=hp;
+				scr=actor;
+			}
+		}
+	}
+	parameters->Clear();
+	parameters->AddTarget(scr, 0, ga_flags);
+	return parameters;
+}
+
+//This filter works only on the Party - silly restriction, but the dataset expects this
+Targets *GameScript::WeakestOf(Scriptable* Sender, Targets *parameters, int ga_flags)
+{
+	Map* area = Sender->GetCurrentArea();
+	Game *game = core->GetGame();
+	Scriptable* scr = NULL;
+	int worsthp = 0;
+	int i = game->GetPartySize(false);
+	while (i--) {
+		Actor *actor = game->GetPC(i, false);
+		if(actor->GetCurrentArea() == area) {
+			int hp = actor->GetStat(IE_HITPOINTS);
+			if (!scr || worsthp>hp) {
 				worsthp=hp;
 				scr=actor;
 			}
@@ -690,23 +616,93 @@ Targets *GameScript::MostDamagedOf(Scriptable* Sender, Targets *parameters, int
 	parameters->AddTarget(scr, 0, ga_flags);
 	return parameters;
 }
-Targets *GameScript::LeastDamagedOf(Scriptable* /*Sender*/, Targets *parameters, int ga_flags)
+
+//This filter works only on the Party - silly restriction, but the dataset expects this
+Targets *GameScript::BestAC(Scriptable* Sender, Targets *parameters, int ga_flags)
 {
-	targetlist::iterator m;
-	const targettype *t = parameters->GetFirstTarget(m, ST_ACTOR);
-	if (!t) {
-		return parameters;
+	Map* area = Sender->GetCurrentArea();
+	Game *game = core->GetGame();
+	Scriptable* scr = NULL;
+	int bestac = 0;
+	int i = game->GetPartySize(false);
+	while (i--) {
+		Actor *actor = game->GetPC(i, false);
+		if(actor->GetCurrentArea() == area) {
+			int ac = actor->AC.GetTotal();
+			if (!scr || bestac>ac) {
+				bestac=ac;
+				scr=actor;
+			}
+		}
 	}
-	Scriptable *scr = t->actor;
-	Actor *actor = (Actor *) scr;
-	int besthp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
-	// assignment in while
-	while ( (t = parameters->GetNextTarget(m, ST_ACTOR) ) ) {
-		actor = (Actor *) t->actor;
-		int hp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
-		if (besthp<hp) {
-			besthp=hp;
-			scr=t->actor;
+	parameters->Clear();
+	parameters->AddTarget(scr, 0, ga_flags);
+	return parameters;
+}
+
+//This filter works only on the Party - silly restriction, but the dataset expects this
+Targets *GameScript::WorstAC(Scriptable* Sender, Targets *parameters, int ga_flags)
+{
+	Map* area = Sender->GetCurrentArea();
+	Game *game = core->GetGame();
+	Scriptable* scr = NULL;
+	int worstac = 0;
+	int i = game->GetPartySize(false);
+	while (i--) {
+		Actor *actor = game->GetPC(i, false);
+		if(actor->GetCurrentArea() == area) {
+			int ac = actor->AC.GetTotal();
+			if (!scr || worstac<ac) {
+				worstac=ac;
+				scr=actor;
+			}
+		}
+	}
+	parameters->Clear();
+	parameters->AddTarget(scr, 0, ga_flags);
+	return parameters;
+}
+
+//This filter works only on the Party - silly restriction, but the dataset expects this
+Targets *GameScript::MostDamagedOf(Scriptable* Sender, Targets *parameters, int ga_flags)
+{
+	Map* area = Sender->GetCurrentArea();
+	Game *game = core->GetGame();
+	Scriptable* scr = NULL;
+	int worsthp = 0;
+	int i = game->GetPartySize(false);
+	while (i--) {
+		Actor *actor = game->GetPC(i, false);
+		if(actor->GetCurrentArea() == area) {
+			int hp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
+			if (!scr || hp>worsthp) {
+				worsthp=hp;
+				scr=actor;
+			}
+		}
+	}
+	parameters->Clear();
+	parameters->AddTarget(scr, 0, ga_flags);
+	return parameters;
+}
+
+//This filter works only on the Party - silly restriction, but the dataset expects this
+//For example the beholder01 script
+Targets *GameScript::LeastDamagedOf(Scriptable* Sender, Targets *parameters, int ga_flags)
+{
+	Map* area = Sender->GetCurrentArea();
+	Game *game = core->GetGame();
+	Scriptable* scr = NULL;
+	int besthp = 0;
+	int i = game->GetPartySize(false);
+	while (i--) {
+		Actor *actor = game->GetPC(i, false);
+		if(actor->GetCurrentArea() == area) {
+			int hp=actor->GetStat(IE_MAXHITPOINTS)-actor->GetBase(IE_HITPOINTS);
+			if (!scr || besthp<hp) {
+				besthp=hp;
+				scr=actor;
+			}
 		}
 	}
 	parameters->Clear();
@@ -1060,16 +1056,34 @@ inline bool idclass(Actor *actor, int parameter, bool iwd2) {
 		classes = all_bg_classes;
 	}
 
+	// FIXME: unhardcode this ugly mess
+	// in IWD2, mage_all = sorcerer or wizard
+	// fighter_all = fighter, paladin or ranger (but not monk)
+	// cleric_all = druid or cleric
+	// thief_all = thief
+	// bard_all = bard
+	// paladin_all = paladin
+	// druid_all = druid
+	// ranger_all = ranger
+
 	// we got one of the *_ALL values
 	if (parameter == classes[4]) {
 		// MAGE_ALL (also sorcerers)
 		value = actor->GetMageLevel() + actor->GetSorcererLevel();
 	} else if (parameter == classes[3]) {
 		// FIGHTER_ALL (also monks)
-		value = actor->GetFighterLevel() + actor->GetMonkLevel();
+		if (iwd2) {
+			value = actor->GetFighterLevel() + actor->GetPaladinLevel() + actor->GetRangerLevel();
+		} else {
+			value = actor->GetFighterLevel() + actor->GetMonkLevel();
+		}
 	} else if (parameter == classes[1]) {
 		// CLERIC_ALL
-		value = actor->GetClericLevel();
+		if (iwd2) {
+			value = actor->GetClericLevel() + actor->GetDruidLevel();
+		} else {
+			value = actor->GetClericLevel();
+		}
 	} else if (parameter == classes[7]) {
 		// THIEF_ALL
 		value = actor->GetThiefLevel();
@@ -1162,3 +1176,5 @@ int GameScript::ID_Specific(Actor *actor, int parameter)
 	int value = actor->GetStat(IE_SPECIFIC);
 	return parameter==value;
 }
+
+}
