@@ -35,6 +35,8 @@
 #include "GameScript/Matching.h"
 #include "Scriptable/Actor.h"
 
+namespace GemRB {
+
 static const int StatValues[9]={
 IE_EA, IE_FACTION, IE_TEAM, IE_GENERAL, IE_RACE, IE_CLASS, IE_SPECIFIC, 
 IE_SEX, IE_ALIGNMENT };
@@ -51,12 +53,26 @@ IniSpawn::IniSpawn(Map *owner)
 	eventspawns = NULL;
 	eventcount = 0;
 	last_spawndate = 0;
+	//high detail level by default
+	detail_level = 2;
+	core->GetDictionary()->Lookup("Detail Level", detail_level);
 }
 
 IniSpawn::~IniSpawn()
 {
 	if (eventspawns) {
 		delete[] eventspawns;
+		eventspawns = NULL;
+	}
+
+	if (Locals) {
+		delete[] Locals;
+		Locals = NULL;
+	}
+
+	if (NamelessVar) {
+		delete[] NamelessVar;
+		NamelessVar = NULL;
 	}
 }
 
@@ -72,8 +88,7 @@ static Holder<DataFileMgr> GetIniFile(const ieResRef DefaultArea)
 		return NULL;
 	}
 	if (!core->IsAvailable( IE_INI_CLASS_ID )) {
-		printStatus( "ERROR", LIGHT_RED );
-		printMessage( "IniSpawn","No INI Importer Available.\n",LIGHT_RED );
+		Log(ERROR, "IniSpawn", "No INI Importer Available.");
 		return NULL;
 	}
 
@@ -136,7 +151,7 @@ static inline void GetElements(const char *s, ieVariable *storage, int count)
 // BINARY_MORE 10          //left has more bits than right
 // BINARY_LESS 11          //left has less bits than right
 
-int IniSpawn::GetDiffMode(const char *keyword)
+int IniSpawn::GetDiffMode(const char *keyword) const
 {
 	if (!keyword) return NO_OPERATION; //-1
 	if (keyword[0]==0) return NO_OPERATION; //-1
@@ -144,36 +159,72 @@ int IniSpawn::GetDiffMode(const char *keyword)
 	if (!stricmp(keyword,"equal_to") ) return EQUALS; // 1
 	if (!stricmp(keyword,"less_than") ) return LESS_THAN; // 2
 	if (!stricmp(keyword,"greater_than") ) return GREATER_THAN; //3
-	if (!stricmp(keyword,"greater_or_equal_to") ) return GREATER_THAN; //4 (gemrb ext)
+	if (!stricmp(keyword,"greater_or_equal_to") ) return GREATER_OR_EQUALS; //4 (gemrb ext)
 	if (!stricmp(keyword,"not_equal_to") ) return NOT_EQUALS; //5
 	return NO_OPERATION;
 }
 
-//unimplemented tags:
-// check_crowd
-// good_mod, law_mod, lady_mod, murder_mod
+//unimplemented tags (* marks partially implemented, # marks not working in original either):
+//*check_crowd
 // control_var
 // spec_area
-// death_faction
-// death_team
+//*death_faction
+//*death_team
 // check_by_view_port
-// do_not_spawn
-// time_of_day
+//*do_not_spawn
 // hold_selected_point_key
 // inc_spawn_point_index
-// find_safest_point
-// exit
-// spawn_time_of_day
+//*find_safest_point
+//#spawn_time_of_day
+// exit - similar to enter[spawn], this is a spawn branch type (on exiting an area?)
 // PST only
-// auto_buddy
-// detail_level
-void IniSpawn::ReadCreature(DataFileMgr *inifile, const char *crittername, CritterEntry &critter)
+//*auto_buddy
+//*detail_level
+void IniSpawn::ReadCreature(DataFileMgr *inifile, const char *crittername, CritterEntry &critter) const
 {
 	const char *s;
 	int ps;
 	
 	memset(&critter,0,sizeof(critter));
 
+	//first assume it is a simple numeric value
+	critter.TimeOfDay = (ieDword) inifile->GetKeyAsInt(crittername,"time_of_day", 0xffffffff);
+
+	//at this point critter.TimeOfDay is usually 0xffffffff
+	s = inifile->GetKeyAsString(crittername,"time_of_day",NULL);
+	if (s && strlen(s)>=24) {
+		ieDword value = 0;
+		ieDword j = 1;
+		for(int i=0;i<24 && s[i];i++) {
+			if (s[i]=='0' || s[i]=='o') value |= j;
+			j<<=1;
+		}
+		//turn off individual bits marked by a 24 long string scheduling
+		//example: '0000xxxxxxxxxxxxxxxx00000000'
+		critter.TimeOfDay^=value;
+	}
+
+	if (inifile->GetKeyAsBool(crittername,"do_not_spawn",false)) {
+		//if the do not spawn flag is true, ignore this entry
+		return;
+	}
+
+	s = inifile->GetKeyAsString(crittername,"detail_level",NULL);
+	if (s) {
+		ieDword level;
+
+		switch(s[0]) {
+			case 'h': case 'H': level = 2; break;
+			case 'm': case 'M': level = 1; break;
+			default: level = 0; break;
+		}
+		//If the detail level is lower than this creature's detail level,
+		//skip this entry, creature_count is 0, so it will be ignored at evaluation of the spawn
+		if (level>detail_level) {
+			return;
+		}
+	}
+
 	//all specvars are using global, but sometimes it is explicitly given
 	s = inifile->GetKeyAsString(crittername,"spec_var",NULL);
 	if (s) {
@@ -207,7 +258,7 @@ void IniSpawn::ReadCreature(DataFileMgr *inifile, const char *crittername, Critt
 		critter.CreFile=new ieResRef[critter.creaturecount];
 		GetElements(s, critter.CreFile, critter.creaturecount);
 	} else {
-		printMessage("IniSpawn", "Invalid spawn entry: %s\n", LIGHT_RED, crittername);
+		Log(ERROR, "IniSpawn", "Invalid spawn entry: %s", crittername);
 	}
 
 	s = inifile->GetKeyAsString(crittername,"point_select",NULL);
@@ -231,16 +282,25 @@ void IniSpawn::ReadCreature(DataFileMgr *inifile, const char *crittername, Critt
 		}
 		//parse the selected spawnpoint
 		int x,y,o;
+		if (sscanf(s,"[%d,%d:%d]", &x, &y, &o)==3) {
+			critter.SpawnPoint.x=(short) x;
+			critter.SpawnPoint.y=(short) y;
+			critter.Orientation=o;
+		} else
 		if (sscanf(s,"[%d.%d:%d]", &x, &y, &o)==3) {
 			critter.SpawnPoint.x=(short) x;
 			critter.SpawnPoint.y=(short) y;
 			critter.Orientation=o;
-		} else {
-			if (sscanf(s,"[%d.%d]", &x, &y)==2) {
-				critter.SpawnPoint.x=(short) x;
-				critter.SpawnPoint.y=(short) y;
-				critter.Orientation=core->Roll(1,16,-1);
-			}
+		} else
+		if (sscanf(s,"[%d,%d]", &x, &y)==2) {
+			critter.SpawnPoint.x=(short) x;
+			critter.SpawnPoint.y=(short) y;
+			critter.Orientation=core->Roll(1,16,-1);
+		} else
+		if (sscanf(s,"[%d.%d]", &x, &y)==2) {
+			critter.SpawnPoint.x=(short) x;
+			critter.SpawnPoint.y=(short) y;
+			critter.Orientation=core->Roll(1,16,-1);
 		}
 	}
 	
@@ -402,6 +462,36 @@ void IniSpawn::ReadCreature(DataFileMgr *inifile, const char *crittername, Critt
 	if (inifile->GetKeyAsBool(crittername,"death_scriptname",false)) {
 		critter.Flags|=CF_DEATHVAR;
 	}
+	if (inifile->GetKeyAsBool(crittername,"death_faction",false)) {
+		critter.Flags|=CF_FACTION;
+	}
+	if (inifile->GetKeyAsBool(crittername,"death_team",false)) {
+		critter.Flags|=CF_TEAM;
+	}
+	ps = inifile->GetKeyAsInt(crittername,"good_mod",0);
+	if (ps) {
+		critter.Flags|=CF_GOOD;
+		critter.DeathCounters[DC_GOOD] = ps;
+	}
+	ps = inifile->GetKeyAsInt(crittername,"law_mod",0);
+	if (ps) {
+		critter.Flags|=CF_LAW;
+		critter.DeathCounters[DC_LAW] = ps;
+	}
+	ps = inifile->GetKeyAsInt(crittername,"lady_mod",0);
+	if (ps) {
+		critter.Flags|=CF_LADY;
+		critter.DeathCounters[DC_LADY] = ps;
+	}
+	ps = inifile->GetKeyAsInt(crittername,"murder_mod",0);
+	if (ps) {
+		critter.Flags|=CF_MURDER;
+		critter.DeathCounters[DC_MURDER] = ps;
+	}
+	if(inifile->GetKeyAsBool(crittername,"auto_buddy", false)) {
+		critter.Flags|=CF_BUDDY;
+	}
+
 	//don't spawn when spawnpoint is visible
 	if (inifile->GetKeyAsBool(crittername,"ignore_can_see",false)) {
 		critter.Flags|=CF_IGNORECANSEE;
@@ -430,7 +520,7 @@ void IniSpawn::ReadCreature(DataFileMgr *inifile, const char *crittername, Critt
 	}
 }
 
-void IniSpawn::ReadSpawnEntry(DataFileMgr *inifile, const char *entryname, SpawnEntry &entry)
+void IniSpawn::ReadSpawnEntry(DataFileMgr *inifile, const char *entryname, SpawnEntry &entry) const
 {
 	const char *s;
 	
@@ -450,7 +540,7 @@ void IniSpawn::ReadSpawnEntry(DataFileMgr *inifile, const char *entryname, Spawn
 }
 
 /* set by action */
-void IniSpawn::SetNamelessDeath(const ieResRef area, Point &pos, ieDword state)
+void IniSpawn::SetNamelessDeath(const ieResRef area, Point &pos, ieDword state) 
 {
 	strnuprcpy(NamelessSpawnArea, area, 8);
 	NamelessSpawnPoint = pos;
@@ -505,6 +595,12 @@ void IniSpawn::InitSpawn(const ieResRef DefaultArea)
 	if (s) {
 		ReadSpawnEntry(inifile.get(), s, enterspawn);
 	}
+
+	s = inifile->GetKeyAsString("spawn_main","exit",NULL);
+	if (s) {
+		ReadSpawnEntry(inifile.get(), s, exitspawn);
+	}
+
 	s = inifile->GetKeyAsString("spawn_main","events",NULL);
 	if (s) {
 		eventcount = CountElements(s,',');
@@ -553,7 +649,7 @@ void IniSpawn::RespawnNameless()
 	}
 }
 
-void IniSpawn::SpawnCreature(CritterEntry &critter)
+void IniSpawn::SpawnCreature(CritterEntry &critter) const
 {
 	if (!critter.creaturecount) {
 		return;
@@ -640,7 +736,7 @@ void IniSpawn::SpawnCreature(CritterEntry &critter)
 	}
 
 	SetVariable(map, critter.SpecVar, critter.SpecContext, specvar+(ieDword) critter.SpecVarInc);
-	map->AddActor(cre);
+	map->AddActor(cre, true);
 	for (x=0;x<9;x++) {
 		if (critter.SetSpec[x]) {
 			cre->SetBase(StatValues[x], critter.SetSpec[x]);
@@ -651,6 +747,43 @@ void IniSpawn::SpawnCreature(CritterEntry &critter)
 	if (critter.ScriptName[0]) {
 		cre->SetScriptName(critter.ScriptName);
 	}
+	//increases death variable
+	if (critter.Flags&CF_DEATHVAR) {
+		cre->AppearanceFlags|=APP_DEATHVAR;
+	}
+	//increases faction specific variable
+	if (critter.Flags&CF_FACTION) {
+		cre->AppearanceFlags|=APP_FACTION;
+	}
+	//increases team specific variable
+	if (critter.Flags&CF_TEAM) {
+		cre->AppearanceFlags|=APP_TEAM;
+	}
+	//increases good variable
+	if (critter.Flags&CF_GOOD) {
+		cre->DeathCounters[DC_GOOD] = critter.DeathCounters[DC_GOOD];
+		cre->AppearanceFlags|=APP_GOOD;
+	}
+	//increases law variable
+	if (critter.Flags&CF_LAW) {
+		cre->DeathCounters[DC_LAW] = critter.DeathCounters[DC_LAW];
+		cre->AppearanceFlags|=APP_LAW;
+	}
+	//increases lady variable
+	if (critter.Flags&CF_LADY) {
+		cre->DeathCounters[DC_LADY] = critter.DeathCounters[DC_LADY];
+		cre->AppearanceFlags|=APP_LADY;
+	}
+	//increases murder variable
+	if (critter.Flags&CF_MURDER) {
+		cre->DeathCounters[DC_MURDER] = critter.DeathCounters[DC_MURDER];
+		cre->AppearanceFlags|=APP_MURDER;
+	}
+	//triggers help from same group
+	if (critter.Flags&CF_BUDDY) {
+		cre->AppearanceFlags|=APP_BUDDY;
+	}
+
 	if (critter.OverrideScript[0]) {
 		cre->SetScript(critter.OverrideScript, SCR_OVERRIDE);
 	}
@@ -677,6 +810,15 @@ void IniSpawn::SpawnCreature(CritterEntry &critter)
 	}
 }
 
+bool IniSpawn::Schedule(ieDword appearance, ieDword gametime) const
+{
+	ieDword bit = 1<<((gametime/AI_UPDATE_TIME)%7200/300);
+	if (appearance & bit) {
+		return true;
+	}
+	return false;
+}
+
 void IniSpawn::SpawnGroup(SpawnEntry &event)
 {
 	if (!event.critters) {
@@ -692,6 +834,9 @@ void IniSpawn::SpawnGroup(SpawnEntry &event)
 	
 	for(int i=0;i<event.crittercount;i++) {
 		CritterEntry* critter = event.critters+i;
+		if (!Schedule(critter->TimeOfDay, last_spawndate) ) {
+			continue;
+		}
 		for(int j=0;j<critter->SpawnCount;j++) {
 			SpawnCreature(*critter);
 		}
@@ -708,6 +853,12 @@ void IniSpawn::InitialSpawn()
 	}
 }
 
+//FIXME:call this at the right time (this feature is not explored yet, and unused in original dataset)
+void IniSpawn::ExitSpawn()
+{
+	SpawnGroup(exitspawn);
+}
+
 //checks if a respawn event occurred
 void IniSpawn::CheckSpawn()
 {
@@ -715,3 +866,6 @@ void IniSpawn::CheckSpawn()
 		SpawnGroup(eventspawns[i]);
 	}
 }
+
+
+}
