@@ -139,6 +139,13 @@ struct TimeStruct {
 	unsigned int attack_round_size;
 };
 
+struct TLKEncodingStruct
+{
+	std::string encoding;
+	bool multibyte;
+	bool zerospace;
+};
+
 struct SpellDescType {
 	ieResRef resref;
 	ieStrRef value;
@@ -146,6 +153,7 @@ struct SpellDescType {
 #define SP_IDENTIFY  1      //any spell that cannot be cast from the menu
 #define SP_SILENCE   2      //any spell that can be cast in silence
 #define SP_SURGE     4      //any spell that cannot be cast during a wild surge
+#define SP_REST      8      //any spell that is cast upon rest if memorized
 
 struct SurgeSpell {
 	ieResRef spell;
@@ -177,9 +185,11 @@ public:
 
 // Colors of modal window shadow
 // !!! Keep these synchronized with GUIDefines.py !!!
-#define MODAL_SHADOW_NONE	0
-#define MODAL_SHADOW_GRAY	1
-#define MODAL_SHADOW_BLACK	2
+enum MODAL_SHADOW {
+	MODAL_SHADOW_NONE = 0,
+	MODAL_SHADOW_GRAY,
+	MODAL_SHADOW_BLACK
+};
 
 #define WINDOW_INVALID   -1
 #define WINDOW_INVISIBLE 0
@@ -306,6 +316,7 @@ private:
 	EventMgr * evntmgr;
 	Holder<WindowMgr> windowmgr;
 	Window* ModalWindow;
+	MODAL_SHADOW modalShadow;
 	char WindowPack[10];
 	Holder<ScriptEngine> guiscript;
 	SaveGameIterator *sgiterator;
@@ -330,7 +341,6 @@ private:
 	ieResRef ScrollCursorBam;
 	ieResRef GroundCircleBam[MAX_CIRCLE_SIZE];
 	int GroundCircleScale[MAX_CIRCLE_SIZE];
-	ieDword FullScreen;
 	ieResRef TooltipFont;
 	ieResRef TooltipBackResRef;
 	ieResRef MovieFont;
@@ -371,7 +381,7 @@ private:
 	KeyMap *keymap;
 	std::string Encoding;
 public:
-	std::string TLKEncoding;
+	TLKEncodingStruct TLKEncoding;
 	Holder<StringMgr> strings;
 	GlobalTimer * timer;
 	Palette *InfoTextPalette;
@@ -470,7 +480,7 @@ public:
 	/** Set a Window Visible Flag */
 	int SetVisible(unsigned short WindowIndex, int visible);
 	/** Show a Window in Modal Mode */
-	int ShowModal(unsigned short WindowIndex, int Shadow);
+	int ShowModal(unsigned short WindowIndex, MODAL_SHADOW Shadow);
 	/** Set the Status of a Control in a Window */
 	int SetControlStatus(unsigned short WindowIndex, unsigned short ControlIndex, unsigned long Status);
 	/** Get a Window from the Loaded Window List */
@@ -487,8 +497,6 @@ public:
 	void RedrawControls(const char *varname, unsigned int value);
 	/** Popup the Console */
 	void PopupConsole();
-	/** Draws the Console */
-	void DrawConsole();
 	/** Get the SaveGameIterator */
 	SaveGameIterator * GetSaveGameIterator() const;
 	/** Get the Variables Dictionary */
@@ -616,6 +624,7 @@ public:
 	void SetDraggedPortrait(int dp, int cursor=-1);
 	CREItem *ReadItem(DataStream *str);
 	CREItem *ReadItem(DataStream *str, CREItem *itm);
+	void SanitizeItem(CREItem *item) const;
 	bool ResolveRandomItem(CREItem *itm);
 	ieStrRef GetRumour(const ieResRef resname);
 	Container *GetCurrentContainer();
@@ -793,6 +802,7 @@ public:
 	int GUIEnhancements;
 	bool KeepCache;
 	bool MultipleQuickSaves;
+
 	Variables *plugin_flags;
 	/** The Main program loop */
 	void Main(void);
