@@ -30,6 +30,7 @@
 #include "DisplayMessage.h"
 #include "Game.h"
 #include "GameData.h"
+#include "GameScript/GSUtils.h"
 #include "Interface.h"
 #include "Item.h"
 #include "Map.h"
@@ -57,7 +58,6 @@ static int SLOT_ARMOR = -1;
 
 //IWD2 style slots
 static bool IWD2 = false;
-static int MagicBit = 0;
 
 static void InvalidSlot(int slot)
 {
@@ -74,7 +74,7 @@ inline Item *Inventory::GetItemPointer(ieDword slot, CREItem *&item) const
 	return gamedata->GetItem(item->ItemResRef);
 }
 
-void Inventory::Init(int mb)
+void Inventory::Init()
 {
 	SLOT_MAGIC=-1;
 	SLOT_FIST=-1;
@@ -88,7 +88,6 @@ void Inventory::Init(int mb)
 	SLOT_ARMOR=-1;
 	//TODO: set this correctly
 	IWD2 = false;
-	MagicBit = mb;
 }
 
 Inventory::Inventory()
@@ -157,52 +156,10 @@ CREItem *Inventory::GetItem(unsigned int slot)
 	return item;
 }
 
-//Make sure the item attributes are valid
-//we don't update all flags here because some need to be set later (like
-//unmovable items in containers (e.g. the bg2 protal key) so that they
-//can actually be picked up)
-void Inventory::SanitizeItem(CREItem *item)
-{
-	Item *itm = gamedata->GetItem(item->ItemResRef, true);
-	if (itm) {
-		//This hack sets the charge counters for non-rechargeable items,
-		//if their charge is zero
-		for (int i=0;i<3;i++) {
-			if (item->Usages[i]) {
-				continue;
-			}
-			ITMExtHeader *h = itm->GetExtHeader(i);
-			if (h && !(h->RechargeFlags&IE_ITEM_RECHARGE)) {
-				//HACK: the original (bg2) allows for 0 charged gems
-				if (h->Charges) {
-					item->Usages[i] = h->Charges;
-				} else {
-					item->Usages[i] = 1;
-				}
-			}
-		}
-
-		//auto identify basic items
-		if (!itm->LoreToID) {
-			item->Flags |= IE_INV_ITEM_IDENTIFIED;
-		}
-
-		//if item is stacked mark it as so
-		if (itm->MaxStackAmount) {
-			item->Flags |= IE_INV_ITEM_STACKED;
-		}
-
-		item->MaxStackAmount = itm->MaxStackAmount;
-
-		gamedata->FreeItem( itm, item->ItemResRef, false );
-	}
-}
-
 void Inventory::AddItem(CREItem *item)
 {
 	if (!item) return; //invalid items get no slot
 	Slots.push_back(item);
-	SanitizeItem(item);
 }
 
 void Inventory::CalculateWeight() const
@@ -219,35 +176,10 @@ void Inventory::CalculateWeight() const
 		if (slot->Weight == -1) {
 			Item *itm = gamedata->GetItem(slot->ItemResRef, true);
 			if (itm) {
-				//simply adding the item flags to the slot
-				slot->Flags |= (itm->Flags<<8);
-				//some slot flags might be affected by the item flags
-				if (!(slot->Flags & IE_INV_ITEM_CRITICAL)) {
-					slot->Flags |= IE_INV_ITEM_DESTRUCTIBLE;
-				}
-				//this is for converting IWD items magic flag
-				if (MagicBit) {
-					if (slot->Flags&IE_INV_ITEM_UNDROPPABLE) {
-						slot->Flags|=IE_INV_ITEM_MAGICAL;
-						slot->Flags&=~IE_INV_ITEM_UNDROPPABLE;
-					}
-				}
-
-				if (!(slot->Flags & IE_INV_ITEM_MOVABLE)) {
-					slot->Flags |= IE_INV_ITEM_UNDROPPABLE;
-				}
-
-				if (slot->Flags & IE_INV_ITEM_STOLEN2) {
-					slot->Flags |= IE_INV_ITEM_STOLEN;
-				}
-
 				slot->Weight = itm->Weight;
-
 				gamedata->FreeItem( itm, slot->ItemResRef, false );
-			}
-			else {
-				Log(ERROR, "Inventory", "Invalid item: %s!",
-					slot->ItemResRef);
+			} else {
+				Log(ERROR, "Inventory", "Invalid item: %s!", slot->ItemResRef);
 				slot->Weight = 0;
 			}
 		} else {
@@ -272,7 +204,7 @@ void Inventory::AddSlotEffects(ieDword index)
 	ItemExcl|=itm->ItemExcl;
 	ieDword pos = itm->ItemType/32;
 	ieDword bit = itm->ItemType%32;
-	if (pos<4) {
+	if (pos<8) {
 		ItemTypes[pos]|=1<<bit;
 	}
 
@@ -424,6 +356,7 @@ void Inventory::KillSlot(unsigned int index)
 	}
 	ItemExcl &= ~itm->ItemExcl;
 	int eqslot = GetEquippedSlot();
+	ieDword equip;
 
 	switch (effect) {
 		case SLOT_EFFECT_LEFT:
@@ -435,9 +368,11 @@ void Inventory::KillSlot(unsigned int index)
 				if (Equipped < 0) {
 					//always get the projectile weapon header (this quiver was equipped)
 					ITMExtHeader *header = itm->GetWeaponHeader(true);
-					Equipped = FindRangedProjectile(header->ProjectileQualifier);
-					if (Equipped!=IW_NO_EQUIPPED) {
-						EquipItem(GetEquippedSlot());
+					//remove potential launcher effects too
+					RemoveSlotEffects(FindTypedRangedWeapon(header->ProjectileQualifier));
+					equip = FindRangedProjectile(header->ProjectileQualifier);
+					if (equip != IW_NO_EQUIPPED) {
+						EquipItem(GetWeaponSlot(equip));
 					} else {
 						EquipBestWeapon(EQUIP_MELEE);
 					}
@@ -448,9 +383,9 @@ void Inventory::KillSlot(unsigned int index)
 		case SLOT_EFFECT_MAGIC:
 		case SLOT_EFFECT_MELEE:
 			// reset Equipped if it was the removed item
-			if (eqslot == (int)index)
-				Equipped = IW_NO_EQUIPPED;
-			else if (Equipped < 0) {
+			if (eqslot == (int)index) {
+				SetEquippedSlot(IW_NO_EQUIPPED, 0);
+			} else if (Equipped < 0) {
 				//always get the projectile weapon header (this is a bow, because Equipped is negative)
 				ITMExtHeader *header = itm->GetWeaponHeader(true);
 				if (header) {
@@ -458,13 +393,15 @@ void Inventory::KillSlot(unsigned int index)
 					int type = header->ProjectileQualifier;
 					int weaponslot = FindTypedRangedWeapon(type);
 					CREItem *item2 = Slots[weaponslot];
-					if (item2) {
+					if (weaponslot == SLOT_FIST) { // a ranged weapon was not found - freshly unequipped
+						EquipBestWeapon(EQUIP_MELEE);
+					} else if (item2) {
 						Item *itm2 = gamedata->GetItem(item2->ItemResRef, true);
 						if (itm2) {
 							if (type == header->ProjectileQualifier) {
-								Equipped = FindRangedProjectile(header->ProjectileQualifier);
-								if (Equipped!=IW_NO_EQUIPPED) {
-									EquipItem(GetEquippedSlot());
+								equip = FindRangedProjectile(header->ProjectileQualifier);
+								if (equip != IW_NO_EQUIPPED) {
+									EquipItem(GetWeaponSlot(equip));
 								} else {
 									EquipBestWeapon(EQUIP_MELEE);
 								}
@@ -577,11 +514,11 @@ CREItem *Inventory::RemoveItem(unsigned int slot, unsigned int count)
 }
 
 //flags set disable item transfer
-//except for undroppable and equipped, which are opposite (and shouldn't be set)
+//except for undroppable which is opposite (and shouldn't be set)
 int Inventory::RemoveItem(const char *resref, unsigned int flags, CREItem **res_item, int count)
 {
 	size_t slot = Slots.size();
-	unsigned int mask = (flags^(IE_INV_ITEM_UNDROPPABLE|IE_INV_ITEM_EQUIPPED));
+	unsigned int mask = (flags^IE_INV_ITEM_UNDROPPABLE);
 	if (core->HasFeature(GF_NO_DROP_CAN_MOVE) ) {
 		mask &= ~IE_INV_ITEM_UNDROPPABLE;
 	}
@@ -618,8 +555,6 @@ void Inventory::SetSlotItem(CREItem* item, unsigned int slot)
 		delete Slots[slot];
 	}
 
-	SanitizeItem(item);
-
 	Slots[slot] = item;
 
 	//update the action bar next time
@@ -716,8 +651,8 @@ int Inventory::AddStoreItem(STOItem* item, int action)
 		memcpy( temp, item, sizeof( CREItem ) );
 		//except the Expired flag
 		temp->Expired=0;
-		if (action==STA_STEAL) {
-			temp->Flags |= IE_INV_ITEM_STOLEN;
+		if (action==STA_STEAL && !core->HasFeature(GF_PST_STATE_FLAGS)) {
+			temp->Flags |= IE_INV_ITEM_STOLEN; // "steel" in pst
 		}
 		temp->Flags &= ~IE_INV_ITEM_SELECTED;
 
@@ -981,28 +916,22 @@ bool Inventory::EquipItem(ieDword slot)
 		EquippedHeader = 0;
 		header = itm->GetExtHeader(EquippedHeader);
 		if (header) {
+			ieDword equip;
 			if (header->AttackType == ITEM_AT_BOW) {
 				//find the ranged projectile associated with it, this returns equipped code
-				Equipped = FindRangedProjectile(header->ProjectileQualifier);
+				equip = FindRangedProjectile(header->ProjectileQualifier);
 				//this is the real item slot of the quarrel
-				slot = Equipped + SLOT_MELEE;
+				slot = equip + SLOT_MELEE;
 			} else {
 				//this is always 0-3
-				Equipped = weaponslot;
+				equip = weaponslot;
 				slot = GetWeaponSlot(weaponslot);
 			}
-			if (Equipped != IW_NO_EQUIPPED) {
+			if (equip != IW_NO_EQUIPPED) {
 				Owner->SetupQuickSlot(ACT_WEAPON1+weaponslot, slot, EquippedHeader);
 			}
-			SetEquippedSlot(Equipped, EquippedHeader);
-			//don't clear effect in case of a launcher, we need to find it and add its effects too
-			//slot is 'negative' for launchers
-			if ((int) Equipped>=0) {
-				effect = 0; // SetEquippedSlot will already call AddSlotEffects
-			} else {
-				effect = SLOT_EFFECT_MISSILE;
-			}
-			UpdateWeaponAnimation();
+			SetEquippedSlot(equip, EquippedHeader);
+			effect = 0; // SetEquippedSlot will already call AddSlotEffects
 		}
 		break;
 	case SLOT_EFFECT_MISSILE:
@@ -1041,9 +970,6 @@ bool Inventory::EquipItem(ieDword slot)
 		if (item->Flags & IE_INV_ITEM_CURSED) {
 			item->Flags|=IE_INV_ITEM_UNDROPPABLE;
 		}
-		if (effect == SLOT_EFFECT_MISSILE) {
-			slot = FindRangedWeapon();
-		}
 		AddSlotEffects( slot );
 	}
 	return true;
@@ -1139,7 +1065,7 @@ int Inventory::FindTypedRangedWeapon(unsigned int type) const
 		//always look for a ranged header when looking for a projectile/projector
 		ITMExtHeader *ext_header = itm->GetWeaponHeader(true);
 		int weapontype = 0;
-		if (ext_header) {
+		if (ext_header && (ext_header->AttackType == ITEM_AT_BOW)) {
 			weapontype = ext_header->ProjectileQualifier;
 		}
 		gamedata->FreeItem(itm, Slot->ItemResRef, false);
@@ -1306,11 +1232,21 @@ bool Inventory::SetEquippedSlot(ieWordSigned slotcode, ieWord header)
 	int oldslot = GetEquippedSlot();
 	int newslot = GetWeaponSlot(slotcode);
 
+	//remove previous slot effects
+	if (Equipped != IW_NO_EQUIPPED) {
+		RemoveSlotEffects(oldslot);
+		//for projectiles we may need to remove the launcher effects too
+		int oldeffects = core->QuerySlotEffects(oldslot);
+		if (oldeffects == SLOT_EFFECT_MISSILE) {
+			int launcher = FindSlotRangedWeapon(oldslot);
+			if (launcher != SLOT_FIST) {
+				RemoveSlotEffects(launcher);
+			}
+		}
+	}
+
 	//unequipping (fist slot will be used now)
 	if (slotcode == IW_NO_EQUIPPED || !HasItemInSlot("", newslot)) {
-		if (Equipped != IW_NO_EQUIPPED) {
-			RemoveSlotEffects( oldslot);
-		}
 		Equipped = IW_NO_EQUIPPED;
 		//fist slot equipping effects
 		AddSlotEffects(SLOT_FIST);
@@ -1318,11 +1254,7 @@ bool Inventory::SetEquippedSlot(ieWordSigned slotcode, ieWord header)
 		return true;
 	}
 
-	//equipping a weapon, but remove its effects first
-	if (Equipped != IW_NO_EQUIPPED) {
-		RemoveSlotEffects( oldslot);
-	}
-
+	//equipping a weapon
 	Equipped = slotcode;
 	int effects = core->QuerySlotEffects( newslot);
 	if (effects) {
@@ -1332,6 +1264,12 @@ bool Inventory::SetEquippedSlot(ieWordSigned slotcode, ieWord header)
 			item->Flags|=IE_INV_ITEM_UNDROPPABLE;
 		}
 		AddSlotEffects(newslot);
+
+		//in case of missiles also look for an appropriate launcher
+		if (effects == SLOT_EFFECT_MISSILE) {
+			newslot = FindRangedWeapon();
+			AddSlotEffects(newslot);
+		}
 	}
 	UpdateWeaponAnimation();
 	return true;
@@ -1347,6 +1285,12 @@ int Inventory::GetEquippedHeader() const
 	return EquippedHeader;
 }
 
+void Inventory::SetEquipped(ieWordSigned slot, ieWord header)
+{
+	Equipped = slot;
+	EquippedHeader = header;
+}
+
 //returns the fist weapon if there is nothing else
 //This will return the actual weapon, I mean the bow in the case of bow+arrow combination
 CREItem *Inventory::GetUsedWeapon(bool leftorright, int &slot) const
@@ -1429,13 +1373,7 @@ int Inventory::FindCandidateSlot(int slottype, size_t first_slot, const char *re
 void Inventory::AddSlotItemRes(const ieResRef ItemResRef, int SlotID, int Charge0, int Charge1, int Charge2)
 {
 	CREItem *TmpItem = new CREItem();
-	strnlwrcpy(TmpItem->ItemResRef, ItemResRef, 8);
-	TmpItem->Expired=0;
-	TmpItem->Usages[0]=(ieWord) Charge0;
-	TmpItem->Usages[1]=(ieWord) Charge1;
-	TmpItem->Usages[2]=(ieWord) Charge2;
-	TmpItem->Flags=0;
-	if (core->ResolveRandomItem(TmpItem)) {
+	if (CreateItemCore(TmpItem, ItemResRef, Charge0, Charge1, Charge2)) {
 		int ret = AddSlotItem( TmpItem, SlotID );
 		if (ret != ASI_SUCCESS) {
 			// put the remainder on the ground
@@ -1448,23 +1386,17 @@ void Inventory::AddSlotItemRes(const ieResRef ItemResRef, int SlotID, int Charge
 				delete TmpItem;
 			}
 		}
+		CalculateWeight();
 	} else {
 		delete TmpItem;
 	}
-	CalculateWeight();
 }
 
 void Inventory::SetSlotItemRes(const ieResRef ItemResRef, int SlotID, int Charge0, int Charge1, int Charge2)
 {
 	if(ItemResRef[0]) {
 		CREItem *TmpItem = new CREItem();
-		strnlwrcpy(TmpItem->ItemResRef, ItemResRef, 8);
-		TmpItem->Expired=0;
-		TmpItem->Usages[0]=(ieWord) Charge0;
-		TmpItem->Usages[1]=(ieWord) Charge1;
-		TmpItem->Usages[2]=(ieWord) Charge2;
-		TmpItem->Flags=0;
-		if (core->ResolveRandomItem(TmpItem)) {
+		if (CreateItemCore(TmpItem, ItemResRef, Charge0, Charge1, Charge2)) {
 			SetSlotItem( TmpItem, SlotID );
 		} else {
 			delete TmpItem;
