@@ -37,7 +37,10 @@
 
 #include <vector>
 
+namespace GemRB {
+
 class Map;
+class StringBuffer;
 
 //AddSlotItem return values
 #define ASI_FAILED     0
@@ -82,7 +85,7 @@ typedef enum ieInventoryType {
 typedef enum ieCREItemFlagBits {
 	IE_INV_ITEM_IDENTIFIED = 1,
 	IE_INV_ITEM_UNSTEALABLE = 2,
-	IE_INV_ITEM_STOLEN = 4,
+	IE_INV_ITEM_STOLEN = 4, // denotes steel items in pst
 	//in iwd/iwd2 this flag means 'magical', some hack is needed
 	IE_INV_ITEM_UNDROPPABLE =8,
 	//just recently acquired
@@ -135,12 +138,13 @@ struct ItemExtHeader {
 	ieByte Target;
 	ieByte TargetNumber;
 	ieWord Range;
-	ieWord ProjectileType;
+	//This was commented out in ITMExtHeader
+	//ieWord ProjectileType;
 	ieWord Speed;
 	ieWord THAC0Bonus;
 	ieWord DiceSides;
 	ieWord DiceThrown;
-	ieWord DamageBonus;
+	ieWordSigned DamageBonus; //this must be signed!!!
 	ieWord DamageType;
 	ieWord FeatureCount;
 	ieWord FeatureOffset;
@@ -192,15 +196,16 @@ private:
 	std::vector<CREItem*> Slots;
 	Actor* Owner;
 	int InventoryType;
-	int Changed;
+	/// Flag indicating whether weight needs to be recalculated
+	mutable int Changed;
 	/** Total weight of all items in Inventory */
-	int Weight;
+	mutable int Weight;
 
 	ieWordSigned Equipped;
 	ieWord EquippedHeader;
 	/** this isn't saved */
 	ieDword ItemExcl;
-	ieDword ItemTypes[4]; //256 bits
+	ieDword ItemTypes[8]; //256 bits
 public: 
 	Inventory();
 	virtual ~Inventory();
@@ -218,11 +223,11 @@ public:
 	bool HasItemInSlot(const char *resref, unsigned int slot) const;
 	/** returns true if contains one itemtype equipped */
 	bool HasItemType(ieDword type) const;
-	/** Looks for a particular item in the inventory. */
-	/* flags: see ieCREItemFlagBits */
+	/** Looks for a particular item in the inventory.
+	 * flags: see ieCREItemFlagBits */
 	bool HasItem(const char *resref, ieDword flags) const;
 
-	void CalculateWeight(void);
+	void CalculateWeight(void) const;
 	void SetInventoryType(int arg);
 	void SetOwner(Actor* act) { Owner = act; }
 
@@ -266,12 +271,13 @@ public:
 	//charges recharging items
 	void ChargeAllItems(int hours);
 	/** Finds the first slot of named item, if resref is empty, finds the first filled! slot */
-	int FindItem(const char *resref, unsigned int flags) const;
+	int FindItem(const char *resref, unsigned int flags, unsigned int skip=0) const;
 	bool DropItemAtLocation(unsigned int slot, unsigned int flags, Map *map, const Point &loc);
 	bool DropItemAtLocation(const char *resref, unsigned int flags, Map *map, const Point &loc);
 	bool SetEquippedSlot(ieWordSigned slotcode, ieWord header);
 	int GetEquipped() const;
 	int GetEquippedHeader() const;
+	void SetEquipped(ieWordSigned slot, ieWord header);
 	//right hand
 	int GetEquippedSlot() const;
 	//left hand
@@ -301,10 +307,16 @@ public:
 	void SetSlotItemRes(const ieResRef ItemResRef, int Slot, int Charge0=1, int Charge1=0, int Charge2=0);
 	/** Adds item to slot*/
 	void AddSlotItemRes(const ieResRef ItemResRef, int Slot, int Charge0=1, int Charge1=0, int Charge2=0);
+	/** returns the itemtype held in the left hand */
+	ieWord GetShieldItemType() const;
+	/** returns the itemtype of the item in the armor slot, mostly used in IWD2 */
+	ieWord GetArmorItemType() const;
 	/** breaks the item (weapon) in slot */
 	void BreakItemSlot(ieDword slot);
 	/** Lists all items in the Inventory on terminal for debugging */
-	void dump();
+	void dump() const;
+	/// List all items in the Inventory to the given buffer */
+	void dump(StringBuffer&) const;
 	/** Equips best weapon */
 	void EquipBestWeapon(int flags);
 	/** returns the struct of the usable items, returns true if there are more */
@@ -321,8 +333,11 @@ public:
 	unsigned int FindStealableItem();
 	/** checks if any equipped item provides critical hit aversion */
 	bool ProvidesCriticalAversion();
+	/** tries to merge the passed item with the on in the passed slot */
+	int MergeItems(int slot, CREItem *item);
 	//setting important constants
-	static void Init(int mb);
+	static void Init();
+	static void SetArmorSlot(int arg);
 	static void SetHeadSlot(int arg);
 	static void SetFistSlot(int arg);
 	static void SetMagicSlot(int arg);
@@ -331,10 +346,13 @@ public:
 	static void SetQuickSlot(int arg);
 	static void SetInventorySlot(int arg);
 	static void SetShieldSlot(int arg);
+	static int GetArmorSlot();
 	static int GetHeadSlot();
 	static int GetFistSlot();
 	static int GetMagicSlot();
 	static int GetWeaponSlot();
+	static int GetWeaponQuickSlot(int weaponslot);
+	static int GetWeaponSlot(int quickslot);
 	static int GetRangedSlot();
 	static int GetQuickSlot();
 	static int GetInventorySlot();
@@ -348,4 +366,6 @@ private:
 	void UpdateShieldAnimation(Item *it);
 };
 
+}
+
 #endif
