@@ -33,6 +33,10 @@
 
 #include "EffectQueue.h"
 
+#include <vector>
+
+namespace GemRB {
+
 class Projectile;
 
 // Item Flags bits
@@ -51,17 +55,19 @@ class Projectile;
 #define IE_ITEM_CONVERSABLE  0x00000800
 #define IE_ITEM_PULSATING    0x00001000
 #define IE_ITEM_UNSELLABLE   ( IE_ITEM_CRITICAL | IE_ITEM_STOLEN )
-
-//modder extensions
+//tobex modder extensions, please note, these are not copied into the local slot bits
 #define IE_ITEM_NO_DISPEL    0x01000000 //disables destruction by dispelling
 #define IE_ITEM_TOGGLE_CRITS 0x02000000 //toggles critical hit avertion
+#define IE_ITEM_NO_INVIS     0x04000000 //don't target invisible
+
 
 //Extended header recharge flags
 #define IE_ITEM_USESTRENGTH  1          //weapon
 #define IE_ITEM_BREAKABLE    2          //weapon
+#define IE_ITEM_USEDEXTERITY 4          //gemrb weapon (move this if tobex implements it elsewhere)
 #define IE_ITEM_HOSTILE      0x400      //equipment
 #define IE_ITEM_RECHARGE     0x800      //equipment
-#define IE_ITEM_IGNORESHIELD 0x10000    //weapon
+#define IE_ITEM_BYPASS       0x10000    //weapon (bypass shield and armor bonus)
 #define IE_ITEM_KEEN         0x20000    //weapon
 
 //modder extensions
@@ -101,6 +107,15 @@ class Projectile;
 
 //items caster level is hardcoded to 10
 #define ITEM_CASTERLEVEL   10
+
+struct DMGOpcodeInfo {
+	const char *TypeName;
+	int DiceThrown;
+	int DiceSides;
+	int DiceBonus;
+	int Chance;
+};
+
 /**
  * @class ITMExtHeader
  * Header for special effects and uses of an Item.
@@ -180,7 +195,7 @@ public:
 	ieByte unknown3;
 	ieDword KitUsability;
 	ieDword Price;
-	ieWord StackAmount;
+	ieWord MaxStackAmount;
 	ieResRef ItemIcon;
 	ieWord LoreToID;
 	ieResRef GroundIcon;
@@ -262,7 +277,11 @@ public:
 	int GetWeaponHeaderNumber(bool ranged) const;
 	int GetEquipmentHeaderNumber(int cnt) const;
 	unsigned int GetCastingDistance(int header) const;
+	// returns  a vector with details about any extended headers containing fx_damage with a 100% probability
+	std::vector<DMGOpcodeInfo> GetDamageOpcodesDetails(ITMExtHeader *header) const;
 private:
 };
 
+}
+
 #endif // ! ITEM_H
