@@ -46,6 +46,7 @@ class Projectile;
 class ScriptedAnimation;
 class SpriteCover;
 class TileMap;
+class VEFObject;
 class Wall_Polygon;
 
 //distance of actors from spawn point
@@ -230,7 +231,7 @@ private:
 	Animation *GetAnimationPiece(AnimationFactory *af, int animCycle);
 };
 
-enum AnimationObjectType {AOT_AREA, AOT_SCRIPTED, AOT_ACTOR, AOT_SPARK, AOT_PROJECTILE};
+enum AnimationObjectType {AOT_AREA, AOT_SCRIPTED, AOT_ACTOR, AOT_SPARK, AOT_PROJECTILE, AOT_PILE};
 
 //i believe we need only the active actors/visible inactive actors queues
 #define QUEUE_COUNT 2
@@ -241,7 +242,7 @@ enum AnimationObjectType {AOT_AREA, AOT_SCRIPTED, AOT_ACTOR, AOT_SPARK, AOT_PROJ
 #define PR_IGNORE  2
 
 typedef std::list<AreaAnimation*>::iterator aniIterator;
-typedef std::list<ScriptedAnimation*>::iterator scaIterator;
+typedef std::list<VEFObject*>::iterator scaIterator;
 typedef std::list<Projectile*>::iterator proIterator;
 typedef std::list<Particles*>::iterator spaIterator;
 
@@ -280,7 +281,7 @@ private:
 	std::vector< Actor*> actors;
 	Wall_Polygon **Walls;
 	unsigned int WallCount;
-	std::list< ScriptedAnimation*> vvcCells;
+	std::list< VEFObject*> vvcCells;
 	std::list< Projectile*> projectiles;
 	std::list< Particles*> particles;
 	std::vector< Entrance*> entrances;
@@ -317,7 +318,7 @@ public:
 	void MoveVisibleGroundPiles(const Point &Pos);
 	/* draws stationary vvc graphics */
 	//void DrawVideocells(Region screen);
-	void DrawHighlightables(Region screen);
+	void DrawHighlightables();
 	void DrawMap(Region screen);
 	void PlayAreaSong(int SongType, bool restart = true, bool hard = false);
 	void AddAnimation(AreaAnimation* anim);
@@ -340,7 +341,7 @@ public:
 		Walls = walls;
 	}
 	SpriteCover* BuildSpriteCover(int x, int y, int xpos, int ypos,
-		unsigned int width, unsigned int height, int flag);
+		unsigned int width, unsigned int height, int flag, bool areaanim = false);
 	void ActivateWallgroups(unsigned int baseindex, unsigned int count, int flg);
 	void Shout(Actor* actor, int shoutID, unsigned int radius);
 	void ActorSpottedByPlayer(Actor *actor);
@@ -367,6 +368,7 @@ public:
 	Actor* GetActorByResource(const char* resref);
 	Actor* GetActorByScriptName(const char* name);
 	bool HasActor(Actor *actor);
+	bool SpawnsAlive() const;
 	void RemoveActor(Actor* actor);
 	//returns actors in rect (onlyparty could be more sophisticated)
 	int GetActorInRect(Actor**& actors, Region& rgn, bool onlyparty);
@@ -380,6 +382,7 @@ public:
 
 	SongHeaderType SongHeader;
 	RestHeaderType RestHeader;
+	int AreaDifficulty;
 
 	//count of all projectiles that are saved
 	size_t GetProjectileCount(proIterator &iter);
@@ -395,7 +398,7 @@ public:
 
 	//returns the duration of a VVC cell set in the area (point may be set to empty)
 	ieDword HasVVCCell(const ieResRef resource, const Point &p);
-	void AddVVCell(ScriptedAnimation* vvc);
+	void AddVVCell(VEFObject* vvc);
 	bool CanFree();
 	int GetCursor( const Point &p);
 	//adds a sparkle puff of colour to a point in the area
@@ -506,8 +509,10 @@ public:
 private:
 	AreaAnimation *GetNextAreaAnimation(aniIterator &iter, ieDword gametime);
 	Particles *GetNextSpark(spaIterator &iter);
-	ScriptedAnimation *GetNextScriptedAnimation(scaIterator &iter);
+	VEFObject *GetNextScriptedAnimation(scaIterator &iter);
 	Actor *GetNextActor(int &q, int &index);
+	Container *GetNextPile (int &index) const;
+	void DrawPile (Region screen, int pileidx);
 	void DrawSearchMap(const Region &screen);
 	void GenerateQueues();
 	void SortQueues();
