@@ -21,6 +21,8 @@
 
 #include "ie_types.h"
 
+namespace GemRB {
+
 struct GEM_EXPORT PolymorphCache {
 	ieResRef Resource;
 	ieDword *stats;
@@ -31,4 +33,6 @@ struct GEM_EXPORT PolymorphCache {
 	}
 };
 
+}
+
 #endif
