@@ -31,6 +31,8 @@
 #include "exports.h"
 #include "ie_types.h"
 
+namespace GemRB {
+
 /**
  * @class Point
  * Point in 2d-space: [x, y]
@@ -73,10 +75,8 @@ public:
 class GEM_EXPORT Region {
 public:
 	Region(void);
-	int x;
-	int y;
-	int w;
-	int h;
+	int x, y;
+	int w, h;
 	Region(const Point& p, int w, int h);
 	bool operator==(const Region& rgn);
 	bool operator!=(const Region& rgn);
@@ -87,4 +87,6 @@ public:
 	void Normalize();
 };
 
+}
+
 #endif  // ! REGION_H
