@@ -33,7 +33,6 @@
 #include "EffectQueue.h"
 #include "Palette.h"
 
-#include <cstring>
 #include <vector>
 
 namespace GemRB {
@@ -149,6 +148,8 @@ namespace GemRB {
 
 // Detect() mode: IDS matching ignores invisibility
 #define GA_DETECT 8192
+//cannot target unscheduled actors
+#define GA_NO_UNSCHEDULED 16384
 
 #define VCONST_COUNT 100
 
@@ -168,6 +169,7 @@ namespace GemRB {
 #define UI_SILENT    1       //no sound when used up
 #define UI_MISS      2       //ranged miss (projectile has no effects)
 #define UI_CRITICAL  4       //a critical hit happened
+#define UI_FAKE      8       //deplete the item but don't actually apply its effects
 
 //used to mask off current profs
 #define PROFS_MASK  0x07
@@ -300,8 +302,6 @@ public:
 	ieByte SetDeathVar, IncKillCount, UnknownField; // boolean fields from iwd1 and iwd2
 
 	Inventory inventory;
-	ieWordSigned Equipped;         //the equipped weapon slot
-	ieWord EquippedHeader;         //the used extended header
 	Spellbook spellbook;
 	//savefile version (creatures embedded in area)
 	int version;
@@ -332,6 +332,7 @@ public:
 	int LastDamage;
 	Point FollowOffset;//follow lastfollowed at this offset
 	Point HomeLocation;//spawnpoint, return here after rest
+	bool Spawned;      //has been created by a spawn point
 
 	ieDword TargetDoor;
 
@@ -347,13 +348,13 @@ public:
 	ieDword nextComment;         //do something random (area comment, interaction)
 	ieDword nextBored;           //do something when bored
 	ieDword lastInit;
-	bool no_more_steps;
 	int speed;
 	//how many attacks left in this round, must be public for cleave opcode
 	int attackcount;
 
 	PolymorphCache *polymorphCache; // fx_polymorph etc
 	WildSurgeSpellMods wildSurgeMods;
+	ieByte DifficultyMargin;
 private:
 	//this stuff doesn't get saved
 	CharAnimations* anims;
@@ -439,7 +440,7 @@ public:
 	/** gets saving throws */
 	void RollSaves();
 	/** returns a saving throw */
-	bool GetSavingThrow(ieDword type, int modifier);
+	bool GetSavingThrow(ieDword type, int modifier, int spellLevel=0, int saveBonus=0);
 	/** Returns true if the actor is targetable */
 	bool ValidTarget(int ga_flags, Scriptable *checker = NULL) const;
 	/** Clamps a stat value to the valid range for the respective stat */
@@ -498,6 +499,7 @@ public:
 
 	/** Gets the Dialog ResRef */
 	const char* GetDialog(int flags=GD_NORMAL) const;
+	void SetDialog(const ieResRef resref);
 	/** Gets the Portrait ResRef */
 	const char* GetPortrait(int which) const
 	{
@@ -520,7 +522,7 @@ public:
 	/* call this on gui selects */
 	void PlaySelectionSound();
 	/* call this when adding actions via gui */
-	void CommandActor();
+	void CommandActor(Action* action);
 	/** handle panic and other involuntary actions that mess with scripting */
 	bool OverrideActions();
 	/** handle idle actions, that shouldn't mess with scripting */
@@ -694,6 +696,7 @@ public:
 	void UpdateAnimations();
 	/* if necessary, draw actor */
 	void Draw(const Region &screen);
+	bool DoStep(unsigned int walk_speed, ieDword time = 0);
 
 	/* add mobile vvc (spell effects) to actor's list */
 	void AddVVCell(ScriptedAnimation* vvc);
