@@ -23,32 +23,27 @@
 #include "strrefs.h"
 #include "ie_cursors.h"
 
-#include "Audio.h"
 #include "DisplayMessage.h"
 #include "Game.h"
 #include "GameData.h"
-#include "Interface.h"
-#include "Item.h"
 #include "Font.h"
-#include "Map.h"
 #include "Projectile.h"
 #include "Spell.h"
 #include "SpriteCover.h"
-#include "TileMap.h"
 #include "Video.h"
 #include "GameScript/GSUtils.h"
 #include "GameScript/Matching.h" // MatchActor
 #include "GUI/GameControl.h"
-//#include "GUI/Window.h"
 #include "Scriptable/InfoPoint.h"
 
-#include <cassert>
-#include <cmath>
+namespace GemRB {
 
 #define YESNO(x) ( (x)?"Yes":"No")
 
 // we start this at a non-zero value to make debugging easier
 static ieDword globalActorCounter = 10000;
+static bool startActive = false;
+static bool third = false;
 
 /***********************
  *  Scriptable Class   *
@@ -64,6 +59,7 @@ Scriptable::Scriptable(ScriptableType type)
 	textDisplaying = 0;
 	timeStartDisplaying = 0;
 	scriptName[0] = 0;
+	scriptlevel = 0;
 
 	LastAttacker = 0;
 	LastCommander = 0;
@@ -97,10 +93,16 @@ Scriptable::Scriptable(ScriptableType type)
 	Dialog[0] = 0;
 
 	globalID = ++globalActorCounter;
+	if (globalActorCounter == 0) {
+		error("Scriptable", "GlobalID overflowed, quitting due to too many actors.");
+	}
 
 	WaitCounter = 0;
 	if (Type == ST_ACTOR) {
 		InternalFlags = IF_VISIBLE | IF_USEDSAVE;
+		if (startActive) {
+			InternalFlags |= IF_ACTIVE;
+		}
 	} else {
 		InternalFlags = IF_ACTIVE | IF_VISIBLE | IF_NOINT;
 	}
@@ -109,10 +111,7 @@ Scriptable::Scriptable(ScriptableType type)
 	Pos.y = 0;
 
 	LastSpellOnMe = 0xffffffff;
-	SpellHeader = -1;
-	SpellResRef[0] = 0;
-	LastTarget = 0;
-	LastTargetPos.empty();
+	ResetCastingState(NULL);
 	InterruptCasting = false;
 	locals = new Variables();
 	locals->SetType( GEM_VARIABLES_INT );
@@ -121,6 +120,8 @@ Scriptable::Scriptable(ScriptableType type)
 	AddTrigger(TriggerEntry(trigger_oncreation));
 
 	memset( script_timers,0, sizeof(script_timers));
+	startActive = core->HasFeature(GF_START_ACTIVE);
+	third = core->HasFeature(GF_3ED_RULES);
 }
 
 Scriptable::~Scriptable(void)
@@ -201,7 +202,7 @@ void Scriptable::SetScript(const ieResRef aScript, int idx, bool ai)
 void Scriptable::SetScript(int index, GameScript* script)
 {
 	if (index >= MAX_SCRIPTS) {
-		printMessage("Scriptable","Invalid script index!\n",LIGHT_RED);
+		Log(ERROR, "Scriptable", "Invalid script index!");
 		return;
 	}
 	if (Scripts[index] ) {
@@ -219,9 +220,9 @@ void Scriptable::DisplayHeadText(const char* text)
 	if (overHeadText) {
 		core->FreeString( overHeadText );
 	}
-	overHeadText = (char *) text;
 	overHeadTextPos.empty();
 	if (text) {
+		overHeadText = strdup(text);
 		timeStartDisplaying = core->GetGame()->Ticks;
 		textDisplaying = 1;
 	}
@@ -238,8 +239,6 @@ void Scriptable::FixHeadTextPos()
 }
 
 #define MAX_DELAY  6000
-static const Color black={0,0,0,0};
-
 void Scriptable::DrawOverheadText(const Region &screen)
 {
 	unsigned long time = core->GetGame()->Ticks;
@@ -257,8 +256,9 @@ void Scriptable::DrawOverheadText(const Region &screen)
 	} else {
 		time = (MAX_DELAY-time)/10;
 		if (time<256) {
-			const Color overHeadColor = {time,time,time,time};
-			palette = core->CreatePalette(overHeadColor, black);
+			ieByte time2 = time; // shut up narrowing warnings
+			const Color overHeadColor = {time2,time2,time2,time2};
+			palette = core->CreatePalette(overHeadColor, ColorBlack);
 		}
 	}
 
@@ -335,7 +335,7 @@ void Scriptable::TickScripting()
 		actorState = ((Actor *)this)->Modified[IE_STATE_ID];
 
 	// Dead actors only get one chance to run a new script.
-	if ((actorState & STATE_DEAD) && !(InternalFlags & IF_JUSTDIED))
+	if ( (InternalFlags& (IF_REALLYDIED|IF_JUSTDIED))==IF_REALLYDIED)
 		return;
 
 	ScriptTicks++;
@@ -387,7 +387,7 @@ void Scriptable::ExecuteScript(int scriptCount)
 	if (!CurrentActionInterruptable) {
 		// sanity check
 		if (!CurrentAction && !GetNextAction())
-			abort();
+			error("Scriptable", "No current action and no next action.\n");
 		return;
 	}
 
@@ -425,11 +425,14 @@ void Scriptable::ExecuteScript(int scriptCount)
 void Scriptable::AddAction(Action* aC)
 {
 	if (!aC) {
-		print( "[GameScript]: NULL action encountered for %s!\n",scriptName );
+		print("[GameScript]: NULL action encountered for %s!", scriptName);
 		return;
 	}
 
 	InternalFlags|=IF_ACTIVE;
+	if (startActive) {
+		InternalFlags &= ~IF_IDLE;
+	}
 	aC->IncRef();
 	if (actionflags[aC->actionID] & AF_SCRIPTLEVEL) {
 		aC->int0Parameter = scriptlevel;
@@ -437,7 +440,8 @@ void Scriptable::AddAction(Action* aC)
 
 	// attempt to handle 'instant' actions, from instant.ids, which run immediately
 	// when added if the action queue is empty, even on actors which are Held/etc
-	if (!CurrentAction && !GetNextAction()) {
+	// FIXME: area check hack until fuzzie fixes scripts here
+	if (!CurrentAction && !GetNextAction() && area) {
 		if (actionflags[aC->actionID] & AF_INSTANT) {
 			CurrentAction = aC;
 			GameScript::ExecuteAction( this, CurrentAction );
@@ -451,7 +455,7 @@ void Scriptable::AddAction(Action* aC)
 void Scriptable::AddActionInFront(Action* aC)
 {
 	if (!aC) {
-		print( "[GameScript]: NULL action encountered for %s!\n",scriptName );
+		print("[GameScript]: NULL action encountered for %s!", scriptName);
 		return;
 	}
 	InternalFlags|=IF_ACTIVE;
@@ -488,6 +492,7 @@ void Scriptable::ClearActions()
 	actionQueue.clear();
 	WaitCounter = 0;
 	LastTarget = 0;
+	LastTargetPos.empty();
 
 	if (Type == ST_ACTOR) {
 		Interrupt();
@@ -496,6 +501,11 @@ void Scriptable::ClearActions()
 	}
 }
 
+void Scriptable::Stop()
+{
+	ClearActions();
+}
+
 void Scriptable::ReleaseCurrentAction()
 {
 	if (CurrentAction) {
@@ -522,7 +532,7 @@ void Scriptable::ProcessActions()
 		CurrentActionInterruptable = true;
 		if (!CurrentAction) {
 			if (! (CurrentActionTicks == 0 && CurrentActionState == 0)) {
-				print("Last action: %d\n", lastAction);
+				print("Last action: %d", lastAction);
 			}
 			assert(CurrentActionTicks == 0 && CurrentActionState == 0);
 			CurrentAction = PopNextAction();
@@ -621,11 +631,22 @@ void Scriptable::PartyRested()
 	AddTrigger(TriggerEntry(trigger_partyrested));
 }
 
-ieDword Scriptable::GetInternalFlag()
+ieDword Scriptable::GetInternalFlag() const
 {
 	return InternalFlags;
 }
 
+void Scriptable::SetInternalFlag(int value, int mode)
+{
+	switch (mode) {
+		case BM_OR: InternalFlags|=value; break;
+		case BM_NAND: InternalFlags&=~value; break;
+		case BM_SET: InternalFlags=value; break;
+		case BM_AND: InternalFlags&=value; break;
+		case BM_XOR: InternalFlags^=value; break;
+	}
+}
+
 void Scriptable::InitTriggers()
 {
 	triggers.clear();
@@ -637,8 +658,11 @@ void Scriptable::AddTrigger(TriggerEntry trigger)
 	ImmediateEvent();
 
 	assert(trigger.triggerID < MAX_TRIGGERS);
-	if (triggerflags[trigger.triggerID] & TF_SAVED)
+	if (triggerflags[trigger.triggerID] & TF_SAVED) {
+		//TODO: if LastTrigger is still overwritten by script action blocks, store this in a separate field and copy it back when the block ends
+		//Log(WARNING, "Scriptable", "%s: Added LastTrigger: %d for trigger %d\n", scriptName, trigger.param1, trigger.triggerID);
 		LastTrigger = trigger.param1;
+	}
 }
 
 bool Scriptable::MatchTrigger(unsigned short id, ieDword param) {
@@ -689,18 +713,17 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 
 	//PST has a weird effect, called Enoll Eva's duplication
 	//it creates every projectile of the affected actor twice
-	int duplicate = 1;
+	int projectileCount = 1;
 	if (Type == ST_ACTOR) {
 		caster = (Actor *) this;
-		caster->CureInvisibility();
 		if (spl->Flags&SF_HOSTILE) {
 			caster->CureSanctuary();
 		}
 
-		//FIXME: 1 duplicate is no duplicate, right?
-		duplicate = caster->wildSurgeMods.num_castings;
-		if (!duplicate) {
-			duplicate = 1;
+		// check if a wild surge ordered us to replicate the projectile
+		projectileCount = caster->wildSurgeMods.num_castings;
+		if (!projectileCount) {
+			projectileCount = 1;
 		}
 	}
 
@@ -708,13 +731,13 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 		if (caster->GetStat(IE_STATE_ID)&STATE_EE_DUPL) {
 			//seriously, wild surges and EE in the same game?
 			//anyway, it would be too many duplications
-			duplicate = 2;
+			projectileCount = 2;
 		}
 	}
 
-	while(duplicate --) {
+	while(projectileCount --) {
 		Projectile *pro = NULL;
-		// jump through hoops to skip applying selftargetting spells to the caster
+		// jump through hoops to skip applying selftargeting spells to the caster
 		// if we'll be changing the target
 		int tct = 0;
 		if (caster) {
@@ -755,7 +778,7 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 					break;
 				case WSTC_ADDTYPE:
 					// TODO: unhardcode to allow for mixing all the target types
-					// caster gets selftargetting fx when the projectile is fetched above
+					// caster gets selftargeting fx when the projectile is fetched above
 					seh = &spl->ext_headers[SpellHeader];
 					for (i=0; i < seh->FeatureCount; i++) {
 						if (seh->features[i].Target == FX_TARGET_SELF) {
@@ -767,6 +790,7 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 						}
 					}
 					// we need to refetch the projectile, so the effect queue is created
+					delete pro; // don't leak the original one
 					pro = spl->GetProjectile(this, SpellHeader, level, LastTargetPos);
 					pro->SetCaster(GetGlobalID(), level);
 					break;
@@ -786,7 +810,7 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 						LastTargetPos = newact->Pos;
 					}
 
-					// make it also work for self-targetting spells:
+					// make it also work for self-targeting spells:
 					// change the payload or this was all in vain
 					seh = &spl->ext_headers[SpellHeader];
 					for (i=0; i < seh->FeatureCount; i++) {
@@ -814,7 +838,7 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 			// change the projectile
 			if (caster->wildSurgeMods.projectile_id) {
 				spl->ext_headers[SpellHeader].ProjectileAnimation = caster->wildSurgeMods.projectile_id;
-				// make it also work for self-targetting spells:
+				// make it also work for self-targeting spells:
 				// change the payload or this was all in vain
 				seh = &spl->ext_headers[SpellHeader];
 				for (i=0; i < seh->FeatureCount; i++) {
@@ -823,6 +847,7 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 					}
 				}
 				// we need to refetch the projectile, so the new one is used
+				delete pro; // don't leak the original one
 				pro = spl->GetProjectile(this, SpellHeader, level, LastTargetPos);
 				pro->SetCaster(GetGlobalID(), level);
 			}
@@ -847,51 +872,101 @@ void Scriptable::CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int l
 	if (spellnum!=0xffffffff) {
 		area->SeeSpellCast(this, spellnum);
 
-		// caster - Casts spellname : target OR
-		// caster - spellname : target (repeating spells)
-		Scriptable *target = NULL;
-		if (tgt) {
-			target = area->GetActorByGlobalID(tgt);
-			if (!target) {
-				target=core->GetGame()->GetActorByGlobalID(tgt);
+		// spellcasting feedback
+		if (third) {
+			// only display it for party friendly creatures - enemies require a successful spellcraft check
+			if (Type == ST_ACTOR) {
+				Actor *caster = ((Actor *) this);
+				if (caster->GetStat(IE_EA) <= EA_CONTROLLABLE) {
+					DisplaySpellCastMessage(tgt, spl);
+				}
 			}
+		} else {
+			DisplaySpellCastMessage(tgt, spl);
 		}
-		char* spell = core->GetString(spl->SpellName);
-		if (stricmp(spell, "")) {
-			char* msg = core->GetString(displaymsg->GetStringReference(STR_ACTION_CAST), 0);
-			char *tmp;
-			if (target) {
-				tmp = (char *) malloc(strlen(msg)+strlen(spell)+strlen(target->GetName(-1))+5);
-				sprintf(tmp, "%s %s : %s", msg, spell, target->GetName(-1));
-			} else {
-				tmp = (char *) malloc(strlen(msg)+strlen(spell)+4);
-				sprintf(tmp, "%s : %s", spell, GetName(-1));
-			}
-			displaymsg->DisplayStringName(tmp, DMC_WHITE, this);
-			core->FreeString(msg);
-			free(tmp);
-		}
-		core->FreeString(spell);
 	}
-	core->Autopause(AP_SPELLCAST);
+	// only trigger the autopause when in combat or buffing gets very annoying
+	if (core->GetGame()->CombatCounter) {
+		core->Autopause(AP_SPELLCAST, this);
+	}
 
 	gamedata->FreeSpell(spl, SpellResRef, false);
 }
 
-void Scriptable::CastSpellPointEnd(int level)
+void Scriptable::DisplaySpellCastMessage(ieDword tgt, Spell *spl)
+{
+	// caster - Casts spellname : target OR
+	// caster - spellname : target (repeating spells)
+	Scriptable *target = NULL;
+	if (tgt) {
+		target = area->GetActorByGlobalID(tgt);
+		if (!target) {
+			target=core->GetGame()->GetActorByGlobalID(tgt);
+		}
+	}
+	char* spell = core->GetString(spl->SpellName);
+	if (stricmp(spell, "") && Type == ST_ACTOR) {
+		char* msg = core->GetString(displaymsg->GetStringReference(STR_ACTION_CAST), 0);
+		char *tmp;
+		if (target) {
+			tmp = (char *) malloc(strlen(msg)+strlen(spell)+strlen(target->GetName(-1))+5);
+			sprintf(tmp, "%s %s : %s", msg, spell, target->GetName(-1));
+		} else {
+			tmp = (char *) malloc(strlen(spell)+strlen(GetName(-1))+4);
+			sprintf(tmp, "%s : %s", spell, GetName(-1));
+		}
+		displaymsg->DisplayStringName(tmp, DMC_WHITE, this);
+		core->FreeString(msg);
+		free(tmp);
+	}
+	core->FreeString(spell);
+}
+
+void Scriptable::SendTriggerToAll(TriggerEntry entry)
+{
+	Actor** nearActors = area->GetAllActorsInRadius(Pos, GA_NO_DEAD, 15*10);
+	int i=0;
+	while(nearActors[i]!=NULL) {
+		nearActors[i]->AddTrigger(entry);
+		++i;
+	}
+	area->AddTrigger(entry);
+	free(nearActors);
+}
+
+inline void Scriptable::ResetCastingState(Actor *caster) {
+	SpellHeader = -1;
+	SpellResRef[0] = 0;
+	LastTarget = 0;
+	LastTargetPos.empty();
+	if (caster) {
+		memset(&(caster->wildSurgeMods), 0, sizeof(caster->wildSurgeMods));
+	}
+}
+
+void Scriptable::CastSpellPointEnd(int level, int no_stance)
 {
 	Actor *caster = NULL;
+	Spell* spl = gamedata->GetSpell(SpellResRef); // this was checked before we got here
+	if (!spl) {
+		return;
+	}
+	int nSpellType = spl->SpellType;
+	gamedata->FreeSpell(spl, SpellResRef, false);
+
 	if (Type == ST_ACTOR) {
 		caster = ((Actor *) this);
-		caster->SetStance(IE_ANI_CONJURE);
-		if (level == 0) {
-			Spell* spl = gamedata->GetSpell(SpellResRef); // this was checked before we got here
-			Actor *actor = NULL;
-			if (Type == ST_ACTOR) {
-				actor = (Actor *) this;
-				level = actor->GetCasterLevel(spl->SpellType);
-			}
-			gamedata->FreeSpell(spl, SpellResRef, false);
+		if (!no_stance) {
+			caster->SetStance(IE_ANI_CONJURE);
+			caster->CureInvisibility();
+		}
+	}
+	if (level == 0) {
+		if (caster) {
+			level = caster->GetCasterLevel(nSpellType);
+		} else {
+			//default caster level is 1
+			level = 1;
 		}
 	}
 
@@ -908,32 +983,57 @@ void Scriptable::CastSpellPointEnd(int level)
 	if (!SpellResRef[0]) {
 		return;
 	}
+	if (!area) {
+		Log(ERROR, "Scriptable", "CastSpellPointEnd: lost area, skipping %s!", SpellResRef);
+		ResetCastingState(caster);
+		return;
+	}
+
+	if (caster && caster->PCStats) {
+		caster->PCStats->RegisterFavourite(SpellResRef, FAV_SPELL);
+	}
 
 	CreateProjectile(SpellResRef, 0, level, false);
-
-	SpellHeader = -1;
-	SpellResRef[0] = 0;
-	LastTarget = 0;
-	LastTargetPos.empty();
-	if (caster) {
-		memset(&(caster->wildSurgeMods), 0, sizeof(caster->wildSurgeMods));
+	//FIXME: this trigger affects actors whom the caster sees, not just the caster itself
+	// the original engine saves lasttrigger only in case of SpellCast, so we have to differentiate
+	ieDword spellID = ResolveSpellNumber(SpellResRef);
+	switch (nSpellType) {
+	case 1:
+		SendTriggerToAll(TriggerEntry(trigger_spellcast, GetGlobalID(), spellID));
+		break;
+	case 2:
+		SendTriggerToAll(TriggerEntry(trigger_spellcastpriest, GetGlobalID(), spellID));
+		break;
+	default:
+		SendTriggerToAll(TriggerEntry(trigger_spellcastinnate, GetGlobalID(), spellID));
+		break;
 	}
+
+	ResetCastingState(caster);
 }
 
-void Scriptable::CastSpellEnd(int level)
+void Scriptable::CastSpellEnd(int level, int no_stance)
 {
 	Actor *caster = NULL;
+	Spell* spl = gamedata->GetSpell(SpellResRef); // this was checked before we got here
+	if (!spl) {
+		return;
+	}
+	int nSpellType = spl->SpellType;
+	gamedata->FreeSpell(spl, SpellResRef, false);
 	if (Type == ST_ACTOR) {
 		caster = ((Actor *) this);
-		caster->SetStance(IE_ANI_CONJURE);
-		if (level == 0) {
-			Spell* spl = gamedata->GetSpell(SpellResRef); // this was checked before we got here
-			Actor *actor = NULL;
-			if (Type == ST_ACTOR) {
-				actor = (Actor *) this;
-				level = actor->GetCasterLevel(spl->SpellType);
-			}
-			gamedata->FreeSpell(spl, SpellResRef, false);
+		if (!no_stance) {
+			caster->SetStance(IE_ANI_CONJURE);
+			caster->CureInvisibility();
+		}
+	}
+	if (level == 0) {
+		if (caster) {
+			level = caster->GetCasterLevel(nSpellType);
+		} else {
+			//default caster level is 1
+			level = 1;
 		}
 	}
 
@@ -948,24 +1048,49 @@ void Scriptable::CastSpellEnd(int level)
 	if (!SpellResRef[0]) {
 		return;
 	}
+	if (!area) {
+		Log(ERROR, "Scriptable", "CastSpellEnd: lost area, skipping %s!", SpellResRef);
+		ResetCastingState(caster);
+		return;
+	}
+
+	if (caster && caster->PCStats) {
+		caster->PCStats->RegisterFavourite(SpellResRef, FAV_SPELL);
+	}
 
 	//if the projectile doesn't need to follow the target, then use the target position
 	CreateProjectile(SpellResRef, LastTarget, level, GetSpellDistance(SpellResRef, this)==0xffffffff);
-	SpellHeader = -1;
-	SpellResRef[0] = 0;
-	LastTarget = 0;
-	LastTargetPos.empty();
-	if (caster) {
-		memset(&(caster->wildSurgeMods), 0, sizeof(caster->wildSurgeMods));
+	//FIXME: this trigger affects actors whom the caster sees, not just the caster itself
+	// the original engine saves lasttrigger only in case of SpellCast, so we have to differentiate
+	ieDword spellID = ResolveSpellNumber(SpellResRef);
+	switch (nSpellType) {
+	case 1:
+		SendTriggerToAll(TriggerEntry(trigger_spellcast, GetGlobalID(), spellID));
+		break;
+	case 2:
+		SendTriggerToAll(TriggerEntry(trigger_spellcastpriest, GetGlobalID(), spellID));
+		break;
+	default:
+		SendTriggerToAll(TriggerEntry(trigger_spellcastinnate, GetGlobalID(), spellID));
+		break;
 	}
+
+	// TODO: maybe it should be set on effect application, since the data uses it with dispel magic and true sight a lot
+	Actor *target = area->GetActorByGlobalID(LastTarget);
+	if (target) {
+		target->AddTrigger(TriggerEntry(trigger_spellcastonme, GetGlobalID(), spellID));
+		target->LastSpellOnMe = spellID;
+	}
+
+	ResetCastingState(caster);
 }
 
 // check for input sanity and good casting conditions
-int Scriptable::CanCast(const ieResRef SpellResRef) {
+int Scriptable::CanCast(const ieResRef SpellResRef, bool verbose) {
 	Spell* spl = gamedata->GetSpell(SpellResRef);
 	if (!spl) {
 		SpellHeader = -1;
-		printMessage("Scriptable", "Spell not found, aborting cast!\n", LIGHT_RED);
+		Log(ERROR, "Scriptable", "Spell not found, aborting cast!");
 		return 0;
 	}
 
@@ -990,16 +1115,15 @@ int Scriptable::CanCast(const ieResRef SpellResRef) {
 		// only a handful of spells don't have a verbal component -
 		// the original hardcoded vocalize and a few more
 		// we (also) ignore nonmagic spells
-		if (actor->Modified[IE_STATE_ID] & STATE_SILENCED) {
+		if (actor->CheckSilenced()) {
 			if (!(core->GetSpecialSpell(spl->Name)&SP_SILENCE) && !(spl->Flags&SF_HLA)) {
-				printMessage("Scriptable", "Tried to cast while silenced!\n", YELLOW);
+				Log(WARNING, "Scriptable", "Tried to cast while silenced!");
 				return 0;
 			}
 		}
 
 		// check for personal dead magic
 		if (actor->Modified[IE_DEADMAGIC]) {
-			// TODO: display fizzling animation
 			displaymsg->DisplayConstantStringName(STR_DEADMAGIC_FAIL, DMC_WHITE, this);
 			return 0;
 		}
@@ -1007,26 +1131,33 @@ int Scriptable::CanCast(const ieResRef SpellResRef) {
 		// check for miscast magic and similar
 		ieDword roll = actor->LuckyRoll(1, 100, 0, 0);
 		bool failed = false;
+		ieDword chance = 0;
 		switch(spl->SpellType)
 		{
 		case IE_SPL_PRIEST:
-			if (actor->Modified[IE_SPELLFAILUREPRIEST] >= roll) {
+			chance = actor->GetSpellFailure(false);
+			if (chance >= roll) {
 				failed = true;
 			}
 			break;
 		case IE_SPL_WIZARD:
-			if (actor->Modified[IE_SPELLFAILUREMAGE] >= roll) {
+			chance = actor->GetSpellFailure(true);
+			if (chance >= roll) {
 				failed = true;
 			}
 			break;
 		case IE_SPL_INNATE:
-			if (actor->Modified[IE_SPELLFAILUREINNATE] >= roll) {
+			chance = actor->Modified[IE_SPELLFAILUREINNATE];
+			if (chance >= roll) {
 				failed = true;
 			}
 			break;
 		}
+		if (verbose && chance && third) {
+			// ~Spell Failure check: Roll d100 %d vs. Spell failure chance %d~
+			displaymsg->DisplayRollStringName(40955, DMC_LIGHTGREY, actor, roll, chance);
+		}
 		if (failed) {
-			// TODO: display fizzling animation
 			displaymsg->DisplayConstantStringName(STR_MISCASTMAGIC, DMC_WHITE, this);
 			return 0;
 		}
@@ -1035,6 +1166,66 @@ int Scriptable::CanCast(const ieResRef SpellResRef) {
 	return 1;
 }
 
+// checks if a party-friendly actor is nearby and if so, if it reckognizes the spell
+// this enemy just started casting
+void Scriptable::SpellcraftCheck(const Actor *caster, const ieResRef SpellResRef)
+{
+	if (!third || !caster || caster->GetStat(IE_EA) <= EA_CONTROLLABLE || !area) {
+		return;
+	}
+
+	Spell* spl = gamedata->GetSpell(SpellResRef);
+	assert(spl); // only a bad surge could make this fail and we want to catch it
+	int AdjustedSpellLevel = spl->SpellLevel + 15;
+	Actor **neighbours = area->GetAllActorsInRadius(caster->Pos, GA_NO_DEAD|GA_NO_ENEMY|GA_NO_SELF|GA_NO_UNSCHEDULED, 10*caster->GetBase(IE_VISUALRANGE));
+	Actor **poi = neighbours;
+	while (*poi) {
+		Actor *detective = *poi;
+		// disallow neutrals from helping the party
+		if (detective->GetStat(IE_EA) > EA_CONTROLLABLE) {
+			poi++;
+			continue;
+		}
+		if ((signed)detective->GetSkill(IE_SPELLCRAFT) <= 0) {
+			poi++;
+			continue;
+		}
+
+		// ~Spellcraft check (d20 roll + Spellcraft level + int mod) %d vs. (spell level + 15)  = %d.   (Int mod = %d)~
+		int Spellcraft = core->Roll(1, 20, 0) + detective->GetStat(IE_SPELLCRAFT);
+		int IntMod = detective->GetAbilityBonus(IE_INT);
+
+		if ((Spellcraft + IntMod) > AdjustedSpellLevel) {
+			char tmpstr[100];
+			memset(tmpstr, 0, sizeof(tmpstr));
+			// eg. .:Casts Word of Recall:.
+			snprintf(tmpstr, sizeof(tmpstr), ".:%s %s:.", core->GetString(displaymsg->GetStringReference(STR_CASTS)), core->GetString(spl->SpellName));
+			DisplayHeadText(tmpstr);
+			displaymsg->DisplayRollStringName(39306, DMC_LIGHTGREY, detective, Spellcraft+IntMod, AdjustedSpellLevel, IntMod);
+			break;
+		}
+		poi++;
+	}
+	gamedata->FreeSpell(spl, SpellResRef, false);
+	free(neighbours);
+}
+
+// shortcut for internal use when there is no wait
+void Scriptable::DirectlyCastSpellPoint(const Point &target, ieResRef spellref, int level, int no_stance, bool deplete, bool instant, bool nointerrupt)
+{
+	SetSpellResRef(spellref);
+	CastSpellPoint(target, deplete, instant, nointerrupt);
+	CastSpellPointEnd(level, no_stance);
+}
+
+// shortcut for internal use
+void Scriptable::DirectlyCastSpell(Scriptable *target, ieResRef spellref, int level, int no_stance, bool deplete, bool instant, bool nointerrupt)
+{
+	SetSpellResRef(spellref);
+	CastSpell(target, deplete, instant, nointerrupt);
+	CastSpellEnd(level, no_stance);
+}
+
 //set target as point
 //if spell needs to be depleted, do it
 //if spell is illegal stop casting
@@ -1045,8 +1236,8 @@ int Scriptable::CastSpellPoint( const Point &target, bool deplete, bool instant,
 	Actor *actor = NULL;
 	if (Type == ST_ACTOR) {
 		actor = (Actor *) this;
-		if (actor->HandleCastingStance(SpellResRef, deplete) ) {
-			printMessage("Scriptable", "Spell not known or memorized, aborting cast!\n", LIGHT_RED);
+		if (actor->HandleCastingStance(SpellResRef, deplete, instant) ) {
+			Log(ERROR, "Scriptable", "Spell not known or memorized, aborting cast!");
 			return -1;
 		}
 	}
@@ -1063,6 +1254,9 @@ int Scriptable::CastSpellPoint( const Point &target, bool deplete, bool instant,
 	if(!CheckWildSurge()) {
 		return -1;
 	}
+	if (!instant) {
+		SpellcraftCheck(actor, SpellResRef);
+	}
 	return SpellCast(instant);
 }
 
@@ -1076,14 +1270,13 @@ int Scriptable::CastSpell( Scriptable* target, bool deplete, bool instant, bool
 	Actor *actor = NULL;
 	if (Type == ST_ACTOR) {
 		actor = (Actor *) this;
-		if (actor->HandleCastingStance(SpellResRef, deplete) ) {
-			printMessage("Scriptable", "Spell not known or memorized, aborting cast!\n", LIGHT_RED);
+		if (actor->HandleCastingStance(SpellResRef, deplete, instant) ) {
+			Log(ERROR, "Scriptable", "Spell not known or memorized, aborting cast!");
 			return -1;
 		}
 	}
 
-	// FIXME: fishy
-	if (!target) target = this;
+	assert(target);
 
 	if(!nointerrupt && !CanCast(SpellResRef)) {
 		SpellResRef[0] = 0;
@@ -1102,6 +1295,9 @@ int Scriptable::CastSpell( Scriptable* target, bool deplete, bool instant, bool
 		return -1;
 	}
 
+	if (!instant) {
+		SpellcraftCheck(actor, SpellResRef);
+	}
 	return SpellCast(instant);
 }
 
@@ -1145,8 +1341,9 @@ int Scriptable::SpellCast(bool instant)
 		//cfb
 		EffectQueue *fxqueue = spl->GetEffectBlock(this, this->Pos, -1, level);
 		fxqueue->SetOwner(actor);
-		if (!actor->Modified[IE_AVATARREMOVAL]) {
-			spl->AddCastingGlow(fxqueue, duration, actor->Modified[IE_SEX]);
+		if (!(actor->Modified[IE_AVATARREMOVAL] || instant)) {
+			ieDword gender = actor->GetCGGender();
+			spl->AddCastingGlow(fxqueue, duration, gender);
 		}
 		fxqueue->AddAllEffects(actor, actor->Pos);
 		delete fxqueue;
@@ -1156,6 +1353,7 @@ int Scriptable::SpellCast(bool instant)
 			// we have to remove it manually
 			actor->fxqueue.RemoveAllEffectsWithParam(fx_force_surge_modifier_ref, 1);
 		}
+		actor->ResetCommentTime();
 	}
 
 	gamedata->FreeSpell(spl, SpellResRef, false);
@@ -1192,15 +1390,13 @@ int Scriptable::CheckWildSurge()
 			int check = roll + caster->GetCasterLevel(spl->SpellType) + caster->Modified[IE_SURGEMOD];
 			if (caster->Modified[IE_CHAOSSHIELD]) {
 				//avert the surge and decrease the chaos shield counter
-				//FIXME: if 0 is also good, use that
-				check = 100;
+				check = 0;
 				caster->fxqueue.DecreaseParam1OfEffect(fx_chaosshield_ref,1);
 				displaymsg->DisplayConstantStringName(STR_CHAOSSHIELD,DMC_LIGHTGREY,caster);
 			}
 
-			// hundred or more means a normal cast
-			//FIXME: what happens if check is 0
-			if (check && (check < 100) ) {
+			// hundred or more means a normal cast; same for negative values (for absurd antisurge modifiers)
+			if ((check > 0) && (check < 100) ) {
 				// display feedback: Wild Surge: bla bla
 				char *s1 = core->GetString(displaymsg->GetStringReference(STR_WILDSURGE), 0);
 				char *s2 = core->GetString(core->SurgeSpells[check-1].message, 0);
@@ -1213,8 +1409,7 @@ int Scriptable::CheckWildSurge()
 
 				// lookup the spell in the "check" row of wildmag.2da
 				ieResRef surgeSpellRef;
-				memset(surgeSpellRef, 0, sizeof(surgeSpellRef));
-				strncpy(surgeSpellRef, core->SurgeSpells[check-1].spell, 8);
+				CopyResRef(surgeSpellRef, core->SurgeSpells[check-1].spell);
 
 				if (!gamedata->Exists(surgeSpellRef, IE_SPL_CLASS_ID)) {
 					// handle the hardcoded cases - they'll also fail here
@@ -1226,7 +1421,7 @@ int Scriptable::CheckWildSurge()
 				} else {
 					// finally change the spell
 					// the hardcoded bunch does it on its own when needed
-					strncpy(SpellResRef, surgeSpellRef, 8);
+					CopyResRef(SpellResRef, surgeSpellRef);
 				}
 			}
 		}
@@ -1295,17 +1490,17 @@ bool Scriptable::HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor
 			for (i=0; i<count; i++) {
 				if (target) {
 					caster->CastSpell(target, false, true);
-					strncpy(newspl, SpellResRef, 8);
+					CopyResRef(newspl, SpellResRef);
 					caster->WMLevelMod = tmp3;
-					caster->CastSpellEnd(level);
+					caster->CastSpellEnd(level, 1);
 				} else {
 					caster->CastSpellPoint(targetpos, false, true);
-					strncpy(newspl, SpellResRef, 8);
+					CopyResRef(newspl, SpellResRef);
 					caster->WMLevelMod = tmp3;
-					caster->CastSpellPointEnd(level);
+					caster->CastSpellPointEnd(level, 1);
 				}
 				// reset the ref, since CastSpell*End destroyed it
-				strncpy(SpellResRef, newspl, 8);
+				CopyResRef(SpellResRef, newspl);
 			}
 			caster->Modified[IE_FORCESURGE] = tmp;
 			caster->Modified[IE_SURGEMOD] = tmp2;
@@ -1332,7 +1527,7 @@ bool Scriptable::HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor
 				int id = core->Roll(1, spellCount, -1);
 				CREKnownSpell *ck = caster->spellbook.GetKnownSpell(i, lvl, id);
 				if (ck) {
-					strncpy(SpellResRef, ck->SpellResRef, 8);
+					CopyResRef(SpellResRef, ck->SpellResRef);
 					break;
 				}
 			}
@@ -1345,7 +1540,7 @@ bool Scriptable::HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor
 		default:
 			SpellHeader = -1;
 			SpellResRef[0] = 0;
-			printMessage("Scriptable", "New spell not found, aborting cast mid-surge!\n", LIGHT_RED);
+			Log(ERROR, "Scriptable", "New spell not found, aborting cast mid-surge!");
 			caster->SetStance(IE_ANI_READY);
 			return false;
 	}
@@ -1406,7 +1601,7 @@ bool Scriptable::TimerExpired(ieDword ID)
 void Scriptable::StartTimer(ieDword ID, ieDword expiration)
 {
 	if (ID>=MAX_TIMER) {
-		printMessage("Scriptable", "Timer id %d exceeded MAX_TIMER %d\n", RED,
+		Log(ERROR, "Scriptable", "Timer id %d exceeded MAX_TIMER %d",
 			ID, MAX_TIMER);
 		return;
 	}
@@ -1470,7 +1665,7 @@ void Selectable::DrawCircle(const Region &vp)
 		//if it is too fast, increase the 6 to 7
 		unsigned long step;
 		step = GetTickCount();
-		step = tp_steps [(step >> 6) & 7];
+		step = tp_steps [(step >> 7) & 7]*2;
 		mix.a = overColor.a;
 		mix.r = (overColor.r*step+selectedColor.r*(8-step))/8;
 		mix.g = (overColor.g*step+selectedColor.g*(8-step))/8;
@@ -1545,7 +1740,7 @@ void Selectable::SetCircle(int circlesize, const Color &color, Sprite2D* normal_
 }
 
 //used for creatures
-int Selectable::WantDither()
+int Selectable::WantDither() const
 {
 	//if dithering is disabled globally, don't do it
 	if (core->FogOfWar&FOG_DITHERSPRITES) {
@@ -1663,13 +1858,27 @@ bool Highlightable::TryUnlock(Actor *actor, bool removekey) {
 //detect this trap, using a skill, skill could be set to 256 for 'sure'
 //skill is the all around modified trap detection skill
 //a trapdetectiondifficulty of 100 means impossible detection short of a spell
-void Highlightable::DetectTrap(int skill)
+void Highlightable::DetectTrap(int skill, ieDword actorID)
 {
 	if (!CanDetectTrap()) return;
 	if (!Scripts[0]) return;
 	if ((skill>=100) && (skill!=256) ) skill = 100;
-	if (skill/2+core->Roll(1,skill/2,0)>TrapDetectionDiff) {
+	int check = 0;
+	if (third) {
+		//~Search (detect traps) check. Search skill %d vs. trap's difficulty %d (searcher's %d INT bonus).~
+		Actor *detective = core->GetGame()->GetActorByGlobalID(actorID);
+		int bonus = 0;
+		if (detective) {
+			bonus = detective->GetAbilityBonus(IE_INT);
+			displaymsg->DisplayRollStringName(39303, DMC_LIGHTGREY, detective, skill-bonus, TrapDetectionDiff, bonus);
+		}
+		check = (skill + bonus)*7;
+	} else {
+		check = skill/2 + core->Roll(1, skill/2, 0);
+	}
+	if (check > TrapDetectionDiff) {
 		SetTrapDetected(1); //probably could be set to the player #?
+		AddTrigger(TriggerEntry(trigger_detected, actorID));
 	}
 }
 
@@ -1752,7 +1961,7 @@ void Movable::SetStance(unsigned int arg)
 	//don't modify stance from dead back to anything if the actor is dead
 	if ((StanceID==IE_ANI_TWITCH || StanceID==IE_ANI_DIE) && (arg!=IE_ANI_TWITCH) ) {
 		if (GetInternalFlag()&IF_REALLYDIED) {
-			printMessage("Movable","Stance overridden by death\n", YELLOW);
+			Log(WARNING, "Movable", "Stance overridden by death");
 			return;
 		}
 	}
@@ -1783,7 +1992,7 @@ void Movable::SetStance(unsigned int arg)
 
 	} else {
 		StanceID=IE_ANI_AWAKE; //
-		print("Tried to set invalid stance id (%u)\n", arg);
+		print("Tried to set invalid stance id(%u)", arg);
 	}
 }
 
@@ -1794,18 +2003,15 @@ void Movable::SetAttackMoveChances(ieWord *amc)
 	AttackMovements[2]=amc[2];
 }
 
-
-
 //this could be used for WingBuffet as well
 void Movable::MoveLine(int steps, int Pass, ieDword orient)
 {
-	//remove previous path
-	ClearPath();
-	if (!steps)
+	if (path || !steps) {
 		return;
+	}
 	Point p = Pos;
 	p.x/=16;
-	p.y/=14;
+	p.y/=12;
 	path = area->GetLine( p, steps, orient, Pass );
 }
 
@@ -1824,6 +2030,24 @@ void AdjustPositionTowards(Point &Pos, ieDword time_diff, unsigned int walk_spee
 			( ( ( Pos.y - ( ( desty * 12 ) + 6 ) ) * ( time_diff ) ) / walk_speed );
 }
 
+unsigned char Movable::GetNextFace()
+{
+	//slow turning
+	if (timeStartStep==core->GetGame()->Ticks) {
+		return Orientation;
+	}
+	if (Orientation != NewOrientation) {
+		if ( ( (NewOrientation-Orientation) & (MAX_ORIENT-1) ) <= MAX_ORIENT/2) {
+			Orientation++;
+		} else {
+			Orientation--;
+		}
+		Orientation = Orientation&(MAX_ORIENT-1);
+	}
+
+	return Orientation;
+}
+
 // returns whether we made all pending steps (so, false if we must be called again this tick)
 // we can't just do them all here because the caller might have to update searchmap etc
 bool Movable::DoStep(unsigned int walk_speed, ieDword time)
@@ -1842,11 +2066,11 @@ bool Movable::DoStep(unsigned int walk_speed, ieDword time)
 		step = path;
 		timeStartStep = time;
 	} else if (step->Next && (( time - timeStartStep ) >= walk_speed)) {
-		//print("[New Step] : Orientation = %d\n", step->orient);
+		//print("[New Step] : Orientation = %d", step->orient);
 		step = step->Next;
 		timeStartStep = timeStartStep + walk_speed;
 	}
-	SetOrientation (step->orient, false);
+	SetOrientation (step->orient, true);
 	StanceID = IE_ANI_WALK;
 	if ((Type == ST_ACTOR) && (InternalFlags & IF_RUNNING)) {
 		StanceID = IE_ANI_RUN;
@@ -2009,9 +2233,9 @@ void Movable::RandomWalk(bool can_stop, bool run)
 	//p.x+=x;
 	//p.y+=(int) sqrt(100-x*x);
 
-	//selecting points in a square around actor
-	p.x+=core->Roll(1,50,-25);
-	p.y+=core->Roll(1,50,-25);
+	//selecting points in a square around actor (-24 to +24)
+	p.x+=core->Roll(1,49,-25);
+	p.y+=core->Roll(1,49,-25);
 	//the 5th parameter is controlling the orientation of the actor
 	//0 - back away, 1 - face direction
 	path = area->RunAway( Pos, p, size, 50, 1 );
@@ -2027,6 +2251,12 @@ void Movable::MoveTo(const Point &Des)
 	}
 }
 
+void Movable::Stop()
+{
+	Scriptable::Stop();
+	ClearPath();
+}
+
 void Movable::ClearPath()
 {
 	//this is to make sure attackers come to us
@@ -2089,3 +2319,5 @@ void TileObject::SetClosedTiles(unsigned short* Tiles, int cnt)
 	closedcount = cnt;
 }
 
+
+}
