@@ -108,6 +108,7 @@ class SpriteCover;
 #define IF_IDLE          0x100000
 //#define IF_PARTYRESTED   0x200000 //party rested trigger event
 #define IF_FORCEUPDATE   0x400000
+#define IF_TRIGGER_AP    0x800000
 
 //the actor should stop attacking
 #define IF_STOPATTACK (IF_JUSTDIED|IF_REALLYDIED|IF_CLEANUP|IF_IDLE)
@@ -307,6 +308,7 @@ public:
 	void Deactivate();
 	void PartyRested();
 	ieDword GetInternalFlag() const;
+	void SetInternalFlag(int value, int mode);
 	const char* GetScriptName() const;
 	Map* GetCurrentArea() const;
 	void SetMap(Map *map);
@@ -326,6 +328,7 @@ public:
 	Action* GetNextAction() const;
 	Action* PopNextAction();
 	void ClearActions();
+	virtual void Stop();
 	virtual void ReleaseCurrentAction();
 	bool InMove() const;
 	void ProcessActions();
@@ -344,6 +347,9 @@ public:
 	/* check for and trigger a wild surge */
 	int CheckWildSurge();
 	void SpellcraftCheck(const Actor *caster, const ieResRef SpellResRef);
+	/* internal spellcasting shortcuts */
+	void DirectlyCastSpellPoint(const Point &target, ieResRef spellref, int level, int no_stance, bool deplete, bool instant = false, bool nointerrupt = false);
+	void DirectlyCastSpell(Scriptable *target, ieResRef spellref, int level, int no_stance, bool deplete, bool instant = false, bool nointerrupt = false);
 	/* actor/scriptable casts spell */
 	int CastSpellPoint( const Point &Target, bool deplete, bool instant = false, bool nointerrupt = false );
 	int CastSpell( Scriptable* Target, bool deplete, bool instant = false, bool nointerrupt = false );
@@ -449,11 +455,12 @@ private: //these seem to be sensitive, so get protection
 
 	PathNode* path; //whole path
 	PathNode* step; //actual step
+protected:
+	ieDword timeStartStep;
 public:
 	Movable(ScriptableType type);
 	virtual ~Movable(void);
 	Point Destination;
-	ieDword timeStartStep;
 	Sprite2D* lastFrame;
 	ieResRef Area;
 public:
@@ -487,7 +494,7 @@ public:
 
 	void SetStance(unsigned int arg);
 	void SetAttackMoveChances(ieWord *amc);
-	bool DoStep(unsigned int walk_speed, ieDword time = 0);
+	virtual bool DoStep(unsigned int walk_speed, ieDword time = 0);
 	void AddWayPoint(const Point &Des);
 	void RunAwayFrom(const Point &Des, int PathLength, int flags);
 	void RandomWalk(bool can_stop, bool run);
@@ -495,11 +502,12 @@ public:
 	void FixPosition();
 	void WalkTo(const Point &Des, int MinDistance = 0);
 	void MoveTo(const Point &Des);
+	void Stop();
 	void ClearPath();
+
 	/* returns the most likely position of this actor */
 	Point GetMostLikelyPosition();
 	virtual bool BlocksSearchMap() const = 0;
-
 };
 
 //Tiled objects are not used (and maybe not even implemented correctly in IE)
