@@ -20,15 +20,20 @@
 
 #include "Spellbook.h"
 
+#include "ActorMgr.h"
 #include "GameData.h"
 #include "Interface.h"
+#include "PluginMgr.h"
 #include "Projectile.h"
 #include "Spell.h"
 #include "TableMgr.h"
 #include "Scriptable/Actor.h"
+#include "System/StringBuffer.h"
 
 #include <cstdio>
 
+namespace GemRB {
+
 static bool SBInitialized = false;
 static int NUM_BOOK_TYPES = 3;
 static bool IWD2Style = false;
@@ -45,6 +50,12 @@ Spellbook::Spellbook()
 		InitializeSpellbook();
 	}
 	spells = new std::vector<CRESpellMemorization*> [NUM_BOOK_TYPES];
+	sorcerer = 0;
+	if (IWD2Style) {
+		innate = 1<<IE_IWD2_SPELL_INNATE;
+	} else {
+		innate = 1<<IE_SPELL_TYPE_INNATE;
+	}
 }
 
 void Spellbook::InitializeSpellbook()
@@ -53,8 +64,10 @@ void Spellbook::InitializeSpellbook()
 		SBInitialized=true;
 		if (core->HasFeature(GF_HAS_SPELLLIST)) {
 			NUM_BOOK_TYPES=NUM_IWD2_SPELLTYPES; //iwd2 spell types
+			IWD2Style = true;
 		} else {
 			NUM_BOOK_TYPES=NUM_SPELLTYPES; //bg/pst/iwd1 spell types
+			IWD2Style = false;
 		}
 	}
 	return;
@@ -120,8 +133,8 @@ void Spellbook::CopyFrom(const Actor *source)
 			CRESpellMemorization *sm = new CRESpellMemorization();
 			spells[t].push_back(sm);
 			sm->Level = wm->Level;
-			sm->Number = wm->Number;
-			sm->Number2 = wm->Number2;
+			sm->SlotCount = wm->SlotCount;
+			sm->SlotCountWithBonus = wm->SlotCountWithBonus;
 			sm->Type = wm->Type;
 			for (k = 0; k < wm->known_spells.size(); k++) {
 				CREKnownSpell *tmp_known = new CREKnownSpell();
@@ -141,21 +154,65 @@ void Spellbook::CopyFrom(const Actor *source)
 
 //ITEM, SPPR, SPWI, SPIN, SPCL
 int sections[]={3,0,1,2,2};
+// domain spells are of all types, so look them up in all cases
+// ignore songs and shapes altogether
+int arcanetypes[] = {IE_IWD2_SPELL_BARD, IE_IWD2_SPELL_SORCERER, IE_IWD2_SPELL_WIZARD, IE_IWD2_SPELL_DOMAIN};
+int divinetypes[] = {IE_IWD2_SPELL_CLERIC, IE_IWD2_SPELL_DRUID, IE_IWD2_SPELL_PALADIN, IE_IWD2_SPELL_RANGER, IE_IWD2_SPELL_DOMAIN};
+int *alltypes[2] = {divinetypes, arcanetypes};
+
+int inline GetType(int spellid, unsigned int &bookcount, int &idx)
+{
+	int type = spellid/1000;
+	if (type>4) {
+		return -1;
+	}
+	if (IWD2Style) {
+		if (type == 1) {
+			// check divine
+			idx = 0;
+			bookcount = sizeof(divinetypes)/sizeof(int);
+		} else if (type == 2) {
+			// check arcane
+			idx = 1;
+			bookcount = sizeof(arcanetypes)/sizeof(int);
+		} else if (type == 3) {
+			type = IE_IWD2_SPELL_INNATE;
+		}
+	} else {
+		type = sections[type];
+		if (type >= NUM_BOOK_TYPES) {
+			return -1;
+		}
+	}
+	return type;
+}
 
 //flags bits
 // 1 - unmemorize it
 bool Spellbook::HaveSpell(int spellid, ieDword flags)
 {
-	int type = spellid/1000;
-	if (type>4) {
-		return false;
-	}
-	type = sections[type];
-	if (type >= NUM_BOOK_TYPES) {
+	int idx = -1;
+	unsigned int bookcount;
+	int type = GetType(spellid, bookcount, idx);
+	if (type == -1) {
 		return false;
 	}
+
 	spellid = spellid % 1000;
 
+	if (idx == -1) {
+		return HaveSpell(spellid, type, flags);
+	} else {
+		for (unsigned int book = 0; book < bookcount; book++) {
+			if (HaveSpell(spellid, alltypes[idx][book], flags)) {
+				return true;
+			}
+		}
+	}
+	return false;
+}
+bool Spellbook::HaveSpell(int spellid, int type, ieDword flags)
+{
 	for (unsigned int j = 0; j < GetSpellLevelCount(type); j++) {
 		CRESpellMemorization* sm = spells[type][j];
 		for (unsigned int k = 0; k < sm->memorized_spells.size(); k++) {
@@ -175,21 +232,64 @@ bool Spellbook::HaveSpell(int spellid, ieDword flags)
 	return false;
 }
 
-bool Spellbook::KnowSpell(int spellid)
+//returns count of memorized spells of a given name/type
+int Spellbook::CountSpells(const char *resref, unsigned int type, int flag) const
 {
-	int type = spellid/1000;
-	if (type>4) {
-		return false;
+	int i, max;
+	int count = 0;
+
+	if (type==0xffffffff) {
+		i=0;
+		max = NUM_BOOK_TYPES;
+	} else {
+		i = type;
+		max = i+1;
 	}
-	type = sections[type];
-	if (type >= NUM_BOOK_TYPES) {
+
+	while(i < max) {
+		for (unsigned int j = 0; j < spells[i].size(); j++) {
+			CRESpellMemorization* sm = spells[i][j];
+			for (unsigned int k = 0; k < sm->memorized_spells.size(); k++) {
+				CREMemorizedSpell* ms = sm->memorized_spells[k];
+				if (resref[0] && !stricmp(ms->SpellResRef, resref) ) {
+					if (flag || ms->Flags) {
+						count++;
+					}
+				}
+			}
+		}
+		i++;
+	}
+	return count;
+}
+
+bool Spellbook::KnowSpell(int spellid) const
+{
+	int idx = -1;
+	unsigned int bookcount;
+	int type = GetType(spellid, bookcount, idx);
+	if (type == -1) {
 		return false;
 	}
 	spellid = spellid % 1000;
 
+	if (idx == -1) {
+		return KnowSpell(spellid, type);
+	} else {
+		for (unsigned int book = 0; book < bookcount; book++) {
+			if (KnowSpell(spellid, alltypes[idx][book])) {
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+bool Spellbook::KnowSpell(int spellid, int type) const
+{
 	for (unsigned int j = 0; j < GetSpellLevelCount(type); j++) {
 		CRESpellMemorization* sm = spells[type][j];
-		for (unsigned int k = 0; k < sm->memorized_spells.size(); k++) {
+		for (unsigned int k = 0; k < sm->known_spells.size(); k++) {
 			CREKnownSpell* ks = sm->known_spells[k];
 			if (atoi(ks->SpellResRef+4)==spellid) {
 				return true;
@@ -200,12 +300,12 @@ bool Spellbook::KnowSpell(int spellid)
 }
 
 //if resref=="" then it is a knownanyspell
-bool Spellbook::KnowSpell(const char *resref)
+bool Spellbook::KnowSpell(const char *resref) const
 {
 	for (int i = 0; i < NUM_BOOK_TYPES; i++) {
 		for (unsigned int j = 0; j < spells[i].size(); j++) {
 			CRESpellMemorization* sm = spells[i][j];
-			for (unsigned int k = 0; k < sm->memorized_spells.size(); k++) {
+			for (unsigned int k = 0; k < sm->known_spells.size(); k++) {
 				CREKnownSpell* ks = sm->known_spells[k];
 				if (resref[0] && stricmp(ks->SpellResRef, resref) ) {
 					continue;
@@ -343,15 +443,26 @@ bool Spellbook::RemoveSpell(CREKnownSpell* spell)
 //IWD2 clab files use it
 void Spellbook::RemoveSpell(int spellid)
 {
-	int type = spellid/1000;
-	if (type>4) {
-		return;
-	}
-	type = sections[type];
-	if (type >= NUM_BOOK_TYPES) {
+	int idx = -1;
+	unsigned int bookcount;
+	int type = GetType(spellid, bookcount, idx);
+	if (type == -1) {
 		return;
 	}
 	spellid = spellid % 1000;
+
+	if (idx == -1) {
+		RemoveSpell(spellid, type);
+	} else {
+		for (unsigned int book = 0; book < bookcount; book++) {
+			RemoveSpell(spellid, alltypes[idx][book]);
+		}
+	}
+
+}
+
+void Spellbook::RemoveSpell(int spellid, int type)
+{
 	std::vector< CRESpellMemorization* >::iterator sm;
 	for (sm = spells[type].begin(); sm != spells[type].end(); sm++) {
 		std::vector< CREKnownSpell* >::iterator ks;
@@ -405,30 +516,30 @@ void Spellbook::SetBookType(int bt)
 //wildshapes are marked as innate, they need some hack to get stored
 //in the right group
 //the rest are stored as innate
-int Spellbook::GetSpellType(int spelltype)
-{
-	if (IWD2Style) return spelltype;
 
-	if (spelltype<6) {
-		return spelltypes[spelltype];
-	}
-	return IE_SPELL_TYPE_INNATE;
-}
-
-int Spellbook::LearnSpell(Spell *spell, int memo)
+int Spellbook::LearnSpell(Spell *spell, int memo, unsigned int clsmsk, unsigned int kit)
 {
 	CREKnownSpell *spl = new CREKnownSpell();
-	strncpy(spl->SpellResRef, spell->Name, 8);
-	spl->Type = (ieWord) GetSpellType(spell->SpellType);
-	if ( spl->Type == IE_SPELL_TYPE_INNATE) {
-		spl->Level = 0;
+	CopyResRef(spl->SpellResRef, spell->Name);
+	spl->Level = 0;
+	if (IWD2Style) {
+		PluginHolder<ActorMgr> gm(IE_CRE_CLASS_ID);
+		spl->Type = gm->FindSpellType(spell->Name, spl->Level, clsmsk, kit);
+		return spell->SpellLevel;
 	}
-	else {
-		spl->Level = (ieWord) (spell->SpellLevel-1);
+
+	//not IWD2
+	if (spell->SpellType<6) {
+		spl->Type = spelltypes[spell->SpellType];
+		spl->Level = spell->SpellLevel-1;
+	} else {
+		spl->Type = IE_SPELL_TYPE_INNATE;
 	}
+
 	bool ret=AddKnownSpell(spl, memo);
 	if (!ret) {
 		delete spl;
+		return 0;
 	}
 	return spell->SpellLevel; // return only the spell level (xp is based on xpbonus)
 }
@@ -445,7 +556,7 @@ bool Spellbook::AddKnownSpell(CREKnownSpell *spl, int flg)
 		CRESpellMemorization *sm = new CRESpellMemorization();
 		sm->Type = (ieWord) type;
 		sm->Level = (ieWord) level;
-		sm->Number = sm->Number2 = 0;
+		sm->SlotCount = sm->SlotCountWithBonus = 0;
 		if ( !AddSpellMemorization(sm) ) {
 			delete sm;
 			return false;
@@ -454,8 +565,8 @@ bool Spellbook::AddKnownSpell(CREKnownSpell *spl, int flg)
 
 	spells[type][level]->known_spells.push_back(spl);
 	if (type==IE_SPELL_TYPE_INNATE) {
-		spells[type][level]->Number++;
-		spells[type][level]->Number2++;
+		spells[type][level]->SlotCount++;
+		spells[type][level]->SlotCountWithBonus++;
 	}
 	if (flg) {
 		MemorizeSpell(spl, true);
@@ -559,7 +670,7 @@ bool Spellbook::AddSpellMemorization(CRESpellMemorization* sm)
 		CRESpellMemorization *newsm = new CRESpellMemorization();
 		newsm->Type = sm->Type;
 		newsm->Level = (ieWord) s->size();
-		newsm->Number = newsm->Number2 = 0;
+		newsm->SlotCount = newsm->SlotCountWithBonus = 0;
 		s->push_back( newsm );
 	}
 
@@ -577,7 +688,10 @@ void Spellbook::BonusSpells(int type, int count, int *bonuses)
 	if (level>count) level=count;
 	for (int i = 0; i < level; i++) {
 		CRESpellMemorization* sm = GetSpellMemorization(type, i);
-		sm->Number2+=bonuses[i];
+		// don't give access to new spell levels through these boni
+		if (sm->SlotCountWithBonus) {
+			sm->SlotCountWithBonus+=bonuses[i];
+		}
 	}
 }
 
@@ -591,7 +705,7 @@ void Spellbook::ClearBonus()
 		int level = GetSpellLevelCount(type);
 		for (int i = 0; i < level; i++) {
 			CRESpellMemorization* sm = GetSpellMemorization(type, i);
-			sm->Number2=sm->Number;
+			sm->SlotCountWithBonus=sm->SlotCount;
 		}
 	}
 }
@@ -606,7 +720,7 @@ CRESpellMemorization *Spellbook::GetSpellMemorization(unsigned int type, unsigne
 		sm = new CRESpellMemorization();
 		sm->Type = (ieWord) type;
 		sm->Level = (ieWord) level;
-		sm->Number = sm->Number2 = 0;
+		sm->SlotCount = sm->SlotCountWithBonus = 0;
 		if ( !AddSpellMemorization(sm) ) {
 			delete sm;
 			return NULL;
@@ -632,14 +746,14 @@ void Spellbook::SetMemorizableSpellsCount(int Value, int type, unsigned int leve
 	CRESpellMemorization* sm = GetSpellMemorization(type, level);
 	if (bonus) {
 		if (!Value) {
-			Value=sm->Number;
+			Value=sm->SlotCount;
 		}
-		sm->Number2=(ieWord) (sm->Number2+Value);
+		sm->SlotCountWithBonus=(ieWord) (sm->SlotCountWithBonus+Value);
 	}
 	else {
-		diff=sm->Number2-sm->Number;
-		sm->Number=(ieWord) Value;
-		sm->Number2=(ieWord) (Value+diff);
+		diff=sm->SlotCountWithBonus-sm->SlotCount;
+		sm->SlotCount=(ieWord) Value;
+		sm->SlotCountWithBonus=(ieWord) (Value+diff);
 	}
 }
 
@@ -649,21 +763,22 @@ int Spellbook::GetMemorizableSpellsCount(int type, unsigned int level, bool bonu
 		return 0;
 	CRESpellMemorization* sm = spells[type][level];
 	if (bonus)
-		return sm->Number2;
-	return sm->Number;
+		return sm->SlotCountWithBonus;
+	return sm->SlotCount;
 }
 
 bool Spellbook::MemorizeSpell(CREKnownSpell* spell, bool usable)
 {
-	CRESpellMemorization* sm = spells[spell->Type][spell->Level];
-	if (sm->Number2 <= sm->memorized_spells.size()) {
+	ieWord spellType = spell->Type;
+	CRESpellMemorization* sm = spells[spellType][spell->Level];
+	if (sm->SlotCountWithBonus <= sm->memorized_spells.size() && !(innate & (1<<spellType))) {
 		//it is possible to have sorcerer type spellbooks for any spellbook type
-		if (! (sorcerer & (1<<spell->Type) ) )
+		if (! (sorcerer & (1<<spellType) ) )
 			return false;
 	}
 
 	CREMemorizedSpell* mem_spl = new CREMemorizedSpell();
-	strncpy( mem_spl->SpellResRef, spell->SpellResRef, 8 );
+	CopyResRef( mem_spl->SpellResRef, spell->SpellResRef );
 	mem_spl->Flags = usable ? 1 : 0; // FIXME: is it all it's used for?
 
 	sm->memorized_spells.push_back( mem_spl );
@@ -691,7 +806,7 @@ bool Spellbook::UnmemorizeSpell(CREMemorizedSpell* spell)
 	return false;
 }
 
-bool Spellbook::UnmemorizeSpell(const ieResRef ResRef, bool deplete)
+bool Spellbook::UnmemorizeSpell(const ieResRef ResRef, bool deplete, bool onlydepleted)
 {
 	for (int type = 0; type<NUM_BOOK_TYPES; type++) {
 		std::vector< CRESpellMemorization* >::iterator sm;
@@ -701,6 +816,9 @@ bool Spellbook::UnmemorizeSpell(const ieResRef ResRef, bool deplete)
 				if (strnicmp(ResRef, (*s)->SpellResRef, sizeof(ieResRef) ) ) {
 					continue;
 				}
+				if (onlydepleted && (*s)->Flags != 0) {
+					continue;
+				}
 				if (deplete) {
 					(*s)->Flags = 0;
 				} else {
@@ -718,7 +836,7 @@ bool Spellbook::UnmemorizeSpell(const ieResRef ResRef, bool deplete)
 
 //bitfield disabling type: 1 - mage, 2 - cleric etc
 //level: if set, then finds that level only
-CREMemorizedSpell* Spellbook::FindUnchargedSpell(int type, int level)
+CREMemorizedSpell* Spellbook::FindUnchargedSpell(int type, int level) const
 {
 	int mask=1;
 
@@ -758,7 +876,7 @@ void Spellbook::CreateSorcererMemory(int type)
 		sm->memorized_spells.clear();
 		for (unsigned int k = 0; k < sm->known_spells.size(); k++) {
 			CREKnownSpell *ck = sm->known_spells[k];
-			cnt = sm->Number2;
+			cnt = sm->SlotCountWithBonus;
 			while(cnt--) {
 				MemorizeSpell(ck, true);
 			}
@@ -926,18 +1044,25 @@ unsigned int Spellbook::GetSpellInfoSize(int type)
 	return count;
 }
 
-bool Spellbook::FindSpellInfo(SpellExtHeader *array, ieResRef spellname)
+//type = 0 means any type
+int Spellbook::FindSpellInfo(SpellExtHeader *array, const ieResRef spellname, unsigned int type)
 {
 	memset(array, 0, sizeof(SpellExtHeader) );
 	if (spellinfo.size() == 0) {
 		GenerateSpellInfo();
 	}
+	int offset = 0;
 	for (unsigned int i = 0; i<spellinfo.size(); i++) {
+		// take the offset into account, since we need per-type indices
+		if (type && !(1<<(spellinfo[i]->type) & type)) {
+			offset++;
+			continue;
+		}
 		if (strnicmp(spellinfo[i]->spellname, spellname, sizeof(ieResRef) ) ) continue;
 		memcpy(array, spellinfo[i], sizeof(SpellExtHeader));
-		return true;
+		return i-offset+1;
 	}
-	return false;
+	return 0;
 }
 
 SpellExtHeader *Spellbook::FindSpellInfo(unsigned int level, unsigned int type, const ieResRef spellname)
@@ -955,7 +1080,7 @@ SpellExtHeader *Spellbook::FindSpellInfo(unsigned int level, unsigned int type,
 
 void Spellbook::AddSpellInfo(unsigned int sm_level, unsigned int sm_type, const ieResRef spellname, unsigned int idx)
 {
-	Spell *spl = gamedata->GetSpell(spellname);
+	Spell *spl = gamedata->GetSpell(spellname, true);
 	if (!spl)
 		return;
 	if (spl->ExtHeaderCount<1)
@@ -1018,7 +1143,7 @@ void Spellbook::SetCustomSpellInfo(ieResRef *data, ieResRef spell, int type)
 					if (!slot)
 						continue;
 					//skip the spell itself
-					if (!strnicmp(slot->SpellResRef, spell, sizeof(ieResRef)))
+					if (spell && !strnicmp(slot->SpellResRef, spell, sizeof(ieResRef)))
 						continue;
 					AddSpellInfo(sm->Level, sm->Type, slot->SpellResRef, -1);
 				}
@@ -1047,32 +1172,41 @@ void Spellbook::GenerateSpellInfo()
 	}
 }
 
-void Spellbook::dump()
+void Spellbook::dump() const
+{
+	StringBuffer buffer;
+	dump(buffer);
+	Log(DEBUG, "Spellbook", buffer);
+}
+
+void Spellbook::dump(StringBuffer& buffer) const
 {
 	unsigned int k;
 
-	print( "SPELLBOOK:\n" );
+	buffer.append( "SPELLBOOK:\n" );
 	for (int i = 0; i < NUM_BOOK_TYPES; i++) {
 		for (unsigned int j = 0; j < spells[i].size(); j++) {
 			CRESpellMemorization* sm = spells[i][j];
 
 			if (sm->known_spells.size())
-				print( " Known spells:\n" );
+				buffer.append( " Known spells:\n" );
 			for (k = 0; k < sm->known_spells.size(); k++) {
 				CREKnownSpell* spl = sm->known_spells[k];
 				if (!spl) continue;
 
-				print ( " %2d: %8s L: %d T: %d\n", k, spl->SpellResRef, spl->Level, spl->Type );
+				buffer.appendFormatted ( " %2d: %8s L: %d T: %d\n", k, spl->SpellResRef, spl->Level, spl->Type );
 			}
 
 			if (sm->memorized_spells.size())
-				print( " Memorized spells:\n" );
+				buffer.append( " Memorized spells:\n" );
 			for (k = 0; k < sm->memorized_spells.size (); k++) {
 				CREMemorizedSpell* spl = sm->memorized_spells[k];
 				if (!spl) continue;
 
-				print ( " %2u: %8s %x\n", k, spl->SpellResRef, spl->Flags );
+				buffer.appendFormatted ( " %2u: %8s %x\n", k, spl->SpellResRef, spl->Flags );
 			}
 		}
 	}
 }
+
+}
