@@ -31,6 +31,8 @@
 #include "Plugin.h"
 #include "System/DataStream.h"
 
+namespace GemRB {
+
 /**
  * @struct StringBlock
  * Text and its associated sound.
@@ -52,11 +54,13 @@ public:
 	virtual ~StringMgr(void);
 	virtual void OpenAux() = 0;
 	virtual void CloseAux() = 0;
-	virtual bool Open(DataStream* stream, bool autoFree = true) = 0;
+	virtual bool Open(DataStream* stream) = 0;
 	virtual char* GetString(ieStrRef strref, unsigned int flags = 0) = 0;
 	virtual StringBlock GetStringBlock(ieStrRef strref, unsigned int flags = 0) = 0;
 	virtual void FreeString(char *str) = 0;
 	virtual ieStrRef UpdateString(ieStrRef strref, const char *text) = 0;
 };
 
+}
+
 #endif  // ! STRINGMGR_H
