@@ -26,6 +26,8 @@
 
 #include "Animation.h"
 
+namespace GemRB {
+
 class GEM_EXPORT Tile {
 public:
 	Tile(Animation* anim, Animation* sec = NULL);
@@ -39,4 +41,6 @@ public:
 	Animation* anim[2];
 };
 
+}
+
 #endif
