@@ -30,6 +30,8 @@
 #ifndef IE_STATS_H
 #define IE_STATS_H
 
+namespace GemRB {
+
 //EA values
 #define EA_INANIMATE   		1
 #define EA_PC  			2
@@ -70,6 +72,7 @@
 #define SEX_SUMMON    6
 #define SEX_ILLUSION  7 // bg2
 #define SEX_EXTRA     8 // bg2
+#define SEX_SUMMON_DEMON 9 //bg2
 #define SEX_EXTRA2    0xa // ToB
 #define SEX_MAXEXTRA  0x12 // ToB (extra10)
 
@@ -139,9 +142,9 @@
 #define STATE_EMBALM     0x40000000
 #define STATE_CONFUSED   0x80000000
 
-#define STATE_STILL      (STATE_STUNNED | STATE_FROZEN | STATE_PETRIFIED) //not animated
+#define STATE_STILL      (STATE_STUNNED | STATE_FROZEN | STATE_PETRIFIED) //0xc8: not animated
 
-#define STATE_CANTMOVE   0x80180fef
+#define STATE_CANTMOVE   0x180fef   //can't walk or attack - confused characters can do these
 #define STATE_CANTLISTEN 0x80080fef
 #define STATE_CANTSTEAL  0x00180fc0 //can't steal from
 #define STATE_CANTSEE    0x00080fc0 //can't explore (even itself)
@@ -190,11 +193,13 @@
 #define MC_FALLEN_PALADIN	0x0200
 #define MC_FALLEN_RANGER	0x0400
 #define MC_EXPORTABLE           0x0800
-#define MC_HIDE_HP              0x1000 
-#define MC_PLOT_CRITICAL        0x2000  //if dies, it means game over
+#define MC_HIDE_HP              0x1000  //also 'large creature' according to IE dev info
+#define MC_PLOT_CRITICAL        0x2000  //if dies, it means game over (IWD2)
+#define MC_LARGE_CREATURE       0x2000  //creature is subject to alternative melee damage - semi invulnerability (BG2)
+#define MC_LIMBO_CREATURE       0x4000
 #define MC_BEENINPARTY          0x8000
-#define MC_HIDDEN               0x10000 //iwd
-
+#define MC_SEENPARTY            0x10000 //iwd2
+#define MC_INVULNERABLE         0x20000 //iwd2
 #define MC_NO_TALK              0x80000 //ignore dialoginterrupt
 
 //stats
@@ -341,6 +346,7 @@
 #define IE_PROFICIENCYSLING			107
 #define IE_PROFICIENCYBLACKJACK			108
 #define IE_PROFICIENCYGUN			109
+#define IE_UNDEADLEVEL                          109 //if i calculated correctly, this is the 21th byte in the profs array
 #define IE_PROFICIENCYMARTIALARTS		110
 #define IE_PROFICIENCY2HANDED		   	111 
 #define IE_PROFICIENCYSWORDANDSHIELD		112
@@ -388,7 +394,7 @@
 #define IE_PUPPETID 				  141
 #define IE_CHECKFORBERSERK  		  142
 #define IE_BERSERKSTAGE1			  143
-#define IE_BERSERKSTAGE2			  144
+#define IE_BERSERKSTAGE2			  144 //attack anyone
 #define IE_DAMAGELUCK   			  145
 #define IE_CRITICALHITBONUS 		  146
 #define IE_VISUALRANGE  			  147
@@ -494,7 +500,8 @@
 #define IE_TEAM			220
 #define IE_FACTION		221
 #define IE_SUBRACE		222
-#define IE_SPECIES              223
+#define IE_SPECIES              223 // pst specific
+#define IE_UNUSED_SKILLPTS 223 // iwd2 specific
 //temporarily here for iwd2
 #define IE_HATEDRACE2       224
 #define IE_HATEDRACE3       225
@@ -521,11 +528,12 @@
 #define IE_LEVELMONK             244
 #define IE_LEVELPALADIN          245
 #define IE_LEVELRANGER           246
-#define IE_LEVELSORCEROR         247
+#define IE_LEVELSORCERER         247
 // place for 2 more classes
 #define IE_LEVELCLASS12          248
 #define IE_LEVELCLASS13          249
 // these are iwd2 spell states, iwd2 uses ~180, we have place for 192
+// TODO: consider dropping these (move them to class variable) if unused by guiscript
 #define IE_SPLSTATE_ID1          250
 #define IE_SPLSTATE_ID2          251
 #define IE_SPLSTATE_ID3          252
@@ -533,4 +541,6 @@
 #define IE_SPLSTATE_ID5          254
 #define IE_SPLSTATE_ID6          255
 
+}
+
 #endif  // ! IE_STATS_H
