@@ -37,7 +37,6 @@
 #include "PluginMgr.h"
 #include "ProjectileServer.h"
 #include "TileMapMgr.h"
-#include "Video.h"
 #include "GameScript/GameScript.h"
 #include "Scriptable/Container.h"
 #include "Scriptable/Door.h"
@@ -176,6 +175,23 @@ bool AREImporter::Open(DataStream* stream)
 	str->ReadWord( &WFog );
 	str->ReadWord( &WLightning );
 	str->ReadWord( &WUnknown );
+	AreaDifficulty = 0;
+	if (bigheader) {
+		// are9.1 difficulty bits for level2/level3
+		// TODO: there must be more advanced logic here for sure (guessing vs party level)
+		// ar4000 for example has a bunch of actors for all area difficulty levels, so these here are likely just the allowed levels
+		AreaDifficulty = 1;
+		ieByte tmp = 0;
+		str->Read( &tmp, 1);
+		if (tmp) {
+			AreaDifficulty = 2;
+		}
+		tmp = 0;
+		str->Read( &tmp, 1);
+		if (tmp) {
+			AreaDifficulty = 4;
+		}
+	}
 	//bigheader gap is here
 	str->Seek( 0x54 + bigheader, GEM_STREAM_START );
 	str->ReadDword( &ActorOffset );
@@ -333,6 +349,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	map->Lightning = WLightning;
 	map->AreaType = AreaType;
 	map->DayNight = day_or_night;
+	map->AreaDifficulty = AreaDifficulty;
 	strnlwrcpy( map->WEDResRef, WEDResRef, 8);
 	strnlwrcpy( map->Dream[0], Dream1, 8);
 	strnlwrcpy( map->Dream[1], Dream2, 8);
@@ -497,18 +514,18 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		//maybe we have to store this
 		str->Seek( 36, GEM_CURRENT_POS );
 
-		if (core->HasFeature(GF_CLEAR_UNUSED_AREA)) {
-			memset( WavResRef, 0, sizeof(WavResRef));
-			TalkX = 0;
-			TalkY = 0;
-			DialogName = -1;
-			memset( DialogResRef, 0, sizeof(DialogResRef));
-		}	else {
+		if (core->HasFeature(GF_INFOPOINT_DIALOGS)) {
 			str->ReadResRef( WavResRef );
 			str->ReadWord( &TalkX);
 			str->ReadWord( &TalkY);
 			str->ReadDword( &DialogName );
 			str->ReadResRef( DialogResRef );
+		} else {
+			memset(WavResRef, 0, sizeof(WavResRef));
+			TalkX = 0;
+			TalkY = 0;
+			DialogName = -1;
+			memset(DialogResRef, 0, sizeof(DialogResRef));
 		}
 
 		char* string = core->GetString( StrRef );
@@ -550,6 +567,11 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		memcpy( ip->KeyResRef, KeyResRef, sizeof(KeyResRef) );
 
 		//these appear only in PST, but we could support them everywhere
+		// HOWEVER they did not use them as witnessed in ar0101 (0101prt1 and 0101prt2) :(
+		if (core->HasFeature(GF_PST_STATE_FLAGS)) {
+			TalkX = ip->Pos.x;
+			TalkY = ip->Pos.y;
+		}
 		ip->TalkPos.x=TalkX;
 		ip->TalkPos.y=TalkY;
 		ip->DialogName=DialogName;
@@ -943,10 +965,12 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			ieResRef CreResRef;
 			ieDword TalkCount;
 			ieDword Orientation, Schedule, RemovalTime;
-			ieWord XPos, YPos, XDes, YDes;
+			ieWord XPos, YPos, XDes, YDes, Spawned;
 			ieResRef Dialog;
 			ieResRef Scripts[8]; //the original order
 			ieDword Flags;
+			ieByte DifficultyMargin;
+
 			str->Read( DefaultName, 32);
 			DefaultName[32]=0;
 			str->ReadWord( &XPos );
@@ -954,7 +978,10 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			str->ReadWord( &XDes );
 			str->ReadWord( &YDes );
 			str->ReadDword( &Flags );
-			str->Seek( 8, GEM_CURRENT_POS );
+			str->ReadWord( &Spawned );
+			str->Seek( 1, GEM_CURRENT_POS ); // one letter of a ResRef, TODO: purpose unknown (portraits?)
+			str->Read( &DifficultyMargin, 1 ); // iwd2 only
+			str->Seek( 4, GEM_CURRENT_POS ); //actor animation, unused
 			str->ReadDword( &Orientation );
 			str->ReadDword( &RemovalTime );
 			str->Seek( 4, GEM_CURRENT_POS );
@@ -1004,13 +1031,15 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			ab->Destination.y = YPos;
 			ab->HomeLocation.x = XDes;
 			ab->HomeLocation.y = YDes;
+			ab->Spawned = Spawned;
+			ab->appearance = Schedule;
 			//copying the scripting name into the actor
 			//if the CreatureAreaFlag was set to 8
 			if ((Flags&AF_NAME_OVERRIDE) || (core->HasFeature(GF_IWD2_SCRIPTNAME)) ) {
 				ab->SetScriptName(DefaultName);
 			}
 			//IWD2 specific hacks
-			if (core->HasFeature(GF_IWD2_SCRIPTNAME)) {
+			if (core->HasFeature(GF_3ED_RULES)) {
 				//This flag is used for something else in IWD2
 				if (Flags&AF_NAME_OVERRIDE) {
 					ab->BaseStats[IE_EA]=EA_EVILCUTOFF;
@@ -1021,7 +1050,18 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 				if (Flags&AF_INVULNERABLE) {
 					ab->SetMCFlag(MC_INVULNERABLE,BM_OR);
 				}
+				if (!(Flags&AF_ENABLED)) {
+					// DifficultyMargin - only enable actors that are difficult enough vs the area difficulty
+					// 1 - area difficulty 1
+					// 2 - area difficulty 2
+					// 4 - area difficulty 3
+					if (DifficultyMargin && !(DifficultyMargin & map->AreaDifficulty)) {
+						// iwd2 has GF_START_ACTIVE off, but that only touches IF_IDLE
+						ab->appearance = 0;
+					}
+				}
 			}
+			ab->DifficultyMargin = DifficultyMargin;
 
 			if (Dialog[0]) {
 				ab->SetDialog(Dialog);
@@ -1032,7 +1072,6 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 				}
 			}
 			ab->SetOrientation( Orientation,0 );
-			ab->appearance = Schedule;
 			ab->TalkCount = TalkCount;
 			// TODO: remove corpse at removal time?
 			ab->RemovalTime = RemovalTime;
@@ -1495,8 +1534,18 @@ int AREImporter::PutHeader(DataStream *stream, Map *map)
 	stream->WriteWord( &map->Lightning);
 	stream->WriteWord( &tmpWord);
 
-	if (map->version==16) { //writing 16 bytes of 0's
-		stream->Write( Signature, 8);
+	if (map->version == 16) { //writing 14 bytes of 0's
+		char tmp[1] = { '0' };
+		if (map->AreaDifficulty == 2) {
+			tmp[0] = 1;
+		}
+		stream->Write( tmp, 1);
+		tmp[0] = 0;
+		if (map->AreaDifficulty == 4) {
+			tmp[0] = 1;
+		}
+		stream->Write( tmp, 1);
+		stream->Write( Signature, 6);
 		stream->Write( Signature, 8);
 	}
 
@@ -1898,6 +1947,7 @@ int AREImporter::PutActors( DataStream *stream, Map *map)
 {
 	ieDword tmpDword = 0;
 	ieWord tmpWord;
+	ieByte tmpByte;
 	ieDword CreatureOffset = EmbeddedCreOffset;
 	char filling[120];
 	unsigned int i;
@@ -1918,7 +1968,11 @@ int AREImporter::PutActors( DataStream *stream, Map *map)
 		stream->WriteWord( &tmpWord);
 
 		stream->WriteDword( &tmpDword); //used fields flag always 0 for saved areas
-		stream->WriteDword( &tmpDword); //unknown2c
+		tmpWord = ac->Spawned;
+		stream->WriteWord( &tmpWord);
+		stream->Write(filling, 1); // letter
+		tmpByte = ac->DifficultyMargin;
+		stream->Write( &tmpByte, 1 );
 		stream->WriteDword( &tmpDword); //actor animation, unused
 		tmpWord = ac->GetOrientation();
 		stream->WriteWord( &tmpWord);
