@@ -26,6 +26,8 @@
 #include "RGBAColor.h"
 #include "globals.h"
 
+namespace GemRB {
+
 struct FrameEntry {
 	ieWord Width;
 	ieWord  Height;
@@ -60,8 +62,6 @@ public:
 	int GetCycleSize(unsigned char Cycle);
 	AnimationFactory* GetAnimationFactory(const char* ResRef,
 		unsigned char mode = IE_NORMAL);
-	/** This function will load the Animation as a Font */
-	Font* GetFont();
 	/** Debug Function: Returns the Global Animation Palette as a Sprite2D Object.
 	If the Global Animation Palette is NULL, returns NULL. */
 	Sprite2D* GetPalette();
@@ -86,4 +86,6 @@ public:
 	}
 };
 
+}
+
 #endif
