@@ -30,10 +30,6 @@
  * Copyright (c) 2009 Konstantin Shishkov
 */
 
-#if defined(__HAIKU__)
-#include <unistd.h>
-#endif
-
 #include "BIKPlayer.h"
 
 #include "rational.h"
@@ -48,6 +44,12 @@
 #include <cassert>
 #include <cstdio>
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+using namespace GemRB;
+
 static int g_truecolor;
 static ieDword *cbAtFrame = NULL;
 static ieDword *strRef = NULL;
@@ -218,7 +220,8 @@ int BIKPlayer::Play()
 	frameCount = 0;
 	int ret = doPlay( );
 
-	EndAudio();
+	if (s_stream > -1)
+		EndAudio();
 	EndVideo();
 	av_freep((void **) &inbuff);
 	return ret;
@@ -286,7 +289,7 @@ bool BIKPlayer::next_frame()
 	ieDword audframesize;
 	str->ReadDword(&audframesize);
 	frame.size = str->Read( inbuff, frame.size - 4 );
-	if (DecodeAudioFrame(inbuff, audframesize)) {
+	if (s_stream > -1 && DecodeAudioFrame(inbuff, audframesize)) {
 		//buggy frame, we stop immediately
 		//return false;
 	}
@@ -329,6 +332,7 @@ int BIKPlayer::doPlay()
 		done = video->PollMovieEvents();
 	}
 
+	video->DestroyMovieScreen();
 	return 0;
 }
 
@@ -1152,7 +1156,8 @@ int BIKPlayer::read_colors(Bundle *b)
 int BIKPlayer::read_dcs(Bundle *b, int start_bits, int has_sign)
 {
 	int i, j, len, len2, bsize, v, v2;
-	SET_INT_TYPE *dst = (SET_INT_TYPE*)b->cur_dec;
+
+	SET_INT_TYPE *dst = reinterpret_cast<SET_INT_TYPE*>(b->cur_dec);
 	//int16_t *dst = (int16_t*)b->cur_dec;
 
 	CHECK_READ_VAL(v_gb, b, len);
@@ -1503,7 +1508,7 @@ int BIKPlayer::DecodeVideoFrame(void *data, int data_size)
 						}
 						break;
 					default:
-						printf("Incorrect 16x16 block type!\n");
+						print("Incorrect 16x16 block type!");
 						return -1;
 					}
 					bx++;
@@ -1586,7 +1591,7 @@ int BIKPlayer::DecodeVideoFrame(void *data, int data_size)
 					c_bundle[BINK_SRC_COLORS].cur_ptr += 64;
 					break;
 				default:
-					printf("Unknown block type!\n");
+					print("Unknown block type!");
 					return -1;
 				}
 			}
@@ -1613,5 +1618,5 @@ int BIKPlayer::DecodeVideoFrame(void *data, int data_size)
 #include "plugindef.h"
 
 GEMRB_PLUGIN(0x316E2EDE, "BIK Video Player")
-PLUGIN_RESOURCE(BIKPlayer, ".mve")
+PLUGIN_RESOURCE(BIKPlayer, "mve")
 END_PLUGIN()
