@@ -32,6 +32,8 @@
 
 #include <cassert>
 
+using namespace GemRB;
+
 #define MAXCOLOR 12
 typedef unsigned char ColorSet[MAXCOLOR];
 
@@ -55,8 +57,8 @@ static int shpcount=-1;
 
 struct LevelAndKit
 {
-	int level;
-	int kit;
+	unsigned int level;
+	unsigned int kit;
 };
 
 class SpellEntry
@@ -64,10 +66,12 @@ class SpellEntry
 public:
 	~SpellEntry();
 	SpellEntry();
-	const ieResRef *FindSpell(int level, int kit) const;
+	const ieResRef *GetSpell() const;
+	const ieResRef *FindSpell(unsigned int level, unsigned int kit) const;
+	int FindSpell(unsigned int kit) const;
 	bool Equals(const char *spl) const;
 	void SetSpell(const char *spl);
-	void AddLevel(int level, int kit);
+	void AddLevel(unsigned int level, unsigned int kit);
 private:
 	ieResRef spell;
 	LevelAndKit *levels;
@@ -87,7 +91,12 @@ SpellEntry::~SpellEntry()
 	levels = NULL;
 }
 
-const ieResRef *SpellEntry::FindSpell(int level, int kit) const
+const ieResRef *SpellEntry::GetSpell() const
+{
+	return &spell;
+}
+
+const ieResRef *SpellEntry::FindSpell(unsigned int level, unsigned int kit) const
 {
 	int i = count;
 	while(i--) {
@@ -98,6 +107,28 @@ const ieResRef *SpellEntry::FindSpell(int level, int kit) const
 	return NULL;
 }
 
+int SpellEntry::FindSpell(unsigned int kit) const
+{
+	int i = count;
+	while(i--) {
+		if (levels[i].kit==kit) {
+			return levels[i].level;
+		}
+	}
+	return 0;
+}
+
+static int FindSpell(ieResRef spellref, SpellEntry* list, int listsize)
+{
+	int i = listsize;
+	while (i--) {
+		if (list[i].Equals(spellref)) {
+			return i;
+		}
+	}
+	return -1;
+}
+
 bool SpellEntry::Equals(const char *spl) const
 {
 	return !strnicmp(spell, spl, sizeof(ieResRef));
@@ -108,15 +139,16 @@ void SpellEntry::SetSpell(const char *spl)
 	strnlwrcpy(spell, spl, 8);
 }
 
-void SpellEntry::AddLevel(int level, int kit)
+void SpellEntry::AddLevel(unsigned int level,unsigned int kit)
 {
 	if(!level) {
 		return;
 	}
 
-	level--;
+	level--; // convert to 0-based for internal use
 	for(int i=0;i<count;i++) {
 		if(levels[i].kit==kit && levels[i].level==level) {
+			Log(WARNING, "CREImporter", "Skipping duplicate spell list table entry for: %s", spell);
 			return;
 		}
 	}
@@ -126,6 +158,36 @@ void SpellEntry::AddLevel(int level, int kit)
 	count++;
 }
 
+static int IsInnate(ieResRef name)
+{
+	for(int i=0;i<inncount;i++) {
+		if(!strnicmp(name, innlist[i], 8) ) {
+			return i;
+		}
+	}
+	return -1;
+}
+
+static int IsSong(ieResRef name)
+{
+	for(int i=0;i<sngcount;i++) {
+		if(!strnicmp(name, snglist[i], 8) ) {
+			return i;
+		}
+	}
+	return -1;
+}
+
+static int IsShape(ieResRef name)
+{
+	for(int i=0;i<shpcount;i++) {
+		if(!strnicmp(name, shplist[i], 8) ) {
+			return i;
+		}
+	}
+	return -1;
+}
+
 static SpellEntry* spllist=NULL;
 static int splcount=-1;
 static SpellEntry* domlist=NULL;
@@ -133,6 +195,54 @@ static int domcount=-1;
 static SpellEntry* maglist=NULL;
 static int magcount=-1;
 
+static int IsDomain(ieResRef name, unsigned short &level, unsigned int kit)
+{
+	for(int i=0;i<domcount;i++) {
+		if (domlist[i].Equals(name) ) {
+			level = domlist[i].FindSpell(kit);
+				return i;
+		}
+	}
+	return -1;
+}
+
+static int IsSpecial(ieResRef name, unsigned short &level, unsigned int kit)
+{
+	for(int i=0;i<magcount;i++) {
+		if (maglist[i].Equals(name) ) {
+			level = maglist[i].FindSpell(kit);
+				return i;
+		}
+	}
+	return -1;
+}
+
+static int SpellType(ieResRef name, unsigned short &level, unsigned int clsmsk)
+{
+	for (int i = 0;i<splcount;i++) {
+		if (spllist[i].Equals(name) ) {
+			for(int type=0;type<7;type++) {
+				if (clsmsk & i) {
+					level = spllist[i].FindSpell(type);
+				}
+			}
+		}
+	}
+	return 0;
+}
+
+int CREImporter::FindSpellType(char *name, unsigned short &level, unsigned int clsmsk, unsigned int kit) const
+{
+	level = 0;
+	if (IsSong(name)>=0) return IE_IWD2_SPELL_SONG;
+	if (IsShape(name)>=0) return IE_IWD2_SPELL_SHAPE;
+	if (IsInnate(name)>=0) return IE_IWD2_SPELL_INNATE;
+	if (IsDomain(name, level, kit)>=0) return IE_IWD2_SPELL_DOMAIN;
+	if (IsSpecial(name, level, kit)>=0) return IE_IWD2_SPELL_WIZARD;
+	return SpellType(name, level, clsmsk);
+}
+
+//int CREImporter::ResolveSpellName(ieResRef name, int level, ieIWD2SpellType type) const
 int ResolveSpellName(ieResRef name, int level, ieIWD2SpellType type)
 {
 	int i;
@@ -164,13 +274,26 @@ int ResolveSpellName(ieResRef name, int level, ieIWD2SpellType type)
 		}
 		break;
 	case IE_IWD2_SPELL_DOMAIN:
-		return -1;
 	default:
-		return -1;
+		for(i=0;i<splcount;i++) {
+			if (spllist[i].Equals(name) ) return i;
+		}
 	}
 	return -1;
 }
 
+// just returns the integer part of the log
+// perfect for deducing kit values, since they are bitfields and we don't care about any noise
+static int log2(int value)
+{
+	int pow = -1;
+	while (value) {
+		value = value>>1;
+		pow++;
+	}
+	return pow;
+}
+
 //input: index, level, type, kit
 static const ieResRef *ResolveSpellIndex(int index, int level, ieIWD2SpellType type, int kit)
 {
@@ -198,14 +321,26 @@ static const ieResRef *ResolveSpellIndex(int index, int level, ieIWD2SpellType t
 		}
 		return &shplist[index];
 	case IE_IWD2_SPELL_DOMAIN:
-		if (index>=domcount) {
+		if (index>=splcount) {
 			return NULL;
 		}
-		return domlist[index].FindSpell(level, kit);
+		// translate the actual kit to a column index to make them comparable
+		// luckily they are in order
+		kit = log2(kit/0x8000); // 0x8000 is the first cleric kit
+		ret = domlist[index].FindSpell(level, kit);
+		if (ret) {
+			return ret;
+		}
+		// sigh, retry with wizard spells, since the table does not cover everything npcs have
+		kit = -1;
+		type = IE_IWD2_SPELL_WIZARD;
+		break;
 	case IE_IWD2_SPELL_WIZARD:
-		if (index>=magcount) {
+		if (index>=splcount) {
 			break;
 		}
+		// translate the actual kit to a column index to make them comparable
+		kit = log2(kit/0x40); // 0x40 is the first mage kit
 		//if it is a specialist spell, return it now
 		ret = maglist[index].FindSpell(level, kit);
 		if ( ret) {
@@ -218,16 +353,32 @@ static const ieResRef *ResolveSpellIndex(int index, int level, ieIWD2SpellType t
 		break;
 	}
 
+	// type matches the table columns (0-bard to 6-wizard)
 	ret = spllist[index].FindSpell(level, type);
-	if ( !ret || (kit==-1) ) {
+	if (!ret) {
+		// some npcs have spells at odd levels, so the lookup just failed
+		// eg. slayer knights of xvim with sppr325 at level 2 instead of 3
+		Log(ERROR, "CREImporter", "Spell (%d of type %d) found at unexpected level (%d)!", index, type, level);
+		int level2 = spllist[index].FindSpell(type);
+		// grrr, some rows have no levels set - they're all 0, but with a valid resref, so just return that
+		if (!level2) {
+			Log(DEBUG, "CREImporter", "Spell entry (%d) without any levels set!", index);
+			return spllist[index].GetSpell();
+		}
+		ret = spllist[index].FindSpell(level2, type);
+		if (ret) Log(DEBUG, "CREImporter", "The spell was found at level %d!", level2);
+	}
+	if (ret || (kit==-1) ) {
 		return ret;
 	}
 
+	error("CREImporter", "Doing extra mage spell lookups!");
+	// FIXME: is this really needed? reachable only if wizard index was too high
+	kit = log2(kit/0x40); // 0x40 is the first mage kit
 	int i;
-
 	for(i=0;i<magcount;i++) {
 		if (maglist[i].Equals(*ret)) {
-			return maglist[i].FindSpell(level, kit);    
+			return maglist[i].FindSpell(level, kit);
 		}
 	}
 	return NULL;
@@ -298,6 +449,7 @@ static ieResRef *GetSpellTable(const ieResRef tableresref, int &count)
 	return reslist;
 }
 
+// different tables, but all use listspll.2da for the spell indices
 static SpellEntry *GetKitSpell(const ieResRef tableresref, int &count)
 {
 	count = 0;
@@ -311,11 +463,32 @@ static SpellEntry *GetKitSpell(const ieResRef tableresref, int &count)
 	}
 
 	count = tab->GetRowCount();
-	SpellEntry *reslist = new SpellEntry[count];
+	SpellEntry *reslist;
+	bool indexlist = false;
+	if (!strnicmp(tableresref, "listspll", 8)) {
+		indexlist = true;
+		reslist = new SpellEntry[count];
+	} else {
+		reslist = new SpellEntry[splcount]; // needs to be the same size for the simple index lookup we do!
+	}
+	int index;
 	for(int i = 0;i<count;i++) {
-		reslist[i].SetSpell(tab->QueryField(i,column));
+		if (indexlist) {
+			index = i;
+		} else {
+			// find the correct index in listspll.2da
+			ieResRef spellref;
+			strnlwrcpy(spellref, tab->QueryField(i, column), 8);
+			// the table has disabled spells in it and they all have the first two chars replaced by '*'
+			if (spellref[0] == '*') {
+				continue;
+			}
+			index = FindSpell(spellref, spllist, splcount);
+			assert (index != -1);
+		}
+		reslist[index].SetSpell(tab->QueryField(i, column));
 		for(int col=0;col<column;col++) {
-			reslist[i].AddLevel(atoi(tab->QueryField(i,col)), col);
+			reslist[index].AddLevel(atoi(tab->QueryField(i, col)), col);
 		}
 	}
 	return reslist;
@@ -331,7 +504,7 @@ static void InitSpellbook()
 		innlist = GetSpellTable("listinnt", inncount);
 		snglist = GetSpellTable("listsong", sngcount);
 		shplist = GetSpellTable("listshap", shpcount);
-		spllist = GetKitSpell("listspll", splcount);
+		spllist = GetKitSpell("listspll", splcount); // need to init this one first, since the other two rely on it
 		maglist = GetKitSpell("listmage", magcount);
 		domlist = GetKitSpell("listdomn", domcount);
 	}
@@ -390,7 +563,7 @@ bool CREImporter::Open(DataStream* stream)
 		return true;
 	}
 
-	printMessage("CREImporter", "Not a CRE File or File Version not supported: %8.8s\n", LIGHT_RED, Signature);
+	Log(ERROR, "CREImporter", "Not a CRE File or File Version not supported: %8.8s", Signature);
 	return false;
 }
 
@@ -423,10 +596,10 @@ void CREImporter::SetupSlotCounts()
 void CREImporter::WriteChrHeader(DataStream *stream, Actor *act)
 {
 	char Signature[8];
+	char filling[10];
 	ieVariable name;
 	ieDword tmpDword, CRESize;
 	ieWord tmpWord;
-	ieByte tmpByte;
 
 	CRESize = GetStoredFileSize (act);
 	switch (CREVersion) {
@@ -450,7 +623,7 @@ void CREImporter::WriteChrHeader(DataStream *stream, Actor *act)
 			tmpDword = 0x68; //headersize
 			TotSCEFF = 0;
 			break;
-		case IE_CRE_V2_2:  //iwd2
+		case IE_CRE_V2_2: //iwd2
 			memcpy(Signature, "CHR V2.2",8);
 			tmpDword = 0x21c; //headersize
 			TotSCEFF = 1;
@@ -461,7 +634,7 @@ void CREImporter::WriteChrHeader(DataStream *stream, Actor *act)
 			TotSCEFF = 1;
 			break;
 		default:
-			printMessage("CREImporter","Unknown CHR version!\n",LIGHT_RED);
+			Log(ERROR, "CREImporter", "Unknown CHR version!");
 			return;
 	}
 	stream->Write( Signature, 8);
@@ -490,11 +663,12 @@ void CREImporter::WriteChrHeader(DataStream *stream, Actor *act)
 	if (QSPCount==9) {
 		//NOTE: the gemrb internal format stores
 		//0xff or 0xfe in case of innates and bardsongs
-		//if IWD2 doesn't tolerate this, then this field
-		//should be sanitized for IWD2 (not for GemRB)
-		stream->Write( act->PCStats->QuickSpellClass,9);
-		tmpByte = 0;
-		stream->Write( &tmpByte, 1);
+		memset(filling,0,sizeof(filling));
+		memcpy(filling,act->PCStats->QuickSpellClass,MAX_QSLOTS);
+		for(i=0;i<MAX_QSLOTS;i++) {
+			if ( (ieByte) filling[i]>=0xfe) filling[i]=0;
+		}
+		stream->Write( filling, 10);
 	}
 	for (i=0;i<QITCount;i++) {
 		tmpWord = act->PCStats->QuickItemSlots[i];
@@ -526,7 +700,7 @@ void CREImporter::WriteChrHeader(DataStream *stream, Actor *act)
 		//fallthrough
 	case IE_CRE_GEMRB:
 		for (i=0;i<QSPCount;i++) {
-			tmpDword = act->PCStats->QSlots[i];
+			tmpDword = act->PCStats->QSlots[i+3];
 			stream->WriteDword( &tmpDword);
 		}
 		for (i=0;i<13;i++) {
@@ -534,7 +708,13 @@ void CREImporter::WriteChrHeader(DataStream *stream, Actor *act)
 		}
 		stream->Write( act->PCStats->SoundFolder, 32);
 		stream->Write( act->PCStats->SoundSet, 8);
-		for (i=0;i<32;i++) {
+		for (i=0;i<ES_COUNT;i++) {
+			tmpDword = act->PCStats->ExtraSettings[i];
+			stream->WriteDword( &tmpDword);
+		}
+		//Reserved
+		tmpDword = 0;
+		for (i=0;i<16;i++) {
 			stream->WriteDword( &tmpDword);
 		}
 		break;
@@ -548,6 +728,7 @@ void CREImporter::ReadChrHeader(Actor *act)
 	ieVariable name;
 	char Signature[8];
 	ieDword offset, size;
+	ieDword tmpDword;
 	ieWord tmpWord;
 	ieByte tmpByte;
 
@@ -586,7 +767,43 @@ void CREImporter::ReadChrHeader(Actor *act)
 		str->ReadWord (&tmpWord);
 		act->PCStats->QuickItemHeaders[i]=tmpWord;
 	}
+
 	//here comes the version specific read
+	switch (CREVersion) {
+	case IE_CRE_V2_2:
+		//gemrb format doesn't save these redundantly
+		for (i=0;i<QSPCount;i++) {
+			str->ReadResRef(Signature);
+			if (Signature[0]) {
+				act->PCStats->QuickSpellClass[i]=0xff;
+				memcpy(act->PCStats->QuickSpells[i], Signature, sizeof(ieResRef));
+			}
+		}
+		for (i=0;i<QSPCount;i++) {
+			str->ReadResRef(Signature);
+			if (Signature[0]) {
+				act->PCStats->QuickSpellClass[i]=0xfe;
+				memcpy(act->PCStats->QuickSpells[i], Signature, sizeof(ieResRef));
+			}
+		}
+		//fallthrough
+	case IE_CRE_GEMRB:
+		for (i=0;i<QSPCount;i++) {
+			str->ReadDword( &tmpDword);
+			act->PCStats->QSlots[i+3] = (ieByte) tmpDword;
+		}
+		str->Seek(26, GEM_CURRENT_POS);
+		str->Read( act->PCStats->SoundFolder, 32);
+		str->Read( act->PCStats->SoundSet, 8);
+		for (i=0;i<ES_COUNT;i++) {
+			str->ReadDword( &act->PCStats->ExtraSettings[i] );
+		}
+		//Reserved
+		str->Seek(64, GEM_CURRENT_POS);
+		break;
+	default:
+		break;
+	}
 }
 
 bool CREImporter::SeekCreHeader(char *Signature)
@@ -638,9 +855,9 @@ CRESpellMemorization* CREImporter::GetSpellMemorization(Actor *act)
 	str->ReadDword( &MemorizedCount );
 
 	CRESpellMemorization* spl = act->spellbook.GetSpellMemorization(Type, Level);
-	assert(spl && spl->Number == 0 && spl->Number2 == 0); // unused
-	spl->Number = Number;
-	spl->Number2 = Number;
+	assert(spl && spl->SlotCount == 0 && spl->SlotCountWithBonus == 0); // unused
+	spl->SlotCount = Number;
+	spl->SlotCountWithBonus = Number;
 
 	return spl;
 }
@@ -650,7 +867,7 @@ void CREImporter::SetupColor(ieDword &stat)
 	if (RandColor==-1) {
 		RandColor=0;
 		RandRows=0;
-		AutoTable rndcol("randcolr");
+		AutoTable rndcol("randcolr", true);
 		if (rndcol) {
 			RandColor = rndcol->GetColumnCount();
 			RandRows = rndcol->GetRowCount();
@@ -679,13 +896,13 @@ void CREImporter::SetupColor(ieDword &stat)
 		int i;
 		for (i=(int) stat;i>=0;i--) {
 			if (randcolors[i][0]==stat) {
-				stat=randcolors[i][ rand()%RandRows + 1];
+				stat=randcolors[i][rand()%RandRows];
 				return;
 			}
 		}
 		for (i=(int) stat+1;i<RandColor;i++) {
 			if (randcolors[i][0]==stat) {
-				stat=randcolors[i][ rand()%RandRows + 1];
+				stat=randcolors[i][rand()%RandRows];
 				return;
 			}
 		}
@@ -788,15 +1005,16 @@ Actor* CREImporter::GetActor(unsigned char is_in_party)
 			GetActorIWD1(act);
 			break;
 		default:
-			Inventory_Size=0;
-			error("CREImporter", "Unknown creature signature: %d\n", CREVersion);
+			Log(ERROR, "CREImporter", "Unknown creature signature: %d\n", CREVersion);
+			delete act;
+			return NULL;
 	}
 
 	// Read saved effects
 	if (core->IsAvailable(IE_EFF_CLASS_ID) ) {
 		ReadEffects( act );
 	} else {
-		printMessage("CREImporter", "Effect importer is unavailable!\n", RED);
+		Log(ERROR, "CREImporter", "Effect importer is unavailable!");
 	}
 	// Reading inventory, spellbook, etc
 	ReadInventory( act, Inventory_Size );
@@ -823,7 +1041,7 @@ void CREImporter::GetActorPST(Actor *act)
 	str->ReadWord( &tmpWord );
 	//skipping a word
 	str->ReadWord( &tmpWord );
-	act->BaseStats[IE_ARMORCLASS]=(ieWordSigned) tmpWord;
+	act->AC.SetNatural((ieWordSigned) tmpWord);
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACCRUSHINGMOD]=(ieWordSigned) tmpWord;
 	str->ReadWord( &tmpWord );
@@ -833,7 +1051,7 @@ void CREImporter::GetActorPST(Actor *act)
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACSLASHINGMOD]=(ieWordSigned) tmpWord;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_TOHIT]=(ieByteSigned) tmpByte;
+	act->ToHit.SetBase((ieByteSigned) tmpByte);
 	str->Read( &tmpByte, 1 );
 	tmpByte = tmpByte * 2;
 	if (tmpByte>10) tmpByte-=11;
@@ -890,13 +1108,15 @@ void CREImporter::GetActorPST(Actor *act)
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_INTOXICATION]=tmpByte;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_LUCK]=tmpByte;
+	act->BaseStats[IE_LUCK]=(ieByteSigned) tmpByte;
+	//last byte is actually an undead level (according to IE dev info)
 	for (i=0;i<21;i++) {
 		str->Read( &tmpByte, 1 );
 		act->BaseStats[IE_PROFICIENCYBASTARDSWORD+i]=tmpByte;
 	}
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_TRACKING]=tmpByte;
+	//scriptname of tracked creature (according to IE dev info)
 	str->Seek( 32, GEM_CURRENT_POS );
 	for (i=0;i<100;i++) {
 		str->ReadDword( &act->StrRefs[i] );
@@ -964,8 +1184,6 @@ void CREImporter::GetActorPST(Actor *act)
 
 	str->Read( &tmpByte, 1 );
 
-	//str->ReadWord( &act->AppearanceFlags1 );
-	//str->ReadWord( &act->AppearanceFlags2 );
 	str->ReadDword( &act->AppearanceFlags );
 
 	for (i = 0; i < 7; i++) {
@@ -1025,6 +1243,8 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 	ieWord *indices = (ieWord *) calloc(Inventory_Size, sizeof(ieWord));
 	//CREItem** items;
 	unsigned int i,j,k;
+	ieWordSigned eqslot;
+	ieWord eqheader;
 
 	act->inventory.SetSlotCount(Inventory_Size+1);
 	str->Seek( ItemSlotsOffset+CREOffset, GEM_STREAM_START );
@@ -1038,10 +1258,11 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 	// 1000 - fist
 	// -24,-23,-22,-21 - quiver
 	//the equipping effects are delayed until the actor gets an area
-	str->ReadWordSigned( &act->Equipped );
+	str->ReadWordSigned(&eqslot);
 	//the equipped slot's selected ability is stored here
-	str->ReadWord( &act->EquippedHeader );
-	
+	str->ReadWord(&eqheader);
+	act->inventory.SetEquipped(eqslot, eqheader);
+
 	//read the item entries based on the previously read indices
 	//an item entry may be read multiple times if the indices are repeating
 	for (i = 0;i<Inventory_Size;) {
@@ -1049,7 +1270,7 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 		ieWord index = indices[i++];
 		if (index != 0xffff) {
 			if (index>=ItemsCount) {
-				printMessage("CREImporter", "Invalid item index (%d) in creature!\n", LIGHT_RED, index);
+				Log(ERROR, "CREImporter", "Invalid item index (%d) in creature!", index);
 				continue;
 			}
 			//20 is the size of CREItem on disc (8+2+3x2+4)
@@ -1060,7 +1281,7 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 			if (item) {
 				act->inventory.SetSlotItem(item, Slot);
 			} else {
-				printMessage("CREImporter", "Invalid item index (%d) in creature!\n", LIGHT_RED, index);
+				Log(ERROR, "CREImporter", "Invalid item index (%d) in creature!", index);
 			}
 		}
 	}
@@ -1069,7 +1290,7 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 
 	// Reading spellbook
 	CREKnownSpell **known_spells=(CREKnownSpell **) calloc(KnownSpellsCount, sizeof(CREKnownSpell *) );
-	CREMemorizedSpell **memorized_spells=(CREMemorizedSpell **) calloc(MemorizedSpellsCount, sizeof(CREKnownSpell *) );
+	CREMemorizedSpell **memorized_spells=(CREMemorizedSpell **) calloc(MemorizedSpellsCount, sizeof(CREMemorizedSpell *) );
 
 	str->Seek( KnownSpellsOffset+CREOffset, GEM_STREAM_START );
 	for (i = 0; i < KnownSpellsCount; i++) {
@@ -1105,14 +1326,14 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 				memorized_spells[k] = NULL;
 				continue;
 			}
-			print("[CREImporter]: Duplicate memorized spell (%d) in creature!\n", k);
+			Log(WARNING, "CREImporter", "Duplicate memorized spell(%d) in creature!", k);
 		}
 	}
 
 	i=KnownSpellsCount;
 	while(i--) {
 		if (known_spells[i]) {
-			printMessage("CREImporter", "Dangling spell in creature: %s!\n", YELLOW,
+			Log(WARNING, "CREImporter", "Dangling spell in creature: %s!",
 				known_spells[i]->SpellResRef);
 			delete known_spells[i];
 		}
@@ -1122,7 +1343,7 @@ void CREImporter::ReadInventory(Actor *act, unsigned int Inventory_Size)
 	i=MemorizedSpellsCount;
 	while(i--) {
 		if (memorized_spells[i]) {
-			printMessage("CREImporter", "Dangling spell in creature: %s!\n", YELLOW,
+			Log(WARNING, "CREImporter", "Dangling spell in creature: %s!",
 				memorized_spells[i]->SpellResRef);
 			delete memorized_spells[i];
 		}
@@ -1168,7 +1389,7 @@ ieDword CREImporter::GetActorGemRB(Actor *act)
 	//skipping a word( useful for something)
 	str->ReadWord( &tmpWord );
 	str->ReadWord( &tmpWord );
-	act->BaseStats[IE_ARMORCLASS]=(ieWordSigned) tmpWord;
+	act->AC.SetNatural((ieWordSigned) tmpWord);
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACCRUSHINGMOD]=(ieWordSigned) tmpWord;
 	str->ReadWord( &tmpWord );
@@ -1178,7 +1399,7 @@ ieDword CREImporter::GetActorGemRB(Actor *act)
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACSLASHINGMOD]=(ieWordSigned) tmpWord;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_TOHIT]=(ieByteSigned) tmpByte;
+	act->ToHit.SetBase((ieByteSigned) tmpByte);
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_NUMBEROFATTACKS]=tmpByte;
 	str->Read( &tmpByte, 1 );
@@ -1232,7 +1453,7 @@ ieDword CREImporter::GetActorGemRB(Actor *act)
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_INTOXICATION]=tmpByte;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_LUCK]=tmpByte;
+	act->BaseStats[IE_LUCK]=(ieByteSigned) tmpByte;
 	str->Read( &tmpByte, 1 );
 	//these could be used to save iwd2 skills
 	//TODO: gemrb format
@@ -1256,7 +1477,7 @@ void CREImporter::GetActorBG(Actor *act)
 	str->ReadWord( &tmpWord );
 	//skipping a word
 	str->ReadWord( &tmpWord );
-	act->BaseStats[IE_ARMORCLASS]=(ieWordSigned) tmpWord;
+	act->AC.SetNatural((ieWordSigned) tmpWord);
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACCRUSHINGMOD]=(ieWordSigned) tmpWord;
 	str->ReadWord( &tmpWord );
@@ -1266,7 +1487,7 @@ void CREImporter::GetActorBG(Actor *act)
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACSLASHINGMOD]=(ieWordSigned) tmpWord;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_TOHIT]=(ieByteSigned) tmpByte;
+	act->ToHit.SetBase((ieByteSigned) tmpByte);
 	str->Read( &tmpByte, 1 );
 	tmpWord = tmpByte * 2;
 	if (tmpWord>10) tmpWord-=11;
@@ -1322,7 +1543,7 @@ void CREImporter::GetActorBG(Actor *act)
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_INTOXICATION]=tmpByte;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_LUCK]=tmpByte;
+	act->BaseStats[IE_LUCK]=(ieByteSigned) tmpByte;
 	for (i=0;i<21;i++) {
 		str->Read( &tmpByte, 1 );
 		act->BaseStats[IE_PROFICIENCYBASTARDSWORD+i]=tmpByte;
@@ -1424,7 +1645,7 @@ void CREImporter::GetIWD2Spellpage(Actor *act, ieIWD2SpellType type, int level,
 
 	int check = 0;
 	CRESpellMemorization* sm = act->spellbook.GetSpellMemorization(type, level);
-	assert(sm && sm->Number == 0 && sm->Number2 == 0); // unused
+	assert(sm && sm->SlotCount == 0 && sm->SlotCountWithBonus == 0); // unused
 	while(count--) {
 		str->ReadDword(&spellindex);
 		str->ReadDword(&totalcount);
@@ -1442,7 +1663,7 @@ void CREImporter::GetIWD2Spellpage(Actor *act, ieIWD2SpellType type, int level,
 				if(totalcount) {
 					totalcount--;
 				} else {
-					printMessage("CREImporter", "More spells still known than memorised.\n", LIGHT_RED);
+					Log(ERROR, "CREImporter", "More spells still known than memorised.");
 					break;
 				}
 				CREMemorizedSpell *memory = new CREMemorizedSpell;
@@ -1450,7 +1671,7 @@ void CREImporter::GetIWD2Spellpage(Actor *act, ieIWD2SpellType type, int level,
 				strnlwrcpy(memory->SpellResRef,*tmp,8);
 				sm->memorized_spells.push_back(memory);
 			}
-			
+
 			while(totalcount--) {
 				CREMemorizedSpell *memory = new CREMemorizedSpell;
 				memory->Flags=0;
@@ -1458,14 +1679,24 @@ void CREImporter::GetIWD2Spellpage(Actor *act, ieIWD2SpellType type, int level,
 				sm->memorized_spells.push_back(memory);
 			}
 		} else {
-			printMessage("CREImporter", "Unresolved spell index: %d level:%d, type: %d\n", LIGHT_RED,
+			error("CREImporter", "Unresolved spell index: %d level:%d, type: %d",
 				spellindex, level+1, type);
 		}
 	}
+	// hacks for domain spells, since their count is not stored and also always 1
+	// NOTE: luckily this does not cause save game incompatibility
 	str->ReadDword(&tmpDword);
-	sm->Number = (ieWord) tmpDword;
+	if (type == IE_IWD2_SPELL_DOMAIN) {
+		sm->SlotCount = 1;
+	} else {
+		sm->SlotCount = (ieWord) tmpDword;
+	}
 	str->ReadDword(&tmpDword);
-	sm->Number2 = (ieWord) tmpDword;
+	if (type == IE_IWD2_SPELL_DOMAIN) {
+		sm->SlotCountWithBonus = 1;
+	} else {
+		sm->SlotCountWithBonus = (ieWord) tmpDword;
+	}
 }
 
 void CREImporter::GetActorIWD2(Actor *act)
@@ -1479,7 +1710,7 @@ void CREImporter::GetActorIWD2(Actor *act)
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_HIDEINSHADOWS]=tmpByte;
 	str->ReadWord( &tmpWord );
-	act->BaseStats[IE_ARMORCLASS]=(ieWordSigned) tmpWord;
+	act->AC.SetNatural((ieWordSigned) tmpWord);
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACCRUSHINGMOD]=(ieWordSigned) tmpWord;
 	str->ReadWord( &tmpWord );
@@ -1489,7 +1720,7 @@ void CREImporter::GetActorIWD2(Actor *act)
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACSLASHINGMOD]=(ieWordSigned) tmpWord;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_TOHIT]=(ieByteSigned) tmpByte;//Unknown in CRE V2.2
+	act->ToHit.SetBase((ieByteSigned) tmpByte);//Unknown in CRE V2.2
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_NUMBEROFATTACKS]=tmpByte;//Unknown in CRE V2.2
 	str->Read( &tmpByte, 1 );
@@ -1528,7 +1759,7 @@ void CREImporter::GetActorIWD2(Actor *act)
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_INTOXICATION]=tmpByte;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_LUCK]=tmpByte;
+	act->BaseStats[IE_LUCK]=(ieByteSigned) tmpByte;
 	str->Seek( 34, GEM_CURRENT_POS ); //unknowns
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_CLASSLEVELSUM]=tmpByte; //total levels
@@ -1551,7 +1782,7 @@ void CREImporter::GetActorIWD2(Actor *act)
 	str->Read( & tmpByte, 1 );
 	act->BaseStats[IE_LEVELTHIEF]=tmpByte;
 	str->Read( & tmpByte, 1 );
-	act->BaseStats[IE_LEVELSORCEROR]=tmpByte;
+	act->BaseStats[IE_LEVELSORCERER]=tmpByte;
 	str->Read( & tmpByte, 1 );
 	act->BaseStats[IE_LEVELMAGE]=tmpByte;
 	str->Seek( 22, GEM_CURRENT_POS ); //levels for classes
@@ -1672,7 +1903,17 @@ void CREImporter::GetActorIWD2(Actor *act)
 	act->BaseStats[IE_SAVEDYPOS] = tmpWord;
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_SAVEDFACE] = tmpWord;
-	str->Seek( 146, GEM_CURRENT_POS );
+
+	str->Seek( 15, GEM_CURRENT_POS );
+	str->Read( &tmpByte, 1);
+	act->BaseStats[IE_TRANSLUCENT]=tmpByte;
+	str->Read( &tmpByte, 1); //fade speed
+	str->Read( &tmpByte, 1); //spec. flags
+	str->Read( &tmpByte, 1); //invisible
+	str->ReadWord( &tmpWord); //unknown
+	str->Read( &tmpByte, 1); //unused skill points
+	act->BaseStats[IE_UNUSED_SKILLPTS] = tmpByte;
+	str->Seek( 124, GEM_CURRENT_POS );
 	str->Read( &tmpByte, 1);
 	act->BaseStats[IE_EA]=tmpByte;
 	str->Read( &tmpByte, 1);
@@ -1746,8 +1987,8 @@ void CREImporter::GetActorIWD2(Actor *act)
 
 	for(i=0;i<8;i++) {
 		for(int lev=0;lev<9;lev++) {
-			//if everything is alright, then this seek is not needed
-			str->Seek(CREOffset+ClassSpellOffsets[i*9+lev], GEM_STREAM_START);
+			//if everything is alright, then seeking is not needed
+			assert(str->GetPos() == CREOffset+ClassSpellOffsets[i*9+lev]);
 			GetIWD2Spellpage(act, (ieIWD2SpellType) i, lev, ClassSpellCounts[i*9+lev]);
 		}
 	}
@@ -1774,7 +2015,7 @@ void CREImporter::GetActorIWD1(Actor *act) //9.0
 	str->ReadWord( &tmpWord );
 	//skipping a word
 	str->ReadWord( &tmpWord );
-	act->BaseStats[IE_ARMORCLASS]=(ieWordSigned) tmpWord;
+	act->AC.SetNatural((ieWordSigned) tmpWord);
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACCRUSHINGMOD]=(ieWordSigned) tmpWord;
 	str->ReadWord( &tmpWord );
@@ -1784,7 +2025,7 @@ void CREImporter::GetActorIWD1(Actor *act) //9.0
 	str->ReadWord( &tmpWord );
 	act->BaseStats[IE_ACSLASHINGMOD]=(ieWordSigned) tmpWord;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_TOHIT]=(ieByteSigned) tmpByte;
+	act->ToHit.SetBase((ieByteSigned) tmpByte);
 	str->Read( &tmpByte, 1 );
 	tmpByte = tmpByte * 2;
 	if (tmpByte>10) tmpByte-=11;
@@ -1840,7 +2081,7 @@ void CREImporter::GetActorIWD1(Actor *act) //9.0
 	str->Read( &tmpByte, 1 );
 	act->BaseStats[IE_INTOXICATION]=tmpByte;
 	str->Read( &tmpByte, 1 );
-	act->BaseStats[IE_LUCK]=tmpByte;
+	act->BaseStats[IE_LUCK]=(ieByteSigned) tmpByte;
 	for (i=0;i<21;i++) {
 		str->Read( &tmpByte, 1 );
 		act->BaseStats[IE_PROFICIENCYBASTARDSWORD+i]=tmpByte;
@@ -2002,9 +2243,17 @@ int CREImporter::GetStoredFileSize(Actor *actor)
 	KnownSpellsOffset = headersize;
 
 	if (actor->version==IE_CRE_V2_2) { //iwd2
-		//adding spellbook header sizes
-		//class spells, domains, (shapes,songs,innates)
-		headersize += 7*9*8 + 9*8 + 3*8;
+		int type, level;
+
+		for (type=IE_IWD2_SPELL_BARD;type<IE_IWD2_SPELL_DOMAIN;type++) for(level=0;level<9;level++) {
+			headersize += GetIWD2SpellpageSize(actor, (ieIWD2SpellType) type, level)*16+8;
+		}
+		for(level=0;level<9;level++) {
+			headersize += GetIWD2SpellpageSize(actor, IE_IWD2_SPELL_DOMAIN, level)*16+8;
+		}
+		for (type=IE_IWD2_SPELL_INNATE;type<NUM_IWD2_SPELLTYPES;type++) {
+			headersize += GetIWD2SpellpageSize(actor, (ieIWD2SpellType) type, 0)*16+8;
+		}
 	} else {//others
 		//adding known spells
 		KnownSpellsCount = actor->spellbook.GetTotalKnownSpellsCount();
@@ -2142,12 +2391,12 @@ int CREImporter::PutHeader(DataStream *stream, Actor *actor)
 	tmpByte = actor->BaseStats[IE_HIDEINSHADOWS];
 	stream->Write( &tmpByte, 1 );
 	//from here it differs, slightly
-	tmpWord = actor->BaseStats[IE_ARMORCLASS];
+	tmpWord = actor->AC.GetNatural();
 	stream->WriteWord( &tmpWord);
 	//iwd2 doesn't store this a second time,
 	//probably gemrb format shouldn't either?
 	if (actor->version != IE_CRE_V2_2) {
-		tmpWord = actor->BaseStats[IE_ARMORCLASS];
+		tmpWord = actor->AC.GetNatural();
 		stream->WriteWord( &tmpWord);
 	}
 	tmpWord = actor->BaseStats[IE_ACCRUSHINGMOD];
@@ -2158,7 +2407,7 @@ int CREImporter::PutHeader(DataStream *stream, Actor *actor)
 	stream->WriteWord( &tmpWord);
 	tmpWord = actor->BaseStats[IE_ACSLASHINGMOD];
 	stream->WriteWord( &tmpWord);
-	tmpByte = actor->BaseStats[IE_TOHIT];
+	tmpByte = actor->ToHit.GetBase();
 	stream->Write( &tmpByte, 1);
 	tmpByte = actor->BaseStats[IE_NUMBEROFATTACKS];
 	if (actor->version == IE_CRE_V2_2) {
@@ -2262,7 +2511,7 @@ int CREImporter::PutHeader(DataStream *stream, Actor *actor)
 		stream->Write( &tmpByte, 1);
 		tmpByte = actor->BaseStats[IE_LEVELTHIEF];
 		stream->Write( &tmpByte, 1);
-		tmpByte = actor->BaseStats[IE_LEVELSORCEROR];
+		tmpByte = actor->BaseStats[IE_LEVELSORCERER];
 		stream->Write( &tmpByte, 1);
 		tmpByte = actor->BaseStats[IE_LEVELMAGE];
 		stream->Write( &tmpByte, 1);
@@ -2509,8 +2758,6 @@ int CREImporter::PutActorPST(DataStream *stream, Actor *actor)
 	stream->Write( filling,3); //unknown
 	tmpByte=actor->BaseStats[IE_COLORCOUNT];
 	stream->Write( &tmpByte, 1);
-	//stream->WriteWord( &actor->AppearanceFlags1);
-	//stream->WriteWord( &actor->AppearanceFlags2);
 	stream->WriteDword( &actor->AppearanceFlags);
 
 	for (i=0;i<7;i++) {
@@ -2598,13 +2845,13 @@ int CREImporter::PutActorIWD2(DataStream *stream, Actor *actor)
 	ieByte tmpByte;
 	ieWord tmpWord;
 	int i;
-	char filling[146];
+	char filling[124];
 
 	memset(filling,0,sizeof(filling));
 	tmpByte=(ieByte) actor->BaseStats[IE_AVATARREMOVAL];
 	stream->Write( &tmpByte, 1);
 	stream->Write( &actor->SetDeathVar, 1);
-	stream->Write( &actor->IncKillCount, 1);	
+	stream->Write( &actor->IncKillCount, 1);
 	stream->Write( &actor->UnknownField, 1); //unknown
 	for (i=0;i<5;i++) {
 		tmpWord = actor->BaseStats[IE_INTERNAL_0+i];
@@ -2619,7 +2866,13 @@ int CREImporter::PutActorIWD2(DataStream *stream, Actor *actor)
 	stream->WriteWord( &tmpWord);
 	tmpWord = actor->BaseStats[IE_SAVEDFACE];
 	stream->WriteWord( &tmpWord);
-	stream->Write( filling, 146);
+	stream->Write( filling, 15);
+	tmpByte = actor->BaseStats[IE_TRANSLUCENT];
+	stream->Write(&tmpByte, 1);
+	stream->Write( filling, 5); //fade speed, spec flags, invisible
+	tmpByte = actor->BaseStats[IE_UNUSED_SKILLPTS];
+	stream->Write( &tmpByte, 1);
+	stream->Write( filling, 124);
 	//similar in all engines
 	tmpByte = actor->BaseStats[IE_EA];
 	stream->Write( &tmpByte, 1);
@@ -2741,9 +2994,9 @@ int CREImporter::PutEffects( DataStream *stream, Actor *actor)
 			tmpByte = (ieByte) fx->Resistance;
 			stream->Write(&tmpByte, 1);
 			stream->WriteDword( &fx->Duration);
-			tmpByte = (ieByte) fx->Probability1;
+			tmpByte = (ieByte) fx->ProbabilityRangeMax;
 			stream->Write(&tmpByte, 1);
-			tmpByte = (ieByte) fx->Probability2;
+			tmpByte = (ieByte) fx->ProbabilityRangeMin;
 			stream->Write(&tmpByte, 1);
 			stream->Write(fx->Resource, 8);
 			stream->WriteDword( &fx->DiceThrown );
@@ -2788,6 +3041,39 @@ int CREImporter::PutVariables( DataStream *stream, Actor *actor)
 	return 0;
 }
 
+//Don't forget to add 8 for the totals/bonus fields
+ieDword CREImporter::GetIWD2SpellpageSize(Actor *actor, ieIWD2SpellType type, int level) const
+{
+	CRESpellMemorization* sm = actor->spellbook.GetSpellMemorization(type, level);
+	ieDword cnt = sm->known_spells.size();
+	return cnt;
+}
+
+int CREImporter::PutIWD2Spellpage(DataStream *stream, Actor *actor, ieIWD2SpellType type, int level)
+{
+	ieDword ID, max, known;
+
+	CRESpellMemorization* sm = actor->spellbook.GetSpellMemorization(type, level);
+	for (unsigned int k = 0; k < sm->known_spells.size(); k++) {
+		CREKnownSpell *ck = sm->known_spells[k];
+		ID = ResolveSpellName(ck->SpellResRef, level, type);
+		stream->WriteDword ( &ID);
+		max = actor->spellbook.CountSpells(ck->SpellResRef, type, 1);
+		known = actor->spellbook.CountSpells(ck->SpellResRef, type, 0);
+		stream->WriteDword ( &max);
+		stream->WriteDword ( &known);
+		//unknown field (always 0)
+		known = 0;
+		stream->WriteDword (&known);
+	}
+
+	max = sm->SlotCount;
+	known = sm->SlotCountWithBonus;
+	stream->WriteDword ( &max);
+	stream->WriteDword ( &known);
+	return 0;
+}
+
 /* this function expects GetStoredFileSize to be called before */
 int CREImporter::PutActor(DataStream *stream, Actor *actor, bool chr)
 {
@@ -2843,30 +3129,35 @@ int CREImporter::PutActor(DataStream *stream, Actor *actor, bool chr)
 		return ret;
 	}
 
-	//writing offsets
+	//writing offsets and counts
 	if (actor->version==IE_CRE_V2_2) {
-		int i;
+		int type, level;
 
 		//class spells
-		for (i=0;i<7*9;i++) {
+		for (type=IE_IWD2_SPELL_BARD;type<IE_IWD2_SPELL_DOMAIN;type++) for(level=0;level<9;level++) {
+			tmpDword = GetIWD2SpellpageSize(actor, (ieIWD2SpellType) type, level);
 			stream->WriteDword(&KnownSpellsOffset);
-			KnownSpellsOffset+=8;
+			KnownSpellsOffset+=tmpDword*16+8;
 		}
-		for (i=0;i<7*9;i++) {
+		for (type=IE_IWD2_SPELL_BARD;type<IE_IWD2_SPELL_DOMAIN;type++) for(level=0;level<9;level++) {
+			tmpDword = GetIWD2SpellpageSize(actor, (ieIWD2SpellType) type, level);
 			stream->WriteDword(&tmpDword);
 		}
 		//domain spells
-		for (i=0;i<9;i++) {
+		for (level=0;level<9;level++) {
+			tmpDword = GetIWD2SpellpageSize(actor, IE_IWD2_SPELL_DOMAIN, level);
 			stream->WriteDword(&KnownSpellsOffset);
-			KnownSpellsOffset+=8;
+			KnownSpellsOffset+=tmpDword*16+8;
 		}
-		for (i=0;i<9;i++) {
+		for (level=0;level<9;level++) {
+			tmpDword = GetIWD2SpellpageSize(actor, IE_IWD2_SPELL_DOMAIN, level);
 			stream->WriteDword(&tmpDword);
 		}
 		//innates, shapes, songs
-		for (i=0;i<3;i++) {
+		for (type=IE_IWD2_SPELL_INNATE;type<NUM_IWD2_SPELLTYPES;type++) {
+			tmpDword = GetIWD2SpellpageSize(actor, (ieIWD2SpellType) type, 0);
 			stream->WriteDword(&KnownSpellsOffset);
-			KnownSpellsOffset+=8;
+			KnownSpellsOffset+=tmpDword*16+8;
 			stream->WriteDword(&tmpDword);
 		}
 	} else {
@@ -2888,22 +3179,21 @@ int CREImporter::PutActor(DataStream *stream, Actor *actor, bool chr)
 	//spells, spellbook etc
 
 	if (actor->version==IE_CRE_V2_2) {
-		int i;
+		int type, level;
 
-		//putting out book headers
-		for (i=0;i<7*9;i++) {
-			stream->WriteDword(&tmpDword);
-			stream->WriteDword(&tmpDword);
+		//writing out spell page headers
+		for (type=IE_IWD2_SPELL_BARD;type<IE_IWD2_SPELL_DOMAIN;type++) for(level=0;level<9;level++) {
+			PutIWD2Spellpage(stream, actor, (ieIWD2SpellType) type, level);
 		}
-		//domain headers
-		for (i=0;i<9;i++) {
-			stream->WriteDword(&tmpDword);
-			stream->WriteDword(&tmpDword);
+
+		//writing out domain page headers
+		for (level=0;level<9;level++) {
+			PutIWD2Spellpage(stream, actor, IE_IWD2_SPELL_DOMAIN, level);
 		}
+
 		//innates, shapes, songs
-		for (i=0;i<3;i++) {
-			stream->WriteDword(&tmpDword);
-			stream->WriteDword(&tmpDword);
+		for (type = IE_IWD2_SPELL_INNATE; type<NUM_IWD2_SPELLTYPES; type ++) {
+			PutIWD2Spellpage(stream, actor, (ieIWD2SpellType) type, 0);
 		}
 	} else {
 		assert(stream->GetPos() == CREOffset+KnownSpellsOffset);
