@@ -27,6 +27,8 @@
 #include "PluginMgr.h"
 #include "TableMgr.h" //needed for autotable
 
+using namespace GemRB;
+
 int *profs = NULL;
 int profcount = -1;
 
@@ -40,14 +42,13 @@ static void Initializer()
 	profcount = 0;
 	AutoTable tm("proftype");
 	if (!tm) {
-		printStatus( "ERROR", LIGHT_RED );
-		print( "Cannot find proftype.2da.\n");
+		Log(ERROR, "ITMImporter", "Cannot find proftype.2da.");
 		return;
 	}
 	profcount = tm->GetRowCount();
 	profs = (int *) calloc( profcount, sizeof(int) );
 	for (int i = 0; i < profcount; i++) {
-		profs[i] = atoi(tm->QueryField( i, 1 ) );
+		profs[i] = atoi(tm->QueryField( i, 0 ) );
 	}
 }
 
@@ -97,7 +98,7 @@ bool ITMImporter::Open(DataStream* stream)
 	} else if (strncmp( Signature, "ITM V2.0", 8 ) == 0) {
 		version = 20;
 	} else {
-		print( "[ITMImporter]: This file is not a valid ITM File\n" );
+		print("[ITMImporter]: This file is not a valid ITM File");
 		return false;
 	}
 
@@ -149,6 +150,12 @@ Item* ITMImporter::GetItem(Item *s)
 	str->Read( &s->unknown3, 1 );
 	str->ReadDword( &s->Price );
 	str->ReadWord( &s->MaxStackAmount );
+
+	//hack for non stacked items, so MaxStackAmount could be used as a boolean
+	if (s->MaxStackAmount==1) {
+		s->MaxStackAmount = 0;
+	}
+
 	str->ReadResRef( s->ItemIcon );
 	str->ReadWord( &s->LoreToID );
 	str->ReadResRef( s->GroundIcon );
@@ -207,12 +214,16 @@ Item* ITMImporter::GetItem(Item *s)
 
 
 	if (!core->IsAvailable( IE_BAM_CLASS_ID )) {
-		print( "[ITMImporter]: No BAM Importer Available.\n" );
+		print("[ITMImporter]: No BAM Importer Available.");
 		return NULL;
 	}
 	return s;
 }
 
+//unfortunately, i couldn't avoid this hack, unless adding another array
+#define IT_DAGGER     0x10
+#define IT_SHORTSWORD 0x13
+
 void ITMImporter::GetExtHeader(Item *s, ITMExtHeader* eh)
 {
 	ieByte tmpByte;
@@ -243,11 +254,20 @@ void ITMImporter::GetExtHeader(Item *s, ITMExtHeader* eh)
 	str->ReadWord( &eh->Charges );
 	str->ReadWord( &eh->ChargeDepletion );
 	str->ReadDword( &eh->RechargeFlags );
+
+	//hack for default weapon finesse
+	if (s->ItemType==IT_DAGGER || s->ItemType==IT_SHORTSWORD) eh->RechargeFlags^=IE_ITEM_USEDEXTERITY;
+
 	str->ReadWord( &eh->ProjectileAnimation );
 	//for some odd reasons 0 and 1 are the same
 	if (eh->ProjectileAnimation) {
 		eh->ProjectileAnimation--;
 	}
+	// bg2 ignored the projectile for melee weapons (rarely set, but gives staf13 AOE effects)
+	if (!core->HasFeature(GF_MELEEHEADER_USESPROJECTILE) && eh->AttackType == ITEM_AT_MELEE) {
+		// HACK: use invtrav, so the effects are still applied on the attack target
+		eh->ProjectileAnimation = 78;
+	}
 
 	unsigned int i; //msvc6.0 can't cope with index variable scope
 
