@@ -25,24 +25,24 @@
 #include <cassert>
 #include <cstdio>
 
-bool checkALError(const char* msg, const char* status) {
+using namespace GemRB;
+
+bool checkALError(const char* msg, log_level level) {
 	int error = alGetError();
 	if (error != AL_NO_ERROR) {
-		printMessage("OpenAL", "%s", WHITE, msg);
-		print (": 0x%x ", error);
-		printStatus(status, YELLOW);
+		Log(level, "OpenAL", "%s: 0x%x - %s", msg, error, alGetString(error));
 		return true;
 	}
 	return false;
 }
 
-void showALCError(const char* msg, const char* status, ALCdevice *device) {
+void showALCError(const char* msg, log_level level, ALCdevice *device) {
 	int error = alcGetError(device);
-	printMessage("OpenAL", "%s", WHITE, msg );
 	if (error != AL_NO_ERROR) {
-		print (": 0x%x ", error);
+		Log(level, "OpenAL", "%s: 0x%x", msg, error);
+	} else {
+		Log(level, "OpenAL", "%s", msg);
 	}
-	printStatus(status, YELLOW);
 }
 
 void OpenALSoundHandle::SetPos(int XPos, int YPos) {
@@ -53,6 +53,7 @@ void OpenALSoundHandle::SetPos(int XPos, int YPos) {
 	};
 
 	alSourcefv(parent->Source, AL_POSITION, SourcePos);
+	checkALError("Unable to set source position", WARNING);
 }
 
 bool OpenALSoundHandle::Playing() {
@@ -72,19 +73,19 @@ void OpenALSoundHandle::StopLooping() {
 	if (!parent) return;
 
 	alSourcei(parent->Source, AL_LOOPING, 0);
+	checkALError("Unable to stop audio loop", WARNING);
 }
 
 void AudioStream::ClearProcessedBuffers()
 {
 	ALint processed = 0;
 	alGetSourcei( Source, AL_BUFFERS_PROCESSED, &processed );
-
-	checkALError("Failed to get processed buffers", "WARNING");
+	checkALError("Failed to get processed buffers", WARNING);
 
 	if (processed > 0) {
 		ALuint * b = new ALuint[processed];
 		alSourceUnqueueBuffers( Source, processed, b );
-		checkALError("Failed to unqueue buffers", "WARNING");
+		checkALError("Failed to unqueue buffers", WARNING);
 
 		if (delete_buffers) {
 #ifdef __APPLE__ // mac os x and iOS
@@ -98,7 +99,7 @@ void AudioStream::ClearProcessedBuffers()
 			}while(alGetError() != AL_NO_ERROR);
 #else
 			alDeleteBuffers(processed, b);
-			checkALError("Failed to delete buffers", "WARNING");
+			checkALError("Failed to delete buffers", WARNING);
 #endif
 		}
 
@@ -111,16 +112,19 @@ void AudioStream::ClearIfStopped()
 {
 	if (free || locked) return;
 
-	if (!alIsSource(Source)) return;
+	if (!Source || !alIsSource(Source)) {
+		checkALError("No AL Context", WARNING);
+		return;
+	}
 
 	ALint state;
 	alGetSourcei( Source, AL_SOURCE_STATE, &state );
-	if (!checkALError("Failed to check source state", "WARNING") &&
+	if (!checkALError("Failed to check source state", WARNING) &&
 			state == AL_STOPPED)
 	{
 		ClearProcessedBuffers();
 		alDeleteSources( 1, &Source );
-		checkALError("Failed to delete source", "WARNING");
+		checkALError("Failed to delete source", WARNING);
 		Source = 0;
 		Buffer = 0;
 		free = true;
@@ -133,10 +137,10 @@ void AudioStream::ClearIfStopped()
 
 void AudioStream::ForceClear()
 {
-	if (!alIsSource(Source)) return;
+	if (!Source || !alIsSource(Source)) return;
 
 	alSourceStop(Source);
-	checkALError("Failed to stop source", "WARNING");
+	checkALError("Failed to stop source", WARNING);
 	ClearProcessedBuffers();
 	ClearIfStopped();
 }
@@ -152,26 +156,47 @@ OpenALAudioDriver::OpenALAudioDriver(void)
 	ambim = NULL;
 }
 
+void OpenALAudioDriver::PrintDeviceList ()
+{
+	char *deviceList;
+
+	if (alcIsExtensionPresent(NULL, (ALchar*)"ALC_ENUMERATION_EXT") == AL_TRUE) { // try out enumeration extension
+		Log(MESSAGE, "OpenAL", "Usable audio output devices:");
+		deviceList = (char *)alcGetString(NULL, ALC_DEVICE_SPECIFIER);
+
+		while(deviceList && *deviceList) {
+			Log(MESSAGE,"OpenAL", "Devices: %s", deviceList);
+			deviceList+=strlen(deviceList)+1;
+		}
+		return;
+	}
+	Log(MESSAGE, "OpenAL", "No device enumeration present.");
+}
+
 bool OpenALAudioDriver::Init(void)
 {
+	Log(MESSAGE, "OpenAL", "Initializing OpenAL driver:\nAL Version:%s\nAL Renderer:%s\nAL Vendor:%s",
+		alGetString(AL_VERSION), alGetString(AL_RENDERER), alGetString(AL_VENDOR));
+
 	ALCdevice *device;
 	ALCcontext *context;
 
 	device = alcOpenDevice (NULL);
 	if (device == NULL) {
-		showALCError("Failed to open device", "ERROR", device);
+		showALCError("Failed to open device", ERROR, device);
+		PrintDeviceList();
 		return false;
 	}
 
 	context = alcCreateContext (device, NULL);
 	if (context == NULL) {
-		showALCError("Failed to create context", "ERROR", device);
+		showALCError("Failed to create context", ERROR, device);
 		alcCloseDevice (device);
 		return false;
 	}
 
 	if (!alcMakeContextCurrent (context)) {
-		showALCError("Failed to select context", "ERROR", device);
+		showALCError("Failed to select context", ERROR, device);
 		alcDestroyContext (context);
 		alcCloseDevice (device);
 		return false;
@@ -182,11 +207,16 @@ bool OpenALAudioDriver::Init(void)
 	int sources = CountAvailableSources(MAX_STREAMS+1);
 	num_streams = sources - 1;
 
-	printMessage( "OpenAL", "Allocated %d streams.%s", WHITE,
+	Log(MESSAGE, "OpenAL", "Allocated %d streams.%s",
 		num_streams, (num_streams < MAX_STREAMS ? " (Fewer than desired.)" : "" ));
 
 	stayAlive = true;
+#if	SDL_VERSION_ATLEAST(1, 3, 0)
+	/* as of changeset 3a041d215edc SDL_CreateThread has a 'name' parameter */
+	musicThread = SDL_CreateThread( MusicManager, "OpenALAudio", this );
+#else
 	musicThread = SDL_CreateThread( MusicManager, this );
+#endif
 
 	ambim = new AmbientMgrAL;
 	speech.free = true;
@@ -211,7 +241,7 @@ int OpenALAudioDriver::CountAvailableSources(int limit)
 	// (Might not be strictly necessary...)
 	i -= 2;
 
-	checkALError("Error while auto-detecting number of sources", "WARNING");
+	checkALError("Error while auto-detecting number of sources", WARNING);
 
 	// Return number of succesfully allocated sources
 	return i;
@@ -224,6 +254,12 @@ OpenALAudioDriver::~OpenALAudioDriver(void)
 		return;
 	}
 
+	stayAlive = false;
+// AmigaOS4 can't kill threads and would just wait forever
+#ifndef __amigaos4__
+	SDL_WaitThread(musicThread, NULL);
+#endif
+
 	for(int i =0; i<num_streams; i++) {
 		streams[i].ForceClear();
 	}
@@ -241,8 +277,6 @@ OpenALAudioDriver::~OpenALAudioDriver(void)
 		alcCloseDevice (device);
 	}
 	alutContext = NULL;
-	stayAlive = false;
-	SDL_WaitThread(musicThread, NULL);
 
 	SDL_DestroyMutex(musicMutex);
 	musicMutex = NULL;
@@ -271,13 +305,14 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 
 	//no cache entry...
 	alGenBuffers(1, &Buffer);
-	if (checkALError("Unable to create sound buffer", "ERROR")) {
+	if (checkALError("Unable to create sound buffer", ERROR)) {
 		return 0;
 	}
 
 	ResourceHolder<SoundMgr> acm(ResRef);
 	if (!acm) {
 		alDeleteBuffers( 1, &Buffer );
+		checkALError("Unable to delete buffer!", ERROR);
 		return 0;
 	}
 	int cnt = acm->get_length();
@@ -294,9 +329,9 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 	alBufferData( Buffer, GetFormatEnum( riff_chans, 16 ), memory, cnt1, samplerate );
 	free(memory);
 
-	if (checkALError("Unable to fill buffer", "ERROR")) {
+	if (checkALError("Unable to fill buffer", ERROR)) {
 		alDeleteBuffers( 1, &Buffer );
-		checkALError("Error deleting buffer", "WARNING");
+		checkALError("Error deleting buffer", WARNING);
 		return 0;
 	}
 
@@ -305,7 +340,7 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 	e->Length = ((cnt / riff_chans) * 1000) / samplerate;
 
 	buffercache.SetAt(ResRef, (void*)e);
-	//print("LoadSound: added %s to cache: %d. Cache size now %d\n", ResRef, e->Buffer, buffercache.GetCount());
+	//print("LoadSound: added %s to cache: %d. Cache size now %d", ResRef, e->Buffer, buffercache.GetCount());
 
 	if (buffercache.GetCount() > BUFFER_CACHE_SIZE) {
 		evictBuffer();
@@ -319,10 +354,10 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, int XPos, int YP
 	unsigned int time_length;
 
 	if(ResRef == NULL) {
-		if((flags & GEM_SND_SPEECH) && alIsSource(speech.Source)) {
+		if((flags & GEM_SND_SPEECH) && (speech.Source && alIsSource(speech.Source))) {
 			//So we want him to be quiet...
 			alSourceStop( speech.Source );
-			checkALError("Unable to stop speech", "WARNING");
+			checkALError("Unable to stop speech", WARNING);
 			speech.ClearProcessedBuffers();
 		}
 		return Holder<SoundHandle>();
@@ -337,7 +372,6 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, int XPos, int YP
 		*length = time_length;
 	}
 
-	ALuint Source;
 	ALfloat SourcePos[] = {
 		(float) XPos, (float) YPos, 0.0f
 	};
@@ -346,99 +380,76 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, int XPos, int YP
 	};
 
 	ieDword volume = 100;
+	ALint loop = (flags & GEM_SND_LOOPING ? 1 : 0);
+
+	AudioStream* stream = NULL;
 
 	if (flags & GEM_SND_SPEECH) {
-		//speech has a single channel, if a new speech started
-		//we stop the previous one
-		if(!speech.free && alIsSource(speech.Source)) {
-			alSourceStop( speech.Source );
-			checkALError("Unable to stop speech", "WARNING");
-			speech.ClearProcessedBuffers();
-		}
-		if(!alIsSource(speech.Source)) {
-			alGenSources( 1, &speech.Source );
-			if (checkALError("Error creating source for speech", "ERROR")) {
-				return Holder<SoundHandle>();
+		stream = &speech;
+
+		if (!(flags & GEM_SND_QUEUE)) {
+			//speech has a single channel, if a new speech started
+			//we stop the previous one
+
+			if(!speech.free && (speech.Source && alIsSource(speech.Source))) {
+				alSourceStop( speech.Source );
+				checkALError("Unable to stop speech", WARNING);
+				speech.ClearProcessedBuffers();
 			}
 		}
 
-		alSourcef( speech.Source, AL_PITCH, 1.0f );
-		alSourcefv( speech.Source, AL_VELOCITY, SourceVel );
-		alSourcei( speech.Source, AL_LOOPING, 0 );
-		alSourcef( speech.Source, AL_REFERENCE_DISTANCE, REFERENCE_DISTANCE );
-		checkALError("Unable to set speech parameters", "WARNING");
-		speech.free = false;
-		print("speech.free: %d source:%d\n", speech.free,speech.Source);
-
 		core->GetDictionary()->Lookup( "Volume Voices", volume );
-		alSourcef( speech.Source, AL_GAIN, 0.01f * volume );
-		alSourcei( speech.Source, AL_SOURCE_RELATIVE, flags & GEM_SND_RELATIVE );
-		alSourcefv( speech.Source, AL_POSITION, SourcePos );
-		assert(!speech.delete_buffers);
-		alSourcei( speech.Source, AL_BUFFER, Buffer );
-		checkALError("Unable to set speech parameters", "WARNING");
-		speech.Buffer = Buffer;
-		alSourcePlay( speech.Source );
-		if (checkALError("Unable to play speech", "ERROR")) {
+
+		loop = 0; // Speech ignores GEM_SND_LOOPING
+	} else {
+		// do we want to be able to queue sfx too? not so far. How would we?
+		for (int i = 0; i < num_streams; i++) {
+			streams[i].ClearIfStopped();
+			if (streams[i].free) {
+				stream = &streams[i];
+				break;
+			}
+		}
+
+		core->GetDictionary()->Lookup( "Volume SFX", volume );
+
+		if (stream == NULL) {
+			// Failed to assign new sound.
+			// The buffercache will handle deleting Buffer.
 			return Holder<SoundHandle>();
 		}
-		speech.handle = new OpenALSoundHandle(&speech);
-		return speech.handle.get();
 	}
 
-	int stream = -1;
-	for (int i = 0; i < num_streams; i++) {
-		streams[i].ClearIfStopped();
-		if (streams[i].free) {
-			stream = i;
-			break;
+	assert(stream);
+	ALuint Source = stream->Source;
+
+	if(!Source || !alIsSource(Source)) {
+		alGenSources( 1, &Source );
+		if (checkALError("Error creating source", ERROR)) {
+			return Holder<SoundHandle>();
 		}
 	}
 
-	if (stream == -1) {
-		// Failed to assign new sound.
-		// The buffercache will handle deleting Buffer.
-		return Holder<SoundHandle>();
-	}
-
-	// not speech
-	alGenSources( 1, &Source );
-	if (checkALError("Unable to create source", "ERROR")) {
-		return Holder<SoundHandle>();
-	}
-
 	alSourcef( Source, AL_PITCH, 1.0f );
 	alSourcefv( Source, AL_VELOCITY, SourceVel );
-	alSourcei( Source, AL_LOOPING, (flags & GEM_SND_LOOPING ? 1 : 0) );
+	alSourcei( Source, AL_LOOPING, loop);
 	alSourcef( Source, AL_REFERENCE_DISTANCE, REFERENCE_DISTANCE );
-	core->GetDictionary()->Lookup( "Volume SFX", volume );
 	alSourcef( Source, AL_GAIN, 0.01f * volume );
 	alSourcei( Source, AL_SOURCE_RELATIVE, flags & GEM_SND_RELATIVE );
 	alSourcefv( Source, AL_POSITION, SourcePos );
-	assert(!streams[stream].delete_buffers);
-	alSourcei( Source, AL_BUFFER, Buffer );
+	checkALError("Unable to set audio parameters", WARNING);
 
-	if (checkALError("Unable to set sound parameters", "ERROR")) {
+	assert(!stream->delete_buffers);
+
+	stream->Source = Source;
+	stream->free = false;
+
+	if (QueueALBuffer(Source, Buffer) != GEM_OK) {
 		return Holder<SoundHandle>();
 	}
 
-	streams[stream].Buffer = Buffer;
-	streams[stream].Source = Source;
-	streams[stream].free = false;
-	alSourcePlay( Source );
-
-	if (checkALError("Unable to play sound", "ERROR")) {
-		return Holder<SoundHandle>();
-	}
-
-	streams[stream].handle = new OpenALSoundHandle(&streams[stream]);
-	return streams[stream].handle.get();
-}
-
-bool OpenALAudioDriver::IsSpeaking()
-{
-	speech.ClearIfStopped();
-	return !speech.free;
+	stream->handle = new OpenALSoundHandle(stream);
+	return stream->handle.get();
 }
 
 void OpenALAudioDriver::UpdateVolume(unsigned int flags)
@@ -448,7 +459,7 @@ void OpenALAudioDriver::UpdateVolume(unsigned int flags)
 	if (flags & GEM_SND_VOL_MUSIC) {
 		SDL_mutexP( musicMutex );
 		core->GetDictionary()->Lookup("Volume Music", volume);
-		if (alIsSource(MusicSource))
+		if (MusicSource && alIsSource(MusicSource))
 			alSourcef(MusicSource, AL_GAIN, volume * 0.01f);
 		SDL_mutexV(musicMutex);
 	}
@@ -468,16 +479,16 @@ void OpenALAudioDriver::ResetMusics()
 {
 	MusicPlaying = false;
 	SDL_mutexP( musicMutex );
-	if (alIsSource(MusicSource)) {
+	if (MusicSource && alIsSource(MusicSource)) {
 		alSourceStop(MusicSource);
-		checkALError("Unable to stop music source", "WARNING");
+		checkALError("Unable to stop music source", WARNING);
 		alDeleteSources(1, &MusicSource );
-		checkALError("Unable to delete music source", "WARNING");
+		checkALError("Unable to delete music source", WARNING);
 		MusicSource = 0;
 		for (int i=0; i<MUSICBUFFERS; i++) {
 			if (alIsBuffer(MusicBuffer[i])) {
 				alDeleteBuffers(1, MusicBuffer+i);
-				checkALError("Unable to delete music buffer", "WARNING");
+				checkALError("Unable to delete music buffer", WARNING);
 			}
 		}
 	}
@@ -499,15 +510,15 @@ bool OpenALAudioDriver::Play()
 bool OpenALAudioDriver::Stop()
 {
 	SDL_mutexP( musicMutex );
-	if (!alIsSource( MusicSource )) {
+	if (!MusicSource || !alIsSource( MusicSource )) {
 		SDL_mutexV( musicMutex );
 		return false;
 	}
 	alSourceStop( MusicSource );
-	checkALError("Unable to stop music source", "WARNING");
+	checkALError("Unable to stop music source", WARNING);
 	MusicPlaying = false;
 	alDeleteSources( 1, &MusicSource );
-	checkALError("Unable to delete music source", "WARNING");
+	checkALError("Unable to delete music source", WARNING);
 	MusicSource = 0;
 	SDL_mutexV( musicMutex );
 	return true;
@@ -516,17 +527,19 @@ bool OpenALAudioDriver::Stop()
 bool OpenALAudioDriver::Pause()
 {
 	SDL_mutexP( musicMutex );
-	if (!alIsSource( MusicSource )) {
+	if (!MusicSource || !alIsSource( MusicSource )) {
 		SDL_mutexV( musicMutex );
 		return false;
 	}
 	alSourcePause(MusicSource);
-	checkALError("Unable to pause music source", "WARNING");
+	checkALError("Unable to pause music source", WARNING);
 	MusicPlaying = false;
 	SDL_mutexV( musicMutex );
 	((AmbientMgrAL*) ambim)->deactivate();
 #ifdef ANDROID
+#if SDL_COMPILEDVERSION < SDL_VERSIONNUM(1,3,0)
 	al_android_pause_playback(); //call AudioTrack.pause() from JNI
+#endif
 #endif
 	return true;
 }
@@ -534,15 +547,17 @@ bool OpenALAudioDriver::Pause()
 bool OpenALAudioDriver::Resume()
 {
 #ifdef ANDROID
+#if SDL_COMPILEDVERSION < SDL_VERSIONNUM(1,3,0)
 	al_android_resume_playback(); //call AudioTrack.play() from JNI
+#endif
 #endif
 	SDL_mutexP( musicMutex );
-	if (!alIsSource( MusicSource )) {
+	if (!MusicSource || !alIsSource( MusicSource )) {
 		SDL_mutexV( musicMutex );
 		return false;
 	}
 	alSourcePlay(MusicSource);
-	checkALError("Unable to resume music source", "WARNING");
+	checkALError("Unable to resume music source", WARNING);
 	MusicPlaying = true;
 	SDL_mutexV( musicMutex );
 	((AmbientMgrAL*) ambim)->activate();
@@ -559,18 +574,16 @@ int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
 		MusicPlaying = false;
 	}
 
-	checkALError("Uncaught AL error.", "WARNING"); // clear any previous errors
-
 	if (MusicBuffer[0] == 0) {
 		alGenBuffers( MUSICBUFFERS, MusicBuffer );
-		if (checkALError("Unable to create music buffers", "ERROR")) {
+		if (checkALError("Unable to create music buffers", ERROR)) {
 			return -1;
 		}
 	}
 
 	if (MusicSource == 0) {
 		alGenSources( 1, &MusicSource );
-		if (checkALError("Unable to create music source", "ERROR")) {
+		if (checkALError("Unable to create music source", ERROR)) {
 			return -1;
 		}
 
@@ -589,7 +602,7 @@ int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
 		alSourcefv( MusicSource, AL_POSITION, SourcePos );
 		alSourcefv( MusicSource, AL_VELOCITY, SourceVel );
 		alSourcei( MusicSource, AL_LOOPING, 0 );
-		checkALError("Unable to set music parameters", "WARNING");
+		checkALError("Unable to set music parameters", WARNING);
 	}
 
 	return 0;
@@ -598,13 +611,14 @@ int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
 void OpenALAudioDriver::UpdateListenerPos(int XPos, int YPos )
 {
 	alListener3f( AL_POSITION, (float) XPos, (float) YPos, 0.0f );
+	checkALError("Unable to update listener position.", WARNING);
 }
 
 void OpenALAudioDriver::GetListenerPos(int &XPos, int &YPos )
 {
 	ALfloat listen[3];
 	alGetListenerfv( AL_POSITION, listen );
-	if (checkALError("Unable to get listener pos", "ERROR")) return;
+	if (checkALError("Unable to get listener pos", ERROR)) return;
 	XPos = (int) listen[0];
 	YPos = (int) listen[1];
 }
@@ -621,9 +635,8 @@ bool OpenALAudioDriver::ReleaseStream(int stream, bool HardStop)
 
 	ALuint Source = streams[stream].Source;
 	alSourceStop(Source);
-	checkALError("Unable to stop source", "WARNING");
+	checkALError("Unable to stop source", WARNING);
 	streams[stream].ClearIfStopped();
-
 	return true;
 }
 
@@ -640,12 +653,14 @@ int OpenALAudioDriver::SetupNewStream( ieWord x, ieWord y, ieWord z,
 			break;
 		}
 	}
-	if (stream == -1) return -1;
+	if (stream == -1) {
+		Log(ERROR, "OpenAL", "No available audio streams out of %d", num_streams);
+		return -1;
+	}
 
-	checkALError("Uncaught AL error.", "WARNING"); // clear any previous errors
 	ALuint source;
 	alGenSources(1, &source);
-	if (checkALError("Unable to create new source", "ERROR")) {
+	if (checkALError("Unable to create new source", ERROR)) {
 		return -1;
 	}
 
@@ -656,7 +671,7 @@ int OpenALAudioDriver::SetupNewStream( ieWord x, ieWord y, ieWord z,
 	alSourcei( source, AL_REFERENCE_DISTANCE, REFERENCE_DISTANCE );
 	alSourcei( source, AL_ROLLOFF_FACTOR, point ? 1 : 0 );
 	alSourcei( source, AL_LOOPING, 0 );
-	checkALError("Unable to set stream parameters", "WARNING");
+	checkALError("Unable to set stream parameters", WARNING);
 
 	streams[stream].Buffer = 0;
 	streams[stream].Source = source;
@@ -688,20 +703,8 @@ int OpenALAudioDriver::QueueAmbient(int stream, const char* sound)
 
 	assert(!streams[stream].delete_buffers);
 
-	alSourceQueueBuffers(source, 1, &Buffer);
-	if (checkALError("Unable to queue ambient buffer","ERROR")) {
-		return -1;
-	}
-
-	// play
-	ALint state;
-	alGetSourcei( source, AL_SOURCE_STATE, &state );
-	if (!checkALError("Unable to query ambient source state", "ERROR") &&
-			state != AL_PLAYING)
-	{ // play on playing source would rewind it
-		alSourcePlay( source );
-		if (checkALError("Unable to play ambient source", "ERROR"))
-			return -1;
+	if (QueueALBuffer(source, Buffer) != GEM_OK) {
+		return GEM_ERROR;
 	}
 
 	return time_length;
@@ -714,7 +717,7 @@ void OpenALAudioDriver::SetAmbientStreamVolume(int stream, int volume)
 
 	ALuint source = streams[stream].Source;
 	alSourcef( source, AL_GAIN, 0.01f * volume );
-	checkALError("Unable to set ambient volume", "WARNING");
+	checkALError("Unable to set ambient volume", WARNING);
 }
 
 bool OpenALAudioDriver::evictBuffer()
@@ -739,7 +742,7 @@ bool OpenALAudioDriver::evictBuffer()
 			delete e;
 			buffercache.Remove(k);
 
-			//print("Removed buffer %s from ACMImp cache\n", k);
+			//print("Removed buffer %s from ACMImp cache", k);
 			break;
 		}
 		++n;
@@ -797,38 +800,52 @@ int OpenALAudioDriver::MusicManager(void* arg)
 		if (driver->MusicPlaying) {
 			ALint state;
 			alGetSourcei( driver->MusicSource, AL_SOURCE_STATE, &state );
-			if (checkALError("Unable to query music source state", "ERROR")) {
+			if (checkALError("Unable to query music source state", ERROR)) {
 				driver->MusicPlaying = false;
 				return -1;
 			}
 			switch (state) {
 				default:
-					printMessage("OpenAL", "WARNING: Unhandled Music state", WHITE );
-					printStatus("ERROR", YELLOW);
+					Log(ERROR, "OpenAL", "Unhandled Music state '%d'.", state);
+				//no break
+				case AL_PAUSED:
 					driver->MusicPlaying = false;
 					return -1;
 				case AL_INITIAL:
 					 {
-						printMessage("OPENAL", "Music in INITIAL State. AutoStarting\n", WHITE );
+						Log(MESSAGE, "OPENAL", "Music in INITIAL State. AutoStarting");
+						// ensure that MusicSource has no buffers attached by passing "NULL" buffer
+						alSourcei(driver->MusicSource, AL_BUFFER, 0);
+						checkALError("Unable to detach buffers from music source.", WARNING);
 						for (int i = 0; i < MUSICBUFFERS; i++) {
 							driver->MusicReader->read_samples( driver->music_memory, ACM_BUFFERSIZE >> 1 );
 							alBufferData( driver->MusicBuffer[i], AL_FORMAT_STEREO16,
 								driver->music_memory, ACM_BUFFERSIZE,
 								driver->MusicReader->get_samplerate() );
 						}
+						checkALError("Unable to buffer data.", ERROR);
+						// FIXME: determine if we should error out if any data fails to buffer
 						alSourceQueueBuffers( driver->MusicSource, MUSICBUFFERS, driver->MusicBuffer );
-						if (alIsSource( driver->MusicSource )) {
-							alSourcePlay( driver->MusicSource );
-							checkALError("Error playing music source", "ERROR");
+						if (!checkALError("Unable to queue buffer.", ERROR)) {
+							if (driver->MusicSource && alIsSource( driver->MusicSource )) {
+								alSourcePlay( driver->MusicSource );
+								if (!checkALError("Error playing music source", ERROR)) {
+									// no errors happened
+									bFinished = AL_FALSE;
+									break;
+								}
+							}
 						}
-						bFinished = AL_FALSE;
+						// if all had gone well we would have broken out
+						driver->MusicPlaying = false;
+						return -1;
 					}
 					break;
 				case AL_STOPPED:
-					printMessage("OpenAL", "WARNING: Buffer Underrun. AutoRestarting Stream Playback\n", WHITE );
-					if (alIsSource( driver->MusicSource )) {
+					Log(MESSAGE, "OpenAL", "WARNING: Buffer Underrun. AutoRestarting Stream Playback");
+					if (driver->MusicSource && alIsSource( driver->MusicSource )) {
 						alSourcePlay( driver->MusicSource );
-						checkALError("Error playing music source", "ERROR");
+						checkALError("Error playing music source", ERROR);
 					}
 					break;
 				case AL_PLAYING:
@@ -836,7 +853,7 @@ int OpenALAudioDriver::MusicManager(void* arg)
 			}
 			ALint processed;
 			alGetSourcei( driver->MusicSource, AL_BUFFERS_PROCESSED, &processed );
-			if (checkALError("Unable to query music source state", "ERROR")) {
+			if (checkALError("Unable to query music source state", ERROR)) {
 				driver->MusicPlaying = false;
 				return -1;
 			}
@@ -845,7 +862,7 @@ int OpenALAudioDriver::MusicManager(void* arg)
 				while (processed) {
 					ALuint BufferID;
 					alSourceUnqueueBuffers( driver->MusicSource, 1, &BufferID );
-					if (checkALError("Unable to unqueue music buffers", "ERROR")) {
+					if (checkALError("Unable to unqueue music buffers", ERROR)) {
 						driver->MusicPlaying = false;
 						return -1;
 					}
@@ -856,26 +873,26 @@ int OpenALAudioDriver::MusicManager(void* arg)
 						if (size != 0)
 							bFinished = AL_TRUE;
 						if (bFinished) {
-							printMessage("OpenAL", "Playing Next Music\n", WHITE );
+							Log(MESSAGE, "OpenAL", "Playing Next Music");
 							core->GetMusicMgr()->PlayNext();
 							if (driver->MusicPlaying) {
-								printMessage( "OpenAL", "Queuing New Music\n", WHITE );
+								Log(MESSAGE, "OpenAL", "Queuing New Music");
 								driver->MusicReader->read_samples( ( driver->music_memory + cnt ), size >> 1 );
 								bFinished = AL_FALSE;
 							} else {
-								printMessage( "OpenAL", "No Other Music to play\n", WHITE );
+								Log(MESSAGE, "OpenAL", "No Other Music to play");
 								memset( driver->music_memory + cnt, 0, size );
 								driver->MusicPlaying = false;
 								break;
 							}
 						}
 						alBufferData( BufferID, AL_FORMAT_STEREO16, driver->music_memory, ACM_BUFFERSIZE, driver->MusicReader->get_samplerate() );
-						if (checkALError("Unable to buffer music data", "ERROR")) {
+						if (checkALError("Unable to buffer music data", ERROR)) {
 							driver->MusicPlaying = false;
 							return -1;
 						}
 						alSourceQueueBuffers( driver->MusicSource, 1, &BufferID );
-						if (checkALError("Unable to queue music buffers", "ERROR")) {
+						if (checkALError("Unable to queue music buffers", ERROR)) {
 							driver->MusicPlaying = false;
 							return -1;
 						}
@@ -896,35 +913,60 @@ void OpenALAudioDriver::QueueBuffer(int stream, unsigned short bits,
 	ALuint Buffer;
 
 	alGenBuffers(1, &Buffer);
-	if (checkALError("Unable to create buffer", "ERROR")) {
+	if (checkALError("Unable to create buffer", ERROR)) {
 		return;
 	}
 
 	alBufferData(Buffer, GetFormatEnum(channels, bits), memory, size, samplerate);
-	if (checkALError("Unable to buffer data", "ERROR")) {
+	if (checkALError("Unable to buffer data", ERROR)) {
 		return;
 	}
 
 	streams[stream].delete_buffers = true;
 	streams[stream].ClearProcessedBuffers();
 
-	alSourceQueueBuffers(streams[stream].Source, 1, &Buffer );
-	if (checkALError("Unable to queue buffer", "ERROR")) {
-		return;
+	QueueALBuffer(streams[stream].Source, Buffer);
+}
+
+// !!!!!!!!!!!!!!!
+// Private Methods
+// !!!!!!!!!!!!!!!
+
+int OpenALAudioDriver::QueueALBuffer(ALuint source, ALuint buffer)
+{
+#ifdef _DEBUG
+	ALint frequency, bits, channels;
+	alGetBufferi(buffer, AL_FREQUENCY, &frequency);
+	alGetBufferi(buffer, AL_BITS, &bits);
+	alGetBufferi(buffer, AL_CHANNELS, &channels);
+	checkALError("Error querying buffer properties.", WARNING);
+	Log(DEBUG, "OpenAL", "Attempting to buffer audio source:%d\nFrequency:%d\nBits:%d\nChannels:%d",
+		source, frequency, bits, channels);
+#endif
+	ALint type;
+	alGetSourcei(source, AL_SOURCE_TYPE, &type);
+	if (type == AL_STATIC || checkALError("Cannot get AL source type.", ERROR)) {
+		Log(ERROR, "OpenAL", "Cannot queue a buffer to a static source.");
+		return GEM_ERROR;
+	}
+	alSourceQueueBuffers(source, 1, &buffer);
+	if (checkALError("Unable to queue buffer", ERROR)) {
+		return GEM_ERROR;
 	}
 
 	ALenum state;
-	alGetSourcei(streams[stream].Source, AL_SOURCE_STATE, &state);
-	if (checkALError("Unable to query source state", "ERROR")) {
-		return;
+	alGetSourcei(source, AL_SOURCE_STATE, &state);
+	if (checkALError("Unable to query source state", ERROR)) {
+		return GEM_ERROR;
 	}
 
+	// queueing always implies playing for us
 	if (state != AL_PLAYING ) {
-		alSourcePlay(streams[stream].Source);
-		checkALError("Unable to play source", "ERROR");
+		alSourcePlay(source);
+		checkALError("Unable to play source", ERROR);
+		return GEM_ERROR;
 	}
-
-	return;
+	return GEM_OK;
 }
 
 #include "plugindef.h"
