@@ -25,10 +25,11 @@
 
 #include "TlkOverride.h"
 
+namespace GemRB {
+
 class TLKImporter : public StringMgr {
 private:
 	DataStream* str;
-	bool autoFree;
 
 	//Data
 	ieDword StrRefCount, Offset;
@@ -41,7 +42,7 @@ public:
 	void OpenAux();
 	/** purge string defs coming from saved game */
 	void CloseAux();
-	bool Open(DataStream* stream, bool autoFree = true);
+	bool Open(DataStream* stream);
 	/** construct a new custom string */
 	ieStrRef UpdateString(ieStrRef strref, const char *newvalue);
 	/** resolve a string reference */
@@ -59,10 +60,13 @@ private:
 	/**returns the decoded length of the built-in token
 		 if dest is not NULL it also returns the decoded value */
 	int BuiltinToken(char* Token, char* dest);
+	int ClassStrRef(int slot);
 	int RaceStrRef(int slot);
 	int GenderStrRef(int slot, int malestrref, int femalestrref);
 	char *Gabber();
 	char *CharName(int slot);
 };
 
+}
+
 #endif
